/* Z Kit - hardware/storage medium/ROM cartridge/SNES Game Pak.h
 _____  _______________
/_   /_/  -_/_   _/  _ |
 /____/\___/ /__//___/_| Kit
Copyright (C) 2006-2018 Manuel Sainz de Baranda y Goñi.
Released under the terms of the GNU Lesser General Public License v3. */

#ifndef _Z_hardware_storage_medium_ROM_cartridge_SNES_Game_Pak_H_
#define _Z_hardware_storage_medium_ROM_cartridge_SNES_Game_Pak_H_

#include <Z/types/base.h>

Z_DEFINE_STRICT_STRUCTURE (
	zuint8	name[21];
	zuint8	layout;
	zuint8	type;
	zuint8	rom_size;
	zuint8	ram_size;
	zuint8	region;
	zuint8	license;
	zuint8	version;
	zuint16	checksum;
	zuint16	snes_checksum;
	zuint8	unknown1[4];
	zuint8	native_cop_interrupt_rutine_pointer;
	zuint8	native_brk_interrupt_rutine_pointer;
	zuint8	native_abort_interrupt_rutine_pointer;
	zuint8	native_unused;
	zuint8	native_irq_interrupt_rutine_pointer;
	zuint8	unknown2[4];
	zuint8	emulation_cop_interrupt_rutine_pointer;
	zuint8	emulation_unused;
	zuint8	emulation_abort_interrupt_rutine_pointer;
	zuint8	emulation_nmi_interrupt_rutine_pointer;
	zuint8	emulation_reset_interrupt_rutine_pointer;
	zuint8	emulation_irq_or_brk_interrupt_rutine_pointer;
) ZSNESGamePakHeader;

#define Z_SNES_GAME_PAK_LAYOUT_LO_ROM	   0x20
#define Z_SNES_GAME_PAK_LAYOUT_HI_ROM	   0x21
#define Z_SNES_GAME_PAK_LAYOUT_LO_ROM_FAST 0x30
#define Z_SNES_GAME_PAK_LAYOUT_HI_ROM_FAST 0x31

#define Z_SNES_GAME_PAK_TYPE_ROM_ONLY			 0x00
#define Z_SNES_GAME_PAK_TYPE_ROM_AND_RAM		 0x01
#define Z_SNES_GAME_PAK_TYPE_ROM_AND_SAVE_RAM		 0x02
#define Z_SNES_GAME_PAK_TYPE_ROM_AND_SUPER_FX_A		 0x13
#define Z_SNES_GAME_PAK_TYPE_ROM_AND_SUPER_FX_B		 0x14
#define Z_SNES_GAME_PAK_TYPE_ROM_SAVE_RAM_AND_SUPER_FX_A 0x15
#define Z_SNES_GAME_PAK_TYPE_ROM_SAVE_RAM_AND_SUPER_FX_B 0x1A
#define Z_SNES_GAME_PAK_TYPE_ROM_AND_SA_1_A		 0x34
#define Z_SNES_GAME_PAK_TYPE_ROM_AND_SA_1_B		 0x35

#define Z_SNES_GAME_PAK_MEMORY_SIZE_0	    0x00
#define Z_SNES_GAME_PAK_MEMORY_SIZE_2_KIB   0x01
#define Z_SNES_GAME_PAK_MEMORY_SIZE_4_KIB   0x02
#define Z_SNES_GAME_PAK_MEMORY_SIZE_8_KIB   0x03
#define Z_SNES_GAME_PAK_MEMORY_SIZE_16_KIB  0x04
#define Z_SNES_GAME_PAK_MEMORY_SIZE_32_KIB  0x05
#define Z_SNES_GAME_PAK_MEMORY_SIZE_64_KIB  0x06
#define Z_SNES_GAME_PAK_MEMORY_SIZE_128_KIB 0x07
#define Z_SNES_GAME_PAK_MEMORY_SIZE_256_KIB 0x08
#define Z_SNES_GAME_PAK_MEMORY_SIZE_512_KIB 0x09
#define Z_SNES_GAME_PAK_MEMORY_SIZE_1_MIB   0x0A
#define Z_SNES_GAME_PAK_MEMORY_SIZE_2_MIB   0x0B
#define Z_SNES_GAME_PAK_MEMORY_SIZE_4_MIB   0x0C

#define Z_SNES_GAME_PAK_REGION_JP		0
#define Z_SNES_GAME_PAK_REGION_US		1
#define Z_SNES_GAME_PAK_REGION_AU_EU_OC_AND_AS	2
#define Z_SNES_GAME_PAK_REGION_SE		3
#define Z_SNES_GAME_PAK_REGION_FI		4
#define Z_SNES_GAME_PAK_REGION_DK		5
#define Z_SNES_GAME_PAK_REGION_FR		6
#define Z_SNES_GAME_PAK_REGION_NL		7
#define Z_SNES_GAME_PAK_REGION_ES		8
#define Z_SNES_GAME_PAK_REGION_DE_AT_AND_CH	9
#define Z_SNES_GAME_PAK_REGION_IT	       10
#define Z_SNES_GAME_PAK_REGION_HK_AND_CN       11
#define Z_SNES_GAME_PAK_REGION_ID	       12
#define Z_SNES_GAME_PAK_REGION_KR	       13

#define Z_SNES_GAME_PAK_LICENSE_NINTENDO			   1
#define Z_SNES_GAME_PAK_LICENSE_IMAGINEER_ZOOM			   3
#define Z_SNES_GAME_PAK_LICENSE_ZAMUSE				   5
#define Z_SNES_GAME_PAK_LICENSE_FALCOM				   6
#define Z_SNES_GAME_PAK_LICENSE_CAPCOM				   8
#define Z_SNES_GAME_PAK_LICENSE_HOT_B				   9
#define Z_SNES_GAME_PAK_LICENSE_JALECO				  10
#define Z_SNES_GAME_PAK_LICENSE_COCONUTS			  11
#define Z_SNES_GAME_PAK_LICENSE_RAGE_SOFTWARE			  12
#define Z_SNES_GAME_PAK_LICENSE_TECHNOS				  14
#define Z_SNES_GAME_PAK_LICENSE_MEBIO_SOFTWARE			  15
#define Z_SNES_GAME_PAK_LICENSE_GREMLIN_GRAPHICS		  18
#define Z_SNES_GAME_PAK_LICENSE_ELECTRONIC_ARTS			  19
#define Z_SNES_GAME_PAK_LICENSE_COBRA_TEAM			  21
#define Z_SNES_GAME_PAK_LICENSE_HUMAN_FIELD			  22
#define Z_SNES_GAME_PAK_LICENSE_KOEI				  23
#define Z_SNES_GAME_PAK_LICENSE_HUDSON_SOFT			  24
#define Z_SNES_GAME_PAK_LICENSE_YANOMAN				  26
#define Z_SNES_GAME_PAK_LICENSE_TECMO				  28
#define Z_SNES_GAME_PAK_LICENSE_OPEN_SYSTEM			  30
#define Z_SNES_GAME_PAK_LICENSE_VIRGIN_GAMES			  31
#define Z_SNES_GAME_PAK_LICENSE_KSS				  32
#define Z_SNES_GAME_PAK_LICENSE_SUNSOFT				  33
#define Z_SNES_GAME_PAK_LICENSE_POW				  34
#define Z_SNES_GAME_PAK_LICENSE_MICRO_WORLD			  35
#define Z_SNES_GAME_PAK_LICENSE_ENIX				  38
#define Z_SNES_GAME_PAK_LICENSE_LORICIEL_ELECTRO_BRAIN		  39
#define Z_SNES_GAME_PAK_LICENSE_KEMCO				  40
#define Z_SNES_GAME_PAK_LICENSE_SETA				  41
#define Z_SNES_GAME_PAK_LICENSE_VISIT				  45
#define Z_SNES_GAME_PAK_LICENSE_CARROZZERIA			  49
#define Z_SNES_GAME_PAK_LICENSE_DYNAMIC				  50
#define Z_SNES_GAME_PAK_LICENSE_NINTENDO			  51
#define Z_SNES_GAME_PAK_LICENSE_MAGIFACT			  52
#define Z_SNES_GAME_PAK_LICENSE_HECT				  53
#define Z_SNES_GAME_PAK_LICENSE_EMPIRE_SOFTWARE			  60
#define Z_SNES_GAME_PAK_LICENSE_LORICIEL			  61
#define Z_SNES_GAME_PAK_LICENSE_SEIKA				  64
#define Z_SNES_GAME_PAK_LICENSE_UBI_SOFT			  65
#define Z_SNES_GAME_PAK_LICENSE_SYSTEM_3			  70
#define Z_SNES_GAME_PAK_LICENSE_SPECTRUM_HOLOBYTE		  71
#define Z_SNES_GAME_PAK_LICENSE_IREM				  73
#define Z_SNES_GAME_PAK_LICENSE_RAYA_SYSTEMS_SCULPTURED_SOFTWARE  75
#define Z_SNES_GAME_PAK_LICENSE_RENOVATION_PRODUCTS		  76
#define Z_SNES_GAME_PAK_LICENSE_MALIBU_GAMES_BLACK_PEARL	  77
#define Z_SNES_GAME_PAK_LICENSE_US_GOLD				  79
#define Z_SNES_GAME_PAK_LICENSE_ABSOLUTE_ENTERTAINMENT		  80
#define Z_SNES_GAME_PAK_LICENSE_ACCLAIM				  81
#define Z_SNES_GAME_PAK_LICENSE_ACTIVISION			  82
#define Z_SNES_GAME_PAK_LICENSE_AMERICAN_SAMMY			  83
#define Z_SNES_GAME_PAK_LICENSE_GAMETEK				  84
#define Z_SNES_GAME_PAK_LICENSE_HI_TECH_EXPRESSIONS		  85
#define Z_SNES_GAME_PAK_LICENSE_LJN_TOYS			  86
#define Z_SNES_GAME_PAK_LICENSE_MINDSCAPE			  90
#define Z_SNES_GAME_PAK_LICENSE_TRADEWEST			  93
#define Z_SNES_GAME_PAK_LICENSE_AMERICAN_SOFTWORKS		  95
#define Z_SNES_GAME_PAK_LICENSE_TITUS				  96
#define Z_SNES_GAME_PAK_LICENSE_VIRGIN_INTERACTIVE_ENTERTAINMENT  97
#define Z_SNES_GAME_PAK_LICENSE_MAXIS				  98
#define Z_SNES_GAME_PAK_LICENSE_OCEAN				 103
#define Z_SNES_GAME_PAK_LICENSE_ELECTRONIC_ARTS			 105
#define Z_SNES_GAME_PAK_LICENSE_LASER_BEAM			 107
#define Z_SNES_GAME_PAK_LICENSE_ELITE				 110
#define Z_SNES_GAME_PAK_LICENSE_ELECTRO_BRAIN			 111
#define Z_SNES_GAME_PAK_LICENSE_INFOGRAMES			 112
#define Z_SNES_GAME_PAK_LICENSE_INTERPLAY			 113
#define Z_SNES_GAME_PAK_LICENSE_LUCASARTS			 114
#define Z_SNES_GAME_PAK_LICENSE_PARKER_BROTHERS			 115
#define Z_SNES_GAME_PAK_LICENSE_STORM				 117
#define Z_SNES_GAME_PAK_LICENSE_THZ_SOFTWARE			 120
#define Z_SNES_GAME_PAK_LICENSE_ACCOLADE			 121
#define Z_SNES_GAME_PAK_LICENSE_TRIFFIX_ENTERTAINMENT		 122
#define Z_SNES_GAME_PAK_LICENSE_MICROPROSE			 124
#define Z_SNES_GAME_PAK_LICENSE_KEMCO				 127
#define Z_SNES_GAME_PAK_LICENSE_MISAWA				 128
#define Z_SNES_GAME_PAK_LICENSE_TEICHIO				 129
#define Z_SNES_GAME_PAK_LICENSE_NAMCO				 130
#define Z_SNES_GAME_PAK_LICENSE_LOZC				 131
#define Z_SNES_GAME_PAK_LICENSE_KOEI				 132
#define Z_SNES_GAME_PAK_LICENSE_TOKUMA_SHOTEN_INTERMEDIA	 134
#define Z_SNES_GAME_PAK_LICENSE_DATAM_POLYSTAR			 136
#define Z_SNES_GAME_PAK_LICENSE_BULLET_PROOF_SOFTWARE		 139
#define Z_SNES_GAME_PAK_LICENSE_VIC_TOKAI			 140
#define Z_SNES_GAME_PAK_LICENSE_CHARACTER_SOFT			 142
#define Z_SNES_GAME_PAK_LICENSE_I_MAX				 143
#define Z_SNES_GAME_PAK_LICENSE_TAKARA				 144
#define Z_SNES_GAME_PAK_LICENSE_CHUN_SOFT			 145
#define Z_SNES_GAME_PAK_LICENSE_VIDEO_SYSTEM			 146
#define Z_SNES_GAME_PAK_LICENSE_BEC				 147
#define Z_SNES_GAME_PAK_LICENSE_VARIE				 149
#define Z_SNES_GAME_PAK_LICENSE_KANECO				 151
#define Z_SNES_GAME_PAK_LICENSE_PACK_IN_VIDEO			 153
#define Z_SNES_GAME_PAK_LICENSE_NICHIBUTSU			 154
#define Z_SNES_GAME_PAK_LICENSE_TECMO				 155
#define Z_SNES_GAME_PAK_LICENSE_IMAGINEER			 156
#define Z_SNES_GAME_PAK_LICENSE_TELENET				 160
#define Z_SNES_GAME_PAK_LICENSE_KONAMI				 164
#define Z_SNES_GAME_PAK_LICENSE_K_AMUSEMENT_LEASING		 165
#define Z_SNES_GAME_PAK_LICENSE_TAKARA				 167
#define Z_SNES_GAME_PAK_LICENSE_TECHNOS_JAPAN			 169
#define Z_SNES_GAME_PAK_LICENSE_JVC				 170
#define Z_SNES_GAME_PAK_LICENSE_TOEI_ANIMATION			 172
#define Z_SNES_GAME_PAK_LICENSE_TOHO				 173
#define Z_SNES_GAME_PAK_LICENSE_NAMCO				 175
#define Z_SNES_GAME_PAK_LICENSE_ASCII_ACTIVISON			 177
#define Z_SNES_GAME_PAK_LICENSE_BANDAI_AMERICA			 178
#define Z_SNES_GAME_PAK_LICENSE_ENIX				 180
#define Z_SNES_GAME_PAK_LICENSE_HALKEN				 182
#define Z_SNES_GAME_PAK_LICENSE_CULTURE_BRAIN			 186
#define Z_SNES_GAME_PAK_LICENSE_SUNSOFT				 187
#define Z_SNES_GAME_PAK_LICENSE_TOSHIBA_EMI			 188
#define Z_SNES_GAME_PAK_LICENSE_SONY_IMAGESOFT			 189
#define Z_SNES_GAME_PAK_LICENSE_SAMMY				 191
#define Z_SNES_GAME_PAK_LICENSE_TAITO				 192
#define Z_SNES_GAME_PAK_LICENSE_KEMCO				 194
#define Z_SNES_GAME_PAK_LICENSE_SQUARE				 195
#define Z_SNES_GAME_PAK_LICENSE_TOKUMA_SOFT			 196
#define Z_SNES_GAME_PAK_LICENSE_DATA_EAST			 197
#define Z_SNES_GAME_PAK_LICENSE_TONKIN_HOUSE			 198
#define Z_SNES_GAME_PAK_LICENSE_KOEI				 200
#define Z_SNES_GAME_PAK_LICENSE_KONAMI_USA			 202
#define Z_SNES_GAME_PAK_LICENSE_NTVIC				 203
#define Z_SNES_GAME_PAK_LICENSE_MELDAC				 205
#define Z_SNES_GAME_PAK_LICENSE_PONY_CANYON			 206
#define Z_SNES_GAME_PAK_LICENSE_SOTSU_AGENCY_SUNRISE		 207
#define Z_SNES_GAME_PAK_LICENSE_DISCO_TAITO			 208
#define Z_SNES_GAME_PAK_LICENSE_SOFEL				 209
#define Z_SNES_GAME_PAK_LICENSE_QUEST				 210
#define Z_SNES_GAME_PAK_LICENSE_SIGMA				 211
#define Z_SNES_GAME_PAK_LICENSE_NAXAT				 214
#define Z_SNES_GAME_PAK_LICENSE_CAPCOM				 216
#define Z_SNES_GAME_PAK_LICENSE_BANPRESTO			 217
#define Z_SNES_GAME_PAK_LICENSE_TOMY				 218
#define Z_SNES_GAME_PAK_LICENSE_ACCLAIM				 219
#define Z_SNES_GAME_PAK_LICENSE_NCS				 221
#define Z_SNES_GAME_PAK_LICENSE_HUMAN_ENTERTAINMENT		 222
#define Z_SNES_GAME_PAK_LICENSE_ALTRON				 223
#define Z_SNES_GAME_PAK_LICENSE_JALECO				 224
#define Z_SNES_GAME_PAK_LICENSE_YUTAKA				 226
#define Z_SNES_GAME_PAK_LICENSE_T_AND_ESOFT			 228
#define Z_SNES_GAME_PAK_LICENSE_EPOCH				 229
#define Z_SNES_GAME_PAK_LICENSE_ATHENA				 231
#define Z_SNES_GAME_PAK_LICENSE_ASMIK				 232
#define Z_SNES_GAME_PAK_LICENSE_NATSUME				 233
#define Z_SNES_GAME_PAK_LICENSE_KING_RECORDS			 234
#define Z_SNES_GAME_PAK_LICENSE_ATLUS				 235
#define Z_SNES_GAME_PAK_LICENSE_SONY_MUSIC_ENTERTAINMENT	 236
#define Z_SNES_GAME_PAK_LICENSE_IGS				 238
#define Z_SNES_GAME_PAK_LICENSE_MOTOWN_SOFTWARE			 241
#define Z_SNES_GAME_PAK_LICENSE_LEFT_FIELD_ENTERTAINMENT	 242
#define Z_SNES_GAME_PAK_LICENSE_BEAM_SOFTWARE			 243
#define Z_SNES_GAME_PAK_LICENSE_TEC_MAGIK			 244
#define Z_SNES_GAME_PAK_LICENSE_CYBERSOFT			 249
#define Z_SNES_GAME_PAK_LICENSE_HUDSON_SOFT			 255

#endif /* _Z_hardware_storage_medium_ROM_cartridge_SNES_Game_Pak_H_ */
