#include <stdlib.h>

#include "emulation/CPU/Z80.h"

#include "debug.h"

#include "memory.h"
#include "io.h"
#include "interrupt.h"

/* Allocate and initialize a Z80 instance.
 * Return a pointer to the instance so that it can be used
 * by other functions.
 */
Z80 * zemu_init(void)
{
    Z80 * instance = malloc(sizeof(Z80));

    /* Not sure what context needs to be initialized to.
     * NULL for now. This will inevitably break.
     */
    instance->context = NULL;

    /* Memory read and write callbacks.
     * These are autogenerated in memory.c/memory.h.
     */
    instance->read = zemu_memory_read;
    instance->write = zemu_memory_write;

    /* IO read and write callbacks.
     * These are autogenerated in io.c/io.h
     */
    instance->in = zemu_io_in;
    instance->out = zemu_io_out;

    /* Interrupt opcode fetch callback.
     * This is autogenerated in interrupt.c/interrupt.h.
     */
    instance->int_data = zemu_interrupt_int_data;

    /* HALT callback.
     * Set to NULL for now.
     * This will need to be set so we can see if the CPU is halted.
     */
    instance->halt = zemu_debug_halt;

    /* Return the now-initialized instance. */
    return instance;
}

void zemu_free(Z80 * instance)
{
    free(instance);
}

void zemu_power_on(Z80 * instance)
{
    z80_power(instance, TRUE);
}

void zemu_power_off(Z80 * instance)
{
    z80_power(instance, FALSE);
}

void zemu_reset(Z80 * instance)
{
    z80_reset(instance);
}
