/* Z Kit - constants/chemical elements.h
 _____  _______________
/_   /_/  -_/_   _/  _ |
 /____/\___/ /__//___/_| Kit
Copyright (C) 2006-2018 Manuel Sainz de Baranda y Goñi.
Released under the terms of the GNU Lesser General Public License v3. */

#ifndef _Z_constants_chemical_elements_H_
#define _Z_constants_chemical_elements_H_

#include <Z/keys/science/chemical elements.h>
#include <Z/keys/science/electricity.h>
#include <Z/keys/science/magnetism.h>

/*  01											 18
  .----.									       .----.
  |1   |									       |2   |
1 | H  | 02							13   14   15   16   17 | He |
  |----+----.						      .------------------------+----|
  |3   |4   |						      |5   |6   |7   |8   |9   |10  |
2 | Li | Be |						      | B  | C  | N  | O  | F  | Ne |
  |----+----|						      |----+----+----+----+----+----|
  |11  |12  |						      |13  |14  |15  |16  |17  |18  |
3 | Na | Mg | 03   04	05   06   07   08   09	 10   11   12 | Al | Si | P  | S  | Cl | Ar |
  |----+----+-------------------------------------------------+----+----+----+----+----+----|
  |19  |20  |21  |22  |23  |24	|25  |26  |27  |28  |29  |30  |31  |32  |33  |34  |35  |36  |
4 | K  | Ca | Sc | Ti | V  | Cr | Mn | Fe | Co | Ni | Cu | Zn | Ga | Ge | As | Se | Br | Kr |
  |----+----+----+----+----+----+----+----+----+----+----+----+----+----+----+----+----+----|
  |37  |38  |39  |40  |41  |42	|43  |44  |45  |46  |47  |48  |49  |50  |51  |52  |53  |54  |
5 | Rb | Sr | Y  | Zr | Nb | Mo | Tc | Ru | Rh | Pd | Ag | Cd | In | Sn | Sb | Te | I  | Xe |
  |----+----+----+----+----+----+----+----+----+----+----+----+----+----+----+----+----+----|
  |55  |56  |57  |72  |73  |74	|75  |76  |77  |78  |79  |80  |81  |82  |83  |84  |85  |86  |
6 | Cs | Ba | La | Hf | Ta | W	| Re | Os | Ir | Pt | Au | Hg | Tl | Pb | Bi | Po | At | Rn |
  |----+----+----+----+----+----+----+----+----+----+----+----+----+----+----+----+----+----|
  |87  |88  |89  |104 |105 |106 |107 |108 |109 |110 |111 |112 |113 |114 |115 |116 |117 |118 |
7 | Fr | Ra | Ac | Rf | Db | Sg | Bh | Hs | Mt | Ds | Rg | Cn | Nh | Fl | Mc | Lv | Ts | Og |
  '-----------------------------------------------------------------------------------------'

		 .---------------------------------------------------------------------.
		 |58  |59  |60	|61  |62  |63  |64  |65  |66  |67  |68  |69  |70  |71  |
     Lanthanides | Ce | Pr | Nd | Pm | Sm | Eu | Gd | Tb | Dy | Ho | Er | Tm | Yb | Lu |
		 |----+----+----+----+----+----+----+----+----+----+----+----+----+----|
		 |90  |91  |92	|93  |94  |95  |96  |97  |98  |99  |100 |101 |102 |103 |
     Actinides	 | Th | Pa | U	| Np | Pu | Am | Cm | Bk | Cf | Es | Fm | Md | No | Lr |
		 '---------------------------------------------------------------------'

 ___________________________________________________________________________
/\									    \
\_| Atomic Structure		       Bulk physical properties		     |
  | -----------------------	       ------------------------		     |
  | Atomic mass:	 u    	       Density:		  		     |
  | Atomic radius:	 pm	       Density (liquid):  		     |
  | Covalent radius:	 pm	       Molar volume:	  		     |
  | Van de Walls radius: pm 	       Brinell hardness: MPa		     |
  |				       Vickers hardness: MPa		     |
  | Nuclear properties		       Bulk modulus:	 GPa		     |
  | ------------------------	       Shear modulus:	 GPa		     |
  | Neutron cross section: b	       Young's modulus:  GPa		     |
  |				       Sound speed:	 m/s		     |
  | Reactivity								     |
  | --------------------------------   Electrical properties		     |
  | Electronegativity: Pauling scale   ----------------------------	     |
  | Electron affinity: kJ/mol	       Electrical conductivity: S/m	     |
  |				       Electrical resistivity:	Ω·m	     |
  | Thermal properties		       Superconducting point:	K	     |
  | ------------------------------					     |
  | Melting point:	  °C	       Magnetic properties		     |
  | Boiling point:	  °C	       ------------------------------------- |
  | Critical pressure:	  MPa	       Mass magnetic susceptibility:  m³/kg  |
  | Critical temperature: K	       Molar magnetic susceptibility: m³/mol |
  | Fusion Heat:	  kJ/mol					     |
  | Vaporization heat:	  kJ/mol					     |
  | Specific heat:	  J/(kg·K)					     |
  | Neel point:		  K						     |
  | Thermal conductivity: W/(m·K)					     |
  | Thermal expansion:	  K⁻¹						     |
  |   _______________________________________________________________________|_
   \_/_______________________________________________________________________*/

/* MARK: - 1 - H */

#define Z_HYDROGEN_SYMBOL				H
#define Z_HYDROGEN_ATOMIC_NUMBER			1
#define Z_HYDROGEN_GROUP				1
#define Z_HYDROGEN_PERIOD				1
#define Z_HYDROGEN_BLOCK				Z_CHEMICAL_ELEMENT_BLOCK_S
#define Z_HYDROGEN_CLASS				Z_CHEMICAL_ELEMENT_CLASS_NON_METAL
#define Z_HYDROGEN_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_NON_METAL
#define Z_HYDROGEN_SPACE_GROUP_NUMBER			194
#define Z_HYDROGEN_ATOMIC_MASS				1.00794
#define Z_HYDROGEN_ATOMIC_RADIUS			53.0
#define Z_HYDROGEN_COVALENT_RADIUS			31.0
#define Z_HYDROGEN_VAN_DER_WAALS_RADIUS			120.0
#define Z_HYDROGEN_DECAY_MODE				KEY_NOT_APPLICABLE
#define Z_HYDROGEN_NEUTRON_CROSS_SECTION		0.332
#define Z_HYDROGEN_NEUTRON_MASS_ABSORPTION		0.011
#define Z_HYDROGEN_VALENCE				1
#define Z_HYDROGEN_ELECTRONEGATIVITY			2.20
#define Z_HYDROGEN_ELECTRON_AFFINITY			72.8
#define Z_HYDROGEN_MELTING_POINT			-259.14
#define Z_HYDROGEN_BOILING_POINT			-252.87
#define Z_HYDROGEN_CRITICAL_PRESSURE			1.293
#define Z_HYDROGEN_CRITICAL_TEMPERATURE			32.97
#define Z_HYDROGEN_FUSION_HEAT				0.558
#define Z_HYDROGEN_VAPORIZATION_HEAT			0.452
#define Z_HYDROGEN_SPECIFIC_HEAT			14300.0
#define Z_HYDROGEN_ADIABATIC_INDEX			Z_CHEMICAL_ELEMENT_ADIABATIC_INDEX_7_5
#define Z_HYDROGEN_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_HYDROGEN_THERMAL_CONDUCTIVITY			0.1805
#define Z_HYDROGEN_THERMAL_EXPANSION			REAL_NOT_APPLICABLE
#define Z_HYDROGEN_DENSITY				0.0899
#define Z_HYDROGEN_LIQUID_DENSITY			REAL_MISSING
#define Z_HYDROGEN_MOLAR_VOLUME				0.0112
#define Z_HYDROGEN_BRINELL_HARDNESS			REAL_NOT_APPLICABLE
#define Z_HYDROGEN_MOHS_HARDNESS			REAL_NOT_APPLICABLE
#define Z_HYDROGEN_VICKERS_HARDNESS			REAL_NOT_APPLICABLE
#define Z_HYDROGEN_BULK_MODULUS				REAL_NOT_APPLICABLE
#define Z_HYDROGEN_SHEAR_MODULUS			REAL_NOT_APPLICABLE
#define Z_HYDROGEN_YOUNG_MODULUS			REAL_NOT_APPLICABLE
#define Z_HYDROGEN_POISSON_RATIO			REAL_NOT_APPLICABLE
#define Z_HYDROGEN_REFRACTIVE_INDEX			1.000132
#define Z_HYDROGEN_SOUND_SPEED				1270.0
#define Z_HYDROGEN_ELECTRICAL_TYPE			KEY_MISSING
#define Z_HYDROGEN_ELECTRICAL_CONDUCTIVITY		REAL_MISSING
#define Z_HYDROGEN_ELECTRICAL_RESISTIVITY		REAL_NOT_APPLICABLE
#define Z_HYDROGEN_SUPERCONDUCTING_POINT		REAL_MISSING
#define Z_HYDROGEN_MAGNETIC_TYPE			Z_MAGNETIC_TYPE_DIAMAGNETIC
#define Z_HYDROGEN_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_HYDROGEN_MASS_MAGNETIC_SUSCEPTIBILITY		-2.480e-8
#define Z_HYDROGEN_MOLAR_MAGNETIC_SUSCEPTIBILITY	-4.999e-11
#define Z_HYDROGEN_VOLUME_MAGNETIC_SUSCEPTIBILITY	-2.230e-9

/* MARK: - 2 - He */

#define Z_HELIUM_SYMBOL					He
#define Z_HELIUM_ATOMIC_NUMBER				2
#define Z_HELIUM_GROUP					18
#define Z_HELIUM_PERIOD					1
#define Z_HELIUM_BLOCK					Z_CHEMICAL_ELEMENT_BLOCK_S
#define Z_HELIUM_CLASS					Z_CHEMICAL_ELEMENT_CLASS_NOBLE_GAS
#define Z_HELIUM_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_NON_METAL
#define Z_HELIUM_SPACE_GROUP_NUMBER			225
#define Z_HELIUM_ATOMIC_MASS				4.002602
#define Z_HELIUM_ATOMIC_RADIUS				31.0
#define Z_HELIUM_COVALENT_RADIUS			28.0
#define Z_HELIUM_VAN_DER_WAALS_RADIUS			140.0
#define Z_HELIUM_DECAY_MODE				KEY_NOT_APPLICABLE
#define Z_HELIUM_NEUTRON_CROSS_SECTION			0.007
#define Z_HELIUM_NEUTRON_MASS_ABSORPTION		1.0e-5
#define Z_HELIUM_VALENCE				0
#define Z_HELIUM_ELECTRONEGATIVITY			REAL_NOT_APPLICABLE
#define Z_HELIUM_ELECTRON_AFFINITY			0.0
#define Z_HELIUM_MELTING_POINT				REAL_NOT_APPLICABLE
#define Z_HELIUM_BOILING_POINT				-268.93
#define Z_HELIUM_CRITICAL_PRESSURE			0.227
#define Z_HELIUM_CRITICAL_TEMPERATURE			5.19
#define Z_HELIUM_FUSION_HEAT				0.02
#define Z_HELIUM_VAPORIZATION_HEAT			0.083
#define Z_HELIUM_SPECIFIC_HEAT				5193.1
#define Z_HELIUM_ADIABATIC_INDEX			Z_CHEMICAL_ELEMENT_ADIABATIC_INDEX_5_3
#define Z_HELIUM_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_HELIUM_THERMAL_CONDUCTIVITY			0.1513
#define Z_HELIUM_THERMAL_EXPANSION			REAL_NOT_APPLICABLE
#define Z_HELIUM_DENSITY				0.1785
#define Z_HELIUM_LIQUID_DENSITY				REAL_MISSING
#define Z_HELIUM_MOLAR_VOLUME				0.02242
#define Z_HELIUM_BRINELL_HARDNESS			REAL_NOT_APPLICABLE
#define Z_HELIUM_MOHS_HARDNESS				REAL_NOT_APPLICABLE
#define Z_HELIUM_VICKERS_HARDNESS			REAL_NOT_APPLICABLE
#define Z_HELIUM_BULK_MODULUS				REAL_NOT_APPLICABLE
#define Z_HELIUM_SHEAR_MODULUS				REAL_NOT_APPLICABLE
#define Z_HELIUM_YOUNG_MODULUS				REAL_NOT_APPLICABLE
#define Z_HELIUM_POISSON_RATIO				REAL_NOT_APPLICABLE
#define Z_HELIUM_REFRACTIVE_INDEX			1.000035
#define Z_HELIUM_SOUND_SPEED				970.0
#define Z_HELIUM_ELECTRICAL_TYPE			KEY_MISSING
#define Z_HELIUM_ELECTRICAL_CONDUCTIVITY		REAL_MISSING
#define Z_HELIUM_ELECTRICAL_RESISTIVITY			REAL_NOT_APPLICABLE
#define Z_HELIUM_SUPERCONDUCTING_POINT			REAL_MISSING
#define Z_HELIUM_MAGNETIC_TYPE				Z_MAGNETIC_TYPE_DIAMAGNETIC
#define Z_HELIUM_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_HELIUM_MASS_MAGNETIC_SUSCEPTIBILITY		-5.90e-9
#define Z_HELIUM_MOLAR_MAGNETIC_SUSCEPTIBILITY		-2.36e-11
#define Z_HELIUM_VOLUME_MAGNETIC_SUSCEPTIBILITY		-1.05e-9

/* MARK: - 3 - Li */

#define Z_LITHIUM_SYMBOL				Li
#define Z_LITHIUM_ATOMIC_NUMBER				3
#define Z_LITHIUM_GROUP					1
#define Z_LITHIUM_PERIOD				2
#define Z_LITHIUM_BLOCK					Z_CHEMICAL_ELEMENT_BLOCK_S
#define Z_LITHIUM_CLASS					Z_CHEMICAL_ELEMENT_CLASS_ALKALI_METAL
#define Z_LITHIUM_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_LITHIUM_SPACE_GROUP_NUMBER			229
#define Z_LITHIUM_ATOMIC_MASS				6.941
#define Z_LITHIUM_ATOMIC_RADIUS				167.0
#define Z_LITHIUM_COVALENT_RADIUS			128.0
#define Z_LITHIUM_VAN_DER_WAALS_RADIUS			182.0
#define Z_LITHIUM_DECAY_MODE				KEY_NOT_APPLICABLE
#define Z_LITHIUM_NEUTRON_CROSS_SECTION			71.0
#define Z_LITHIUM_NEUTRON_MASS_ABSORPTION		REAL_MISSING
#define Z_LITHIUM_VALENCE				1
#define Z_LITHIUM_ELECTRONEGATIVITY			0.98
#define Z_LITHIUM_ELECTRON_AFFINITY			59.6
#define Z_LITHIUM_MELTING_POINT				180.54
#define Z_LITHIUM_BOILING_POINT				1342.0
#define Z_LITHIUM_CRITICAL_PRESSURE			67.0
#define Z_LITHIUM_CRITICAL_TEMPERATURE			3223.0
#define Z_LITHIUM_FUSION_HEAT				3.0
#define Z_LITHIUM_VAPORIZATION_HEAT			147.0
#define Z_LITHIUM_SPECIFIC_HEAT				3570.0
#define Z_LITHIUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_LITHIUM_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_LITHIUM_THERMAL_CONDUCTIVITY			85.0
#define Z_LITHIUM_THERMAL_EXPANSION			0.000046
#define Z_LITHIUM_DENSITY				535.0
#define Z_LITHIUM_LIQUID_DENSITY			512.0
#define Z_LITHIUM_MOLAR_VOLUME				0.000013
#define Z_LITHIUM_BRINELL_HARDNESS			REAL_MISSING
#define Z_LITHIUM_MOHS_HARDNESS				0.6
#define Z_LITHIUM_VICKERS_HARDNESS			REAL_MISSING
#define Z_LITHIUM_BULK_MODULUS				11.0
#define Z_LITHIUM_SHEAR_MODULUS				4.2
#define Z_LITHIUM_YOUNG_MODULUS				4.9
#define Z_LITHIUM_POISSON_RATIO				REAL_MISSING
#define Z_LITHIUM_REFRACTIVE_INDEX			REAL_MISSING
#define Z_LITHIUM_SOUND_SPEED				6.0e3
#define Z_LITHIUM_ELECTRICAL_TYPE			Z_ELECTRICAL_TYPE_CONDUCTOR
#define Z_LITHIUM_ELECTRICAL_CONDUCTIVITY		1.1e7
#define Z_LITHIUM_ELECTRICAL_RESISTIVITY		9.4e-8
#define Z_LITHIUM_SUPERCONDUCTING_POINT			REAL_MISSING
#define Z_LITHIUM_MAGNETIC_TYPE				Z_MAGNETIC_TYPE_PARAMAGNETIC
#define Z_LITHIUM_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_LITHIUM_MASS_MAGNETIC_SUSCEPTIBILITY		2.56e-8
#define Z_LITHIUM_MOLAR_MAGNETIC_SUSCEPTIBILITY		1.78e-10
#define Z_LITHIUM_VOLUME_MAGNETIC_SUSCEPTIBILITY	1.37e-5

/* MARK: - 4 - Be */

#define Z_BERYLLIUM_SYMBOL				Be
#define Z_BERYLLIUM_ATOMIC_NUMBER			4
#define Z_BERYLLIUM_GROUP				2
#define Z_BERYLLIUM_PERIOD				2
#define Z_BERYLLIUM_BLOCK				Z_CHEMICAL_ELEMENT_BLOCK_S
#define Z_BERYLLIUM_CLASS				Z_CHEMICAL_ELEMENT_CLASS_ALKALINE_EARTH_METAL
#define Z_BERYLLIUM_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_BERYLLIUM_SPACE_GROUP_NUMBER			194
#define Z_BERYLLIUM_ATOMIC_MASS				9.012182
#define Z_BERYLLIUM_ATOMIC_RADIUS			112.0
#define Z_BERYLLIUM_COVALENT_RADIUS			96.0
#define Z_BERYLLIUM_VAN_DER_WAALS_RADIUS		REAL_MISSING
#define Z_BERYLLIUM_DECAY_MODE				KEY_NOT_APPLICABLE
#define Z_BERYLLIUM_NEUTRON_CROSS_SECTION		0.0092
#define Z_BERYLLIUM_NEUTRON_MASS_ABSORPTION		0.000030
#define Z_BERYLLIUM_VALENCE				2
#define Z_BERYLLIUM_ELECTRONEGATIVITY			1.57
#define Z_BERYLLIUM_ELECTRON_AFFINITY			0.0
#define Z_BERYLLIUM_MELTING_POINT			1287.0
#define Z_BERYLLIUM_BOILING_POINT			2470.0
#define Z_BERYLLIUM_CRITICAL_PRESSURE			REAL_MISSING
#define Z_BERYLLIUM_CRITICAL_TEMPERATURE		REAL_MISSING
#define Z_BERYLLIUM_FUSION_HEAT				7.95
#define Z_BERYLLIUM_VAPORIZATION_HEAT			297.0
#define Z_BERYLLIUM_SPECIFIC_HEAT			1820.0
#define Z_BERYLLIUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_BERYLLIUM_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_BERYLLIUM_THERMAL_CONDUCTIVITY		190.0
#define Z_BERYLLIUM_THERMAL_EXPANSION			0.0000113
#define Z_BERYLLIUM_DENSITY				1848.0
#define Z_BERYLLIUM_LIQUID_DENSITY			1690.0
#define Z_BERYLLIUM_MOLAR_VOLUME			4.877e-6
#define Z_BERYLLIUM_BRINELL_HARDNESS			6.0e2
#define Z_BERYLLIUM_MOHS_HARDNESS			5.5
#define Z_BERYLLIUM_VICKERS_HARDNESS			1670.0
#define Z_BERYLLIUM_BULK_MODULUS			130.0
#define Z_BERYLLIUM_SHEAR_MODULUS			132.0
#define Z_BERYLLIUM_YOUNG_MODULUS			287.0
#define Z_BERYLLIUM_POISSON_RATIO			0.032
#define Z_BERYLLIUM_REFRACTIVE_INDEX			REAL_MISSING
#define Z_BERYLLIUM_SOUND_SPEED				1.3e4
#define Z_BERYLLIUM_ELECTRICAL_TYPE			Z_ELECTRICAL_TYPE_CONDUCTOR
#define Z_BERYLLIUM_ELECTRICAL_CONDUCTIVITY		2.5e7
#define Z_BERYLLIUM_ELECTRICAL_RESISTIVITY		4.0e-8
#define Z_BERYLLIUM_SUPERCONDUCTING_POINT		0.026
#define Z_BERYLLIUM_MAGNETIC_TYPE			Z_MAGNETIC_TYPE_DIAMAGNETIC
#define Z_BERYLLIUM_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_BERYLLIUM_MASS_MAGNETIC_SUSCEPTIBILITY	-1.260e-8
#define Z_BERYLLIUM_MOLAR_MAGNETIC_SUSCEPTIBILITY	-1.136e-10
#define Z_BERYLLIUM_VOLUME_MAGNETIC_SUSCEPTIBILITY	-2.328e-5

/* MARK: - 5 - B */

#define Z_BORON_SYMBOL					B
#define Z_BORON_ATOMIC_NUMBER				5
#define Z_BORON_GROUP					13
#define Z_BORON_PERIOD					2
#define Z_BORON_BLOCK					Z_CHEMICAL_ELEMENT_BLOCK_P
#define Z_BORON_CLASS					Z_CHEMICAL_ELEMENT_CLASS_METALLOID
#define Z_BORON_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_METALLOID
#define Z_BORON_SPACE_GROUP_NUMBER			166
#define Z_BORON_ATOMIC_MASS				10.811
#define Z_BORON_ATOMIC_RADIUS				87.0
#define Z_BORON_COVALENT_RADIUS				85.0
#define Z_BORON_VAN_DER_WAALS_RADIUS			REAL_MISSING
#define Z_BORON_DECAY_MODE				KEY_NOT_APPLICABLE
#define Z_BORON_NEUTRON_CROSS_SECTION			7.6e2
#define Z_BORON_NEUTRON_MASS_ABSORPTION			2.4
#define Z_BORON_VALENCE					3
#define Z_BORON_ELECTRONEGATIVITY			2.04
#define Z_BORON_ELECTRON_AFFINITY			26.7
#define Z_BORON_MELTING_POINT				2075.0
#define Z_BORON_BOILING_POINT				4000.0
#define Z_BORON_CRITICAL_PRESSURE			REAL_MISSING
#define Z_BORON_CRITICAL_TEMPERATURE			REAL_MISSING
#define Z_BORON_FUSION_HEAT				50.0
#define Z_BORON_VAPORIZATION_HEAT			507.0
#define Z_BORON_SPECIFIC_HEAT				1030.0
#define Z_BORON_ADIABATIC_INDEX				KEY_NOT_APPLICABLE
#define Z_BORON_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_BORON_THERMAL_CONDUCTIVITY			27.0
#define Z_BORON_THERMAL_EXPANSION			6.0e-6
#define Z_BORON_DENSITY					2460.0
#define Z_BORON_LIQUID_DENSITY				2080.0
#define Z_BORON_MOLAR_VOLUME				4.395e-6
#define Z_BORON_BRINELL_HARDNESS			REAL_MISSING
#define Z_BORON_MOHS_HARDNESS				9.3
#define Z_BORON_VICKERS_HARDNESS			4.9e4
#define Z_BORON_BULK_MODULUS				320.0
#define Z_BORON_SHEAR_MODULUS				REAL_MISSING
#define Z_BORON_YOUNG_MODULUS				REAL_MISSING
#define Z_BORON_POISSON_RATIO				REAL_MISSING
#define Z_BORON_REFRACTIVE_INDEX			REAL_MISSING
#define Z_BORON_SOUND_SPEED				1.62e4
#define Z_BORON_ELECTRICAL_TYPE				Z_ELECTRICAL_TYPE_INSULATOR
#define Z_BORON_ELECTRICAL_CONDUCTIVITY			0.0001
#define Z_BORON_ELECTRICAL_RESISTIVITY			10000.0
#define Z_BORON_SUPERCONDUCTING_POINT			REAL_MISSING
#define Z_BORON_MAGNETIC_TYPE				Z_MAGNETIC_TYPE_DIAMAGNETIC
#define Z_BORON_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_BORON_MASS_MAGNETIC_SUSCEPTIBILITY		-8.70e-9
#define Z_BORON_MOLAR_MAGNETIC_SUSCEPTIBILITY		-9.41e-11
#define Z_BORON_VOLUME_MAGNETIC_SUSCEPTIBILITY		-2.14e-5

/* MARK: - 6 - C */

#define Z_CARBON_SYMBOL					C
#define Z_CARBON_ATOMIC_NUMBER				6
#define Z_CARBON_GROUP					14
#define Z_CARBON_PERIOD					2
#define Z_CARBON_BLOCK					Z_CHEMICAL_ELEMENT_BLOCK_P
#define Z_CARBON_CLASS					Z_CHEMICAL_ELEMENT_CLASS_NON_METAL
#define Z_CARBON_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_NON_METAL
#define Z_CARBON_SPACE_GROUP_NUMBER			194
#define Z_CARBON_ATOMIC_MASS				12.0107
#define Z_CARBON_ATOMIC_RADIUS				67.0
#define Z_CARBON_COVALENT_RADIUS			76.0
#define Z_CARBON_VAN_DER_WAALS_RADIUS			170.0
#define Z_CARBON_DECAY_MODE				KEY_NOT_APPLICABLE
#define Z_CARBON_NEUTRON_CROSS_SECTION			0.0035
#define Z_CARBON_NEUTRON_MASS_ABSORPTION		0.000015
#define Z_CARBON_VALENCE				4
#define Z_CARBON_ELECTRONEGATIVITY			2.55
#define Z_CARBON_ELECTRON_AFFINITY			153.9
#define Z_CARBON_MELTING_POINT				3550.0
#define Z_CARBON_BOILING_POINT				4027.0
#define Z_CARBON_CRITICAL_PRESSURE			REAL_MISSING
#define Z_CARBON_CRITICAL_TEMPERATURE			REAL_MISSING
#define Z_CARBON_FUSION_HEAT				105.0
#define Z_CARBON_VAPORIZATION_HEAT			715.0
#define Z_CARBON_SPECIFIC_HEAT				710.0
#define Z_CARBON_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_CARBON_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_CARBON_THERMAL_CONDUCTIVITY			140.0
#define Z_CARBON_THERMAL_EXPANSION			7.1e-6
#define Z_CARBON_DENSITY				2260.0
#define Z_CARBON_LIQUID_DENSITY				REAL_MISSING
#define Z_CARBON_MOLAR_VOLUME				5.314e-6
#define Z_CARBON_BRINELL_HARDNESS			REAL_MISSING
#define Z_CARBON_MOHS_HARDNESS				0.5
#define Z_CARBON_VICKERS_HARDNESS			REAL_MISSING
#define Z_CARBON_BULK_MODULUS				33.0
#define Z_CARBON_SHEAR_MODULUS				REAL_MISSING
#define Z_CARBON_YOUNG_MODULUS				REAL_MISSING
#define Z_CARBON_POISSON_RATIO				REAL_MISSING
#define Z_CARBON_REFRACTIVE_INDEX			2.417
#define Z_CARBON_SOUND_SPEED				18350.0
#define Z_CARBON_ELECTRICAL_TYPE			Z_ELECTRICAL_TYPE_CONDUCTOR
#define Z_CARBON_ELECTRICAL_CONDUCTIVITY		1.0e5
#define Z_CARBON_ELECTRICAL_RESISTIVITY			1.0e-5
#define Z_CARBON_SUPERCONDUCTING_POINT			REAL_MISSING
#define Z_CARBON_MAGNETIC_TYPE				Z_MAGNETIC_TYPE_DIAMAGNETIC
#define Z_CARBON_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_CARBON_MASS_MAGNETIC_SUSCEPTIBILITY		-6.20e-9
#define Z_CARBON_MOLAR_MAGNETIC_SUSCEPTIBILITY		-7.45e-11
#define Z_CARBON_VOLUME_MAGNETIC_SUSCEPTIBILITY		-0.0000140

/* MARK: - 7 - N */

#define Z_NITROGEN_SYMBOL				N
#define Z_NITROGEN_ATOMIC_NUMBER			7
#define Z_NITROGEN_GROUP				15
#define Z_NITROGEN_PERIOD				2
#define Z_NITROGEN_BLOCK				Z_CHEMICAL_ELEMENT_BLOCK_P
#define Z_NITROGEN_CLASS				Z_CHEMICAL_ELEMENT_CLASS_NON_METAL
#define Z_NITROGEN_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_NON_METAL
#define Z_NITROGEN_SPACE_GROUP_NUMBER			194
#define Z_NITROGEN_ATOMIC_MASS				14.0067
#define Z_NITROGEN_ATOMIC_RADIUS			56.0
#define Z_NITROGEN_COVALENT_RADIUS			71.0
#define Z_NITROGEN_VAN_DER_WAALS_RADIUS			155.0
#define Z_NITROGEN_DECAY_MODE				KEY_NOT_APPLICABLE
#define Z_NITROGEN_NEUTRON_CROSS_SECTION		1.91
#define Z_NITROGEN_NEUTRON_MASS_ABSORPTION		0.0048
#define Z_NITROGEN_VALENCE				3
#define Z_NITROGEN_ELECTRONEGATIVITY			3.04
#define Z_NITROGEN_ELECTRON_AFFINITY			7.0
#define Z_NITROGEN_MELTING_POINT			-210.1
#define Z_NITROGEN_BOILING_POINT			-195.79
#define Z_NITROGEN_CRITICAL_PRESSURE			3.39
#define Z_NITROGEN_CRITICAL_TEMPERATURE			126.21
#define Z_NITROGEN_FUSION_HEAT				0.36
#define Z_NITROGEN_VAPORIZATION_HEAT			2.79
#define Z_NITROGEN_SPECIFIC_HEAT			1040.0
#define Z_NITROGEN_ADIABATIC_INDEX			Z_CHEMICAL_ELEMENT_ADIABATIC_INDEX_7_5
#define Z_NITROGEN_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_NITROGEN_THERMAL_CONDUCTIVITY			0.02583
#define Z_NITROGEN_THERMAL_EXPANSION			REAL_NOT_APPLICABLE
#define Z_NITROGEN_DENSITY				1.251
#define Z_NITROGEN_LIQUID_DENSITY			REAL_MISSING
#define Z_NITROGEN_MOLAR_VOLUME				0.01120
#define Z_NITROGEN_BRINELL_HARDNESS			REAL_NOT_APPLICABLE
#define Z_NITROGEN_MOHS_HARDNESS			REAL_NOT_APPLICABLE
#define Z_NITROGEN_VICKERS_HARDNESS			REAL_NOT_APPLICABLE
#define Z_NITROGEN_BULK_MODULUS				REAL_NOT_APPLICABLE
#define Z_NITROGEN_SHEAR_MODULUS			REAL_NOT_APPLICABLE
#define Z_NITROGEN_YOUNG_MODULUS			REAL_NOT_APPLICABLE
#define Z_NITROGEN_POISSON_RATIO			REAL_NOT_APPLICABLE
#define Z_NITROGEN_REFRACTIVE_INDEX			1.000298
#define Z_NITROGEN_SOUND_SPEED				333.6
#define Z_NITROGEN_ELECTRICAL_TYPE			KEY_MISSING
#define Z_NITROGEN_ELECTRICAL_CONDUCTIVITY		REAL_MISSING
#define Z_NITROGEN_ELECTRICAL_RESISTIVITY		REAL_NOT_APPLICABLE
#define Z_NITROGEN_SUPERCONDUCTING_POINT		REAL_MISSING
#define Z_NITROGEN_MAGNETIC_TYPE			Z_MAGNETIC_TYPE_DIAMAGNETIC
#define Z_NITROGEN_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_NITROGEN_MASS_MAGNETIC_SUSCEPTIBILITY		-5.4e-9
#define Z_NITROGEN_MOLAR_MAGNETIC_SUSCEPTIBILITY	-1.5e-10
#define Z_NITROGEN_VOLUME_MAGNETIC_SUSCEPTIBILITY	-6.8e-9

/* MARK: - 8 - O */

#define Z_OXYGEN_SYMBOL					O
#define Z_OXYGEN_ATOMIC_NUMBER				8
#define Z_OXYGEN_GROUP					16
#define Z_OXYGEN_PERIOD					2
#define Z_OXYGEN_BLOCK					Z_CHEMICAL_ELEMENT_BLOCK_P
#define Z_OXYGEN_CLASS					Z_CHEMICAL_ELEMENT_CLASS_NON_METAL
#define Z_OXYGEN_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_NON_METAL
#define Z_OXYGEN_SPACE_GROUP_NUMBER			12
#define Z_OXYGEN_ATOMIC_MASS				15.9994
#define Z_OXYGEN_ATOMIC_RADIUS				48.0
#define Z_OXYGEN_COVALENT_RADIUS			66.0
#define Z_OXYGEN_VAN_DER_WAALS_RADIUS			152.0
#define Z_OXYGEN_DECAY_MODE				KEY_NOT_APPLICABLE
#define Z_OXYGEN_NEUTRON_CROSS_SECTION			0.00028
#define Z_OXYGEN_NEUTRON_MASS_ABSORPTION		1.0e-6
#define Z_OXYGEN_VALENCE				2
#define Z_OXYGEN_ELECTRONEGATIVITY			3.44
#define Z_OXYGEN_ELECTRON_AFFINITY			141.0
#define Z_OXYGEN_MELTING_POINT				-218.3
#define Z_OXYGEN_BOILING_POINT				-182.9
#define Z_OXYGEN_CRITICAL_PRESSURE			5.043
#define Z_OXYGEN_CRITICAL_TEMPERATURE			154.59
#define Z_OXYGEN_FUSION_HEAT				0.222
#define Z_OXYGEN_VAPORIZATION_HEAT			3.41
#define Z_OXYGEN_SPECIFIC_HEAT				919.0
#define Z_OXYGEN_ADIABATIC_INDEX			Z_CHEMICAL_ELEMENT_ADIABATIC_INDEX_7_5
#define Z_OXYGEN_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_OXYGEN_THERMAL_CONDUCTIVITY			0.02658
#define Z_OXYGEN_THERMAL_EXPANSION			REAL_NOT_APPLICABLE
#define Z_OXYGEN_DENSITY				1.429
#define Z_OXYGEN_LIQUID_DENSITY				REAL_MISSING
#define Z_OXYGEN_MOLAR_VOLUME				0.01120
#define Z_OXYGEN_BRINELL_HARDNESS			REAL_NOT_APPLICABLE
#define Z_OXYGEN_MOHS_HARDNESS				REAL_NOT_APPLICABLE
#define Z_OXYGEN_VICKERS_HARDNESS			REAL_NOT_APPLICABLE
#define Z_OXYGEN_BULK_MODULUS				REAL_NOT_APPLICABLE
#define Z_OXYGEN_SHEAR_MODULUS				REAL_NOT_APPLICABLE
#define Z_OXYGEN_YOUNG_MODULUS				REAL_NOT_APPLICABLE
#define Z_OXYGEN_POISSON_RATIO				REAL_NOT_APPLICABLE
#define Z_OXYGEN_REFRACTIVE_INDEX			1.000271
#define Z_OXYGEN_SOUND_SPEED				317.5
#define Z_OXYGEN_ELECTRICAL_TYPE			KEY_MISSING
#define Z_OXYGEN_ELECTRICAL_CONDUCTIVITY		REAL_MISSING
#define Z_OXYGEN_ELECTRICAL_RESISTIVITY			REAL_NOT_APPLICABLE
#define Z_OXYGEN_SUPERCONDUCTING_POINT			REAL_MISSING
#define Z_OXYGEN_MAGNETIC_TYPE				Z_MAGNETIC_TYPE_PARAMAGNETIC
#define Z_OXYGEN_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_OXYGEN_MASS_MAGNETIC_SUSCEPTIBILITY		1.33500e-6
#define Z_OXYGEN_MOLAR_MAGNETIC_SUSCEPTIBILITY		4.27184e-8
#define Z_OXYGEN_VOLUME_MAGNETIC_SUSCEPTIBILITY		1.90772e-6

/* MARK: - 9 - F */

#define Z_FLUORINE_SYMBOL				F
#define Z_FLUORINE_ATOMIC_NUMBER			9
#define Z_FLUORINE_GROUP				17
#define Z_FLUORINE_PERIOD				2
#define Z_FLUORINE_BLOCK				Z_CHEMICAL_ELEMENT_BLOCK_P
#define Z_FLUORINE_CLASS				Z_CHEMICAL_ELEMENT_CLASS_HALOGEN
#define Z_FLUORINE_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_NON_METAL
#define Z_FLUORINE_SPACE_GROUP_NUMBER			15
#define Z_FLUORINE_ATOMIC_MASS				18.9984032
#define Z_FLUORINE_ATOMIC_RADIUS			42.0
#define Z_FLUORINE_COVALENT_RADIUS			57.0
#define Z_FLUORINE_VAN_DER_WAALS_RADIUS			147.0
#define Z_FLUORINE_DECAY_MODE				KEY_NOT_APPLICABLE
#define Z_FLUORINE_NEUTRON_CROSS_SECTION		0.0096
#define Z_FLUORINE_NEUTRON_MASS_ABSORPTION		0.000020
#define Z_FLUORINE_VALENCE				1
#define Z_FLUORINE_ELECTRONEGATIVITY			3.98
#define Z_FLUORINE_ELECTRON_AFFINITY			328.0
#define Z_FLUORINE_MELTING_POINT			-219.6
#define Z_FLUORINE_BOILING_POINT			-188.12
#define Z_FLUORINE_CRITICAL_PRESSURE			5.172
#define Z_FLUORINE_CRITICAL_TEMPERATURE			144.13
#define Z_FLUORINE_FUSION_HEAT				0.26
#define Z_FLUORINE_VAPORIZATION_HEAT			3.27
#define Z_FLUORINE_SPECIFIC_HEAT			824.0
#define Z_FLUORINE_ADIABATIC_INDEX			Z_CHEMICAL_ELEMENT_ADIABATIC_INDEX_7_5
#define Z_FLUORINE_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_FLUORINE_THERMAL_CONDUCTIVITY			0.0277
#define Z_FLUORINE_THERMAL_EXPANSION			REAL_NOT_APPLICABLE
#define Z_FLUORINE_DENSITY				1.696
#define Z_FLUORINE_LIQUID_DENSITY			REAL_MISSING
#define Z_FLUORINE_MOLAR_VOLUME				0.01120
#define Z_FLUORINE_BRINELL_HARDNESS			REAL_NOT_APPLICABLE
#define Z_FLUORINE_MOHS_HARDNESS			REAL_NOT_APPLICABLE
#define Z_FLUORINE_VICKERS_HARDNESS			REAL_NOT_APPLICABLE
#define Z_FLUORINE_BULK_MODULUS				REAL_NOT_APPLICABLE
#define Z_FLUORINE_SHEAR_MODULUS			REAL_NOT_APPLICABLE
#define Z_FLUORINE_YOUNG_MODULUS			REAL_NOT_APPLICABLE
#define Z_FLUORINE_POISSON_RATIO			REAL_NOT_APPLICABLE
#define Z_FLUORINE_REFRACTIVE_INDEX			1.000195
#define Z_FLUORINE_SOUND_SPEED				REAL_MISSING
#define Z_FLUORINE_ELECTRICAL_TYPE			KEY_MISSING
#define Z_FLUORINE_ELECTRICAL_CONDUCTIVITY		REAL_MISSING
#define Z_FLUORINE_ELECTRICAL_RESISTIVITY		REAL_NOT_APPLICABLE
#define Z_FLUORINE_SUPERCONDUCTING_POINT		REAL_MISSING
#define Z_FLUORINE_MAGNETIC_TYPE			KEY_MISSING
#define Z_FLUORINE_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_FLUORINE_MASS_MAGNETIC_SUSCEPTIBILITY		REAL_MISSING
#define Z_FLUORINE_MOLAR_MAGNETIC_SUSCEPTIBILITY	REAL_MISSING
#define Z_FLUORINE_VOLUME_MAGNETIC_SUSCEPTIBILITY	REAL_MISSING

/* MARK: - 10 - Ne */

#define Z_NEON_SYMBOL					Ne
#define Z_NEON_ATOMIC_NUMBER				10
#define Z_NEON_GROUP					18
#define Z_NEON_PERIOD					2
#define Z_NEON_BLOCK					Z_CHEMICAL_ELEMENT_BLOCK_P
#define Z_NEON_CLASS					Z_CHEMICAL_ELEMENT_CLASS_NOBLE_GAS
#define Z_NEON_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_NON_METAL
#define Z_NEON_SPACE_GROUP_NUMBER			225
#define Z_NEON_ATOMIC_MASS				20.1791
#define Z_NEON_ATOMIC_RADIUS				38.0
#define Z_NEON_COVALENT_RADIUS				58.0
#define Z_NEON_VAN_DER_WAALS_RADIUS			154.0
#define Z_NEON_DECAY_MODE				KEY_NOT_APPLICABLE
#define Z_NEON_NEUTRON_CROSS_SECTION			0.04
#define Z_NEON_NEUTRON_MASS_ABSORPTION			0.00059
#define Z_NEON_VALENCE					0
#define Z_NEON_ELECTRONEGATIVITY			REAL_NOT_APPLICABLE
#define Z_NEON_ELECTRON_AFFINITY			0.0
#define Z_NEON_MELTING_POINT				-248.59
#define Z_NEON_BOILING_POINT				-246.08
#define Z_NEON_CRITICAL_PRESSURE			2.76
#define Z_NEON_CRITICAL_TEMPERATURE			44.4
#define Z_NEON_FUSION_HEAT				0.34
#define Z_NEON_VAPORIZATION_HEAT			1.75
#define Z_NEON_SPECIFIC_HEAT				1030.0
#define Z_NEON_ADIABATIC_INDEX				Z_CHEMICAL_ELEMENT_ADIABATIC_INDEX_5_3
#define Z_NEON_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_NEON_THERMAL_CONDUCTIVITY			0.0491
#define Z_NEON_THERMAL_EXPANSION			REAL_NOT_APPLICABLE
#define Z_NEON_DENSITY					0.900
#define Z_NEON_LIQUID_DENSITY				REAL_MISSING
#define Z_NEON_MOLAR_VOLUME				0.0224
#define Z_NEON_BRINELL_HARDNESS				REAL_NOT_APPLICABLE
#define Z_NEON_MOHS_HARDNESS				REAL_NOT_APPLICABLE
#define Z_NEON_VICKERS_HARDNESS				REAL_NOT_APPLICABLE
#define Z_NEON_BULK_MODULUS				REAL_NOT_APPLICABLE
#define Z_NEON_SHEAR_MODULUS				REAL_NOT_APPLICABLE
#define Z_NEON_YOUNG_MODULUS				REAL_NOT_APPLICABLE
#define Z_NEON_POISSON_RATIO				REAL_NOT_APPLICABLE
#define Z_NEON_REFRACTIVE_INDEX				1.000067
#define Z_NEON_SOUND_SPEED				936.0
#define Z_NEON_ELECTRICAL_TYPE				KEY_MISSING
#define Z_NEON_ELECTRICAL_CONDUCTIVITY			REAL_MISSING
#define Z_NEON_ELECTRICAL_RESISTIVITY			REAL_NOT_APPLICABLE
#define Z_NEON_SUPERCONDUCTING_POINT			REAL_MISSING
#define Z_NEON_MAGNETIC_TYPE				Z_MAGNETIC_TYPE_DIAMAGNETIC
#define Z_NEON_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_NEON_MASS_MAGNETIC_SUSCEPTIBILITY		-4.10e-9
#define Z_NEON_MOLAR_MAGNETIC_SUSCEPTIBILITY		-8.27e-11
#define Z_NEON_VOLUME_MAGNETIC_SUSCEPTIBILITY		-3.69e-9

/* MARK: - 11 - Na */

#define Z_SODIUM_SYMBOL					Na
#define Z_SODIUM_ATOMIC_NUMBER				11
#define Z_SODIUM_GROUP					1
#define Z_SODIUM_PERIOD					3
#define Z_SODIUM_BLOCK					Z_CHEMICAL_ELEMENT_BLOCK_S
#define Z_SODIUM_CLASS					Z_CHEMICAL_ELEMENT_CLASS_ALKALI_METAL
#define Z_SODIUM_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_SODIUM_SPACE_GROUP_NUMBER			229
#define Z_SODIUM_ATOMIC_MASS				22.98976928
#define Z_SODIUM_ATOMIC_RADIUS				190.0
#define Z_SODIUM_COVALENT_RADIUS			166.0
#define Z_SODIUM_VAN_DER_WAALS_RADIUS			227.0
#define Z_SODIUM_DECAY_MODE				KEY_NOT_APPLICABLE
#define Z_SODIUM_NEUTRON_CROSS_SECTION			0.53
#define Z_SODIUM_NEUTRON_MASS_ABSORPTION		0.00070
#define Z_SODIUM_VALENCE				1
#define Z_SODIUM_ELECTRONEGATIVITY			0.93
#define Z_SODIUM_ELECTRON_AFFINITY			52.8
#define Z_SODIUM_MELTING_POINT				97.72
#define Z_SODIUM_BOILING_POINT				883.0
#define Z_SODIUM_CRITICAL_PRESSURE			35.0
#define Z_SODIUM_CRITICAL_TEMPERATURE			2573.0
#define Z_SODIUM_FUSION_HEAT				2.60
#define Z_SODIUM_VAPORIZATION_HEAT			97.7
#define Z_SODIUM_SPECIFIC_HEAT				1230.0
#define Z_SODIUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_SODIUM_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_SODIUM_THERMAL_CONDUCTIVITY			140.0
#define Z_SODIUM_THERMAL_EXPANSION			0.000070
#define Z_SODIUM_DENSITY				968.0
#define Z_SODIUM_LIQUID_DENSITY				927.0
#define Z_SODIUM_MOLAR_VOLUME				0.0000237
#define Z_SODIUM_BRINELL_HARDNESS			0.69
#define Z_SODIUM_MOHS_HARDNESS				0.5
#define Z_SODIUM_VICKERS_HARDNESS			REAL_MISSING
#define Z_SODIUM_BULK_MODULUS				6.3
#define Z_SODIUM_SHEAR_MODULUS				3.3
#define Z_SODIUM_YOUNG_MODULUS				10.0
#define Z_SODIUM_POISSON_RATIO				REAL_MISSING
#define Z_SODIUM_REFRACTIVE_INDEX			REAL_MISSING
#define Z_SODIUM_SOUND_SPEED				3.2e3
#define Z_SODIUM_ELECTRICAL_TYPE			Z_ELECTRICAL_TYPE_CONDUCTOR
#define Z_SODIUM_ELECTRICAL_CONDUCTIVITY		2.1e7
#define Z_SODIUM_ELECTRICAL_RESISTIVITY			4.7e-8
#define Z_SODIUM_SUPERCONDUCTING_POINT			REAL_MISSING
#define Z_SODIUM_MAGNETIC_TYPE				Z_MAGNETIC_TYPE_PARAMAGNETIC
#define Z_SODIUM_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_SODIUM_MASS_MAGNETIC_SUSCEPTIBILITY		8.8e-9
#define Z_SODIUM_MOLAR_MAGNETIC_SUSCEPTIBILITY		2.0e-10
#define Z_SODIUM_VOLUME_MAGNETIC_SUSCEPTIBILITY		8.6e-6

/* MARK: - 12 - Mg */

#define Z_MAGNESIUM_SYMBOL				Mg
#define Z_MAGNESIUM_ATOMIC_NUMBER			12
#define Z_MAGNESIUM_GROUP				2
#define Z_MAGNESIUM_PERIOD				3
#define Z_MAGNESIUM_BLOCK				Z_CHEMICAL_ELEMENT_BLOCK_S
#define Z_MAGNESIUM_CLASS				Z_CHEMICAL_ELEMENT_CLASS_ALKALINE_EARTH_METAL
#define Z_MAGNESIUM_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_MAGNESIUM_SPACE_GROUP_NUMBER			194
#define Z_MAGNESIUM_ATOMIC_MASS				24.3050
#define Z_MAGNESIUM_ATOMIC_RADIUS			145.0
#define Z_MAGNESIUM_COVALENT_RADIUS			141.0
#define Z_MAGNESIUM_VAN_DER_WAALS_RADIUS		173.0
#define Z_MAGNESIUM_DECAY_MODE				KEY_NOT_APPLICABLE
#define Z_MAGNESIUM_NEUTRON_CROSS_SECTION		0.063
#define Z_MAGNESIUM_NEUTRON_MASS_ABSORPTION		0.0001
#define Z_MAGNESIUM_VALENCE				2
#define Z_MAGNESIUM_ELECTRONEGATIVITY			1.31
#define Z_MAGNESIUM_ELECTRON_AFFINITY			0.0
#define Z_MAGNESIUM_MELTING_POINT			650.0
#define Z_MAGNESIUM_BOILING_POINT			1090.0
#define Z_MAGNESIUM_CRITICAL_PRESSURE			REAL_UNKNOWN
#define Z_MAGNESIUM_CRITICAL_TEMPERATURE		REAL_UNKNOWN
#define Z_MAGNESIUM_FUSION_HEAT				8.7
#define Z_MAGNESIUM_VAPORIZATION_HEAT			128.0
#define Z_MAGNESIUM_SPECIFIC_HEAT			1020.0
#define Z_MAGNESIUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_MAGNESIUM_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_MAGNESIUM_THERMAL_CONDUCTIVITY		160.0
#define Z_MAGNESIUM_THERMAL_EXPANSION			0.000025
#define Z_MAGNESIUM_DENSITY				1738.0
#define Z_MAGNESIUM_LIQUID_DENSITY			1584.0
#define Z_MAGNESIUM_MOLAR_VOLUME			0.00001398
#define Z_MAGNESIUM_BRINELL_HARDNESS			260.0
#define Z_MAGNESIUM_MOHS_HARDNESS			2.5
#define Z_MAGNESIUM_VICKERS_HARDNESS			REAL_UNKNOWN
#define Z_MAGNESIUM_BULK_MODULUS			45.0
#define Z_MAGNESIUM_SHEAR_MODULUS			17.0
#define Z_MAGNESIUM_YOUNG_MODULUS			45.0
#define Z_MAGNESIUM_POISSON_RATIO			0.29
#define Z_MAGNESIUM_REFRACTIVE_INDEX			REAL_UNKNOWN
#define Z_MAGNESIUM_SOUND_SPEED				4602.0
#define Z_MAGNESIUM_ELECTRICAL_TYPE			Z_ELECTRICAL_TYPE_CONDUCTOR
#define Z_MAGNESIUM_ELECTRICAL_CONDUCTIVITY		2.3e7
#define Z_MAGNESIUM_ELECTRICAL_RESISTIVITY		4.4e-8
#define Z_MAGNESIUM_SUPERCONDUCTING_POINT		REAL_UNKNOWN
#define Z_MAGNESIUM_MAGNETIC_TYPE			Z_MAGNETIC_TYPE_PARAMAGNETIC
#define Z_MAGNESIUM_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_MAGNESIUM_MASS_MAGNETIC_SUSCEPTIBILITY	6.90e-9
#define Z_MAGNESIUM_MOLAR_MAGNETIC_SUSCEPTIBILITY	1.68e-10
#define Z_MAGNESIUM_VOLUME_MAGNETIC_SUSCEPTIBILITY	0.0000120

/* MARK: - 13 - Al */

#define Z_ALUMINIUM_SYMBOL				Al
#define Z_ALUMINIUM_ATOMIC_NUMBER			13
#define Z_ALUMINIUM_GROUP				13
#define Z_ALUMINIUM_PERIOD				3
#define Z_ALUMINIUM_BLOCK				Z_CHEMICAL_ELEMENT_BLOCK_P
#define Z_ALUMINIUM_CLASS				Z_CHEMICAL_ELEMENT_CLASS_POOR_METAL
#define Z_ALUMINIUM_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_ALUMINIUM_SPACE_GROUP_NUMBER			225
#define Z_ALUMINIUM_ATOMIC_MASS				26.9815386
#define Z_ALUMINIUM_ATOMIC_RADIUS			118.0
#define Z_ALUMINIUM_COVALENT_RADIUS			121.0
#define Z_ALUMINIUM_VAN_DER_WAALS_RADIUS		REAL_MISSING
#define Z_ALUMINIUM_DECAY_MODE				KEY_NOT_APPLICABLE
#define Z_ALUMINIUM_NEUTRON_CROSS_SECTION		0.233
#define Z_ALUMINIUM_NEUTRON_MASS_ABSORPTION		0.030
#define Z_ALUMINIUM_VALENCE				3
#define Z_ALUMINIUM_ELECTRONEGATIVITY			1.61
#define Z_ALUMINIUM_ELECTRON_AFFINITY			42.5
#define Z_ALUMINIUM_MELTING_POINT			660.32
#define Z_ALUMINIUM_BOILING_POINT			2519.0
#define Z_ALUMINIUM_CRITICAL_PRESSURE			REAL_MISSING
#define Z_ALUMINIUM_CRITICAL_TEMPERATURE		REAL_MISSING
#define Z_ALUMINIUM_FUSION_HEAT				10.7
#define Z_ALUMINIUM_VAPORIZATION_HEAT			293.0
#define Z_ALUMINIUM_SPECIFIC_HEAT			904.0
#define Z_ALUMINIUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_ALUMINIUM_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_ALUMINIUM_THERMAL_CONDUCTIVITY		235.0
#define Z_ALUMINIUM_THERMAL_EXPANSION			0.0000231
#define Z_ALUMINIUM_DENSITY				2.7e3
#define Z_ALUMINIUM_LIQUID_DENSITY			2375.0
#define Z_ALUMINIUM_MOLAR_VOLUME			1.0e-5
#define Z_ALUMINIUM_BRINELL_HARDNESS			245.0
#define Z_ALUMINIUM_MOHS_HARDNESS			2.75
#define Z_ALUMINIUM_VICKERS_HARDNESS			167.0
#define Z_ALUMINIUM_BULK_MODULUS			76.0
#define Z_ALUMINIUM_SHEAR_MODULUS			26.0
#define Z_ALUMINIUM_YOUNG_MODULUS			70.0
#define Z_ALUMINIUM_POISSON_RATIO			0.35
#define Z_ALUMINIUM_REFRACTIVE_INDEX			REAL_MISSING
#define Z_ALUMINIUM_SOUND_SPEED				5.1e3
#define Z_ALUMINIUM_ELECTRICAL_TYPE			Z_ELECTRICAL_TYPE_CONDUCTOR
#define Z_ALUMINIUM_ELECTRICAL_CONDUCTIVITY		3.8e7
#define Z_ALUMINIUM_ELECTRICAL_RESISTIVITY		2.6e-8
#define Z_ALUMINIUM_SUPERCONDUCTING_POINT		1.175
#define Z_ALUMINIUM_MAGNETIC_TYPE			Z_MAGNETIC_TYPE_PARAMAGNETIC
#define Z_ALUMINIUM_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_ALUMINIUM_MASS_MAGNETIC_SUSCEPTIBILITY	7.80e-9
#define Z_ALUMINIUM_MOLAR_MAGNETIC_SUSCEPTIBILITY	2.10e-10
#define Z_ALUMINIUM_VOLUME_MAGNETIC_SUSCEPTIBILITY	0.0000211

/* MARK: - 14 - Si */

#define Z_SILICON_SYMBOL				Si
#define Z_SILICON_ATOMIC_NUMBER				14
#define Z_SILICON_GROUP					14
#define Z_SILICON_PERIOD				3
#define Z_SILICON_BLOCK					Z_CHEMICAL_ELEMENT_BLOCK_P
#define Z_SILICON_CLASS					Z_CHEMICAL_ELEMENT_CLASS_METALLOID
#define Z_SILICON_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_METALLOID
#define Z_SILICON_SPACE_GROUP_NUMBER			227
#define Z_SILICON_ATOMIC_MASS				28.0855
#define Z_SILICON_ATOMIC_RADIUS				111.0
#define Z_SILICON_COVALENT_RADIUS			111.0
#define Z_SILICON_VAN_DER_WAALS_RADIUS			210.0
#define Z_SILICON_DECAY_MODE				KEY_NOT_APPLICABLE
#define Z_SILICON_NEUTRON_CROSS_SECTION			0.166
#define Z_SILICON_NEUTRON_MASS_ABSORPTION		0.00020
#define Z_SILICON_VALENCE				4
#define Z_SILICON_ELECTRONEGATIVITY			1.90
#define Z_SILICON_ELECTRON_AFFINITY			133.6
#define Z_SILICON_MELTING_POINT				1414.0
#define Z_SILICON_BOILING_POINT				2.9e3
#define Z_SILICON_CRITICAL_PRESSURE			REAL_MISSING
#define Z_SILICON_CRITICAL_TEMPERATURE			REAL_MISSING
#define Z_SILICON_FUSION_HEAT				50.2
#define Z_SILICON_VAPORIZATION_HEAT			359.0
#define Z_SILICON_SPECIFIC_HEAT				710.0
#define Z_SILICON_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_SILICON_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_SILICON_THERMAL_CONDUCTIVITY			150.0
#define Z_SILICON_THERMAL_EXPANSION			2.6e-6
#define Z_SILICON_DENSITY				2330.0
#define Z_SILICON_LIQUID_DENSITY			2570.0
#define Z_SILICON_MOLAR_VOLUME				0.00001205
#define Z_SILICON_BRINELL_HARDNESS			REAL_MISSING
#define Z_SILICON_MOHS_HARDNESS				6.5
#define Z_SILICON_VICKERS_HARDNESS			9.63e3
#define Z_SILICON_BULK_MODULUS				1.0e2
#define Z_SILICON_SHEAR_MODULUS				REAL_MISSING
#define Z_SILICON_YOUNG_MODULUS				47.0
#define Z_SILICON_POISSON_RATIO				REAL_MISSING
#define Z_SILICON_REFRACTIVE_INDEX			REAL_MISSING
#define Z_SILICON_SOUND_SPEED				2.2e3
#define Z_SILICON_ELECTRICAL_TYPE			Z_ELECTRICAL_TYPE_SEMICONDUCTOR
#define Z_SILICON_ELECTRICAL_CONDUCTIVITY		1.0e3
#define Z_SILICON_ELECTRICAL_RESISTIVITY		0.001
#define Z_SILICON_SUPERCONDUCTING_POINT			REAL_MISSING
#define Z_SILICON_MAGNETIC_TYPE				Z_MAGNETIC_TYPE_DIAMAGNETIC
#define Z_SILICON_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_SILICON_MASS_MAGNETIC_SUSCEPTIBILITY		-1.60e-9
#define Z_SILICON_MOLAR_MAGNETIC_SUSCEPTIBILITY		-4.49e-11
#define Z_SILICON_VOLUME_MAGNETIC_SUSCEPTIBILITY	-3.73e-6

/* MARK: - 15 - P */

#define Z_PHOSPHORUS_SYMBOL				P
#define Z_PHOSPHORUS_ATOMIC_NUMBER			15
#define Z_PHOSPHORUS_GROUP				15
#define Z_PHOSPHORUS_PERIOD				3
#define Z_PHOSPHORUS_BLOCK				Z_CHEMICAL_ELEMENT_BLOCK_P
#define Z_PHOSPHORUS_CLASS				Z_CHEMICAL_ELEMENT_CLASS_NON_METAL
#define Z_PHOSPHORUS_METAL_STATE			Z_CHEMICAL_ELEMENT_METAL_STATE_NON_METAL
#define Z_PHOSPHORUS_SPACE_GROUP_NUMBER			2
#define Z_PHOSPHORUS_ATOMIC_MASS			30.973762
#define Z_PHOSPHORUS_ATOMIC_RADIUS			98.0
#define Z_PHOSPHORUS_COVALENT_RADIUS			107.0
#define Z_PHOSPHORUS_VAN_DER_WAALS_RADIUS		180.0
#define Z_PHOSPHORUS_DECAY_MODE				KEY_NOT_APPLICABLE
#define Z_PHOSPHORUS_NEUTRON_CROSS_SECTION		0.18
#define Z_PHOSPHORUS_NEUTRON_MASS_ABSORPTION		0.00020
#define Z_PHOSPHORUS_VALENCE				5
#define Z_PHOSPHORUS_ELECTRONEGATIVITY			2.19
#define Z_PHOSPHORUS_ELECTRON_AFFINITY			71.0
#define Z_PHOSPHORUS_MELTING_POINT			44.2
#define Z_PHOSPHORUS_BOILING_POINT			280.5
#define Z_PHOSPHORUS_CRITICAL_PRESSURE			REAL_MISSING
#define Z_PHOSPHORUS_CRITICAL_TEMPERATURE		REAL_MISSING
#define Z_PHOSPHORUS_FUSION_HEAT			0.64
#define Z_PHOSPHORUS_VAPORIZATION_HEAT			12.4
#define Z_PHOSPHORUS_SPECIFIC_HEAT			769.7
#define Z_PHOSPHORUS_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_PHOSPHORUS_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_PHOSPHORUS_THERMAL_CONDUCTIVITY		0.236
#define Z_PHOSPHORUS_THERMAL_EXPANSION			REAL_MISSING
#define Z_PHOSPHORUS_DENSITY				1823.0
#define Z_PHOSPHORUS_LIQUID_DENSITY			REAL_MISSING
#define Z_PHOSPHORUS_MOLAR_VOLUME			0.00001699
#define Z_PHOSPHORUS_BRINELL_HARDNESS			REAL_MISSING
#define Z_PHOSPHORUS_MOHS_HARDNESS			REAL_MISSING
#define Z_PHOSPHORUS_VICKERS_HARDNESS			REAL_MISSING
#define Z_PHOSPHORUS_BULK_MODULUS			11.0
#define Z_PHOSPHORUS_SHEAR_MODULUS			REAL_MISSING
#define Z_PHOSPHORUS_YOUNG_MODULUS			REAL_MISSING
#define Z_PHOSPHORUS_POISSON_RATIO			REAL_MISSING
#define Z_PHOSPHORUS_REFRACTIVE_INDEX			1.001212
#define Z_PHOSPHORUS_SOUND_SPEED			REAL_MISSING
#define Z_PHOSPHORUS_ELECTRICAL_TYPE			Z_ELECTRICAL_TYPE_CONDUCTOR
#define Z_PHOSPHORUS_ELECTRICAL_CONDUCTIVITY		1.0e7
#define Z_PHOSPHORUS_ELECTRICAL_RESISTIVITY		1.0e-7
#define Z_PHOSPHORUS_SUPERCONDUCTING_POINT		REAL_MISSING
#define Z_PHOSPHORUS_MAGNETIC_TYPE			Z_MAGNETIC_TYPE_DIAMAGNETIC
#define Z_PHOSPHORUS_CURIE_POINT			REAL_NOT_APPLICABLE
#define Z_PHOSPHORUS_MASS_MAGNETIC_SUSCEPTIBILITY	-1.130e-8
#define Z_PHOSPHORUS_MOLAR_MAGNETIC_SUSCEPTIBILITY	-3.500e-10
#define Z_PHOSPHORUS_VOLUME_MAGNETIC_SUSCEPTIBILITY	-0.00002060

/* MARK: - 16 - S */

#define Z_SULFUR_SYMBOL					S
#define Z_SULFUR_ATOMIC_NUMBER				16
#define Z_SULFUR_GROUP					16
#define Z_SULFUR_PERIOD					3
#define Z_SULFUR_BLOCK					Z_CHEMICAL_ELEMENT_BLOCK_P
#define Z_SULFUR_CLASS					Z_CHEMICAL_ELEMENT_CLASS_NON_METAL
#define Z_SULFUR_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_NON_METAL
#define Z_SULFUR_SPACE_GROUP_NUMBER			70
#define Z_SULFUR_ATOMIC_MASS				32.065
#define Z_SULFUR_ATOMIC_RADIUS				87.0
#define Z_SULFUR_COVALENT_RADIUS			105.0
#define Z_SULFUR_VAN_DER_WAALS_RADIUS			180.0
#define Z_SULFUR_DECAY_MODE				KEY_NOT_APPLICABLE
#define Z_SULFUR_NEUTRON_CROSS_SECTION			0.52
#define Z_SULFUR_NEUTRON_MASS_ABSORPTION		0.00055
#define Z_SULFUR_VALENCE				6
#define Z_SULFUR_ELECTRONEGATIVITY			2.58
#define Z_SULFUR_ELECTRON_AFFINITY			2.0e2
#define Z_SULFUR_MELTING_POINT				115.21
#define Z_SULFUR_BOILING_POINT				444.72
#define Z_SULFUR_CRITICAL_PRESSURE			20.7
#define Z_SULFUR_CRITICAL_TEMPERATURE			1314.0
#define Z_SULFUR_FUSION_HEAT				1.73
#define Z_SULFUR_VAPORIZATION_HEAT			9.8
#define Z_SULFUR_SPECIFIC_HEAT				705.0
#define Z_SULFUR_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_SULFUR_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_SULFUR_THERMAL_CONDUCTIVITY			0.205
#define Z_SULFUR_THERMAL_EXPANSION			REAL_MISSING
#define Z_SULFUR_DENSITY				1960.0
#define Z_SULFUR_LIQUID_DENSITY				1819.0
#define Z_SULFUR_MOLAR_VOLUME				0.00001636
#define Z_SULFUR_BRINELL_HARDNESS			REAL_MISSING
#define Z_SULFUR_MOHS_HARDNESS				2.0
#define Z_SULFUR_VICKERS_HARDNESS			REAL_MISSING
#define Z_SULFUR_BULK_MODULUS				7.7
#define Z_SULFUR_SHEAR_MODULUS				REAL_MISSING
#define Z_SULFUR_YOUNG_MODULUS				REAL_MISSING
#define Z_SULFUR_POISSON_RATIO				REAL_MISSING
#define Z_SULFUR_REFRACTIVE_INDEX			1.001111
#define Z_SULFUR_SOUND_SPEED				REAL_MISSING
#define Z_SULFUR_ELECTRICAL_TYPE			Z_ELECTRICAL_TYPE_INSULATOR
#define Z_SULFUR_ELECTRICAL_CONDUCTIVITY		1.0e-15
#define Z_SULFUR_ELECTRICAL_RESISTIVITY			1.0e15
#define Z_SULFUR_SUPERCONDUCTING_POINT			REAL_MISSING
#define Z_SULFUR_MAGNETIC_TYPE				Z_MAGNETIC_TYPE_DIAMAGNETIC
#define Z_SULFUR_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_SULFUR_MASS_MAGNETIC_SUSCEPTIBILITY		-6.20e-9
#define Z_SULFUR_MOLAR_MAGNETIC_SUSCEPTIBILITY		-1.99e-10
#define Z_SULFUR_VOLUME_MAGNETIC_SUSCEPTIBILITY		-0.0000122

/* MARK: - 17 - Cl */

#define Z_CHLORINE_SYMBOL				Cl
#define Z_CHLORINE_ATOMIC_NUMBER			17
#define Z_CHLORINE_GROUP				17
#define Z_CHLORINE_PERIOD				3
#define Z_CHLORINE_BLOCK				Z_CHEMICAL_ELEMENT_BLOCK_P
#define Z_CHLORINE_CLASS				Z_CHEMICAL_ELEMENT_CLASS_HALOGEN
#define Z_CHLORINE_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_NON_METAL
#define Z_CHLORINE_SPACE_GROUP_NUMBER			64
#define Z_CHLORINE_ATOMIC_MASS				35.453
#define Z_CHLORINE_ATOMIC_RADIUS			79.0
#define Z_CHLORINE_COVALENT_RADIUS			102.0
#define Z_CHLORINE_VAN_DER_WAALS_RADIUS			175.0
#define Z_CHLORINE_DECAY_MODE				KEY_NOT_APPLICABLE
#define Z_CHLORINE_NEUTRON_CROSS_SECTION		35.3
#define Z_CHLORINE_NEUTRON_MASS_ABSORPTION		0.033
#define Z_CHLORINE_VALENCE				5
#define Z_CHLORINE_ELECTRONEGATIVITY			3.16
#define Z_CHLORINE_ELECTRON_AFFINITY			349.0
#define Z_CHLORINE_MELTING_POINT			-101.5
#define Z_CHLORINE_BOILING_POINT			-34.04
#define Z_CHLORINE_CRITICAL_PRESSURE			7.991
#define Z_CHLORINE_CRITICAL_TEMPERATURE			416.9
#define Z_CHLORINE_FUSION_HEAT				3.2
#define Z_CHLORINE_VAPORIZATION_HEAT			10.2
#define Z_CHLORINE_SPECIFIC_HEAT			478.2
#define Z_CHLORINE_ADIABATIC_INDEX			Z_CHEMICAL_ELEMENT_ADIABATIC_INDEX_7_5
#define Z_CHLORINE_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_CHLORINE_THERMAL_CONDUCTIVITY			0.0089
#define Z_CHLORINE_THERMAL_EXPANSION			REAL_NOT_APPLICABLE
#define Z_CHLORINE_DENSITY				3.214
#define Z_CHLORINE_LIQUID_DENSITY			REAL_MISSING
#define Z_CHLORINE_MOLAR_VOLUME				0.01103
#define Z_CHLORINE_BRINELL_HARDNESS			REAL_NOT_APPLICABLE
#define Z_CHLORINE_MOHS_HARDNESS			REAL_NOT_APPLICABLE
#define Z_CHLORINE_VICKERS_HARDNESS			REAL_NOT_APPLICABLE
#define Z_CHLORINE_BULK_MODULUS				1.1
#define Z_CHLORINE_SHEAR_MODULUS			REAL_NOT_APPLICABLE
#define Z_CHLORINE_YOUNG_MODULUS			REAL_NOT_APPLICABLE
#define Z_CHLORINE_POISSON_RATIO			REAL_NOT_APPLICABLE
#define Z_CHLORINE_REFRACTIVE_INDEX			1.000773
#define Z_CHLORINE_SOUND_SPEED				206.0
#define Z_CHLORINE_ELECTRICAL_TYPE			Z_ELECTRICAL_TYPE_INSULATOR
#define Z_CHLORINE_ELECTRICAL_CONDUCTIVITY		0.010
#define Z_CHLORINE_ELECTRICAL_RESISTIVITY		100.0
#define Z_CHLORINE_SUPERCONDUCTING_POINT		REAL_MISSING
#define Z_CHLORINE_MAGNETIC_TYPE			Z_MAGNETIC_TYPE_DIAMAGNETIC
#define Z_CHLORINE_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_CHLORINE_MASS_MAGNETIC_SUSCEPTIBILITY		-7.20e-9
#define Z_CHLORINE_MOLAR_MAGNETIC_SUSCEPTIBILITY	-5.11e-10
#define Z_CHLORINE_VOLUME_MAGNETIC_SUSCEPTIBILITY	-2.31e-8

/* MARK: - 18 - Ar */

#define Z_ARGON_SYMBOL					Ar
#define Z_ARGON_ATOMIC_NUMBER				18
#define Z_ARGON_GROUP					18
#define Z_ARGON_PERIOD					3
#define Z_ARGON_BLOCK					Z_CHEMICAL_ELEMENT_BLOCK_P
#define Z_ARGON_CLASS					Z_CHEMICAL_ELEMENT_CLASS_NOBLE_GAS
#define Z_ARGON_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_NON_METAL
#define Z_ARGON_SPACE_GROUP_NUMBER			225
#define Z_ARGON_ATOMIC_MASS				39.948
#define Z_ARGON_ATOMIC_RADIUS				71.0
#define Z_ARGON_COVALENT_RADIUS				106.0
#define Z_ARGON_VAN_DER_WAALS_RADIUS			188.0
#define Z_ARGON_DECAY_MODE				KEY_NOT_APPLICABLE
#define Z_ARGON_NEUTRON_CROSS_SECTION			0.66
#define Z_ARGON_NEUTRON_MASS_ABSORPTION			0.00059
#define Z_ARGON_VALENCE					0
#define Z_ARGON_ELECTRONEGATIVITY			REAL_NOT_APPLICABLE
#define Z_ARGON_ELECTRON_AFFINITY			0.0
#define Z_ARGON_MELTING_POINT				-189.3
#define Z_ARGON_BOILING_POINT				-185.8
#define Z_ARGON_CRITICAL_PRESSURE			4.898
#define Z_ARGON_CRITICAL_TEMPERATURE			150.87
#define Z_ARGON_FUSION_HEAT				1.18
#define Z_ARGON_VAPORIZATION_HEAT			6.5
#define Z_ARGON_SPECIFIC_HEAT				520.33
#define Z_ARGON_ADIABATIC_INDEX				Z_CHEMICAL_ELEMENT_ADIABATIC_INDEX_5_3
#define Z_ARGON_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_ARGON_THERMAL_CONDUCTIVITY			0.01772
#define Z_ARGON_THERMAL_EXPANSION			REAL_NOT_APPLICABLE
#define Z_ARGON_DENSITY					1.784
#define Z_ARGON_LIQUID_DENSITY				REAL_MISSING
#define Z_ARGON_MOLAR_VOLUME				0.02239
#define Z_ARGON_BRINELL_HARDNESS			REAL_NOT_APPLICABLE
#define Z_ARGON_MOHS_HARDNESS				REAL_NOT_APPLICABLE
#define Z_ARGON_VICKERS_HARDNESS			REAL_NOT_APPLICABLE
#define Z_ARGON_BULK_MODULUS				REAL_NOT_APPLICABLE
#define Z_ARGON_SHEAR_MODULUS				REAL_NOT_APPLICABLE
#define Z_ARGON_YOUNG_MODULUS				REAL_NOT_APPLICABLE
#define Z_ARGON_POISSON_RATIO				REAL_NOT_APPLICABLE
#define Z_ARGON_REFRACTIVE_INDEX			1.000281
#define Z_ARGON_SOUND_SPEED				319.0
#define Z_ARGON_ELECTRICAL_TYPE				KEY_MISSING
#define Z_ARGON_ELECTRICAL_CONDUCTIVITY			REAL_MISSING
#define Z_ARGON_ELECTRICAL_RESISTIVITY			REAL_NOT_APPLICABLE
#define Z_ARGON_SUPERCONDUCTING_POINT			REAL_MISSING
#define Z_ARGON_MAGNETIC_TYPE				Z_MAGNETIC_TYPE_DIAMAGNETIC
#define Z_ARGON_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_ARGON_MASS_MAGNETIC_SUSCEPTIBILITY		-6.00e-9
#define Z_ARGON_MOLAR_MAGNETIC_SUSCEPTIBILITY		-2.40e-10
#define Z_ARGON_VOLUME_MAGNETIC_SUSCEPTIBILITY		-1.07e-8

/* MARK: - 19 - K */

#define Z_POTASSIUM_SYMBOL				K
#define Z_POTASSIUM_ATOMIC_NUMBER			19
#define Z_POTASSIUM_GROUP				1
#define Z_POTASSIUM_PERIOD				4
#define Z_POTASSIUM_BLOCK				Z_CHEMICAL_ELEMENT_BLOCK_S
#define Z_POTASSIUM_CLASS				Z_CHEMICAL_ELEMENT_CLASS_ALKALI_METAL
#define Z_POTASSIUM_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_POTASSIUM_SPACE_GROUP_NUMBER			229
#define Z_POTASSIUM_ATOMIC_MASS				39.0983
#define Z_POTASSIUM_ATOMIC_RADIUS			243.0
#define Z_POTASSIUM_COVALENT_RADIUS			203.0
#define Z_POTASSIUM_VAN_DER_WAALS_RADIUS		275.0
#define Z_POTASSIUM_DECAY_MODE				KEY_NOT_APPLICABLE
#define Z_POTASSIUM_NEUTRON_CROSS_SECTION		2.1
#define Z_POTASSIUM_NEUTRON_MASS_ABSORPTION		0.0018
#define Z_POTASSIUM_VALENCE				1
#define Z_POTASSIUM_ELECTRONEGATIVITY			0.82
#define Z_POTASSIUM_ELECTRON_AFFINITY			48.4
#define Z_POTASSIUM_MELTING_POINT			63.38
#define Z_POTASSIUM_BOILING_POINT			759.0
#define Z_POTASSIUM_CRITICAL_PRESSURE			16.0
#define Z_POTASSIUM_CRITICAL_TEMPERATURE		2223.0
#define Z_POTASSIUM_FUSION_HEAT				2.33
#define Z_POTASSIUM_VAPORIZATION_HEAT			76.9
#define Z_POTASSIUM_SPECIFIC_HEAT			757.0
#define Z_POTASSIUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_POTASSIUM_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_POTASSIUM_THERMAL_CONDUCTIVITY		1.0e2
#define Z_POTASSIUM_THERMAL_EXPANSION			REAL_MISSING
#define Z_POTASSIUM_DENSITY				856.0
#define Z_POTASSIUM_LIQUID_DENSITY			828.0
#define Z_POTASSIUM_MOLAR_VOLUME			0.0000457
#define Z_POTASSIUM_BRINELL_HARDNESS			0.363
#define Z_POTASSIUM_MOHS_HARDNESS			0.4
#define Z_POTASSIUM_VICKERS_HARDNESS			REAL_MISSING
#define Z_POTASSIUM_BULK_MODULUS			3.1
#define Z_POTASSIUM_SHEAR_MODULUS			1.3
#define Z_POTASSIUM_YOUNG_MODULUS			REAL_MISSING
#define Z_POTASSIUM_POISSON_RATIO			REAL_MISSING
#define Z_POTASSIUM_REFRACTIVE_INDEX			REAL_MISSING
#define Z_POTASSIUM_SOUND_SPEED				2.0e3
#define Z_POTASSIUM_ELECTRICAL_TYPE			Z_ELECTRICAL_TYPE_CONDUCTOR
#define Z_POTASSIUM_ELECTRICAL_CONDUCTIVITY		1.4e7
#define Z_POTASSIUM_ELECTRICAL_RESISTIVITY		7.0e-8
#define Z_POTASSIUM_SUPERCONDUCTING_POINT		REAL_MISSING
#define Z_POTASSIUM_MAGNETIC_TYPE			Z_MAGNETIC_TYPE_PARAMAGNETIC
#define Z_POTASSIUM_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_POTASSIUM_MASS_MAGNETIC_SUSCEPTIBILITY	6.70e-9
#define Z_POTASSIUM_MOLAR_MAGNETIC_SUSCEPTIBILITY	2.62e-10
#define Z_POTASSIUM_VOLUME_MAGNETIC_SUSCEPTIBILITY	5.74e-6

/* MARK: - 20 - Ca */

#define Z_CALCIUM_SYMBOL				Ca
#define Z_CALCIUM_ATOMIC_NUMBER				20
#define Z_CALCIUM_GROUP					2
#define Z_CALCIUM_PERIOD				4
#define Z_CALCIUM_BLOCK					Z_CHEMICAL_ELEMENT_BLOCK_S
#define Z_CALCIUM_CLASS					Z_CHEMICAL_ELEMENT_CLASS_ALKALINE_EARTH_METAL
#define Z_CALCIUM_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_CALCIUM_SPACE_GROUP_NUMBER			225
#define Z_CALCIUM_ATOMIC_MASS				40.078
#define Z_CALCIUM_ATOMIC_RADIUS				194.0
#define Z_CALCIUM_COVALENT_RADIUS			176.0
#define Z_CALCIUM_VAN_DER_WAALS_RADIUS			REAL_MISSING
#define Z_CALCIUM_DECAY_MODE				KEY_NOT_APPLICABLE
#define Z_CALCIUM_NEUTRON_CROSS_SECTION			0.43
#define Z_CALCIUM_NEUTRON_MASS_ABSORPTION		0.00037
#define Z_CALCIUM_VALENCE				2
#define Z_CALCIUM_ELECTRONEGATIVITY			1.0
#define Z_CALCIUM_ELECTRON_AFFINITY			2.37
#define Z_CALCIUM_MELTING_POINT				842.0
#define Z_CALCIUM_BOILING_POINT				1484.0
#define Z_CALCIUM_CRITICAL_PRESSURE			REAL_MISSING
#define Z_CALCIUM_CRITICAL_TEMPERATURE			REAL_MISSING
#define Z_CALCIUM_FUSION_HEAT				8.54
#define Z_CALCIUM_VAPORIZATION_HEAT			155.0
#define Z_CALCIUM_SPECIFIC_HEAT				631.0
#define Z_CALCIUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_CALCIUM_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_CALCIUM_THERMAL_CONDUCTIVITY			2.0e2
#define Z_CALCIUM_THERMAL_EXPANSION			0.0000223
#define Z_CALCIUM_DENSITY				1550.0
#define Z_CALCIUM_LIQUID_DENSITY			1378.0
#define Z_CALCIUM_MOLAR_VOLUME				0.00002586
#define Z_CALCIUM_BRINELL_HARDNESS			167.0
#define Z_CALCIUM_MOHS_HARDNESS				1.75
#define Z_CALCIUM_VICKERS_HARDNESS			REAL_MISSING
#define Z_CALCIUM_BULK_MODULUS				17.0
#define Z_CALCIUM_SHEAR_MODULUS				7.4
#define Z_CALCIUM_YOUNG_MODULUS				20.0
#define Z_CALCIUM_POISSON_RATIO				0.31
#define Z_CALCIUM_REFRACTIVE_INDEX			REAL_MISSING
#define Z_CALCIUM_SOUND_SPEED				3810.0
#define Z_CALCIUM_ELECTRICAL_TYPE			Z_ELECTRICAL_TYPE_CONDUCTOR
#define Z_CALCIUM_ELECTRICAL_CONDUCTIVITY		2.9e7
#define Z_CALCIUM_ELECTRICAL_RESISTIVITY		3.4e-8
#define Z_CALCIUM_SUPERCONDUCTING_POINT			REAL_MISSING
#define Z_CALCIUM_MAGNETIC_TYPE				Z_MAGNETIC_TYPE_DIAMAGNETIC
#define Z_CALCIUM_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_CALCIUM_MASS_MAGNETIC_SUSCEPTIBILITY		1.380e-8
#define Z_CALCIUM_MOLAR_MAGNETIC_SUSCEPTIBILITY		5.531e-10
#define Z_CALCIUM_VOLUME_MAGNETIC_SUSCEPTIBILITY	0.00002139

/* MARK: - 21 - Sc */

#define Z_SCANDIUM_SYMBOL				Sc
#define Z_SCANDIUM_ATOMIC_NUMBER			21
#define Z_SCANDIUM_GROUP				3
#define Z_SCANDIUM_PERIOD				4
#define Z_SCANDIUM_BLOCK				Z_CHEMICAL_ELEMENT_BLOCK_D
#define Z_SCANDIUM_CLASS				Z_CHEMICAL_ELEMENT_CLASS_TRANSITION_METAL
#define Z_SCANDIUM_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_SCANDIUM_SPACE_GROUP_NUMBER			194
#define Z_SCANDIUM_ATOMIC_MASS				44.955912
#define Z_SCANDIUM_ATOMIC_RADIUS			184.0
#define Z_SCANDIUM_COVALENT_RADIUS			170.0
#define Z_SCANDIUM_VAN_DER_WAALS_RADIUS			REAL_MISSING
#define Z_SCANDIUM_DECAY_MODE				KEY_NOT_APPLICABLE
#define Z_SCANDIUM_NEUTRON_CROSS_SECTION		27.2
#define Z_SCANDIUM_NEUTRON_MASS_ABSORPTION		0.025
#define Z_SCANDIUM_VALENCE				3
#define Z_SCANDIUM_ELECTRONEGATIVITY			1.36
#define Z_SCANDIUM_ELECTRON_AFFINITY			18.1
#define Z_SCANDIUM_MELTING_POINT			1541.0
#define Z_SCANDIUM_BOILING_POINT			2830.0
#define Z_SCANDIUM_CRITICAL_PRESSURE			REAL_MISSING
#define Z_SCANDIUM_CRITICAL_TEMPERATURE			REAL_MISSING
#define Z_SCANDIUM_FUSION_HEAT				16.0
#define Z_SCANDIUM_VAPORIZATION_HEAT			318.0
#define Z_SCANDIUM_SPECIFIC_HEAT			567.0
#define Z_SCANDIUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_SCANDIUM_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_SCANDIUM_THERMAL_CONDUCTIVITY			16.0
#define Z_SCANDIUM_THERMAL_EXPANSION			0.0000102
#define Z_SCANDIUM_DENSITY				2985.0
#define Z_SCANDIUM_LIQUID_DENSITY			2800.0
#define Z_SCANDIUM_MOLAR_VOLUME				0.00001506
#define Z_SCANDIUM_BRINELL_HARDNESS			750.0
#define Z_SCANDIUM_MOHS_HARDNESS			REAL_MISSING
#define Z_SCANDIUM_VICKERS_HARDNESS			REAL_MISSING
#define Z_SCANDIUM_BULK_MODULUS				57.0
#define Z_SCANDIUM_SHEAR_MODULUS			29.0
#define Z_SCANDIUM_YOUNG_MODULUS			74.0
#define Z_SCANDIUM_POISSON_RATIO			0.28
#define Z_SCANDIUM_REFRACTIVE_INDEX			REAL_MISSING
#define Z_SCANDIUM_SOUND_SPEED				REAL_MISSING
#define Z_SCANDIUM_ELECTRICAL_TYPE			Z_ELECTRICAL_TYPE_CONDUCTOR
#define Z_SCANDIUM_ELECTRICAL_CONDUCTIVITY		1.8e6
#define Z_SCANDIUM_ELECTRICAL_RESISTIVITY		5.5e-7
#define Z_SCANDIUM_SUPERCONDUCTING_POINT		0.05
#define Z_SCANDIUM_MAGNETIC_TYPE			Z_MAGNETIC_TYPE_PARAMAGNETIC
#define Z_SCANDIUM_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_SCANDIUM_MASS_MAGNETIC_SUSCEPTIBILITY		8.800e-8
#define Z_SCANDIUM_MOLAR_MAGNETIC_SUSCEPTIBILITY	3.956e-9
#define Z_SCANDIUM_VOLUME_MAGNETIC_SUSCEPTIBILITY	0.0002627

/* MARK: - 22 - Ti */

#define Z_TITANIUM_SYMBOL				Ti
#define Z_TITANIUM_ATOMIC_NUMBER			22
#define Z_TITANIUM_GROUP				4
#define Z_TITANIUM_PERIOD				4
#define Z_TITANIUM_BLOCK				Z_CHEMICAL_ELEMENT_BLOCK_D
#define Z_TITANIUM_CLASS				Z_CHEMICAL_ELEMENT_CLASS_TRANSITION_METAL
#define Z_TITANIUM_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_TITANIUM_SPACE_GROUP_NUMBER			194
#define Z_TITANIUM_ATOMIC_MASS				47.867
#define Z_TITANIUM_ATOMIC_RADIUS			176.0
#define Z_TITANIUM_COVALENT_RADIUS			160.0
#define Z_TITANIUM_VAN_DER_WAALS_RADIUS			REAL_MISSING
#define Z_TITANIUM_DECAY_MODE				KEY_NOT_APPLICABLE
#define Z_TITANIUM_NEUTRON_CROSS_SECTION		6.1
#define Z_TITANIUM_NEUTRON_MASS_ABSORPTION		0.0044
#define Z_TITANIUM_VALENCE				4
#define Z_TITANIUM_ELECTRONEGATIVITY			1.54
#define Z_TITANIUM_ELECTRON_AFFINITY			7.6
#define Z_TITANIUM_MELTING_POINT			1668.0
#define Z_TITANIUM_BOILING_POINT			3287.0
#define Z_TITANIUM_CRITICAL_PRESSURE			REAL_MISSING
#define Z_TITANIUM_CRITICAL_TEMPERATURE			REAL_MISSING
#define Z_TITANIUM_FUSION_HEAT				18.7
#define Z_TITANIUM_VAPORIZATION_HEAT			425.0
#define Z_TITANIUM_SPECIFIC_HEAT			520.0
#define Z_TITANIUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_TITANIUM_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_TITANIUM_THERMAL_CONDUCTIVITY			22.0
#define Z_TITANIUM_THERMAL_EXPANSION			8.6e-6
#define Z_TITANIUM_DENSITY				4507.0
#define Z_TITANIUM_LIQUID_DENSITY			4110.0
#define Z_TITANIUM_MOLAR_VOLUME				0.00001062
#define Z_TITANIUM_BRINELL_HARDNESS			715.0
#define Z_TITANIUM_MOHS_HARDNESS			6.0
#define Z_TITANIUM_VICKERS_HARDNESS			970.0
#define Z_TITANIUM_BULK_MODULUS				110.0
#define Z_TITANIUM_SHEAR_MODULUS			44.0
#define Z_TITANIUM_YOUNG_MODULUS			116.0
#define Z_TITANIUM_POISSON_RATIO			0.32
#define Z_TITANIUM_REFRACTIVE_INDEX			REAL_MISSING
#define Z_TITANIUM_SOUND_SPEED				4140.0
#define Z_TITANIUM_ELECTRICAL_TYPE			Z_ELECTRICAL_TYPE_CONDUCTOR
#define Z_TITANIUM_ELECTRICAL_CONDUCTIVITY		2.5e6
#define Z_TITANIUM_ELECTRICAL_RESISTIVITY		4.0e-7
#define Z_TITANIUM_SUPERCONDUCTING_POINT		0.40
#define Z_TITANIUM_MAGNETIC_TYPE			Z_MAGNETIC_TYPE_PARAMAGNETIC
#define Z_TITANIUM_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_TITANIUM_MASS_MAGNETIC_SUSCEPTIBILITY		4.010e-8
#define Z_TITANIUM_MOLAR_MAGNETIC_SUSCEPTIBILITY	1.919e-9
#define Z_TITANIUM_VOLUME_MAGNETIC_SUSCEPTIBILITY	0.0001807

/* MARK: - 23 - V */

#define Z_VANADIUM_SYMBOL				V
#define Z_VANADIUM_ATOMIC_NUMBER			23
#define Z_VANADIUM_GROUP				5
#define Z_VANADIUM_PERIOD				4
#define Z_VANADIUM_BLOCK				Z_CHEMICAL_ELEMENT_BLOCK_D
#define Z_VANADIUM_CLASS				Z_CHEMICAL_ELEMENT_CLASS_TRANSITION_METAL
#define Z_VANADIUM_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_VANADIUM_SPACE_GROUP_NUMBER			229
#define Z_VANADIUM_ATOMIC_MASS				50.9415
#define Z_VANADIUM_ATOMIC_RADIUS			171.0
#define Z_VANADIUM_COVALENT_RADIUS			153.0
#define Z_VANADIUM_VAN_DER_WAALS_RADIUS			REAL_MISSING
#define Z_VANADIUM_DECAY_MODE				KEY_NOT_APPLICABLE
#define Z_VANADIUM_NEUTRON_CROSS_SECTION		5.06
#define Z_VANADIUM_NEUTRON_MASS_ABSORPTION		0.0033
#define Z_VANADIUM_VALENCE				5
#define Z_VANADIUM_ELECTRONEGATIVITY			1.63
#define Z_VANADIUM_ELECTRON_AFFINITY			50.6
#define Z_VANADIUM_MELTING_POINT			1910.0
#define Z_VANADIUM_BOILING_POINT			3407.0
#define Z_VANADIUM_CRITICAL_PRESSURE			REAL_MISSING
#define Z_VANADIUM_CRITICAL_TEMPERATURE			REAL_MISSING
#define Z_VANADIUM_FUSION_HEAT				22.8
#define Z_VANADIUM_VAPORIZATION_HEAT			453.0
#define Z_VANADIUM_SPECIFIC_HEAT			489.0
#define Z_VANADIUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_VANADIUM_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_VANADIUM_THERMAL_CONDUCTIVITY			31.0
#define Z_VANADIUM_THERMAL_EXPANSION			8.4e-6
#define Z_VANADIUM_DENSITY				6110.0
#define Z_VANADIUM_LIQUID_DENSITY			5500.0
#define Z_VANADIUM_MOLAR_VOLUME				8.337e-6
#define Z_VANADIUM_BRINELL_HARDNESS			628.0
#define Z_VANADIUM_MOHS_HARDNESS			7.0
#define Z_VANADIUM_VICKERS_HARDNESS			628.0
#define Z_VANADIUM_BULK_MODULUS				160.0
#define Z_VANADIUM_SHEAR_MODULUS			47.0
#define Z_VANADIUM_YOUNG_MODULUS			128.0
#define Z_VANADIUM_POISSON_RATIO			0.37
#define Z_VANADIUM_REFRACTIVE_INDEX			REAL_MISSING
#define Z_VANADIUM_SOUND_SPEED				4560.0
#define Z_VANADIUM_ELECTRICAL_TYPE			Z_ELECTRICAL_TYPE_CONDUCTOR
#define Z_VANADIUM_ELECTRICAL_CONDUCTIVITY		5.0e6
#define Z_VANADIUM_ELECTRICAL_RESISTIVITY		2.0e-7
#define Z_VANADIUM_SUPERCONDUCTING_POINT		5.40
#define Z_VANADIUM_MAGNETIC_TYPE			Z_MAGNETIC_TYPE_PARAMAGNETIC
#define Z_VANADIUM_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_VANADIUM_MASS_MAGNETIC_SUSCEPTIBILITY		6.280e-8
#define Z_VANADIUM_MOLAR_MAGNETIC_SUSCEPTIBILITY	3.199e-9
#define Z_VANADIUM_VOLUME_MAGNETIC_SUSCEPTIBILITY	0.0003837

/* MARK: - 24 - Cr */

#define Z_CHROMIUM_SYMBOL				Cr
#define Z_CHROMIUM_ATOMIC_NUMBER			24
#define Z_CHROMIUM_GROUP				6
#define Z_CHROMIUM_PERIOD				4
#define Z_CHROMIUM_BLOCK				Z_CHEMICAL_ELEMENT_BLOCK_D
#define Z_CHROMIUM_CLASS				Z_CHEMICAL_ELEMENT_CLASS_TRANSITION_METAL
#define Z_CHROMIUM_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_CHROMIUM_SPACE_GROUP_NUMBER			229
#define Z_CHROMIUM_ATOMIC_MASS				51.9961
#define Z_CHROMIUM_ATOMIC_RADIUS			166.0
#define Z_CHROMIUM_COVALENT_RADIUS			139.0
#define Z_CHROMIUM_VAN_DER_WAALS_RADIUS			REAL_MISSING
#define Z_CHROMIUM_DECAY_MODE				KEY_NOT_APPLICABLE
#define Z_CHROMIUM_NEUTRON_CROSS_SECTION		3.1
#define Z_CHROMIUM_NEUTRON_MASS_ABSORPTION		0.0021
#define Z_CHROMIUM_VALENCE				6
#define Z_CHROMIUM_ELECTRONEGATIVITY			1.66
#define Z_CHROMIUM_ELECTRON_AFFINITY			64.3
#define Z_CHROMIUM_MELTING_POINT			1907.0
#define Z_CHROMIUM_BOILING_POINT			2671.0
#define Z_CHROMIUM_CRITICAL_PRESSURE			REAL_MISSING
#define Z_CHROMIUM_CRITICAL_TEMPERATURE			REAL_MISSING
#define Z_CHROMIUM_FUSION_HEAT				20.5
#define Z_CHROMIUM_VAPORIZATION_HEAT			339.0
#define Z_CHROMIUM_SPECIFIC_HEAT			448.0
#define Z_CHROMIUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_CHROMIUM_NEEL_POINT				393.0
#define Z_CHROMIUM_THERMAL_CONDUCTIVITY			94.0
#define Z_CHROMIUM_THERMAL_EXPANSION			4.9e-6
#define Z_CHROMIUM_DENSITY				7190.0
#define Z_CHROMIUM_LIQUID_DENSITY			6300.0
#define Z_CHROMIUM_MOLAR_VOLUME				7.232e-6
#define Z_CHROMIUM_BRINELL_HARDNESS			1120.0
#define Z_CHROMIUM_MOHS_HARDNESS			8.5
#define Z_CHROMIUM_VICKERS_HARDNESS			1060.0
#define Z_CHROMIUM_BULK_MODULUS				160.0
#define Z_CHROMIUM_SHEAR_MODULUS			115.0
#define Z_CHROMIUM_YOUNG_MODULUS			279.0
#define Z_CHROMIUM_POISSON_RATIO			0.21
#define Z_CHROMIUM_REFRACTIVE_INDEX			REAL_MISSING
#define Z_CHROMIUM_SOUND_SPEED				5940.0
#define Z_CHROMIUM_ELECTRICAL_TYPE			Z_ELECTRICAL_TYPE_CONDUCTOR
#define Z_CHROMIUM_ELECTRICAL_CONDUCTIVITY		7.9e6
#define Z_CHROMIUM_ELECTRICAL_RESISTIVITY		1.3e-7
#define Z_CHROMIUM_SUPERCONDUCTING_POINT		REAL_MISSING
#define Z_CHROMIUM_MAGNETIC_TYPE			Z_MAGNETIC_TYPE_ANTIFERROMAGNETIC
#define Z_CHROMIUM_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_CHROMIUM_MASS_MAGNETIC_SUSCEPTIBILITY		4.450e-8
#define Z_CHROMIUM_MOLAR_MAGNETIC_SUSCEPTIBILITY	2.314e-9
#define Z_CHROMIUM_VOLUME_MAGNETIC_SUSCEPTIBILITY	0.0003177

/* MARK: - 25 - Mn */

#define Z_MANGANESE_SYMBOL				Mn
#define Z_MANGANESE_ATOMIC_NUMBER			25
#define Z_MANGANESE_GROUP				7
#define Z_MANGANESE_PERIOD				4
#define Z_MANGANESE_BLOCK				Z_CHEMICAL_ELEMENT_BLOCK_D
#define Z_MANGANESE_CLASS				Z_CHEMICAL_ELEMENT_CLASS_TRANSITION_METAL
#define Z_MANGANESE_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_MANGANESE_SPACE_GROUP_NUMBER			217
#define Z_MANGANESE_ATOMIC_MASS				54.938045
#define Z_MANGANESE_ATOMIC_RADIUS			161.0
#define Z_MANGANESE_COVALENT_RADIUS			139.0
#define Z_MANGANESE_VAN_DER_WAALS_RADIUS		REAL_MISSING
#define Z_MANGANESE_DECAY_MODE				KEY_NOT_APPLICABLE
#define Z_MANGANESE_NEUTRON_CROSS_SECTION		13.3
#define Z_MANGANESE_NEUTRON_MASS_ABSORPTION		0.0083
#define Z_MANGANESE_VALENCE				4
#define Z_MANGANESE_ELECTRONEGATIVITY			1.55
#define Z_MANGANESE_ELECTRON_AFFINITY			0.0
#define Z_MANGANESE_MELTING_POINT			1246.0
#define Z_MANGANESE_BOILING_POINT			2061.0
#define Z_MANGANESE_CRITICAL_PRESSURE			REAL_MISSING
#define Z_MANGANESE_CRITICAL_TEMPERATURE		REAL_MISSING
#define Z_MANGANESE_FUSION_HEAT				13.2
#define Z_MANGANESE_VAPORIZATION_HEAT			220.0
#define Z_MANGANESE_SPECIFIC_HEAT			479.0
#define Z_MANGANESE_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_MANGANESE_NEEL_POINT				100.0
#define Z_MANGANESE_THERMAL_CONDUCTIVITY		7.7
#define Z_MANGANESE_THERMAL_EXPANSION			0.0000217
#define Z_MANGANESE_DENSITY				7470.0
#define Z_MANGANESE_LIQUID_DENSITY			5950.0
#define Z_MANGANESE_MOLAR_VOLUME			7.354e-6
#define Z_MANGANESE_BRINELL_HARDNESS			196.0
#define Z_MANGANESE_MOHS_HARDNESS			6.0
#define Z_MANGANESE_VICKERS_HARDNESS			REAL_MISSING
#define Z_MANGANESE_BULK_MODULUS			120.0
#define Z_MANGANESE_SHEAR_MODULUS			REAL_MISSING
#define Z_MANGANESE_YOUNG_MODULUS			198.0
#define Z_MANGANESE_POISSON_RATIO			REAL_MISSING
#define Z_MANGANESE_REFRACTIVE_INDEX			REAL_MISSING
#define Z_MANGANESE_SOUND_SPEED				5150.0
#define Z_MANGANESE_ELECTRICAL_TYPE			Z_ELECTRICAL_TYPE_CONDUCTOR
#define Z_MANGANESE_ELECTRICAL_CONDUCTIVITY		6.2e5
#define Z_MANGANESE_ELECTRICAL_RESISTIVITY		1.6e-6
#define Z_MANGANESE_SUPERCONDUCTING_POINT		REAL_MISSING
#define Z_MANGANESE_MAGNETIC_TYPE			Z_MAGNETIC_TYPE_PARAMAGNETIC
#define Z_MANGANESE_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_MANGANESE_MASS_MAGNETIC_SUSCEPTIBILITY	1.2100e-7
#define Z_MANGANESE_MOLAR_MAGNETIC_SUSCEPTIBILITY	6.6475e-9
#define Z_MANGANESE_VOLUME_MAGNETIC_SUSCEPTIBILITY	0.00090387

/* MARK: - 26 - Fe */

#define Z_IRON_SYMBOL					Fe
#define Z_IRON_ATOMIC_NUMBER				26
#define Z_IRON_GROUP					8
#define Z_IRON_PERIOD					4
#define Z_IRON_BLOCK					Z_CHEMICAL_ELEMENT_BLOCK_D
#define Z_IRON_CLASS					Z_CHEMICAL_ELEMENT_CLASS_TRANSITION_METAL
#define Z_IRON_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_IRON_SPACE_GROUP_NUMBER			229
#define Z_IRON_ATOMIC_MASS				55.845
#define Z_IRON_ATOMIC_RADIUS				156.0
#define Z_IRON_COVALENT_RADIUS				132.0
#define Z_IRON_VAN_DER_WAALS_RADIUS			REAL_UNKNOWN
#define Z_IRON_DECAY_MODE				KEY_NOT_APPLICABLE
#define Z_IRON_NEUTRON_CROSS_SECTION			2.56
#define Z_IRON_NEUTRON_MASS_ABSORPTION			0.0015
#define Z_IRON_VALENCE					3
#define Z_IRON_ELECTRONEGATIVITY			1.83
#define Z_IRON_ELECTRON_AFFINITY			15.7
#define Z_IRON_MELTING_POINT				1538.0
#define Z_IRON_BOILING_POINT				2861.0
#define Z_IRON_CRITICAL_PRESSURE			REAL_UNKNOWN
#define Z_IRON_CRITICAL_TEMPERATURE			REAL_UNKNOWN
#define Z_IRON_FUSION_HEAT				13.8
#define Z_IRON_VAPORIZATION_HEAT			347.0
#define Z_IRON_SPECIFIC_HEAT				449.0
#define Z_IRON_ADIABATIC_INDEX				KEY_NOT_APPLICABLE
#define Z_IRON_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_IRON_THERMAL_CONDUCTIVITY			79.0
#define Z_IRON_THERMAL_EXPANSION			0.0000118
#define Z_IRON_DENSITY					7874.0
#define Z_IRON_LIQUID_DENSITY				6980.0
#define Z_IRON_MOLAR_VOLUME				7.092e-6
#define Z_IRON_BRINELL_HARDNESS				490.0
#define Z_IRON_MOHS_HARDNESS				4.0
#define Z_IRON_VICKERS_HARDNESS				608.0
#define Z_IRON_BULK_MODULUS				170.0
#define Z_IRON_SHEAR_MODULUS				82.0
#define Z_IRON_YOUNG_MODULUS				211.0
#define Z_IRON_POISSON_RATIO				0.29
#define Z_IRON_REFRACTIVE_INDEX				REAL_UNKNOWN
#define Z_IRON_SOUND_SPEED				4910.0
#define Z_IRON_ELECTRICAL_TYPE				Z_ELECTRICAL_TYPE_CONDUCTOR
#define Z_IRON_ELECTRICAL_CONDUCTIVITY			1.0e7
#define Z_IRON_ELECTRICAL_RESISTIVITY			9.7e-8
#define Z_IRON_SUPERCONDUCTING_POINT			REAL_UNKNOWN
#define Z_IRON_MAGNETIC_TYPE				Z_MAGNETIC_TYPE_FERROMAGNETIC
#define Z_IRON_CURIE_POINT				1043.0
#define Z_IRON_MASS_MAGNETIC_SUSCEPTIBILITY		REAL_MISSING
#define Z_IRON_MOLAR_MAGNETIC_SUSCEPTIBILITY		REAL_MISSING
#define Z_IRON_VOLUME_MAGNETIC_SUSCEPTIBILITY		REAL_MISSING

/* MARK: - 27 - Co */

#define Z_COBALT_SYMBOL					Co
#define Z_COBALT_ATOMIC_NUMBER				27
#define Z_COBALT_GROUP					9
#define Z_COBALT_PERIOD					4
#define Z_COBALT_BLOCK					Z_CHEMICAL_ELEMENT_BLOCK_D
#define Z_COBALT_CLASS					Z_CHEMICAL_ELEMENT_CLASS_TRANSITION_METAL
#define Z_COBALT_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_COBALT_SPACE_GROUP_NUMBER			194
#define Z_COBALT_ATOMIC_MASS				58.933195
#define Z_COBALT_ATOMIC_RADIUS				152.0
#define Z_COBALT_COVALENT_RADIUS			126.0
#define Z_COBALT_VAN_DER_WAALS_RADIUS			REAL_UNKNOWN
#define Z_COBALT_DECAY_MODE				KEY_NOT_APPLICABLE
#define Z_COBALT_NEUTRON_CROSS_SECTION			37.2
#define Z_COBALT_NEUTRON_MASS_ABSORPTION		0.021
#define Z_COBALT_VALENCE				4
#define Z_COBALT_ELECTRONEGATIVITY			1.88
#define Z_COBALT_ELECTRON_AFFINITY			63.7
#define Z_COBALT_MELTING_POINT				1495.0
#define Z_COBALT_BOILING_POINT				2927.0
#define Z_COBALT_CRITICAL_PRESSURE			REAL_UNKNOWN
#define Z_COBALT_CRITICAL_TEMPERATURE			REAL_UNKNOWN
#define Z_COBALT_FUSION_HEAT				16.2
#define Z_COBALT_VAPORIZATION_HEAT			375.0
#define Z_COBALT_SPECIFIC_HEAT				421.0
#define Z_COBALT_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_COBALT_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_COBALT_THERMAL_CONDUCTIVITY			1.0e2
#define Z_COBALT_THERMAL_EXPANSION			0.0000130
#define Z_COBALT_DENSITY				8.9e3
#define Z_COBALT_LIQUID_DENSITY				7750.0
#define Z_COBALT_MOLAR_VOLUME				6.6e-6
#define Z_COBALT_BRINELL_HARDNESS			7.0e2
#define Z_COBALT_MOHS_HARDNESS				5.0
#define Z_COBALT_VICKERS_HARDNESS			1043.0
#define Z_COBALT_BULK_MODULUS				180.0
#define Z_COBALT_SHEAR_MODULUS				76.0
#define Z_COBALT_YOUNG_MODULUS				209.0
#define Z_COBALT_POISSON_RATIO				0.31
#define Z_COBALT_REFRACTIVE_INDEX			REAL_UNKNOWN
#define Z_COBALT_SOUND_SPEED				4720.0
#define Z_COBALT_ELECTRICAL_TYPE			Z_ELECTRICAL_TYPE_CONDUCTOR
#define Z_COBALT_ELECTRICAL_CONDUCTIVITY		1.7e7
#define Z_COBALT_ELECTRICAL_RESISTIVITY			6.0e-8
#define Z_COBALT_SUPERCONDUCTING_POINT			REAL_UNKNOWN
#define Z_COBALT_MAGNETIC_TYPE				Z_MAGNETIC_TYPE_FERROMAGNETIC
#define Z_COBALT_CURIE_POINT				1394.0
#define Z_COBALT_MASS_MAGNETIC_SUSCEPTIBILITY		REAL_MISSING
#define Z_COBALT_MOLAR_MAGNETIC_SUSCEPTIBILITY		REAL_MISSING
#define Z_COBALT_VOLUME_MAGNETIC_SUSCEPTIBILITY		REAL_MISSING

/* MARK: - 28 - Ni */

#define Z_NICKEL_SYMBOL					Ni
#define Z_NICKEL_ATOMIC_NUMBER				28
#define Z_NICKEL_GROUP					10
#define Z_NICKEL_PERIOD					4
#define Z_NICKEL_BLOCK					Z_CHEMICAL_ELEMENT_BLOCK_D
#define Z_NICKEL_CLASS					Z_CHEMICAL_ELEMENT_CLASS_TRANSITION_METAL
#define Z_NICKEL_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_NICKEL_SPACE_GROUP_NUMBER			225
#define Z_NICKEL_ATOMIC_MASS				58.6934
#define Z_NICKEL_ATOMIC_RADIUS				149.0
#define Z_NICKEL_COVALENT_RADIUS			124.0
#define Z_NICKEL_VAN_DER_WAALS_RADIUS			163.0
#define Z_NICKEL_DECAY_MODE				KEY_NOT_APPLICABLE
#define Z_NICKEL_NEUTRON_CROSS_SECTION			4.5
#define Z_NICKEL_NEUTRON_MASS_ABSORPTION		0.0026
#define Z_NICKEL_VALENCE				2
#define Z_NICKEL_ELECTRONEGATIVITY			1.91
#define Z_NICKEL_ELECTRON_AFFINITY			112.0
#define Z_NICKEL_MELTING_POINT				1455.0
#define Z_NICKEL_BOILING_POINT				2913.0
#define Z_NICKEL_CRITICAL_PRESSURE			REAL_MISSING
#define Z_NICKEL_CRITICAL_TEMPERATURE			REAL_MISSING
#define Z_NICKEL_FUSION_HEAT				17.2
#define Z_NICKEL_VAPORIZATION_HEAT			378.0
#define Z_NICKEL_SPECIFIC_HEAT				445.0
#define Z_NICKEL_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_NICKEL_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_NICKEL_THERMAL_CONDUCTIVITY			91.0
#define Z_NICKEL_THERMAL_EXPANSION			0.0000134
#define Z_NICKEL_DENSITY				8908.0
#define Z_NICKEL_LIQUID_DENSITY				7810.0
#define Z_NICKEL_MOLAR_VOLUME				6.589e-6
#define Z_NICKEL_BRINELL_HARDNESS			7.0e2
#define Z_NICKEL_MOHS_HARDNESS				4.0
#define Z_NICKEL_VICKERS_HARDNESS			638.0
#define Z_NICKEL_BULK_MODULUS				180.0
#define Z_NICKEL_SHEAR_MODULUS				76.0
#define Z_NICKEL_YOUNG_MODULUS				2.0e2
#define Z_NICKEL_POISSON_RATIO				0.31
#define Z_NICKEL_REFRACTIVE_INDEX			REAL_MISSING
#define Z_NICKEL_SOUND_SPEED				4970.0
#define Z_NICKEL_ELECTRICAL_TYPE			Z_ELECTRICAL_TYPE_CONDUCTOR
#define Z_NICKEL_ELECTRICAL_CONDUCTIVITY		1.4e7
#define Z_NICKEL_ELECTRICAL_RESISTIVITY			7.0e-8
#define Z_NICKEL_SUPERCONDUCTING_POINT			REAL_MISSING
#define Z_NICKEL_MAGNETIC_TYPE				Z_MAGNETIC_TYPE_FERROMAGNETIC
#define Z_NICKEL_CURIE_POINT				631.0
#define Z_NICKEL_MASS_MAGNETIC_SUSCEPTIBILITY		REAL_MISSING
#define Z_NICKEL_MOLAR_MAGNETIC_SUSCEPTIBILITY		REAL_MISSING
#define Z_NICKEL_VOLUME_MAGNETIC_SUSCEPTIBILITY		REAL_MISSING

/* MARK: - 29 - Cu */

#define Z_COPPER_SYMBOL					Cu
#define Z_COPPER_ATOMIC_NUMBER				29
#define Z_COPPER_GROUP					11
#define Z_COPPER_PERIOD					4
#define Z_COPPER_BLOCK					Z_CHEMICAL_ELEMENT_BLOCK_D
#define Z_COPPER_CLASS					Z_CHEMICAL_ELEMENT_CLASS_TRANSITION_METAL
#define Z_COPPER_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_COPPER_SPACE_GROUP_NUMBER			225
#define Z_COPPER_ATOMIC_MASS				63.546
#define Z_COPPER_ATOMIC_RADIUS				145.0
#define Z_COPPER_COVALENT_RADIUS			132.0
#define Z_COPPER_VAN_DER_WAALS_RADIUS			140.0
#define Z_COPPER_DECAY_MODE				KEY_NOT_APPLICABLE
#define Z_COPPER_NEUTRON_CROSS_SECTION			3.78
#define Z_COPPER_NEUTRON_MASS_ABSORPTION		0.0021
#define Z_COPPER_VALENCE				2
#define Z_COPPER_ELECTRONEGATIVITY			1.90
#define Z_COPPER_ELECTRON_AFFINITY			118.4
#define Z_COPPER_MELTING_POINT				1084.62
#define Z_COPPER_BOILING_POINT				2927.0
#define Z_COPPER_CRITICAL_PRESSURE			REAL_MISSING
#define Z_COPPER_CRITICAL_TEMPERATURE			REAL_MISSING
#define Z_COPPER_FUSION_HEAT				13.1
#define Z_COPPER_VAPORIZATION_HEAT			3.0e2
#define Z_COPPER_SPECIFIC_HEAT				384.4
#define Z_COPPER_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_COPPER_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_COPPER_THERMAL_CONDUCTIVITY			4.0e2
#define Z_COPPER_THERMAL_EXPANSION			0.0000165
#define Z_COPPER_DENSITY				8960.0
#define Z_COPPER_LIQUID_DENSITY				8020.0
#define Z_COPPER_MOLAR_VOLUME				7.092e-6
#define Z_COPPER_BRINELL_HARDNESS			874.0
#define Z_COPPER_MOHS_HARDNESS				3.0
#define Z_COPPER_VICKERS_HARDNESS			369.0
#define Z_COPPER_BULK_MODULUS				140.0
#define Z_COPPER_SHEAR_MODULUS				48.0
#define Z_COPPER_YOUNG_MODULUS				130.0
#define Z_COPPER_POISSON_RATIO				0.34
#define Z_COPPER_REFRACTIVE_INDEX			REAL_MISSING
#define Z_COPPER_SOUND_SPEED				3570.0
#define Z_COPPER_ELECTRICAL_TYPE			Z_ELECTRICAL_TYPE_CONDUCTOR
#define Z_COPPER_ELECTRICAL_CONDUCTIVITY		5.9e7
#define Z_COPPER_ELECTRICAL_RESISTIVITY			1.7e-8
#define Z_COPPER_SUPERCONDUCTING_POINT			REAL_MISSING
#define Z_COPPER_MAGNETIC_TYPE				Z_MAGNETIC_TYPE_DIAMAGNETIC
#define Z_COPPER_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_COPPER_MASS_MAGNETIC_SUSCEPTIBILITY		-1.08e-9
#define Z_COPPER_MOLAR_MAGNETIC_SUSCEPTIBILITY		-6.86e-11
#define Z_COPPER_VOLUME_MAGNETIC_SUSCEPTIBILITY		-9.63e-6

/* MARK: - 30 - Zn */

#define Z_ZINC_SYMBOL					Zn
#define Z_ZINC_ATOMIC_NUMBER				30
#define Z_ZINC_GROUP					12
#define Z_ZINC_PERIOD					4
#define Z_ZINC_BLOCK					Z_CHEMICAL_ELEMENT_BLOCK_D
#define Z_ZINC_CLASS					Z_CHEMICAL_ELEMENT_CLASS_TRANSITION_METAL
#define Z_ZINC_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_ZINC_SPACE_GROUP_NUMBER			194
#define Z_ZINC_ATOMIC_MASS				65.38
#define Z_ZINC_ATOMIC_RADIUS				142.0
#define Z_ZINC_COVALENT_RADIUS				122.0
#define Z_ZINC_VAN_DER_WAALS_RADIUS			139.0
#define Z_ZINC_DECAY_MODE				KEY_NOT_APPLICABLE
#define Z_ZINC_NEUTRON_CROSS_SECTION			1.1
#define Z_ZINC_NEUTRON_MASS_ABSORPTION			0.00055
#define Z_ZINC_VALENCE					2
#define Z_ZINC_ELECTRONEGATIVITY			1.65
#define Z_ZINC_ELECTRON_AFFINITY			0.0
#define Z_ZINC_MELTING_POINT				419.53
#define Z_ZINC_BOILING_POINT				907.0
#define Z_ZINC_CRITICAL_PRESSURE			REAL_MISSING
#define Z_ZINC_CRITICAL_TEMPERATURE			REAL_MISSING
#define Z_ZINC_FUSION_HEAT				7.35
#define Z_ZINC_VAPORIZATION_HEAT			119.0
#define Z_ZINC_SPECIFIC_HEAT				388.0
#define Z_ZINC_ADIABATIC_INDEX				KEY_NOT_APPLICABLE
#define Z_ZINC_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_ZINC_THERMAL_CONDUCTIVITY			120.0
#define Z_ZINC_THERMAL_EXPANSION			0.0000302
#define Z_ZINC_DENSITY					7140.0
#define Z_ZINC_LIQUID_DENSITY				6570.0
#define Z_ZINC_MOLAR_VOLUME				9.16e-6
#define Z_ZINC_BRINELL_HARDNESS				412.0
#define Z_ZINC_MOHS_HARDNESS				2.5
#define Z_ZINC_VICKERS_HARDNESS				REAL_MISSING
#define Z_ZINC_BULK_MODULUS				70.0
#define Z_ZINC_SHEAR_MODULUS				43.0
#define Z_ZINC_YOUNG_MODULUS				108.0
#define Z_ZINC_POISSON_RATIO				0.25
#define Z_ZINC_REFRACTIVE_INDEX				1.002050
#define Z_ZINC_SOUND_SPEED				3.7e3
#define Z_ZINC_ELECTRICAL_TYPE				Z_ELECTRICAL_TYPE_CONDUCTOR
#define Z_ZINC_ELECTRICAL_CONDUCTIVITY			1.7e7
#define Z_ZINC_ELECTRICAL_RESISTIVITY			5.9e-8
#define Z_ZINC_SUPERCONDUCTING_POINT			0.85
#define Z_ZINC_MAGNETIC_TYPE				Z_MAGNETIC_TYPE_DIAMAGNETIC
#define Z_ZINC_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_ZINC_MASS_MAGNETIC_SUSCEPTIBILITY		-2.21e-9
#define Z_ZINC_MOLAR_MAGNETIC_SUSCEPTIBILITY		-1.45e-10
#define Z_ZINC_VOLUME_MAGNETIC_SUSCEPTIBILITY		-0.0000158

/* MARK: - 31 - Ga */

#define Z_GALLIUM_SYMBOL				Ga
#define Z_GALLIUM_ATOMIC_NUMBER				31
#define Z_GALLIUM_GROUP					13
#define Z_GALLIUM_PERIOD				4
#define Z_GALLIUM_BLOCK					Z_CHEMICAL_ELEMENT_BLOCK_P
#define Z_GALLIUM_CLASS					Z_CHEMICAL_ELEMENT_CLASS_POOR_METAL
#define Z_GALLIUM_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_GALLIUM_SPACE_GROUP_NUMBER			64
#define Z_GALLIUM_ATOMIC_MASS				69.723
#define Z_GALLIUM_ATOMIC_RADIUS				136.0
#define Z_GALLIUM_COVALENT_RADIUS			122.0
#define Z_GALLIUM_VAN_DER_WAALS_RADIUS			187.0
#define Z_GALLIUM_DECAY_MODE				KEY_NOT_APPLICABLE
#define Z_GALLIUM_NEUTRON_CROSS_SECTION			2.9
#define Z_GALLIUM_NEUTRON_MASS_ABSORPTION		0.0015
#define Z_GALLIUM_VALENCE				3
#define Z_GALLIUM_ELECTRONEGATIVITY			1.81
#define Z_GALLIUM_ELECTRON_AFFINITY			28.9
#define Z_GALLIUM_MELTING_POINT				29.76
#define Z_GALLIUM_BOILING_POINT				2204.0
#define Z_GALLIUM_CRITICAL_PRESSURE			REAL_MISSING
#define Z_GALLIUM_CRITICAL_TEMPERATURE			REAL_MISSING
#define Z_GALLIUM_FUSION_HEAT				5.59
#define Z_GALLIUM_VAPORIZATION_HEAT			256.0
#define Z_GALLIUM_SPECIFIC_HEAT				371.0
#define Z_GALLIUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_GALLIUM_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_GALLIUM_THERMAL_CONDUCTIVITY			29.0
#define Z_GALLIUM_THERMAL_EXPANSION			0.000120
#define Z_GALLIUM_DENSITY				5904.0
#define Z_GALLIUM_LIQUID_DENSITY			6095.0
#define Z_GALLIUM_MOLAR_VOLUME				0.00001181
#define Z_GALLIUM_BRINELL_HARDNESS			60.0
#define Z_GALLIUM_MOHS_HARDNESS				1.5
#define Z_GALLIUM_VICKERS_HARDNESS			REAL_MISSING
#define Z_GALLIUM_BULK_MODULUS				REAL_MISSING
#define Z_GALLIUM_SHEAR_MODULUS				REAL_MISSING
#define Z_GALLIUM_YOUNG_MODULUS				REAL_MISSING
#define Z_GALLIUM_POISSON_RATIO				REAL_MISSING
#define Z_GALLIUM_REFRACTIVE_INDEX			REAL_MISSING
#define Z_GALLIUM_SOUND_SPEED				2740.0
#define Z_GALLIUM_ELECTRICAL_TYPE			Z_ELECTRICAL_TYPE_CONDUCTOR
#define Z_GALLIUM_ELECTRICAL_CONDUCTIVITY		7.1e6
#define Z_GALLIUM_ELECTRICAL_RESISTIVITY		1.4e-7
#define Z_GALLIUM_SUPERCONDUCTING_POINT			1.083
#define Z_GALLIUM_MAGNETIC_TYPE				Z_MAGNETIC_TYPE_DIAMAGNETIC
#define Z_GALLIUM_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_GALLIUM_MASS_MAGNETIC_SUSCEPTIBILITY		-3.00e-9
#define Z_GALLIUM_MOLAR_MAGNETIC_SUSCEPTIBILITY		-2.09e-10
#define Z_GALLIUM_VOLUME_MAGNETIC_SUSCEPTIBILITY	-0.0000177

/* MARK: - 32 - Ge */

#define Z_GERMANIUM_SYMBOL				Ge
#define Z_GERMANIUM_ATOMIC_NUMBER			32
#define Z_GERMANIUM_GROUP				14
#define Z_GERMANIUM_PERIOD				4
#define Z_GERMANIUM_BLOCK				Z_CHEMICAL_ELEMENT_BLOCK_P
#define Z_GERMANIUM_CLASS				Z_CHEMICAL_ELEMENT_CLASS_METALLOID
#define Z_GERMANIUM_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_METALLOID
#define Z_GERMANIUM_SPACE_GROUP_NUMBER			225
#define Z_GERMANIUM_ATOMIC_MASS				72.63
#define Z_GERMANIUM_ATOMIC_RADIUS			125.0
#define Z_GERMANIUM_COVALENT_RADIUS			120.0
#define Z_GERMANIUM_VAN_DER_WAALS_RADIUS		REAL_UNKNOWN
#define Z_GERMANIUM_DECAY_MODE				KEY_NOT_APPLICABLE
#define Z_GERMANIUM_NEUTRON_CROSS_SECTION		2.2
#define Z_GERMANIUM_NEUTRON_MASS_ABSORPTION		0.0011
#define Z_GERMANIUM_VALENCE				4
#define Z_GERMANIUM_ELECTRONEGATIVITY			2.01
#define Z_GERMANIUM_ELECTRON_AFFINITY			119.0
#define Z_GERMANIUM_MELTING_POINT			938.3
#define Z_GERMANIUM_BOILING_POINT			2820.0
#define Z_GERMANIUM_CRITICAL_PRESSURE			REAL_UNKNOWN
#define Z_GERMANIUM_CRITICAL_TEMPERATURE		REAL_UNKNOWN
#define Z_GERMANIUM_FUSION_HEAT				31.8
#define Z_GERMANIUM_VAPORIZATION_HEAT			334.0
#define Z_GERMANIUM_SPECIFIC_HEAT			321.4
#define Z_GERMANIUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_GERMANIUM_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_GERMANIUM_THERMAL_CONDUCTIVITY		60.0
#define Z_GERMANIUM_THERMAL_EXPANSION			6.0e-6
#define Z_GERMANIUM_DENSITY				5323.0
#define Z_GERMANIUM_LIQUID_DENSITY			5600.0
#define Z_GERMANIUM_MOLAR_VOLUME			0.00001364
#define Z_GERMANIUM_BRINELL_HARDNESS			7.27e3
#define Z_GERMANIUM_MOHS_HARDNESS			6.0
#define Z_GERMANIUM_VICKERS_HARDNESS			8.01e3
#define Z_GERMANIUM_BULK_MODULUS			REAL_UNKNOWN
#define Z_GERMANIUM_SHEAR_MODULUS			REAL_UNKNOWN
#define Z_GERMANIUM_YOUNG_MODULUS			REAL_UNKNOWN
#define Z_GERMANIUM_POISSON_RATIO			REAL_UNKNOWN
#define Z_GERMANIUM_REFRACTIVE_INDEX			REAL_UNKNOWN
#define Z_GERMANIUM_SOUND_SPEED				5.4e3
#define Z_GERMANIUM_ELECTRICAL_TYPE			Z_ELECTRICAL_TYPE_SEMICONDUCTOR
#define Z_GERMANIUM_ELECTRICAL_CONDUCTIVITY		2.0e3
#define Z_GERMANIUM_ELECTRICAL_RESISTIVITY		0.00050
#define Z_GERMANIUM_SUPERCONDUCTING_POINT		REAL_UNKNOWN
#define Z_GERMANIUM_MAGNETIC_TYPE			Z_MAGNETIC_TYPE_DIAMAGNETIC
#define Z_GERMANIUM_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_GERMANIUM_MASS_MAGNETIC_SUSCEPTIBILITY	-1.50e-9
#define Z_GERMANIUM_MOLAR_MAGNETIC_SUSCEPTIBILITY	-1.09e-10
#define Z_GERMANIUM_VOLUME_MAGNETIC_SUSCEPTIBILITY	-7.98e-6

/* MARK: - 33 - As */

#define Z_ARSENIC_SYMBOL				As
#define Z_ARSENIC_ATOMIC_NUMBER				33
#define Z_ARSENIC_GROUP					15
#define Z_ARSENIC_PERIOD				4
#define Z_ARSENIC_BLOCK					Z_CHEMICAL_ELEMENT_BLOCK_P
#define Z_ARSENIC_CLASS					Z_CHEMICAL_ELEMENT_CLASS_METALLOID
#define Z_ARSENIC_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_METALLOID
#define Z_ARSENIC_SPACE_GROUP_NUMBER			166
#define Z_ARSENIC_ATOMIC_MASS				74.92160
#define Z_ARSENIC_ATOMIC_RADIUS				114.0
#define Z_ARSENIC_COVALENT_RADIUS			119.0
#define Z_ARSENIC_VAN_DER_WAALS_RADIUS			185.0
#define Z_ARSENIC_DECAY_MODE				KEY_NOT_APPLICABLE
#define Z_ARSENIC_NEUTRON_CROSS_SECTION			4.2
#define Z_ARSENIC_NEUTRON_MASS_ABSORPTION		0.0020
#define Z_ARSENIC_VALENCE				5
#define Z_ARSENIC_ELECTRONEGATIVITY			2.18
#define Z_ARSENIC_ELECTRON_AFFINITY			78.0
#define Z_ARSENIC_MELTING_POINT				817.0
#define Z_ARSENIC_BOILING_POINT				614.0
#define Z_ARSENIC_CRITICAL_PRESSURE			REAL_MISSING
#define Z_ARSENIC_CRITICAL_TEMPERATURE			REAL_MISSING
#define Z_ARSENIC_FUSION_HEAT				27.7
#define Z_ARSENIC_VAPORIZATION_HEAT			32.4
#define Z_ARSENIC_SPECIFIC_HEAT				328.0
#define Z_ARSENIC_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_ARSENIC_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_ARSENIC_THERMAL_CONDUCTIVITY			50.0
#define Z_ARSENIC_THERMAL_EXPANSION			REAL_MISSING
#define Z_ARSENIC_DENSITY				5727.0
#define Z_ARSENIC_LIQUID_DENSITY			5220.0
#define Z_ARSENIC_MOLAR_VOLUME				0.00001308
#define Z_ARSENIC_BRINELL_HARDNESS			1440.0
#define Z_ARSENIC_MOHS_HARDNESS				3.5
#define Z_ARSENIC_VICKERS_HARDNESS			1510.0
#define Z_ARSENIC_BULK_MODULUS				22.0
#define Z_ARSENIC_SHEAR_MODULUS				REAL_MISSING
#define Z_ARSENIC_YOUNG_MODULUS				8.0
#define Z_ARSENIC_POISSON_RATIO				REAL_MISSING
#define Z_ARSENIC_REFRACTIVE_INDEX			1.001552
#define Z_ARSENIC_SOUND_SPEED				REAL_MISSING
#define Z_ARSENIC_ELECTRICAL_TYPE			Z_ELECTRICAL_TYPE_CONDUCTOR
#define Z_ARSENIC_ELECTRICAL_CONDUCTIVITY		3.3e6
#define Z_ARSENIC_ELECTRICAL_RESISTIVITY		3.0e-7
#define Z_ARSENIC_SUPERCONDUCTING_POINT			REAL_MISSING
#define Z_ARSENIC_MAGNETIC_TYPE				Z_MAGNETIC_TYPE_DIAMAGNETIC
#define Z_ARSENIC_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_ARSENIC_MASS_MAGNETIC_SUSCEPTIBILITY		-3.90e-9
#define Z_ARSENIC_MOLAR_MAGNETIC_SUSCEPTIBILITY		-2.92e-10
#define Z_ARSENIC_VOLUME_MAGNETIC_SUSCEPTIBILITY	-0.0000223

/* MARK: - 34 - Se */

#define Z_SELENIUM_SYMBOL				Se
#define Z_SELENIUM_ATOMIC_NUMBER			34
#define Z_SELENIUM_GROUP				16
#define Z_SELENIUM_PERIOD				4
#define Z_SELENIUM_BLOCK				Z_CHEMICAL_ELEMENT_BLOCK_P
#define Z_SELENIUM_CLASS				Z_CHEMICAL_ELEMENT_CLASS_NON_METAL
#define Z_SELENIUM_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_NON_METAL
#define Z_SELENIUM_SPACE_GROUP_NUMBER			14
#define Z_SELENIUM_ATOMIC_MASS				78.96
#define Z_SELENIUM_ATOMIC_RADIUS			103.0
#define Z_SELENIUM_COVALENT_RADIUS			120.0
#define Z_SELENIUM_VAN_DER_WAALS_RADIUS			190.0
#define Z_SELENIUM_DECAY_MODE				KEY_NOT_APPLICABLE
#define Z_SELENIUM_NEUTRON_CROSS_SECTION		11.7
#define Z_SELENIUM_NEUTRON_MASS_ABSORPTION		0.0056
#define Z_SELENIUM_VALENCE				6
#define Z_SELENIUM_ELECTRONEGATIVITY			2.55
#define Z_SELENIUM_ELECTRON_AFFINITY			195.0
#define Z_SELENIUM_MELTING_POINT			221.0
#define Z_SELENIUM_BOILING_POINT			685.0
#define Z_SELENIUM_CRITICAL_PRESSURE			27.2
#define Z_SELENIUM_CRITICAL_TEMPERATURE			1766.0
#define Z_SELENIUM_FUSION_HEAT				5.4
#define Z_SELENIUM_VAPORIZATION_HEAT			26.0
#define Z_SELENIUM_SPECIFIC_HEAT			321.2
#define Z_SELENIUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_SELENIUM_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_SELENIUM_THERMAL_CONDUCTIVITY			0.52
#define Z_SELENIUM_THERMAL_EXPANSION			REAL_MISSING
#define Z_SELENIUM_DENSITY				4819.0
#define Z_SELENIUM_LIQUID_DENSITY			3990.0
#define Z_SELENIUM_MOLAR_VOLUME				0.00001639
#define Z_SELENIUM_BRINELL_HARDNESS			736.0
#define Z_SELENIUM_MOHS_HARDNESS			2.0
#define Z_SELENIUM_VICKERS_HARDNESS			REAL_MISSING
#define Z_SELENIUM_BULK_MODULUS				8.3
#define Z_SELENIUM_SHEAR_MODULUS			3.7
#define Z_SELENIUM_YOUNG_MODULUS			10.0
#define Z_SELENIUM_POISSON_RATIO			0.33
#define Z_SELENIUM_REFRACTIVE_INDEX			1.000895
#define Z_SELENIUM_SOUND_SPEED				3350.0
#define Z_SELENIUM_ELECTRICAL_TYPE			KEY_MISSING
#define Z_SELENIUM_ELECTRICAL_CONDUCTIVITY		REAL_MISSING
#define Z_SELENIUM_ELECTRICAL_RESISTIVITY		REAL_MISSING
#define Z_SELENIUM_SUPERCONDUCTING_POINT		REAL_MISSING
#define Z_SELENIUM_MAGNETIC_TYPE			Z_MAGNETIC_TYPE_DIAMAGNETIC
#define Z_SELENIUM_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_SELENIUM_MASS_MAGNETIC_SUSCEPTIBILITY		-4.00e-9
#define Z_SELENIUM_MOLAR_MAGNETIC_SUSCEPTIBILITY	-3.16e-10
#define Z_SELENIUM_VOLUME_MAGNETIC_SUSCEPTIBILITY	-0.0000193

/* MARK: - 35 - Br */

#define Z_BROMINE_SYMBOL				Br
#define Z_BROMINE_ATOMIC_NUMBER				35
#define Z_BROMINE_GROUP					17
#define Z_BROMINE_PERIOD				4
#define Z_BROMINE_BLOCK					Z_CHEMICAL_ELEMENT_BLOCK_P
#define Z_BROMINE_CLASS					Z_CHEMICAL_ELEMENT_CLASS_HALOGEN
#define Z_BROMINE_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_NON_METAL
#define Z_BROMINE_SPACE_GROUP_NUMBER			64
#define Z_BROMINE_ATOMIC_MASS				79.904
#define Z_BROMINE_ATOMIC_RADIUS				94.0
#define Z_BROMINE_COVALENT_RADIUS			120.0
#define Z_BROMINE_VAN_DER_WAALS_RADIUS			185.0
#define Z_BROMINE_DECAY_MODE				KEY_NOT_APPLICABLE
#define Z_BROMINE_NEUTRON_CROSS_SECTION			6.8
#define Z_BROMINE_NEUTRON_MASS_ABSORPTION		0.0020
#define Z_BROMINE_VALENCE				5
#define Z_BROMINE_ELECTRONEGATIVITY			2.96
#define Z_BROMINE_ELECTRON_AFFINITY			324.6
#define Z_BROMINE_MELTING_POINT				-7.3
#define Z_BROMINE_BOILING_POINT				59.0
#define Z_BROMINE_CRITICAL_PRESSURE			10.34
#define Z_BROMINE_CRITICAL_TEMPERATURE			588.0
#define Z_BROMINE_FUSION_HEAT				5.8
#define Z_BROMINE_VAPORIZATION_HEAT			14.8
#define Z_BROMINE_SPECIFIC_HEAT				947.3
#define Z_BROMINE_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_BROMINE_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_BROMINE_THERMAL_CONDUCTIVITY			0.12
#define Z_BROMINE_THERMAL_EXPANSION			REAL_NOT_APPLICABLE
#define Z_BROMINE_DENSITY				3120.0
#define Z_BROMINE_LIQUID_DENSITY			3120.0
#define Z_BROMINE_MOLAR_VOLUME				0.00002561
#define Z_BROMINE_BRINELL_HARDNESS			REAL_NOT_APPLICABLE
#define Z_BROMINE_MOHS_HARDNESS				REAL_NOT_APPLICABLE
#define Z_BROMINE_VICKERS_HARDNESS			REAL_NOT_APPLICABLE
#define Z_BROMINE_BULK_MODULUS				1.9
#define Z_BROMINE_SHEAR_MODULUS				REAL_NOT_APPLICABLE
#define Z_BROMINE_YOUNG_MODULUS				REAL_NOT_APPLICABLE
#define Z_BROMINE_POISSON_RATIO				REAL_NOT_APPLICABLE
#define Z_BROMINE_REFRACTIVE_INDEX			1.001132
#define Z_BROMINE_SOUND_SPEED				REAL_MISSING
#define Z_BROMINE_ELECTRICAL_TYPE			Z_ELECTRICAL_TYPE_INSULATOR
#define Z_BROMINE_ELECTRICAL_CONDUCTIVITY		1.0e-10
#define Z_BROMINE_ELECTRICAL_RESISTIVITY		1.0e10
#define Z_BROMINE_SUPERCONDUCTING_POINT			REAL_MISSING
#define Z_BROMINE_MAGNETIC_TYPE				Z_MAGNETIC_TYPE_DIAMAGNETIC
#define Z_BROMINE_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_BROMINE_MASS_MAGNETIC_SUSCEPTIBILITY		-4.90e-9
#define Z_BROMINE_MOLAR_MAGNETIC_SUSCEPTIBILITY		-7.83e-10
#define Z_BROMINE_VOLUME_MAGNETIC_SUSCEPTIBILITY	-0.0000153

/* MARK: - 36 - Kr */

#define Z_KRYPTON_SYMBOL				Kr
#define Z_KRYPTON_ATOMIC_NUMBER				36
#define Z_KRYPTON_GROUP					18
#define Z_KRYPTON_PERIOD				4
#define Z_KRYPTON_BLOCK					Z_CHEMICAL_ELEMENT_BLOCK_P
#define Z_KRYPTON_CLASS					Z_CHEMICAL_ELEMENT_CLASS_NOBLE_GAS
#define Z_KRYPTON_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_NON_METAL
#define Z_KRYPTON_SPACE_GROUP_NUMBER			225
#define Z_KRYPTON_ATOMIC_MASS				83.798
#define Z_KRYPTON_ATOMIC_RADIUS				87.0
#define Z_KRYPTON_COVALENT_RADIUS			116.0
#define Z_KRYPTON_VAN_DER_WAALS_RADIUS			202.0
#define Z_KRYPTON_DECAY_MODE				KEY_NOT_APPLICABLE
#define Z_KRYPTON_NEUTRON_CROSS_SECTION			25.0
#define Z_KRYPTON_NEUTRON_MASS_ABSORPTION		0.013
#define Z_KRYPTON_VALENCE				2
#define Z_KRYPTON_ELECTRONEGATIVITY			3.0
#define Z_KRYPTON_ELECTRON_AFFINITY			0.0
#define Z_KRYPTON_MELTING_POINT				-157.36
#define Z_KRYPTON_BOILING_POINT				-153.22
#define Z_KRYPTON_CRITICAL_PRESSURE			5.50
#define Z_KRYPTON_CRITICAL_TEMPERATURE			209.41
#define Z_KRYPTON_FUSION_HEAT				1.64
#define Z_KRYPTON_VAPORIZATION_HEAT			9.02
#define Z_KRYPTON_SPECIFIC_HEAT				248.05
#define Z_KRYPTON_ADIABATIC_INDEX			Z_CHEMICAL_ELEMENT_ADIABATIC_INDEX_5_3
#define Z_KRYPTON_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_KRYPTON_THERMAL_CONDUCTIVITY			0.00943
#define Z_KRYPTON_THERMAL_EXPANSION			REAL_NOT_APPLICABLE
#define Z_KRYPTON_DENSITY				3.75
#define Z_KRYPTON_LIQUID_DENSITY			REAL_MISSING
#define Z_KRYPTON_MOLAR_VOLUME				0.0223
#define Z_KRYPTON_BRINELL_HARDNESS			REAL_NOT_APPLICABLE
#define Z_KRYPTON_MOHS_HARDNESS				REAL_NOT_APPLICABLE
#define Z_KRYPTON_VICKERS_HARDNESS			REAL_NOT_APPLICABLE
#define Z_KRYPTON_BULK_MODULUS				REAL_NOT_APPLICABLE
#define Z_KRYPTON_SHEAR_MODULUS				REAL_NOT_APPLICABLE
#define Z_KRYPTON_YOUNG_MODULUS				REAL_NOT_APPLICABLE
#define Z_KRYPTON_POISSON_RATIO				REAL_NOT_APPLICABLE
#define Z_KRYPTON_REFRACTIVE_INDEX			1.000427
#define Z_KRYPTON_SOUND_SPEED				1120.0
#define Z_KRYPTON_ELECTRICAL_TYPE			KEY_MISSING
#define Z_KRYPTON_ELECTRICAL_CONDUCTIVITY		REAL_MISSING
#define Z_KRYPTON_ELECTRICAL_RESISTIVITY		REAL_NOT_APPLICABLE
#define Z_KRYPTON_SUPERCONDUCTING_POINT			REAL_MISSING
#define Z_KRYPTON_MAGNETIC_TYPE				Z_MAGNETIC_TYPE_DIAMAGNETIC
#define Z_KRYPTON_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_KRYPTON_MASS_MAGNETIC_SUSCEPTIBILITY		-4.40e-9
#define Z_KRYPTON_MOLAR_MAGNETIC_SUSCEPTIBILITY		-3.69e-10
#define Z_KRYPTON_VOLUME_MAGNETIC_SUSCEPTIBILITY	-1.65e-8

/* MARK: - 37 - Rb */

#define Z_RUBIDIUM_SYMBOL				Rb
#define Z_RUBIDIUM_ATOMIC_NUMBER			37
#define Z_RUBIDIUM_GROUP				1
#define Z_RUBIDIUM_PERIOD				5
#define Z_RUBIDIUM_BLOCK				Z_CHEMICAL_ELEMENT_BLOCK_S
#define Z_RUBIDIUM_CLASS				Z_CHEMICAL_ELEMENT_CLASS_ALKALI_METAL
#define Z_RUBIDIUM_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_RUBIDIUM_SPACE_GROUP_NUMBER			229
#define Z_RUBIDIUM_ATOMIC_MASS				85.4678
#define Z_RUBIDIUM_ATOMIC_RADIUS			265.0
#define Z_RUBIDIUM_COVALENT_RADIUS			220.0
#define Z_RUBIDIUM_VAN_DER_WAALS_RADIUS			REAL_MISSING
#define Z_RUBIDIUM_DECAY_MODE				KEY_NOT_APPLICABLE
#define Z_RUBIDIUM_NEUTRON_CROSS_SECTION		0.38
#define Z_RUBIDIUM_NEUTRON_MASS_ABSORPTION		0.00030
#define Z_RUBIDIUM_VALENCE				1
#define Z_RUBIDIUM_ELECTRONEGATIVITY			0.82
#define Z_RUBIDIUM_ELECTRON_AFFINITY			46.9
#define Z_RUBIDIUM_MELTING_POINT			39.31
#define Z_RUBIDIUM_BOILING_POINT			688.0
#define Z_RUBIDIUM_CRITICAL_PRESSURE			16.0
#define Z_RUBIDIUM_CRITICAL_TEMPERATURE			2093.0
#define Z_RUBIDIUM_FUSION_HEAT				2.19
#define Z_RUBIDIUM_VAPORIZATION_HEAT			71.0
#define Z_RUBIDIUM_SPECIFIC_HEAT			364.0
#define Z_RUBIDIUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_RUBIDIUM_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_RUBIDIUM_THERMAL_CONDUCTIVITY			58.0
#define Z_RUBIDIUM_THERMAL_EXPANSION			REAL_MISSING
#define Z_RUBIDIUM_DENSITY				1532.0
#define Z_RUBIDIUM_LIQUID_DENSITY			1460.0
#define Z_RUBIDIUM_MOLAR_VOLUME				0.00005579
#define Z_RUBIDIUM_BRINELL_HARDNESS			0.216
#define Z_RUBIDIUM_MOHS_HARDNESS			0.3
#define Z_RUBIDIUM_VICKERS_HARDNESS			REAL_MISSING
#define Z_RUBIDIUM_BULK_MODULUS				2.5
#define Z_RUBIDIUM_SHEAR_MODULUS			REAL_MISSING
#define Z_RUBIDIUM_YOUNG_MODULUS			2.4
#define Z_RUBIDIUM_POISSON_RATIO			REAL_MISSING
#define Z_RUBIDIUM_REFRACTIVE_INDEX			REAL_MISSING
#define Z_RUBIDIUM_SOUND_SPEED				1.3e3
#define Z_RUBIDIUM_ELECTRICAL_TYPE			Z_ELECTRICAL_TYPE_CONDUCTOR
#define Z_RUBIDIUM_ELECTRICAL_CONDUCTIVITY		8.3e6
#define Z_RUBIDIUM_ELECTRICAL_RESISTIVITY		1.2e-7
#define Z_RUBIDIUM_SUPERCONDUCTING_POINT		REAL_MISSING
#define Z_RUBIDIUM_MAGNETIC_TYPE			Z_MAGNETIC_TYPE_PARAMAGNETIC
#define Z_RUBIDIUM_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_RUBIDIUM_MASS_MAGNETIC_SUSCEPTIBILITY		2.60e-9
#define Z_RUBIDIUM_MOLAR_MAGNETIC_SUSCEPTIBILITY	2.22e-10
#define Z_RUBIDIUM_VOLUME_MAGNETIC_SUSCEPTIBILITY	3.98e-6

/* MARK: - 38 - Sr */

#define Z_STRONTIUM_SYMBOL				Sr
#define Z_STRONTIUM_ATOMIC_NUMBER			38
#define Z_STRONTIUM_GROUP				2
#define Z_STRONTIUM_PERIOD				5
#define Z_STRONTIUM_BLOCK				Z_CHEMICAL_ELEMENT_BLOCK_S
#define Z_STRONTIUM_CLASS				Z_CHEMICAL_ELEMENT_CLASS_ALKALINE_EARTH_METAL
#define Z_STRONTIUM_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_STRONTIUM_SPACE_GROUP_NUMBER			225
#define Z_STRONTIUM_ATOMIC_MASS				87.62
#define Z_STRONTIUM_ATOMIC_RADIUS			219.0
#define Z_STRONTIUM_COVALENT_RADIUS			195.0
#define Z_STRONTIUM_VAN_DER_WAALS_RADIUS		REAL_MISSING
#define Z_STRONTIUM_DECAY_MODE				KEY_NOT_APPLICABLE
#define Z_STRONTIUM_NEUTRON_CROSS_SECTION		1.2
#define Z_STRONTIUM_NEUTRON_MASS_ABSORPTION		0.00050
#define Z_STRONTIUM_VALENCE				2
#define Z_STRONTIUM_ELECTRONEGATIVITY			0.95
#define Z_STRONTIUM_ELECTRON_AFFINITY			5.03
#define Z_STRONTIUM_MELTING_POINT			777.0
#define Z_STRONTIUM_BOILING_POINT			1382.0
#define Z_STRONTIUM_CRITICAL_PRESSURE			REAL_MISSING
#define Z_STRONTIUM_CRITICAL_TEMPERATURE		REAL_MISSING
#define Z_STRONTIUM_FUSION_HEAT				8.0
#define Z_STRONTIUM_VAPORIZATION_HEAT			137.0
#define Z_STRONTIUM_SPECIFIC_HEAT			300.0
#define Z_STRONTIUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_STRONTIUM_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_STRONTIUM_THERMAL_CONDUCTIVITY		35.0
#define Z_STRONTIUM_THERMAL_EXPANSION			0.0000225
#define Z_STRONTIUM_DENSITY				2630.0
#define Z_STRONTIUM_LIQUID_DENSITY			2375.0
#define Z_STRONTIUM_MOLAR_VOLUME			0.00003332
#define Z_STRONTIUM_BRINELL_HARDNESS			REAL_MISSING
#define Z_STRONTIUM_MOHS_HARDNESS			1.5
#define Z_STRONTIUM_VICKERS_HARDNESS			REAL_MISSING
#define Z_STRONTIUM_BULK_MODULUS			REAL_MISSING
#define Z_STRONTIUM_SHEAR_MODULUS			6.1
#define Z_STRONTIUM_YOUNG_MODULUS			REAL_MISSING
#define Z_STRONTIUM_POISSON_RATIO			0.28
#define Z_STRONTIUM_REFRACTIVE_INDEX			REAL_MISSING
#define Z_STRONTIUM_SOUND_SPEED				REAL_MISSING
#define Z_STRONTIUM_ELECTRICAL_TYPE			Z_ELECTRICAL_TYPE_CONDUCTOR
#define Z_STRONTIUM_ELECTRICAL_CONDUCTIVITY		7.7e6
#define Z_STRONTIUM_ELECTRICAL_RESISTIVITY		1.3e-7
#define Z_STRONTIUM_SUPERCONDUCTING_POINT		REAL_MISSING
#define Z_STRONTIUM_MAGNETIC_TYPE			Z_MAGNETIC_TYPE_PARAMAGNETIC
#define Z_STRONTIUM_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_STRONTIUM_MASS_MAGNETIC_SUSCEPTIBILITY	1.32e-9
#define Z_STRONTIUM_MOLAR_MAGNETIC_SUSCEPTIBILITY	1.16e-10
#define Z_STRONTIUM_VOLUME_MAGNETIC_SUSCEPTIBILITY	3.47e-6

/* MARK: - 39 - Y */

#define Z_YTTRIUM_SYMBOL				Y
#define Z_YTTRIUM_ATOMIC_NUMBER				39
#define Z_YTTRIUM_GROUP					3
#define Z_YTTRIUM_PERIOD				5
#define Z_YTTRIUM_BLOCK					Z_CHEMICAL_ELEMENT_BLOCK_D
#define Z_YTTRIUM_CLASS					Z_CHEMICAL_ELEMENT_CLASS_TRANSITION_METAL
#define Z_YTTRIUM_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_YTTRIUM_SPACE_GROUP_NUMBER			194
#define Z_YTTRIUM_ATOMIC_MASS				88.90585
#define Z_YTTRIUM_ATOMIC_RADIUS				212.0
#define Z_YTTRIUM_COVALENT_RADIUS			190.0
#define Z_YTTRIUM_VAN_DER_WAALS_RADIUS			REAL_MISSING
#define Z_YTTRIUM_DECAY_MODE				KEY_NOT_APPLICABLE
#define Z_YTTRIUM_NEUTRON_CROSS_SECTION			1.28
#define Z_YTTRIUM_NEUTRON_MASS_ABSORPTION		0.00059
#define Z_YTTRIUM_VALENCE				3
#define Z_YTTRIUM_ELECTRONEGATIVITY			1.22
#define Z_YTTRIUM_ELECTRON_AFFINITY			29.6
#define Z_YTTRIUM_MELTING_POINT				1526.0
#define Z_YTTRIUM_BOILING_POINT				3345.0
#define Z_YTTRIUM_CRITICAL_PRESSURE			REAL_MISSING
#define Z_YTTRIUM_CRITICAL_TEMPERATURE			REAL_MISSING
#define Z_YTTRIUM_FUSION_HEAT				11.4
#define Z_YTTRIUM_VAPORIZATION_HEAT			380.0
#define Z_YTTRIUM_SPECIFIC_HEAT				298.0
#define Z_YTTRIUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_YTTRIUM_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_YTTRIUM_THERMAL_CONDUCTIVITY			17.0
#define Z_YTTRIUM_THERMAL_EXPANSION			0.0000106
#define Z_YTTRIUM_DENSITY				4472.0
#define Z_YTTRIUM_LIQUID_DENSITY			4240.0
#define Z_YTTRIUM_MOLAR_VOLUME				0.00001988
#define Z_YTTRIUM_BRINELL_HARDNESS			588.0
#define Z_YTTRIUM_MOHS_HARDNESS				REAL_MISSING
#define Z_YTTRIUM_VICKERS_HARDNESS			REAL_MISSING
#define Z_YTTRIUM_BULK_MODULUS				41.0
#define Z_YTTRIUM_SHEAR_MODULUS				26.0
#define Z_YTTRIUM_YOUNG_MODULUS				64.0
#define Z_YTTRIUM_POISSON_RATIO				0.24
#define Z_YTTRIUM_REFRACTIVE_INDEX			REAL_MISSING
#define Z_YTTRIUM_SOUND_SPEED				3.3e3
#define Z_YTTRIUM_ELECTRICAL_TYPE			Z_ELECTRICAL_TYPE_CONDUCTOR
#define Z_YTTRIUM_ELECTRICAL_CONDUCTIVITY		1.8e6
#define Z_YTTRIUM_ELECTRICAL_RESISTIVITY		5.7e-7
#define Z_YTTRIUM_SUPERCONDUCTING_POINT			1.3
#define Z_YTTRIUM_MAGNETIC_TYPE				Z_MAGNETIC_TYPE_PARAMAGNETIC
#define Z_YTTRIUM_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_YTTRIUM_MASS_MAGNETIC_SUSCEPTIBILITY		6.660e-8
#define Z_YTTRIUM_MOLAR_MAGNETIC_SUSCEPTIBILITY		5.921e-9
#define Z_YTTRIUM_VOLUME_MAGNETIC_SUSCEPTIBILITY	0.0002978

/* MARK: - 40 - Zr */

#define Z_ZIRCONIUM_SYMBOL				Zr
#define Z_ZIRCONIUM_ATOMIC_NUMBER			40
#define Z_ZIRCONIUM_GROUP				4
#define Z_ZIRCONIUM_PERIOD				5
#define Z_ZIRCONIUM_BLOCK				Z_CHEMICAL_ELEMENT_BLOCK_D
#define Z_ZIRCONIUM_CLASS				Z_CHEMICAL_ELEMENT_CLASS_TRANSITION_METAL
#define Z_ZIRCONIUM_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_ZIRCONIUM_SPACE_GROUP_NUMBER			194
#define Z_ZIRCONIUM_ATOMIC_MASS				91.224
#define Z_ZIRCONIUM_ATOMIC_RADIUS			206.0
#define Z_ZIRCONIUM_COVALENT_RADIUS			175.0
#define Z_ZIRCONIUM_VAN_DER_WAALS_RADIUS		REAL_UNKNOWN
#define Z_ZIRCONIUM_DECAY_MODE				KEY_NOT_APPLICABLE
#define Z_ZIRCONIUM_NEUTRON_CROSS_SECTION		0.184
#define Z_ZIRCONIUM_NEUTRON_MASS_ABSORPTION		0.00066
#define Z_ZIRCONIUM_VALENCE				4
#define Z_ZIRCONIUM_ELECTRONEGATIVITY			1.33
#define Z_ZIRCONIUM_ELECTRON_AFFINITY			41.1
#define Z_ZIRCONIUM_MELTING_POINT			1855.0
#define Z_ZIRCONIUM_BOILING_POINT			4409.0
#define Z_ZIRCONIUM_CRITICAL_PRESSURE			REAL_UNKNOWN
#define Z_ZIRCONIUM_CRITICAL_TEMPERATURE		REAL_UNKNOWN
#define Z_ZIRCONIUM_FUSION_HEAT				21.0
#define Z_ZIRCONIUM_VAPORIZATION_HEAT			580.0
#define Z_ZIRCONIUM_SPECIFIC_HEAT			278.0
#define Z_ZIRCONIUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_ZIRCONIUM_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_ZIRCONIUM_THERMAL_CONDUCTIVITY		23.0
#define Z_ZIRCONIUM_THERMAL_EXPANSION			5.7e-6
#define Z_ZIRCONIUM_DENSITY				6511.0
#define Z_ZIRCONIUM_LIQUID_DENSITY			5800.0
#define Z_ZIRCONIUM_MOLAR_VOLUME			0.00001401
#define Z_ZIRCONIUM_BRINELL_HARDNESS			650.0
#define Z_ZIRCONIUM_MOHS_HARDNESS			5.0
#define Z_ZIRCONIUM_VICKERS_HARDNESS			904.0
#define Z_ZIRCONIUM_BULK_MODULUS			REAL_UNKNOWN
#define Z_ZIRCONIUM_SHEAR_MODULUS			33.0
#define Z_ZIRCONIUM_YOUNG_MODULUS			67.0
#define Z_ZIRCONIUM_POISSON_RATIO			0.34
#define Z_ZIRCONIUM_REFRACTIVE_INDEX			REAL_UNKNOWN
#define Z_ZIRCONIUM_SOUND_SPEED				3.8e3
#define Z_ZIRCONIUM_ELECTRICAL_TYPE			Z_ELECTRICAL_TYPE_CONDUCTOR
#define Z_ZIRCONIUM_ELECTRICAL_CONDUCTIVITY		2.4e6
#define Z_ZIRCONIUM_ELECTRICAL_RESISTIVITY		4.2e-7
#define Z_ZIRCONIUM_SUPERCONDUCTING_POINT		0.61
#define Z_ZIRCONIUM_MAGNETIC_TYPE			Z_MAGNETIC_TYPE_PARAMAGNETIC
#define Z_ZIRCONIUM_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_ZIRCONIUM_MASS_MAGNETIC_SUSCEPTIBILITY	1.68e-8
#define Z_ZIRCONIUM_MOLAR_MAGNETIC_SUSCEPTIBILITY	1.53e-9
#define Z_ZIRCONIUM_VOLUME_MAGNETIC_SUSCEPTIBILITY	0.000109

/* MARK: - 41 - Nb */

#define Z_NIOBIUM_SYMBOL				Nb
#define Z_NIOBIUM_ATOMIC_NUMBER				41
#define Z_NIOBIUM_GROUP					5
#define Z_NIOBIUM_PERIOD				5
#define Z_NIOBIUM_BLOCK					Z_CHEMICAL_ELEMENT_BLOCK_D
#define Z_NIOBIUM_CLASS					Z_CHEMICAL_ELEMENT_CLASS_TRANSITION_METAL
#define Z_NIOBIUM_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_NIOBIUM_SPACE_GROUP_NUMBER			229
#define Z_NIOBIUM_ATOMIC_MASS				92.90638
#define Z_NIOBIUM_ATOMIC_RADIUS				198.0
#define Z_NIOBIUM_COVALENT_RADIUS			164.0
#define Z_NIOBIUM_VAN_DER_WAALS_RADIUS			REAL_MISSING
#define Z_NIOBIUM_DECAY_MODE				KEY_NOT_APPLICABLE
#define Z_NIOBIUM_NEUTRON_CROSS_SECTION			1.15
#define Z_NIOBIUM_NEUTRON_MASS_ABSORPTION		0.00040
#define Z_NIOBIUM_VALENCE				5
#define Z_NIOBIUM_ELECTRONEGATIVITY			1.6
#define Z_NIOBIUM_ELECTRON_AFFINITY			86.1
#define Z_NIOBIUM_MELTING_POINT				2477.0
#define Z_NIOBIUM_BOILING_POINT				4744.0
#define Z_NIOBIUM_CRITICAL_PRESSURE			REAL_MISSING
#define Z_NIOBIUM_CRITICAL_TEMPERATURE			REAL_MISSING
#define Z_NIOBIUM_FUSION_HEAT				26.8
#define Z_NIOBIUM_VAPORIZATION_HEAT			690.0
#define Z_NIOBIUM_SPECIFIC_HEAT				265.0
#define Z_NIOBIUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_NIOBIUM_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_NIOBIUM_THERMAL_CONDUCTIVITY			54.0
#define Z_NIOBIUM_THERMAL_EXPANSION			7.3e-6
#define Z_NIOBIUM_DENSITY				8570.0
#define Z_NIOBIUM_LIQUID_DENSITY			REAL_MISSING
#define Z_NIOBIUM_MOLAR_VOLUME				0.00001084
#define Z_NIOBIUM_BRINELL_HARDNESS			736.0
#define Z_NIOBIUM_MOHS_HARDNESS				6.0
#define Z_NIOBIUM_VICKERS_HARDNESS			1320.0
#define Z_NIOBIUM_BULK_MODULUS				170.0
#define Z_NIOBIUM_SHEAR_MODULUS				38.0
#define Z_NIOBIUM_YOUNG_MODULUS				105.0
#define Z_NIOBIUM_POISSON_RATIO				0.40
#define Z_NIOBIUM_REFRACTIVE_INDEX			REAL_MISSING
#define Z_NIOBIUM_SOUND_SPEED				3480.0
#define Z_NIOBIUM_ELECTRICAL_TYPE			Z_ELECTRICAL_TYPE_CONDUCTOR
#define Z_NIOBIUM_ELECTRICAL_CONDUCTIVITY		6.7e6
#define Z_NIOBIUM_ELECTRICAL_RESISTIVITY		1.5e-7
#define Z_NIOBIUM_SUPERCONDUCTING_POINT			9.25
#define Z_NIOBIUM_MAGNETIC_TYPE				Z_MAGNETIC_TYPE_PARAMAGNETIC
#define Z_NIOBIUM_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_NIOBIUM_MASS_MAGNETIC_SUSCEPTIBILITY		2.76e-8
#define Z_NIOBIUM_MOLAR_MAGNETIC_SUSCEPTIBILITY		2.56e-9
#define Z_NIOBIUM_VOLUME_MAGNETIC_SUSCEPTIBILITY	0.000237

/* MARK: - 42 - Mo */

#define Z_MOLYBDENUM_SYMBOL				Mo
#define Z_MOLYBDENUM_ATOMIC_NUMBER			42
#define Z_MOLYBDENUM_GROUP				6
#define Z_MOLYBDENUM_PERIOD				5
#define Z_MOLYBDENUM_BLOCK				Z_CHEMICAL_ELEMENT_BLOCK_D
#define Z_MOLYBDENUM_CLASS				Z_CHEMICAL_ELEMENT_CLASS_TRANSITION_METAL
#define Z_MOLYBDENUM_METAL_STATE			Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_MOLYBDENUM_SPACE_GROUP_NUMBER			229
#define Z_MOLYBDENUM_ATOMIC_MASS			95.96
#define Z_MOLYBDENUM_ATOMIC_RADIUS			190.0
#define Z_MOLYBDENUM_COVALENT_RADIUS			154.0
#define Z_MOLYBDENUM_VAN_DER_WAALS_RADIUS		REAL_MISSING
#define Z_MOLYBDENUM_DECAY_MODE				KEY_NOT_APPLICABLE
#define Z_MOLYBDENUM_NEUTRON_CROSS_SECTION		2.6
#define Z_MOLYBDENUM_NEUTRON_MASS_ABSORPTION		0.00090
#define Z_MOLYBDENUM_VALENCE				6
#define Z_MOLYBDENUM_ELECTRONEGATIVITY			2.16
#define Z_MOLYBDENUM_ELECTRON_AFFINITY			71.9
#define Z_MOLYBDENUM_MELTING_POINT			2623.0
#define Z_MOLYBDENUM_BOILING_POINT			4639.0
#define Z_MOLYBDENUM_CRITICAL_PRESSURE			REAL_MISSING
#define Z_MOLYBDENUM_CRITICAL_TEMPERATURE		REAL_MISSING
#define Z_MOLYBDENUM_FUSION_HEAT			36.0
#define Z_MOLYBDENUM_VAPORIZATION_HEAT			6.0e2
#define Z_MOLYBDENUM_SPECIFIC_HEAT			251.0
#define Z_MOLYBDENUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_MOLYBDENUM_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_MOLYBDENUM_THERMAL_CONDUCTIVITY		139.0
#define Z_MOLYBDENUM_THERMAL_EXPANSION			4.8e-6
#define Z_MOLYBDENUM_DENSITY				10280.0
#define Z_MOLYBDENUM_LIQUID_DENSITY			9330.0
#define Z_MOLYBDENUM_MOLAR_VOLUME			9.33e-6
#define Z_MOLYBDENUM_BRINELL_HARDNESS			1.5e3
#define Z_MOLYBDENUM_MOHS_HARDNESS			5.5
#define Z_MOLYBDENUM_VICKERS_HARDNESS			1530.0
#define Z_MOLYBDENUM_BULK_MODULUS			230.0
#define Z_MOLYBDENUM_SHEAR_MODULUS			20.0
#define Z_MOLYBDENUM_YOUNG_MODULUS			329.0
#define Z_MOLYBDENUM_POISSON_RATIO			0.31
#define Z_MOLYBDENUM_REFRACTIVE_INDEX			REAL_MISSING
#define Z_MOLYBDENUM_SOUND_SPEED			6190.0
#define Z_MOLYBDENUM_ELECTRICAL_TYPE			Z_ELECTRICAL_TYPE_CONDUCTOR
#define Z_MOLYBDENUM_ELECTRICAL_CONDUCTIVITY		2.0e7
#define Z_MOLYBDENUM_ELECTRICAL_RESISTIVITY		5.0e-8
#define Z_MOLYBDENUM_SUPERCONDUCTING_POINT		0.915
#define Z_MOLYBDENUM_MAGNETIC_TYPE			Z_MAGNETIC_TYPE_PARAMAGNETIC
#define Z_MOLYBDENUM_CURIE_POINT			REAL_NOT_APPLICABLE
#define Z_MOLYBDENUM_MASS_MAGNETIC_SUSCEPTIBILITY	1.170e-8
#define Z_MOLYBDENUM_MOLAR_MAGNETIC_SUSCEPTIBILITY	1.122e-9
#define Z_MOLYBDENUM_VOLUME_MAGNETIC_SUSCEPTIBILITY	0.0001203

/* MARK: - 43 - Tc */

#define Z_TECHNETIUM_SYMBOL				Tc
#define Z_TECHNETIUM_ATOMIC_NUMBER			43
#define Z_TECHNETIUM_GROUP				7
#define Z_TECHNETIUM_PERIOD				5
#define Z_TECHNETIUM_BLOCK				Z_CHEMICAL_ELEMENT_BLOCK_D
#define Z_TECHNETIUM_CLASS				Z_CHEMICAL_ELEMENT_CLASS_TRANSITION_METAL
#define Z_TECHNETIUM_METAL_STATE			Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_TECHNETIUM_SPACE_GROUP_NUMBER			194
#define Z_TECHNETIUM_ATOMIC_MASS			98.0
#define Z_TECHNETIUM_ATOMIC_RADIUS			183.0
#define Z_TECHNETIUM_COVALENT_RADIUS			147.0
#define Z_TECHNETIUM_VAN_DER_WAALS_RADIUS		REAL_MISSING
#define Z_TECHNETIUM_DECAY_MODE				Z_CHEMICAL_ELEMENT_DECAY_MODE_BETA_DECAY
#define Z_TECHNETIUM_NEUTRON_CROSS_SECTION		22.0
#define Z_TECHNETIUM_NEUTRON_MASS_ABSORPTION		REAL_MISSING
#define Z_TECHNETIUM_VALENCE				6
#define Z_TECHNETIUM_ELECTRONEGATIVITY			1.9
#define Z_TECHNETIUM_ELECTRON_AFFINITY			53.0
#define Z_TECHNETIUM_MELTING_POINT			2157.0
#define Z_TECHNETIUM_BOILING_POINT			4265.0
#define Z_TECHNETIUM_CRITICAL_PRESSURE			REAL_MISSING
#define Z_TECHNETIUM_CRITICAL_TEMPERATURE		REAL_MISSING
#define Z_TECHNETIUM_FUSION_HEAT			23.0
#define Z_TECHNETIUM_VAPORIZATION_HEAT			550.0
#define Z_TECHNETIUM_SPECIFIC_HEAT			63.0
#define Z_TECHNETIUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_TECHNETIUM_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_TECHNETIUM_THERMAL_CONDUCTIVITY		51.0
#define Z_TECHNETIUM_THERMAL_EXPANSION			REAL_MISSING
#define Z_TECHNETIUM_DENSITY				1.15e4
#define Z_TECHNETIUM_LIQUID_DENSITY			REAL_MISSING
#define Z_TECHNETIUM_MOLAR_VOLUME			8.52e-6
#define Z_TECHNETIUM_BRINELL_HARDNESS			REAL_MISSING
#define Z_TECHNETIUM_MOHS_HARDNESS			REAL_MISSING
#define Z_TECHNETIUM_VICKERS_HARDNESS			REAL_MISSING
#define Z_TECHNETIUM_BULK_MODULUS			REAL_MISSING
#define Z_TECHNETIUM_SHEAR_MODULUS			REAL_MISSING
#define Z_TECHNETIUM_YOUNG_MODULUS			REAL_MISSING
#define Z_TECHNETIUM_POISSON_RATIO			REAL_MISSING
#define Z_TECHNETIUM_REFRACTIVE_INDEX			REAL_MISSING
#define Z_TECHNETIUM_SOUND_SPEED			REAL_MISSING
#define Z_TECHNETIUM_ELECTRICAL_TYPE			Z_ELECTRICAL_TYPE_CONDUCTOR
#define Z_TECHNETIUM_ELECTRICAL_CONDUCTIVITY		5.0e6
#define Z_TECHNETIUM_ELECTRICAL_RESISTIVITY		2.0e-7
#define Z_TECHNETIUM_SUPERCONDUCTING_POINT		7.7
#define Z_TECHNETIUM_MAGNETIC_TYPE			Z_MAGNETIC_TYPE_PARAMAGNETIC
#define Z_TECHNETIUM_CURIE_POINT			REAL_NOT_APPLICABLE
#define Z_TECHNETIUM_MASS_MAGNETIC_SUSCEPTIBILITY	3.420e-8
#define Z_TECHNETIUM_MOLAR_MAGNETIC_SUSCEPTIBILITY	3.352e-9
#define Z_TECHNETIUM_VOLUME_MAGNETIC_SUSCEPTIBILITY	0.0003933

/* MARK: - 44 - Ru */

#define Z_RUTHENIUM_SYMBOL				Ru
#define Z_RUTHENIUM_ATOMIC_NUMBER			44
#define Z_RUTHENIUM_GROUP				8
#define Z_RUTHENIUM_PERIOD				5
#define Z_RUTHENIUM_BLOCK				Z_CHEMICAL_ELEMENT_BLOCK_D
#define Z_RUTHENIUM_CLASS				Z_CHEMICAL_ELEMENT_CLASS_TRANSITION_METAL
#define Z_RUTHENIUM_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_RUTHENIUM_SPACE_GROUP_NUMBER			194
#define Z_RUTHENIUM_ATOMIC_MASS				101.07
#define Z_RUTHENIUM_ATOMIC_RADIUS			178.0
#define Z_RUTHENIUM_COVALENT_RADIUS			146.0
#define Z_RUTHENIUM_VAN_DER_WAALS_RADIUS		REAL_MISSING
#define Z_RUTHENIUM_DECAY_MODE				KEY_NOT_APPLICABLE
#define Z_RUTHENIUM_NEUTRON_CROSS_SECTION		2.6
#define Z_RUTHENIUM_NEUTRON_MASS_ABSORPTION		0.00090
#define Z_RUTHENIUM_VALENCE				6
#define Z_RUTHENIUM_ELECTRONEGATIVITY			2.2
#define Z_RUTHENIUM_ELECTRON_AFFINITY			101.3
#define Z_RUTHENIUM_MELTING_POINT			2334.0
#define Z_RUTHENIUM_BOILING_POINT			4150.0
#define Z_RUTHENIUM_CRITICAL_PRESSURE			REAL_MISSING
#define Z_RUTHENIUM_CRITICAL_TEMPERATURE		REAL_MISSING
#define Z_RUTHENIUM_FUSION_HEAT				25.7
#define Z_RUTHENIUM_VAPORIZATION_HEAT			580.0
#define Z_RUTHENIUM_SPECIFIC_HEAT			238.0
#define Z_RUTHENIUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_RUTHENIUM_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_RUTHENIUM_THERMAL_CONDUCTIVITY		120.0
#define Z_RUTHENIUM_THERMAL_EXPANSION			6.4e-6
#define Z_RUTHENIUM_DENSITY				12370.0
#define Z_RUTHENIUM_LIQUID_DENSITY			10650.0
#define Z_RUTHENIUM_MOLAR_VOLUME			8.171e-6
#define Z_RUTHENIUM_BRINELL_HARDNESS			2160.0
#define Z_RUTHENIUM_MOHS_HARDNESS			6.5
#define Z_RUTHENIUM_VICKERS_HARDNESS			2.30e3
#define Z_RUTHENIUM_BULK_MODULUS			220.0
#define Z_RUTHENIUM_SHEAR_MODULUS			173.0
#define Z_RUTHENIUM_YOUNG_MODULUS			447.0
#define Z_RUTHENIUM_POISSON_RATIO			0.30
#define Z_RUTHENIUM_REFRACTIVE_INDEX			REAL_MISSING
#define Z_RUTHENIUM_SOUND_SPEED				5970.0
#define Z_RUTHENIUM_ELECTRICAL_TYPE			Z_ELECTRICAL_TYPE_CONDUCTOR
#define Z_RUTHENIUM_ELECTRICAL_CONDUCTIVITY		1.4e7
#define Z_RUTHENIUM_ELECTRICAL_RESISTIVITY		7.0e-8
#define Z_RUTHENIUM_SUPERCONDUCTING_POINT		0.49
#define Z_RUTHENIUM_MAGNETIC_TYPE			Z_MAGNETIC_TYPE_PARAMAGNETIC
#define Z_RUTHENIUM_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_RUTHENIUM_MASS_MAGNETIC_SUSCEPTIBILITY	5.42e-9
#define Z_RUTHENIUM_MOLAR_MAGNETIC_SUSCEPTIBILITY	5.48e-10
#define Z_RUTHENIUM_VOLUME_MAGNETIC_SUSCEPTIBILITY	0.0000670

/* MARK: - 45 - Rh */

#define Z_RHODIUM_SYMBOL				Rh
#define Z_RHODIUM_ATOMIC_NUMBER				45
#define Z_RHODIUM_GROUP					9
#define Z_RHODIUM_PERIOD				5
#define Z_RHODIUM_BLOCK					Z_CHEMICAL_ELEMENT_BLOCK_D
#define Z_RHODIUM_CLASS					Z_CHEMICAL_ELEMENT_CLASS_TRANSITION_METAL
#define Z_RHODIUM_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_RHODIUM_SPACE_GROUP_NUMBER			225
#define Z_RHODIUM_ATOMIC_MASS				102.90550
#define Z_RHODIUM_ATOMIC_RADIUS				173.0
#define Z_RHODIUM_COVALENT_RADIUS			142.0
#define Z_RHODIUM_VAN_DER_WAALS_RADIUS			REAL_MISSING
#define Z_RHODIUM_DECAY_MODE				KEY_NOT_APPLICABLE
#define Z_RHODIUM_NEUTRON_CROSS_SECTION			145.0
#define Z_RHODIUM_NEUTRON_MASS_ABSORPTION		0.063
#define Z_RHODIUM_VALENCE				6
#define Z_RHODIUM_ELECTRONEGATIVITY			2.28
#define Z_RHODIUM_ELECTRON_AFFINITY			109.7
#define Z_RHODIUM_MELTING_POINT				1964.0
#define Z_RHODIUM_BOILING_POINT				3695.0
#define Z_RHODIUM_CRITICAL_PRESSURE			REAL_MISSING
#define Z_RHODIUM_CRITICAL_TEMPERATURE			REAL_MISSING
#define Z_RHODIUM_FUSION_HEAT				21.7
#define Z_RHODIUM_VAPORIZATION_HEAT			495.0
#define Z_RHODIUM_SPECIFIC_HEAT				240.0
#define Z_RHODIUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_RHODIUM_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_RHODIUM_THERMAL_CONDUCTIVITY			150.0
#define Z_RHODIUM_THERMAL_EXPANSION			8.0e-6
#define Z_RHODIUM_DENSITY				12450.0
#define Z_RHODIUM_LIQUID_DENSITY			10700.0
#define Z_RHODIUM_MOLAR_VOLUME				8.2655e-6
#define Z_RHODIUM_BRINELL_HARDNESS			1.1e3
#define Z_RHODIUM_MOHS_HARDNESS				6.0
#define Z_RHODIUM_VICKERS_HARDNESS			1246.0
#define Z_RHODIUM_BULK_MODULUS				380.0
#define Z_RHODIUM_SHEAR_MODULUS				150.0
#define Z_RHODIUM_YOUNG_MODULUS				275.0
#define Z_RHODIUM_POISSON_RATIO				0.26
#define Z_RHODIUM_REFRACTIVE_INDEX			REAL_MISSING
#define Z_RHODIUM_SOUND_SPEED				4.7e3
#define Z_RHODIUM_ELECTRICAL_TYPE			Z_ELECTRICAL_TYPE_CONDUCTOR
#define Z_RHODIUM_ELECTRICAL_CONDUCTIVITY		2.3e7
#define Z_RHODIUM_ELECTRICAL_RESISTIVITY		4.3e-8
#define Z_RHODIUM_SUPERCONDUCTING_POINT			REAL_MISSING
#define Z_RHODIUM_MAGNETIC_TYPE				Z_MAGNETIC_TYPE_PARAMAGNETIC
#define Z_RHODIUM_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_RHODIUM_MASS_MAGNETIC_SUSCEPTIBILITY		1.360e-8
#define Z_RHODIUM_MOLAR_MAGNETIC_SUSCEPTIBILITY		1.400e-9
#define Z_RHODIUM_VOLUME_MAGNETIC_SUSCEPTIBILITY	0.0001693

/* MARK: - 46 - Pd */

#define Z_PALLADIUM_SYMBOL				Pd
#define Z_PALLADIUM_ATOMIC_NUMBER			46
#define Z_PALLADIUM_GROUP				10
#define Z_PALLADIUM_PERIOD				5
#define Z_PALLADIUM_BLOCK				Z_CHEMICAL_ELEMENT_BLOCK_D
#define Z_PALLADIUM_CLASS				Z_CHEMICAL_ELEMENT_CLASS_TRANSITION_METAL
#define Z_PALLADIUM_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_PALLADIUM_SPACE_GROUP_NUMBER			225
#define Z_PALLADIUM_ATOMIC_MASS				106.42
#define Z_PALLADIUM_ATOMIC_RADIUS			169.0
#define Z_PALLADIUM_COVALENT_RADIUS			139.0
#define Z_PALLADIUM_VAN_DER_WAALS_RADIUS		163.0
#define Z_PALLADIUM_DECAY_MODE				KEY_NOT_APPLICABLE
#define Z_PALLADIUM_NEUTRON_CROSS_SECTION		6.9
#define Z_PALLADIUM_NEUTRON_MASS_ABSORPTION		0.0023
#define Z_PALLADIUM_VALENCE				4
#define Z_PALLADIUM_ELECTRONEGATIVITY			2.20
#define Z_PALLADIUM_ELECTRON_AFFINITY			53.7
#define Z_PALLADIUM_MELTING_POINT			1554.9
#define Z_PALLADIUM_BOILING_POINT			2963.0
#define Z_PALLADIUM_CRITICAL_PRESSURE			REAL_MISSING
#define Z_PALLADIUM_CRITICAL_TEMPERATURE		REAL_MISSING
#define Z_PALLADIUM_FUSION_HEAT				16.7
#define Z_PALLADIUM_VAPORIZATION_HEAT			380.0
#define Z_PALLADIUM_SPECIFIC_HEAT			240.0
#define Z_PALLADIUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_PALLADIUM_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_PALLADIUM_THERMAL_CONDUCTIVITY		71.0
#define Z_PALLADIUM_THERMAL_EXPANSION			0.0000118
#define Z_PALLADIUM_DENSITY				12023.0
#define Z_PALLADIUM_LIQUID_DENSITY			10380.0
#define Z_PALLADIUM_MOLAR_VOLUME			8.851e-6
#define Z_PALLADIUM_BRINELL_HARDNESS			37.2
#define Z_PALLADIUM_MOHS_HARDNESS			4.75
#define Z_PALLADIUM_VICKERS_HARDNESS			461.0
#define Z_PALLADIUM_BULK_MODULUS			180.0
#define Z_PALLADIUM_SHEAR_MODULUS			44.0
#define Z_PALLADIUM_YOUNG_MODULUS			121.0
#define Z_PALLADIUM_POISSON_RATIO			0.39
#define Z_PALLADIUM_REFRACTIVE_INDEX			REAL_MISSING
#define Z_PALLADIUM_SOUND_SPEED				3070.0
#define Z_PALLADIUM_ELECTRICAL_TYPE			Z_ELECTRICAL_TYPE_CONDUCTOR
#define Z_PALLADIUM_ELECTRICAL_CONDUCTIVITY		1.0e7
#define Z_PALLADIUM_ELECTRICAL_RESISTIVITY		1.0e-7
#define Z_PALLADIUM_SUPERCONDUCTING_POINT		REAL_MISSING
#define Z_PALLADIUM_MAGNETIC_TYPE			Z_MAGNETIC_TYPE_PARAMAGNETIC
#define Z_PALLADIUM_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_PALLADIUM_MASS_MAGNETIC_SUSCEPTIBILITY	6.570e-8
#define Z_PALLADIUM_MOLAR_MAGNETIC_SUSCEPTIBILITY	6.992e-9
#define Z_PALLADIUM_VOLUME_MAGNETIC_SUSCEPTIBILITY	0.0007899

/* MARK: - 47 - Ag */

#define Z_SILVER_SYMBOL					Ag
#define Z_SILVER_ATOMIC_NUMBER				47
#define Z_SILVER_GROUP					11
#define Z_SILVER_PERIOD					5
#define Z_SILVER_BLOCK					Z_CHEMICAL_ELEMENT_BLOCK_D
#define Z_SILVER_CLASS					Z_CHEMICAL_ELEMENT_CLASS_TRANSITION_METAL
#define Z_SILVER_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_SILVER_SPACE_GROUP_NUMBER			225
#define Z_SILVER_ATOMIC_MASS				107.8682
#define Z_SILVER_ATOMIC_RADIUS				165.0
#define Z_SILVER_COVALENT_RADIUS			145.0
#define Z_SILVER_VAN_DER_WAALS_RADIUS			172.0
#define Z_SILVER_DECAY_MODE				KEY_NOT_APPLICABLE
#define Z_SILVER_NEUTRON_CROSS_SECTION			63.6
#define Z_SILVER_NEUTRON_MASS_ABSORPTION		0.020
#define Z_SILVER_VALENCE				1
#define Z_SILVER_ELECTRONEGATIVITY			1.93
#define Z_SILVER_ELECTRON_AFFINITY			125.6
#define Z_SILVER_MELTING_POINT				961.78
#define Z_SILVER_BOILING_POINT				2162.0
#define Z_SILVER_CRITICAL_PRESSURE			REAL_MISSING
#define Z_SILVER_CRITICAL_TEMPERATURE			REAL_MISSING
#define Z_SILVER_FUSION_HEAT				11.3
#define Z_SILVER_VAPORIZATION_HEAT			255.0
#define Z_SILVER_SPECIFIC_HEAT				235.0
#define Z_SILVER_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_SILVER_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_SILVER_THERMAL_CONDUCTIVITY			430.0
#define Z_SILVER_THERMAL_EXPANSION			0.0000189
#define Z_SILVER_DENSITY				10490.0
#define Z_SILVER_LIQUID_DENSITY				9320.0
#define Z_SILVER_MOLAR_VOLUME				0.000010283
#define Z_SILVER_BRINELL_HARDNESS			24.5
#define Z_SILVER_MOHS_HARDNESS				2.5
#define Z_SILVER_VICKERS_HARDNESS			251.0
#define Z_SILVER_BULK_MODULUS				1.0e2
#define Z_SILVER_SHEAR_MODULUS				30.0
#define Z_SILVER_YOUNG_MODULUS				85.0
#define Z_SILVER_POISSON_RATIO				0.37
#define Z_SILVER_REFRACTIVE_INDEX			REAL_MISSING
#define Z_SILVER_SOUND_SPEED				2.6e3
#define Z_SILVER_ELECTRICAL_TYPE			Z_ELECTRICAL_TYPE_CONDUCTOR
#define Z_SILVER_ELECTRICAL_CONDUCTIVITY		6.2e7
#define Z_SILVER_ELECTRICAL_RESISTIVITY			1.6e-8
#define Z_SILVER_SUPERCONDUCTING_POINT			REAL_MISSING
#define Z_SILVER_MAGNETIC_TYPE				Z_MAGNETIC_TYPE_DIAMAGNETIC
#define Z_SILVER_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_SILVER_MASS_MAGNETIC_SUSCEPTIBILITY		-2.27e-9
#define Z_SILVER_MOLAR_MAGNETIC_SUSCEPTIBILITY		-2.45e-10
#define Z_SILVER_VOLUME_MAGNETIC_SUSCEPTIBILITY		-0.0000238

/* MARK: - 48 - Cd */

#define Z_CADMIUM_SYMBOL				Cd
#define Z_CADMIUM_ATOMIC_NUMBER				48
#define Z_CADMIUM_GROUP					12
#define Z_CADMIUM_PERIOD				5
#define Z_CADMIUM_BLOCK					Z_CHEMICAL_ELEMENT_BLOCK_D
#define Z_CADMIUM_CLASS					Z_CHEMICAL_ELEMENT_CLASS_TRANSITION_METAL
#define Z_CADMIUM_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_CADMIUM_SPACE_GROUP_NUMBER			194
#define Z_CADMIUM_ATOMIC_MASS				112.411
#define Z_CADMIUM_ATOMIC_RADIUS				161.0
#define Z_CADMIUM_COVALENT_RADIUS			144.0
#define Z_CADMIUM_VAN_DER_WAALS_RADIUS			158.0
#define Z_CADMIUM_DECAY_MODE				KEY_NOT_APPLICABLE
#define Z_CADMIUM_NEUTRON_CROSS_SECTION			2450.0
#define Z_CADMIUM_NEUTRON_MASS_ABSORPTION		1.400000
#define Z_CADMIUM_VALENCE				2
#define Z_CADMIUM_ELECTRONEGATIVITY			1.69
#define Z_CADMIUM_ELECTRON_AFFINITY			0.0
#define Z_CADMIUM_MELTING_POINT				321.07
#define Z_CADMIUM_BOILING_POINT				767.0
#define Z_CADMIUM_CRITICAL_PRESSURE			REAL_MISSING
#define Z_CADMIUM_CRITICAL_TEMPERATURE			REAL_MISSING
#define Z_CADMIUM_FUSION_HEAT				6.3
#define Z_CADMIUM_VAPORIZATION_HEAT			1.0e2
#define Z_CADMIUM_SPECIFIC_HEAT				230.0
#define Z_CADMIUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_CADMIUM_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_CADMIUM_THERMAL_CONDUCTIVITY			96.0
#define Z_CADMIUM_THERMAL_EXPANSION			0.0000308
#define Z_CADMIUM_DENSITY				8650.0
#define Z_CADMIUM_LIQUID_DENSITY			7996.0
#define Z_CADMIUM_MOLAR_VOLUME				0.00001300
#define Z_CADMIUM_BRINELL_HARDNESS			203.0
#define Z_CADMIUM_MOHS_HARDNESS				2.0
#define Z_CADMIUM_VICKERS_HARDNESS			REAL_MISSING
#define Z_CADMIUM_BULK_MODULUS				42.0
#define Z_CADMIUM_SHEAR_MODULUS				19.0
#define Z_CADMIUM_YOUNG_MODULUS				50.0
#define Z_CADMIUM_POISSON_RATIO				0.30
#define Z_CADMIUM_REFRACTIVE_INDEX			REAL_MISSING
#define Z_CADMIUM_SOUND_SPEED				2310.0
#define Z_CADMIUM_ELECTRICAL_TYPE			Z_ELECTRICAL_TYPE_CONDUCTOR
#define Z_CADMIUM_ELECTRICAL_CONDUCTIVITY		1.4e7
#define Z_CADMIUM_ELECTRICAL_RESISTIVITY		7.0e-8
#define Z_CADMIUM_SUPERCONDUCTING_POINT			0.517
#define Z_CADMIUM_MAGNETIC_TYPE				Z_MAGNETIC_TYPE_DIAMAGNETIC
#define Z_CADMIUM_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_CADMIUM_MASS_MAGNETIC_SUSCEPTIBILITY		-2.30e-9
#define Z_CADMIUM_MOLAR_MAGNETIC_SUSCEPTIBILITY		-2.59e-10
#define Z_CADMIUM_VOLUME_MAGNETIC_SUSCEPTIBILITY	-0.0000199

/* MARK: - 49 - In */

#define Z_INDIUM_SYMBOL					In
#define Z_INDIUM_ATOMIC_NUMBER				49
#define Z_INDIUM_GROUP					13
#define Z_INDIUM_PERIOD					5
#define Z_INDIUM_BLOCK					Z_CHEMICAL_ELEMENT_BLOCK_P
#define Z_INDIUM_CLASS					Z_CHEMICAL_ELEMENT_CLASS_POOR_METAL
#define Z_INDIUM_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_INDIUM_SPACE_GROUP_NUMBER			139
#define Z_INDIUM_ATOMIC_MASS				114.818
#define Z_INDIUM_ATOMIC_RADIUS				156.0
#define Z_INDIUM_COVALENT_RADIUS			142.0
#define Z_INDIUM_VAN_DER_WAALS_RADIUS			193.0
#define Z_INDIUM_DECAY_MODE				KEY_NOT_APPLICABLE
#define Z_INDIUM_NEUTRON_CROSS_SECTION			194.0
#define Z_INDIUM_NEUTRON_MASS_ABSORPTION		0.060
#define Z_INDIUM_VALENCE				3
#define Z_INDIUM_ELECTRONEGATIVITY			1.78
#define Z_INDIUM_ELECTRON_AFFINITY			28.9
#define Z_INDIUM_MELTING_POINT				156.6
#define Z_INDIUM_BOILING_POINT				2072.0
#define Z_INDIUM_CRITICAL_PRESSURE			REAL_MISSING
#define Z_INDIUM_CRITICAL_TEMPERATURE			REAL_MISSING
#define Z_INDIUM_FUSION_HEAT				3.26
#define Z_INDIUM_VAPORIZATION_HEAT			230.0
#define Z_INDIUM_SPECIFIC_HEAT				233.0
#define Z_INDIUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_INDIUM_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_INDIUM_THERMAL_CONDUCTIVITY			82.0
#define Z_INDIUM_THERMAL_EXPANSION			0.0000321
#define Z_INDIUM_DENSITY				7310.0
#define Z_INDIUM_LIQUID_DENSITY				7020.0
#define Z_INDIUM_MOLAR_VOLUME				0.00001571
#define Z_INDIUM_BRINELL_HARDNESS			8.83
#define Z_INDIUM_MOHS_HARDNESS				1.2
#define Z_INDIUM_VICKERS_HARDNESS			REAL_MISSING
#define Z_INDIUM_BULK_MODULUS				REAL_MISSING
#define Z_INDIUM_SHEAR_MODULUS				REAL_MISSING
#define Z_INDIUM_YOUNG_MODULUS				11.0
#define Z_INDIUM_POISSON_RATIO				REAL_MISSING
#define Z_INDIUM_REFRACTIVE_INDEX			REAL_MISSING
#define Z_INDIUM_SOUND_SPEED				1215.0
#define Z_INDIUM_ELECTRICAL_TYPE			Z_ELECTRICAL_TYPE_CONDUCTOR
#define Z_INDIUM_ELECTRICAL_CONDUCTIVITY		1.2e7
#define Z_INDIUM_ELECTRICAL_RESISTIVITY			8.0e-8
#define Z_INDIUM_SUPERCONDUCTING_POINT			3.41
#define Z_INDIUM_MAGNETIC_TYPE				Z_MAGNETIC_TYPE_DIAMAGNETIC
#define Z_INDIUM_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_INDIUM_MASS_MAGNETIC_SUSCEPTIBILITY		-1.40e-9
#define Z_INDIUM_MOLAR_MAGNETIC_SUSCEPTIBILITY		-1.61e-10
#define Z_INDIUM_VOLUME_MAGNETIC_SUSCEPTIBILITY		-0.0000102

/* MARK: - 50 - Sn */

#define Z_TIN_SYMBOL					Sn
#define Z_TIN_ATOMIC_NUMBER				50
#define Z_TIN_GROUP					14
#define Z_TIN_PERIOD					5
#define Z_TIN_BLOCK					Z_CHEMICAL_ELEMENT_BLOCK_P
#define Z_TIN_CLASS					Z_CHEMICAL_ELEMENT_CLASS_POOR_METAL
#define Z_TIN_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_TIN_SPACE_GROUP_NUMBER			141
#define Z_TIN_ATOMIC_MASS				118.710
#define Z_TIN_ATOMIC_RADIUS				145.0
#define Z_TIN_COVALENT_RADIUS				139.0
#define Z_TIN_VAN_DER_WAALS_RADIUS			217.0
#define Z_TIN_DECAY_MODE				KEY_NOT_APPLICABLE
#define Z_TIN_NEUTRON_CROSS_SECTION			0.62
#define Z_TIN_NEUTRON_MASS_ABSORPTION			0.00020
#define Z_TIN_VALENCE					4
#define Z_TIN_ELECTRONEGATIVITY				1.96
#define Z_TIN_ELECTRON_AFFINITY				107.3
#define Z_TIN_MELTING_POINT				231.93
#define Z_TIN_BOILING_POINT				2602.0
#define Z_TIN_CRITICAL_PRESSURE				REAL_UNKNOWN
#define Z_TIN_CRITICAL_TEMPERATURE			REAL_UNKNOWN
#define Z_TIN_FUSION_HEAT				7.0
#define Z_TIN_VAPORIZATION_HEAT				290.0
#define Z_TIN_SPECIFIC_HEAT				217.0
#define Z_TIN_ADIABATIC_INDEX				KEY_NOT_APPLICABLE
#define Z_TIN_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_TIN_THERMAL_CONDUCTIVITY			67.0
#define Z_TIN_THERMAL_EXPANSION				0.000022
#define Z_TIN_DENSITY					7310.0
#define Z_TIN_LIQUID_DENSITY				6990.0
#define Z_TIN_MOLAR_VOLUME				0.00001624
#define Z_TIN_BRINELL_HARDNESS				51.0
#define Z_TIN_MOHS_HARDNESS				1.5
#define Z_TIN_VICKERS_HARDNESS				REAL_MISSING
#define Z_TIN_BULK_MODULUS				58.0
#define Z_TIN_SHEAR_MODULUS				18.0
#define Z_TIN_YOUNG_MODULUS				50.0
#define Z_TIN_POISSON_RATIO				0.36
#define Z_TIN_REFRACTIVE_INDEX				REAL_UNKNOWN
#define Z_TIN_SOUND_SPEED				2.5e3
#define Z_TIN_ELECTRICAL_TYPE				Z_ELECTRICAL_TYPE_CONDUCTOR
#define Z_TIN_ELECTRICAL_CONDUCTIVITY			9.1e6
#define Z_TIN_ELECTRICAL_RESISTIVITY			1.1e-7
#define Z_TIN_SUPERCONDUCTING_POINT			3.72
#define Z_TIN_MAGNETIC_TYPE				Z_MAGNETIC_TYPE_DIAMAGNETIC
#define Z_TIN_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_TIN_MASS_MAGNETIC_SUSCEPTIBILITY		-3.10e-9
#define Z_TIN_MOLAR_MAGNETIC_SUSCEPTIBILITY		-3.68e-10
#define Z_TIN_VOLUME_MAGNETIC_SUSCEPTIBILITY		-0.0000227

/* MARK: - 51 - Sb */

#define Z_ANTIMONY_SYMBOL				Sb
#define Z_ANTIMONY_ATOMIC_NUMBER			51
#define Z_ANTIMONY_GROUP				15
#define Z_ANTIMONY_PERIOD				5
#define Z_ANTIMONY_BLOCK				Z_CHEMICAL_ELEMENT_BLOCK_P
#define Z_ANTIMONY_CLASS				Z_CHEMICAL_ELEMENT_CLASS_METALLOID
#define Z_ANTIMONY_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_METALLOID
#define Z_ANTIMONY_SPACE_GROUP_NUMBER			166
#define Z_ANTIMONY_ATOMIC_MASS				121.760
#define Z_ANTIMONY_ATOMIC_RADIUS			133.0
#define Z_ANTIMONY_COVALENT_RADIUS			139.0
#define Z_ANTIMONY_VAN_DER_WAALS_RADIUS			REAL_MISSING
#define Z_ANTIMONY_DECAY_MODE				KEY_NOT_APPLICABLE
#define Z_ANTIMONY_NEUTRON_CROSS_SECTION		5.4
#define Z_ANTIMONY_NEUTRON_MASS_ABSORPTION		0.0016
#define Z_ANTIMONY_VALENCE				5
#define Z_ANTIMONY_ELECTRONEGATIVITY			2.05
#define Z_ANTIMONY_ELECTRON_AFFINITY			103.2
#define Z_ANTIMONY_MELTING_POINT			630.63
#define Z_ANTIMONY_BOILING_POINT			1587.0
#define Z_ANTIMONY_CRITICAL_PRESSURE			REAL_MISSING
#define Z_ANTIMONY_CRITICAL_TEMPERATURE			REAL_MISSING
#define Z_ANTIMONY_FUSION_HEAT				19.7
#define Z_ANTIMONY_VAPORIZATION_HEAT			67.0
#define Z_ANTIMONY_SPECIFIC_HEAT			207.0
#define Z_ANTIMONY_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_ANTIMONY_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_ANTIMONY_THERMAL_CONDUCTIVITY			24.0
#define Z_ANTIMONY_THERMAL_EXPANSION			0.000011
#define Z_ANTIMONY_DENSITY				6697.0
#define Z_ANTIMONY_LIQUID_DENSITY			6530.0
#define Z_ANTIMONY_MOLAR_VOLUME				0.00001818
#define Z_ANTIMONY_BRINELL_HARDNESS			294.0
#define Z_ANTIMONY_MOHS_HARDNESS			3.0
#define Z_ANTIMONY_VICKERS_HARDNESS			REAL_MISSING
#define Z_ANTIMONY_BULK_MODULUS				42.0
#define Z_ANTIMONY_SHEAR_MODULUS			20.0
#define Z_ANTIMONY_YOUNG_MODULUS			55.0
#define Z_ANTIMONY_POISSON_RATIO			REAL_MISSING
#define Z_ANTIMONY_REFRACTIVE_INDEX			REAL_MISSING
#define Z_ANTIMONY_SOUND_SPEED				3420.0
#define Z_ANTIMONY_ELECTRICAL_TYPE			Z_ELECTRICAL_TYPE_CONDUCTOR
#define Z_ANTIMONY_ELECTRICAL_CONDUCTIVITY		2.5e6
#define Z_ANTIMONY_ELECTRICAL_RESISTIVITY		4.0e-7
#define Z_ANTIMONY_SUPERCONDUCTING_POINT		REAL_MISSING
#define Z_ANTIMONY_MAGNETIC_TYPE			Z_MAGNETIC_TYPE_DIAMAGNETIC
#define Z_ANTIMONY_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_ANTIMONY_MASS_MAGNETIC_SUSCEPTIBILITY		-1.090e-8
#define Z_ANTIMONY_MOLAR_MAGNETIC_SUSCEPTIBILITY	-1.327e-9
#define Z_ANTIMONY_VOLUME_MAGNETIC_SUSCEPTIBILITY	-0.00007300

/* MARK: - 52 - Te */

#define Z_TELLURIUM_SYMBOL				Te
#define Z_TELLURIUM_ATOMIC_NUMBER			52
#define Z_TELLURIUM_GROUP				16
#define Z_TELLURIUM_PERIOD				5
#define Z_TELLURIUM_BLOCK				Z_CHEMICAL_ELEMENT_BLOCK_P
#define Z_TELLURIUM_CLASS				Z_CHEMICAL_ELEMENT_CLASS_METALLOID
#define Z_TELLURIUM_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_METALLOID
#define Z_TELLURIUM_SPACE_GROUP_NUMBER			152
#define Z_TELLURIUM_ATOMIC_MASS				127.60
#define Z_TELLURIUM_ATOMIC_RADIUS			123.0
#define Z_TELLURIUM_COVALENT_RADIUS			138.0
#define Z_TELLURIUM_VAN_DER_WAALS_RADIUS		206.0
#define Z_TELLURIUM_DECAY_MODE				KEY_NOT_APPLICABLE
#define Z_TELLURIUM_NEUTRON_CROSS_SECTION		5.4
#define Z_TELLURIUM_NEUTRON_MASS_ABSORPTION		0.0013
#define Z_TELLURIUM_VALENCE				6
#define Z_TELLURIUM_ELECTRONEGATIVITY			2.1
#define Z_TELLURIUM_ELECTRON_AFFINITY			190.2
#define Z_TELLURIUM_MELTING_POINT			449.51
#define Z_TELLURIUM_BOILING_POINT			988.0
#define Z_TELLURIUM_CRITICAL_PRESSURE			REAL_MISSING
#define Z_TELLURIUM_CRITICAL_TEMPERATURE		REAL_MISSING
#define Z_TELLURIUM_FUSION_HEAT				17.5
#define Z_TELLURIUM_VAPORIZATION_HEAT			48.0
#define Z_TELLURIUM_SPECIFIC_HEAT			201.0
#define Z_TELLURIUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_TELLURIUM_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_TELLURIUM_THERMAL_CONDUCTIVITY		3.0
#define Z_TELLURIUM_THERMAL_EXPANSION			REAL_MISSING
#define Z_TELLURIUM_DENSITY				6240.0
#define Z_TELLURIUM_LIQUID_DENSITY			5700.0
#define Z_TELLURIUM_MOLAR_VOLUME			0.00002045
#define Z_TELLURIUM_BRINELL_HARDNESS			180.0
#define Z_TELLURIUM_MOHS_HARDNESS			2.25
#define Z_TELLURIUM_VICKERS_HARDNESS			REAL_MISSING
#define Z_TELLURIUM_BULK_MODULUS			64.0
#define Z_TELLURIUM_SHEAR_MODULUS			16.0
#define Z_TELLURIUM_YOUNG_MODULUS			43.0
#define Z_TELLURIUM_POISSON_RATIO			REAL_MISSING
#define Z_TELLURIUM_REFRACTIVE_INDEX			1.000991
#define Z_TELLURIUM_SOUND_SPEED				2610.0
#define Z_TELLURIUM_ELECTRICAL_TYPE			Z_ELECTRICAL_TYPE_SEMICONDUCTOR
#define Z_TELLURIUM_ELECTRICAL_CONDUCTIVITY		1.0e4
#define Z_TELLURIUM_ELECTRICAL_RESISTIVITY		0.0001
#define Z_TELLURIUM_SUPERCONDUCTING_POINT		REAL_MISSING
#define Z_TELLURIUM_MAGNETIC_TYPE			Z_MAGNETIC_TYPE_DIAMAGNETIC
#define Z_TELLURIUM_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_TELLURIUM_MASS_MAGNETIC_SUSCEPTIBILITY	-3.90e-9
#define Z_TELLURIUM_MOLAR_MAGNETIC_SUSCEPTIBILITY	-4.98e-10
#define Z_TELLURIUM_VOLUME_MAGNETIC_SUSCEPTIBILITY	-0.0000243

/* MARK: - 53 - I */

#define Z_IODINE_SYMBOL					I
#define Z_IODINE_ATOMIC_NUMBER				53
#define Z_IODINE_GROUP					17
#define Z_IODINE_PERIOD					5
#define Z_IODINE_BLOCK					Z_CHEMICAL_ELEMENT_BLOCK_P
#define Z_IODINE_CLASS					Z_CHEMICAL_ELEMENT_CLASS_HALOGEN
#define Z_IODINE_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_NON_METAL
#define Z_IODINE_SPACE_GROUP_NUMBER			64
#define Z_IODINE_ATOMIC_MASS				126.90447
#define Z_IODINE_ATOMIC_RADIUS				115.0
#define Z_IODINE_COVALENT_RADIUS			139.0
#define Z_IODINE_VAN_DER_WAALS_RADIUS			198.0
#define Z_IODINE_DECAY_MODE				KEY_NOT_APPLICABLE
#define Z_IODINE_NEUTRON_CROSS_SECTION			6.3
#define Z_IODINE_NEUTRON_MASS_ABSORPTION		0.0018
#define Z_IODINE_VALENCE				7
#define Z_IODINE_ELECTRONEGATIVITY			2.66
#define Z_IODINE_ELECTRON_AFFINITY			295.2
#define Z_IODINE_MELTING_POINT				113.7
#define Z_IODINE_BOILING_POINT				184.3
#define Z_IODINE_CRITICAL_PRESSURE			11.7
#define Z_IODINE_CRITICAL_TEMPERATURE			819.0
#define Z_IODINE_FUSION_HEAT				7.76
#define Z_IODINE_VAPORIZATION_HEAT			20.9
#define Z_IODINE_SPECIFIC_HEAT				429.0
#define Z_IODINE_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_IODINE_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_IODINE_THERMAL_CONDUCTIVITY			0.449
#define Z_IODINE_THERMAL_EXPANSION			REAL_MISSING
#define Z_IODINE_DENSITY				4940.0
#define Z_IODINE_LIQUID_DENSITY				REAL_MISSING
#define Z_IODINE_MOLAR_VOLUME				0.00002569
#define Z_IODINE_BRINELL_HARDNESS			REAL_MISSING
#define Z_IODINE_MOHS_HARDNESS				REAL_MISSING
#define Z_IODINE_VICKERS_HARDNESS			REAL_MISSING
#define Z_IODINE_BULK_MODULUS				7.7
#define Z_IODINE_SHEAR_MODULUS				REAL_MISSING
#define Z_IODINE_YOUNG_MODULUS				REAL_MISSING
#define Z_IODINE_POISSON_RATIO				REAL_MISSING
#define Z_IODINE_REFRACTIVE_INDEX			REAL_MISSING
#define Z_IODINE_SOUND_SPEED				REAL_MISSING
#define Z_IODINE_ELECTRICAL_TYPE			Z_ELECTRICAL_TYPE_INSULATOR
#define Z_IODINE_ELECTRICAL_CONDUCTIVITY		1.0e-7
#define Z_IODINE_ELECTRICAL_RESISTIVITY			1.0e7
#define Z_IODINE_SUPERCONDUCTING_POINT			REAL_MISSING
#define Z_IODINE_MAGNETIC_TYPE				Z_MAGNETIC_TYPE_DIAMAGNETIC
#define Z_IODINE_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_IODINE_MASS_MAGNETIC_SUSCEPTIBILITY		-4.50e-9
#define Z_IODINE_MOLAR_MAGNETIC_SUSCEPTIBILITY		-1.14e-9
#define Z_IODINE_VOLUME_MAGNETIC_SUSCEPTIBILITY		-0.0000222

/* MARK: - 54 - Xe */

#define Z_XENON_SYMBOL					Xe
#define Z_XENON_ATOMIC_NUMBER				54
#define Z_XENON_GROUP					18
#define Z_XENON_PERIOD					5
#define Z_XENON_BLOCK					Z_CHEMICAL_ELEMENT_BLOCK_P
#define Z_XENON_CLASS					Z_CHEMICAL_ELEMENT_CLASS_NOBLE_GAS
#define Z_XENON_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_NON_METAL
#define Z_XENON_SPACE_GROUP_NUMBER			225
#define Z_XENON_ATOMIC_MASS				131.293
#define Z_XENON_ATOMIC_RADIUS				108.0
#define Z_XENON_COVALENT_RADIUS				140.0
#define Z_XENON_VAN_DER_WAALS_RADIUS			216.0
#define Z_XENON_DECAY_MODE				KEY_NOT_APPLICABLE
#define Z_XENON_NEUTRON_CROSS_SECTION			25.0
#define Z_XENON_NEUTRON_MASS_ABSORPTION			0.0083
#define Z_XENON_VALENCE					6
#define Z_XENON_ELECTRONEGATIVITY			2.6
#define Z_XENON_ELECTRON_AFFINITY			0.0
#define Z_XENON_MELTING_POINT				-111.8
#define Z_XENON_BOILING_POINT				-108.0
#define Z_XENON_CRITICAL_PRESSURE			5.841
#define Z_XENON_CRITICAL_TEMPERATURE			289.77
#define Z_XENON_FUSION_HEAT				2.30
#define Z_XENON_VAPORIZATION_HEAT			12.64
#define Z_XENON_SPECIFIC_HEAT				158.32
#define Z_XENON_ADIABATIC_INDEX				Z_CHEMICAL_ELEMENT_ADIABATIC_INDEX_5_3
#define Z_XENON_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_XENON_THERMAL_CONDUCTIVITY			0.00565
#define Z_XENON_THERMAL_EXPANSION			REAL_NOT_APPLICABLE
#define Z_XENON_DENSITY					5.9
#define Z_XENON_LIQUID_DENSITY				REAL_MISSING
#define Z_XENON_MOLAR_VOLUME				0.022
#define Z_XENON_BRINELL_HARDNESS			REAL_NOT_APPLICABLE
#define Z_XENON_MOHS_HARDNESS				REAL_NOT_APPLICABLE
#define Z_XENON_VICKERS_HARDNESS			REAL_MISSING
#define Z_XENON_BULK_MODULUS				REAL_NOT_APPLICABLE
#define Z_XENON_SHEAR_MODULUS				REAL_NOT_APPLICABLE
#define Z_XENON_YOUNG_MODULUS				REAL_NOT_APPLICABLE
#define Z_XENON_POISSON_RATIO				REAL_NOT_APPLICABLE
#define Z_XENON_REFRACTIVE_INDEX			1.000702
#define Z_XENON_SOUND_SPEED				1090.0
#define Z_XENON_ELECTRICAL_TYPE				KEY_MISSING
#define Z_XENON_ELECTRICAL_CONDUCTIVITY			REAL_MISSING
#define Z_XENON_ELECTRICAL_RESISTIVITY			REAL_NOT_APPLICABLE
#define Z_XENON_SUPERCONDUCTING_POINT			REAL_MISSING
#define Z_XENON_MAGNETIC_TYPE				Z_MAGNETIC_TYPE_DIAMAGNETIC
#define Z_XENON_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_XENON_MASS_MAGNETIC_SUSCEPTIBILITY		-4.30e-9
#define Z_XENON_MOLAR_MAGNETIC_SUSCEPTIBILITY		-5.65e-10
#define Z_XENON_VOLUME_MAGNETIC_SUSCEPTIBILITY		-2.54e-8

/* MARK: - 55 - Cs */

#define Z_CAESIUM_SYMBOL				Cs
#define Z_CAESIUM_ATOMIC_NUMBER				55
#define Z_CAESIUM_GROUP					1
#define Z_CAESIUM_PERIOD				6
#define Z_CAESIUM_BLOCK					Z_CHEMICAL_ELEMENT_BLOCK_S
#define Z_CAESIUM_CLASS					Z_CHEMICAL_ELEMENT_CLASS_ALKALI_METAL
#define Z_CAESIUM_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_CAESIUM_SPACE_GROUP_NUMBER			229
#define Z_CAESIUM_ATOMIC_MASS				132.9054519
#define Z_CAESIUM_ATOMIC_RADIUS				298.0
#define Z_CAESIUM_COVALENT_RADIUS			244.0
#define Z_CAESIUM_VAN_DER_WAALS_RADIUS			REAL_MISSING
#define Z_CAESIUM_DECAY_MODE				KEY_NOT_APPLICABLE
#define Z_CAESIUM_NEUTRON_CROSS_SECTION			29.0
#define Z_CAESIUM_NEUTRON_MASS_ABSORPTION		0.0077
#define Z_CAESIUM_VALENCE				1
#define Z_CAESIUM_ELECTRONEGATIVITY			0.79
#define Z_CAESIUM_ELECTRON_AFFINITY			45.5
#define Z_CAESIUM_MELTING_POINT				28.44
#define Z_CAESIUM_BOILING_POINT				671.0
#define Z_CAESIUM_CRITICAL_PRESSURE			9.4
#define Z_CAESIUM_CRITICAL_TEMPERATURE			1938.0
#define Z_CAESIUM_FUSION_HEAT				2.09
#define Z_CAESIUM_VAPORIZATION_HEAT			64.0
#define Z_CAESIUM_SPECIFIC_HEAT				242.0
#define Z_CAESIUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_CAESIUM_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_CAESIUM_THERMAL_CONDUCTIVITY			36.0
#define Z_CAESIUM_THERMAL_EXPANSION			REAL_MISSING
#define Z_CAESIUM_DENSITY				1879.0
#define Z_CAESIUM_LIQUID_DENSITY			1843.0
#define Z_CAESIUM_MOLAR_VOLUME				0.00007073
#define Z_CAESIUM_BRINELL_HARDNESS			0.14
#define Z_CAESIUM_MOHS_HARDNESS				0.2
#define Z_CAESIUM_VICKERS_HARDNESS			REAL_MISSING
#define Z_CAESIUM_BULK_MODULUS				1.6
#define Z_CAESIUM_SHEAR_MODULUS				REAL_MISSING
#define Z_CAESIUM_YOUNG_MODULUS				1.7
#define Z_CAESIUM_POISSON_RATIO				REAL_MISSING
#define Z_CAESIUM_REFRACTIVE_INDEX			REAL_MISSING
#define Z_CAESIUM_SOUND_SPEED				REAL_MISSING
#define Z_CAESIUM_ELECTRICAL_TYPE			Z_ELECTRICAL_TYPE_CONDUCTOR
#define Z_CAESIUM_ELECTRICAL_CONDUCTIVITY		5.0e6
#define Z_CAESIUM_ELECTRICAL_RESISTIVITY		2.0e-7
#define Z_CAESIUM_SUPERCONDUCTING_POINT			REAL_MISSING
#define Z_CAESIUM_MAGNETIC_TYPE				Z_MAGNETIC_TYPE_PARAMAGNETIC
#define Z_CAESIUM_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_CAESIUM_MASS_MAGNETIC_SUSCEPTIBILITY		2.80e-9
#define Z_CAESIUM_MOLAR_MAGNETIC_SUSCEPTIBILITY		3.72e-10
#define Z_CAESIUM_VOLUME_MAGNETIC_SUSCEPTIBILITY	5.26e-6

/* MARK: - 56 - Ba */

#define Z_BARIUM_SYMBOL					Ba
#define Z_BARIUM_ATOMIC_NUMBER				56
#define Z_BARIUM_GROUP					2
#define Z_BARIUM_PERIOD					6
#define Z_BARIUM_BLOCK					Z_CHEMICAL_ELEMENT_BLOCK_S
#define Z_BARIUM_CLASS					Z_CHEMICAL_ELEMENT_CLASS_ALKALINE_EARTH_METAL
#define Z_BARIUM_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_BARIUM_SPACE_GROUP_NUMBER			229
#define Z_BARIUM_ATOMIC_MASS				137.327
#define Z_BARIUM_ATOMIC_RADIUS				253.0
#define Z_BARIUM_COVALENT_RADIUS			215.0
#define Z_BARIUM_VAN_DER_WAALS_RADIUS			REAL_UNKNOWN
#define Z_BARIUM_DECAY_MODE				KEY_NOT_APPLICABLE
#define Z_BARIUM_NEUTRON_CROSS_SECTION			1.3
#define Z_BARIUM_NEUTRON_MASS_ABSORPTION		0.00027
#define Z_BARIUM_VALENCE				2
#define Z_BARIUM_ELECTRONEGATIVITY			0.89
#define Z_BARIUM_ELECTRON_AFFINITY			13.95
#define Z_BARIUM_MELTING_POINT				727.0
#define Z_BARIUM_BOILING_POINT				1870.0
#define Z_BARIUM_CRITICAL_PRESSURE			REAL_UNKNOWN
#define Z_BARIUM_CRITICAL_TEMPERATURE			REAL_UNKNOWN
#define Z_BARIUM_FUSION_HEAT				8.0
#define Z_BARIUM_VAPORIZATION_HEAT			140.0
#define Z_BARIUM_SPECIFIC_HEAT				205.0
#define Z_BARIUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_BARIUM_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_BARIUM_THERMAL_CONDUCTIVITY			18.0
#define Z_BARIUM_THERMAL_EXPANSION			0.0000206
#define Z_BARIUM_DENSITY				3510.0
#define Z_BARIUM_LIQUID_DENSITY				3338.0
#define Z_BARIUM_MOLAR_VOLUME				0.00003912
#define Z_BARIUM_BRINELL_HARDNESS			REAL_MISSING
#define Z_BARIUM_MOHS_HARDNESS				1.25
#define Z_BARIUM_VICKERS_HARDNESS			REAL_UNKNOWN
#define Z_BARIUM_BULK_MODULUS				9.4
#define Z_BARIUM_SHEAR_MODULUS				4.9
#define Z_BARIUM_YOUNG_MODULUS				13.0
#define Z_BARIUM_POISSON_RATIO				REAL_MISSING
#define Z_BARIUM_REFRACTIVE_INDEX			REAL_UNKNOWN
#define Z_BARIUM_SOUND_SPEED				1620.0
#define Z_BARIUM_ELECTRICAL_TYPE			Z_ELECTRICAL_TYPE_CONDUCTOR
#define Z_BARIUM_ELECTRICAL_CONDUCTIVITY		2.9e6
#define Z_BARIUM_ELECTRICAL_RESISTIVITY			3.5e-7
#define Z_BARIUM_SUPERCONDUCTING_POINT			REAL_UNKNOWN
#define Z_BARIUM_MAGNETIC_TYPE				Z_MAGNETIC_TYPE_PARAMAGNETIC
#define Z_BARIUM_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_BARIUM_MASS_MAGNETIC_SUSCEPTIBILITY		1.130e-8
#define Z_BARIUM_MOLAR_MAGNETIC_SUSCEPTIBILITY		1.552e-9
#define Z_BARIUM_VOLUME_MAGNETIC_SUSCEPTIBILITY		0.00003966

/* MARK: - 57 - La */

#define Z_LANTHANUM_SYMBOL				La
#define Z_LANTHANUM_ATOMIC_NUMBER			57
#define Z_LANTHANUM_GROUP				INTEGER_NOT_APPLICABLE
#define Z_LANTHANUM_PERIOD				6
#define Z_LANTHANUM_BLOCK				Z_CHEMICAL_ELEMENT_BLOCK_F
#define Z_LANTHANUM_CLASS				Z_CHEMICAL_ELEMENT_CLASS_LANTHANIDE
#define Z_LANTHANUM_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_LANTHANUM_SPACE_GROUP_NUMBER			194
#define Z_LANTHANUM_ATOMIC_MASS				138.90547
#define Z_LANTHANUM_ATOMIC_RADIUS			REAL_MISSING
#define Z_LANTHANUM_COVALENT_RADIUS			207.0
#define Z_LANTHANUM_VAN_DER_WAALS_RADIUS		REAL_MISSING
#define Z_LANTHANUM_DECAY_MODE				KEY_NOT_APPLICABLE
#define Z_LANTHANUM_NEUTRON_CROSS_SECTION		8.98
#define Z_LANTHANUM_NEUTRON_MASS_ABSORPTION		0.0023
#define Z_LANTHANUM_VALENCE				3
#define Z_LANTHANUM_ELECTRONEGATIVITY			1.10
#define Z_LANTHANUM_ELECTRON_AFFINITY			48.0
#define Z_LANTHANUM_MELTING_POINT			919.0
#define Z_LANTHANUM_BOILING_POINT			3464.0
#define Z_LANTHANUM_CRITICAL_PRESSURE			REAL_MISSING
#define Z_LANTHANUM_CRITICAL_TEMPERATURE		REAL_MISSING
#define Z_LANTHANUM_FUSION_HEAT				6.3
#define Z_LANTHANUM_VAPORIZATION_HEAT			4.0e2
#define Z_LANTHANUM_SPECIFIC_HEAT			195.0
#define Z_LANTHANUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_LANTHANUM_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_LANTHANUM_THERMAL_CONDUCTIVITY		13.0
#define Z_LANTHANUM_THERMAL_EXPANSION			0.0000121
#define Z_LANTHANUM_DENSITY				6146.0
#define Z_LANTHANUM_LIQUID_DENSITY			5940.0
#define Z_LANTHANUM_MOLAR_VOLUME			0.00002260
#define Z_LANTHANUM_BRINELL_HARDNESS			363.0
#define Z_LANTHANUM_MOHS_HARDNESS			2.5
#define Z_LANTHANUM_VICKERS_HARDNESS			491.0
#define Z_LANTHANUM_BULK_MODULUS			28.0
#define Z_LANTHANUM_SHEAR_MODULUS			14.0
#define Z_LANTHANUM_YOUNG_MODULUS			37.0
#define Z_LANTHANUM_POISSON_RATIO			0.28
#define Z_LANTHANUM_REFRACTIVE_INDEX			REAL_MISSING
#define Z_LANTHANUM_SOUND_SPEED				2475.0
#define Z_LANTHANUM_ELECTRICAL_TYPE			Z_ELECTRICAL_TYPE_CONDUCTOR
#define Z_LANTHANUM_ELECTRICAL_CONDUCTIVITY		1.6e6
#define Z_LANTHANUM_ELECTRICAL_RESISTIVITY		6.1e-7
#define Z_LANTHANUM_SUPERCONDUCTING_POINT		4.88
#define Z_LANTHANUM_MAGNETIC_TYPE			Z_MAGNETIC_TYPE_PARAMAGNETIC
#define Z_LANTHANUM_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_LANTHANUM_MASS_MAGNETIC_SUSCEPTIBILITY	1.100e-8
#define Z_LANTHANUM_MOLAR_MAGNETIC_SUSCEPTIBILITY	1.528e-9
#define Z_LANTHANUM_VOLUME_MAGNETIC_SUSCEPTIBILITY	0.00006761

/* MARK: - 58 - Ce */

#define Z_CERIUM_SYMBOL					Ce
#define Z_CERIUM_ATOMIC_NUMBER				58
#define Z_CERIUM_GROUP					INTEGER_NOT_APPLICABLE
#define Z_CERIUM_PERIOD					6
#define Z_CERIUM_BLOCK					Z_CHEMICAL_ELEMENT_BLOCK_F
#define Z_CERIUM_CLASS					Z_CHEMICAL_ELEMENT_CLASS_LANTHANIDE
#define Z_CERIUM_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_CERIUM_SPACE_GROUP_NUMBER			194
#define Z_CERIUM_ATOMIC_MASS				140.116
#define Z_CERIUM_ATOMIC_RADIUS				REAL_MISSING
#define Z_CERIUM_COVALENT_RADIUS			204.0
#define Z_CERIUM_VAN_DER_WAALS_RADIUS			REAL_MISSING
#define Z_CERIUM_DECAY_MODE				KEY_NOT_APPLICABLE
#define Z_CERIUM_NEUTRON_CROSS_SECTION			0.6
#define Z_CERIUM_NEUTRON_MASS_ABSORPTION		0.00021
#define Z_CERIUM_VALENCE				4
#define Z_CERIUM_ELECTRONEGATIVITY			1.12
#define Z_CERIUM_ELECTRON_AFFINITY			50.0
#define Z_CERIUM_MELTING_POINT				798.0
#define Z_CERIUM_BOILING_POINT				3360.0
#define Z_CERIUM_CRITICAL_PRESSURE			REAL_MISSING
#define Z_CERIUM_CRITICAL_TEMPERATURE			REAL_MISSING
#define Z_CERIUM_FUSION_HEAT				5.5
#define Z_CERIUM_VAPORIZATION_HEAT			350.0
#define Z_CERIUM_SPECIFIC_HEAT				192.0
#define Z_CERIUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_CERIUM_NEEL_POINT				12.5
#define Z_CERIUM_THERMAL_CONDUCTIVITY			11.0
#define Z_CERIUM_THERMAL_EXPANSION			6.3e-6
#define Z_CERIUM_DENSITY				6689.0
#define Z_CERIUM_LIQUID_DENSITY				6550.0
#define Z_CERIUM_MOLAR_VOLUME				0.00002095
#define Z_CERIUM_BRINELL_HARDNESS			412.0
#define Z_CERIUM_MOHS_HARDNESS				2.5
#define Z_CERIUM_VICKERS_HARDNESS			270.0
#define Z_CERIUM_BULK_MODULUS				22.0
#define Z_CERIUM_SHEAR_MODULUS				14.0
#define Z_CERIUM_YOUNG_MODULUS				34.0
#define Z_CERIUM_POISSON_RATIO				0.24
#define Z_CERIUM_REFRACTIVE_INDEX			REAL_MISSING
#define Z_CERIUM_SOUND_SPEED				2.1e3
#define Z_CERIUM_ELECTRICAL_TYPE			Z_ELECTRICAL_TYPE_CONDUCTOR
#define Z_CERIUM_ELECTRICAL_CONDUCTIVITY		1.4e6
#define Z_CERIUM_ELECTRICAL_RESISTIVITY			7.5e-7
#define Z_CERIUM_SUPERCONDUCTING_POINT			0.022
#define Z_CERIUM_MAGNETIC_TYPE				Z_MAGNETIC_TYPE_PARAMAGNETIC
#define Z_CERIUM_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_CERIUM_MASS_MAGNETIC_SUSCEPTIBILITY		2.2000e-7
#define Z_CERIUM_MOLAR_MAGNETIC_SUSCEPTIBILITY		3.0826e-8
#define Z_CERIUM_VOLUME_MAGNETIC_SUSCEPTIBILITY		0.0014716

/* MARK: - 59 - Pr */

#define Z_PRASEODYMIUM_SYMBOL				Pr
#define Z_PRASEODYMIUM_ATOMIC_NUMBER			59
#define Z_PRASEODYMIUM_GROUP				INTEGER_NOT_APPLICABLE
#define Z_PRASEODYMIUM_PERIOD				6
#define Z_PRASEODYMIUM_BLOCK				Z_CHEMICAL_ELEMENT_BLOCK_F
#define Z_PRASEODYMIUM_CLASS				Z_CHEMICAL_ELEMENT_CLASS_LANTHANIDE
#define Z_PRASEODYMIUM_METAL_STATE			Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_PRASEODYMIUM_SPACE_GROUP_NUMBER		194
#define Z_PRASEODYMIUM_ATOMIC_MASS			140.90765
#define Z_PRASEODYMIUM_ATOMIC_RADIUS			247.0
#define Z_PRASEODYMIUM_COVALENT_RADIUS			203.0
#define Z_PRASEODYMIUM_VAN_DER_WAALS_RADIUS		REAL_MISSING
#define Z_PRASEODYMIUM_DECAY_MODE			KEY_NOT_APPLICABLE
#define Z_PRASEODYMIUM_NEUTRON_CROSS_SECTION		11.4
#define Z_PRASEODYMIUM_NEUTRON_MASS_ABSORPTION		0.0029
#define Z_PRASEODYMIUM_VALENCE				4
#define Z_PRASEODYMIUM_ELECTRONEGATIVITY		1.13
#define Z_PRASEODYMIUM_ELECTRON_AFFINITY		50.0
#define Z_PRASEODYMIUM_MELTING_POINT			931.0
#define Z_PRASEODYMIUM_BOILING_POINT			3290.0
#define Z_PRASEODYMIUM_CRITICAL_PRESSURE		REAL_MISSING
#define Z_PRASEODYMIUM_CRITICAL_TEMPERATURE		REAL_MISSING
#define Z_PRASEODYMIUM_FUSION_HEAT			6.9
#define Z_PRASEODYMIUM_VAPORIZATION_HEAT		330.0
#define Z_PRASEODYMIUM_SPECIFIC_HEAT			193.0
#define Z_PRASEODYMIUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_PRASEODYMIUM_NEEL_POINT			REAL_NOT_APPLICABLE
#define Z_PRASEODYMIUM_THERMAL_CONDUCTIVITY		13.0
#define Z_PRASEODYMIUM_THERMAL_EXPANSION		6.7e-6
#define Z_PRASEODYMIUM_DENSITY				6640.0
#define Z_PRASEODYMIUM_LIQUID_DENSITY			6500.0
#define Z_PRASEODYMIUM_MOLAR_VOLUME			0.00002122
#define Z_PRASEODYMIUM_BRINELL_HARDNESS			481.0
#define Z_PRASEODYMIUM_MOHS_HARDNESS			1.4
#define Z_PRASEODYMIUM_VICKERS_HARDNESS			4.0e2
#define Z_PRASEODYMIUM_BULK_MODULUS			29.0
#define Z_PRASEODYMIUM_SHEAR_MODULUS			15.0
#define Z_PRASEODYMIUM_YOUNG_MODULUS			37.0
#define Z_PRASEODYMIUM_POISSON_RATIO			0.28
#define Z_PRASEODYMIUM_REFRACTIVE_INDEX			REAL_MISSING
#define Z_PRASEODYMIUM_SOUND_SPEED			2280.0
#define Z_PRASEODYMIUM_ELECTRICAL_TYPE			Z_ELECTRICAL_TYPE_CONDUCTOR
#define Z_PRASEODYMIUM_ELECTRICAL_CONDUCTIVITY		1.4e6
#define Z_PRASEODYMIUM_ELECTRICAL_RESISTIVITY		7.0e-7
#define Z_PRASEODYMIUM_SUPERCONDUCTING_POINT		REAL_MISSING
#define Z_PRASEODYMIUM_MAGNETIC_TYPE			Z_MAGNETIC_TYPE_PARAMAGNETIC
#define Z_PRASEODYMIUM_CURIE_POINT			REAL_NOT_APPLICABLE
#define Z_PRASEODYMIUM_MASS_MAGNETIC_SUSCEPTIBILITY	4.23e-7
#define Z_PRASEODYMIUM_MOLAR_MAGNETIC_SUSCEPTIBILITY	5.9604e-8
#define Z_PRASEODYMIUM_VOLUME_MAGNETIC_SUSCEPTIBILITY	0.0028087

/* MARK: - 60 - Nd */

#define Z_NEODYMIUM_SYMBOL				Nd
#define Z_NEODYMIUM_ATOMIC_NUMBER			60
#define Z_NEODYMIUM_GROUP				INTEGER_NOT_APPLICABLE
#define Z_NEODYMIUM_PERIOD				6
#define Z_NEODYMIUM_BLOCK				Z_CHEMICAL_ELEMENT_BLOCK_F
#define Z_NEODYMIUM_CLASS				Z_CHEMICAL_ELEMENT_CLASS_LANTHANIDE
#define Z_NEODYMIUM_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_NEODYMIUM_SPACE_GROUP_NUMBER			194
#define Z_NEODYMIUM_ATOMIC_MASS				144.242
#define Z_NEODYMIUM_ATOMIC_RADIUS			206.0
#define Z_NEODYMIUM_COVALENT_RADIUS			201.0
#define Z_NEODYMIUM_VAN_DER_WAALS_RADIUS		REAL_MISSING
#define Z_NEODYMIUM_DECAY_MODE				KEY_NOT_APPLICABLE
#define Z_NEODYMIUM_NEUTRON_CROSS_SECTION		49.0
#define Z_NEODYMIUM_NEUTRON_MASS_ABSORPTION		0.011
#define Z_NEODYMIUM_VALENCE				3
#define Z_NEODYMIUM_ELECTRONEGATIVITY			1.14
#define Z_NEODYMIUM_ELECTRON_AFFINITY			50.0
#define Z_NEODYMIUM_MELTING_POINT			1021.0
#define Z_NEODYMIUM_BOILING_POINT			3.1e3
#define Z_NEODYMIUM_CRITICAL_PRESSURE			REAL_MISSING
#define Z_NEODYMIUM_CRITICAL_TEMPERATURE		REAL_MISSING
#define Z_NEODYMIUM_FUSION_HEAT				7.1
#define Z_NEODYMIUM_VAPORIZATION_HEAT			285.0
#define Z_NEODYMIUM_SPECIFIC_HEAT			190.0
#define Z_NEODYMIUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_NEODYMIUM_NEEL_POINT				19.2
#define Z_NEODYMIUM_THERMAL_CONDUCTIVITY		17.0
#define Z_NEODYMIUM_THERMAL_EXPANSION			9.6e-6
#define Z_NEODYMIUM_DENSITY				7010.0
#define Z_NEODYMIUM_LIQUID_DENSITY			6890.0
#define Z_NEODYMIUM_MOLAR_VOLUME			0.00002058
#define Z_NEODYMIUM_BRINELL_HARDNESS			265.0
#define Z_NEODYMIUM_MOHS_HARDNESS			1.2
#define Z_NEODYMIUM_VICKERS_HARDNESS			343.0
#define Z_NEODYMIUM_BULK_MODULUS			32.0
#define Z_NEODYMIUM_SHEAR_MODULUS			16.0
#define Z_NEODYMIUM_YOUNG_MODULUS			41.0
#define Z_NEODYMIUM_POISSON_RATIO			0.28
#define Z_NEODYMIUM_REFRACTIVE_INDEX			REAL_MISSING
#define Z_NEODYMIUM_SOUND_SPEED				2330.0
#define Z_NEODYMIUM_ELECTRICAL_TYPE			Z_ELECTRICAL_TYPE_CONDUCTOR
#define Z_NEODYMIUM_ELECTRICAL_CONDUCTIVITY		1.6e6
#define Z_NEODYMIUM_ELECTRICAL_RESISTIVITY		6.4e-7
#define Z_NEODYMIUM_SUPERCONDUCTING_POINT		REAL_MISSING
#define Z_NEODYMIUM_MAGNETIC_TYPE			Z_MAGNETIC_TYPE_PARAMAGNETIC
#define Z_NEODYMIUM_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_NEODYMIUM_MASS_MAGNETIC_SUSCEPTIBILITY	4.8e-7
#define Z_NEODYMIUM_MOLAR_MAGNETIC_SUSCEPTIBILITY	6.9235e-8
#define Z_NEODYMIUM_VOLUME_MAGNETIC_SUSCEPTIBILITY	0.0033648

/* MARK: - 61 - Pm */

#define Z_PROMETHIUM_SYMBOL				Pm
#define Z_PROMETHIUM_ATOMIC_NUMBER			61
#define Z_PROMETHIUM_GROUP				INTEGER_NOT_APPLICABLE
#define Z_PROMETHIUM_PERIOD				6
#define Z_PROMETHIUM_BLOCK				Z_CHEMICAL_ELEMENT_BLOCK_F
#define Z_PROMETHIUM_CLASS				Z_CHEMICAL_ELEMENT_CLASS_LANTHANIDE
#define Z_PROMETHIUM_METAL_STATE			Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_PROMETHIUM_SPACE_GROUP_NUMBER			INTEGER_MISSING
#define Z_PROMETHIUM_ATOMIC_MASS			145.0
#define Z_PROMETHIUM_ATOMIC_RADIUS			205.0
#define Z_PROMETHIUM_COVALENT_RADIUS			199.0
#define Z_PROMETHIUM_VAN_DER_WAALS_RADIUS		REAL_MISSING
#define Z_PROMETHIUM_DECAY_MODE				Z_CHEMICAL_ELEMENT_DECAY_MODE_ELECTRON_CAPTURE
#define Z_PROMETHIUM_NEUTRON_CROSS_SECTION		8.4e3
#define Z_PROMETHIUM_NEUTRON_MASS_ABSORPTION		REAL_MISSING
#define Z_PROMETHIUM_VALENCE				3
#define Z_PROMETHIUM_ELECTRONEGATIVITY			REAL_MISSING
#define Z_PROMETHIUM_ELECTRON_AFFINITY			50.0
#define Z_PROMETHIUM_MELTING_POINT			1.1e3
#define Z_PROMETHIUM_BOILING_POINT			3.0e3
#define Z_PROMETHIUM_CRITICAL_PRESSURE			REAL_MISSING
#define Z_PROMETHIUM_CRITICAL_TEMPERATURE		REAL_MISSING
#define Z_PROMETHIUM_FUSION_HEAT			7.7
#define Z_PROMETHIUM_VAPORIZATION_HEAT			290.0
#define Z_PROMETHIUM_SPECIFIC_HEAT			REAL_MISSING
#define Z_PROMETHIUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_PROMETHIUM_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_PROMETHIUM_THERMAL_CONDUCTIVITY		15.0
#define Z_PROMETHIUM_THERMAL_EXPANSION			0.000011
#define Z_PROMETHIUM_DENSITY				7264.0
#define Z_PROMETHIUM_LIQUID_DENSITY			REAL_MISSING
#define Z_PROMETHIUM_MOLAR_VOLUME			0.00001996
#define Z_PROMETHIUM_BRINELL_HARDNESS			REAL_MISSING
#define Z_PROMETHIUM_MOHS_HARDNESS			REAL_MISSING
#define Z_PROMETHIUM_VICKERS_HARDNESS			REAL_MISSING
#define Z_PROMETHIUM_BULK_MODULUS			33.0
#define Z_PROMETHIUM_SHEAR_MODULUS			18.0
#define Z_PROMETHIUM_YOUNG_MODULUS			46.0
#define Z_PROMETHIUM_POISSON_RATIO			0.28
#define Z_PROMETHIUM_REFRACTIVE_INDEX			REAL_MISSING
#define Z_PROMETHIUM_SOUND_SPEED			REAL_MISSING
#define Z_PROMETHIUM_ELECTRICAL_TYPE			Z_ELECTRICAL_TYPE_CONDUCTOR
#define Z_PROMETHIUM_ELECTRICAL_CONDUCTIVITY		1.3e6
#define Z_PROMETHIUM_ELECTRICAL_RESISTIVITY		7.5e-7
#define Z_PROMETHIUM_SUPERCONDUCTING_POINT		REAL_MISSING
#define Z_PROMETHIUM_MAGNETIC_TYPE			KEY_MISSING
#define Z_PROMETHIUM_CURIE_POINT			REAL_NOT_APPLICABLE
#define Z_PROMETHIUM_MASS_MAGNETIC_SUSCEPTIBILITY	REAL_MISSING
#define Z_PROMETHIUM_MOLAR_MAGNETIC_SUSCEPTIBILITY	REAL_MISSING
#define Z_PROMETHIUM_VOLUME_MAGNETIC_SUSCEPTIBILITY	REAL_MISSING

/* MARK: - 62 - Sm */

#define Z_SAMARIUM_SYMBOL				Sm
#define Z_SAMARIUM_ATOMIC_NUMBER			62
#define Z_SAMARIUM_GROUP				INTEGER_NOT_APPLICABLE
#define Z_SAMARIUM_PERIOD				6
#define Z_SAMARIUM_BLOCK				Z_CHEMICAL_ELEMENT_BLOCK_F
#define Z_SAMARIUM_CLASS				Z_CHEMICAL_ELEMENT_CLASS_LANTHANIDE
#define Z_SAMARIUM_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_SAMARIUM_SPACE_GROUP_NUMBER			166
#define Z_SAMARIUM_ATOMIC_MASS				150.36
#define Z_SAMARIUM_ATOMIC_RADIUS			238.0
#define Z_SAMARIUM_COVALENT_RADIUS			198.0
#define Z_SAMARIUM_VAN_DER_WAALS_RADIUS			REAL_MISSING
#define Z_SAMARIUM_DECAY_MODE				KEY_NOT_APPLICABLE
#define Z_SAMARIUM_NEUTRON_CROSS_SECTION		5.9e3
#define Z_SAMARIUM_NEUTRON_MASS_ABSORPTION		4.7
#define Z_SAMARIUM_VALENCE				3
#define Z_SAMARIUM_ELECTRONEGATIVITY			1.17
#define Z_SAMARIUM_ELECTRON_AFFINITY			50.0
#define Z_SAMARIUM_MELTING_POINT			1072.0
#define Z_SAMARIUM_BOILING_POINT			1803.0
#define Z_SAMARIUM_CRITICAL_PRESSURE			REAL_MISSING
#define Z_SAMARIUM_CRITICAL_TEMPERATURE			REAL_MISSING
#define Z_SAMARIUM_FUSION_HEAT				8.5
#define Z_SAMARIUM_VAPORIZATION_HEAT			175.0
#define Z_SAMARIUM_SPECIFIC_HEAT			196.0
#define Z_SAMARIUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_SAMARIUM_NEEL_POINT				106.0
#define Z_SAMARIUM_THERMAL_CONDUCTIVITY			13.0
#define Z_SAMARIUM_THERMAL_EXPANSION			0.0000127
#define Z_SAMARIUM_DENSITY				7353.0
#define Z_SAMARIUM_LIQUID_DENSITY			7160.0
#define Z_SAMARIUM_MOLAR_VOLUME				0.00002045
#define Z_SAMARIUM_BRINELL_HARDNESS			441.0
#define Z_SAMARIUM_MOHS_HARDNESS			1.4
#define Z_SAMARIUM_VICKERS_HARDNESS			412.0
#define Z_SAMARIUM_BULK_MODULUS				38.0
#define Z_SAMARIUM_SHEAR_MODULUS			20.0
#define Z_SAMARIUM_YOUNG_MODULUS			50.0
#define Z_SAMARIUM_POISSON_RATIO			0.27
#define Z_SAMARIUM_REFRACTIVE_INDEX			REAL_MISSING
#define Z_SAMARIUM_SOUND_SPEED				2130.0
#define Z_SAMARIUM_ELECTRICAL_TYPE			Z_ELECTRICAL_TYPE_CONDUCTOR
#define Z_SAMARIUM_ELECTRICAL_CONDUCTIVITY		1.1e6
#define Z_SAMARIUM_ELECTRICAL_RESISTIVITY		9.4e-7
#define Z_SAMARIUM_SUPERCONDUCTING_POINT		REAL_MISSING
#define Z_SAMARIUM_MAGNETIC_TYPE			Z_MAGNETIC_TYPE_PARAMAGNETIC
#define Z_SAMARIUM_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_SAMARIUM_MASS_MAGNETIC_SUSCEPTIBILITY		1.11e-7
#define Z_SAMARIUM_MOLAR_MAGNETIC_SUSCEPTIBILITY	1.669e-8
#define Z_SAMARIUM_VOLUME_MAGNETIC_SUSCEPTIBILITY	0.00081618

/* MARK: - 63 - Eu */

#define Z_EUROPIUM_SYMBOL				Eu
#define Z_EUROPIUM_ATOMIC_NUMBER			63
#define Z_EUROPIUM_GROUP				INTEGER_NOT_APPLICABLE
#define Z_EUROPIUM_PERIOD				6
#define Z_EUROPIUM_BLOCK				Z_CHEMICAL_ELEMENT_BLOCK_F
#define Z_EUROPIUM_CLASS				Z_CHEMICAL_ELEMENT_CLASS_LANTHANIDE
#define Z_EUROPIUM_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_EUROPIUM_SPACE_GROUP_NUMBER			229
#define Z_EUROPIUM_ATOMIC_MASS				151.964
#define Z_EUROPIUM_ATOMIC_RADIUS			231.0
#define Z_EUROPIUM_COVALENT_RADIUS			198.0
#define Z_EUROPIUM_VAN_DER_WAALS_RADIUS			REAL_MISSING
#define Z_EUROPIUM_DECAY_MODE				KEY_NOT_APPLICABLE
#define Z_EUROPIUM_NEUTRON_CROSS_SECTION		4.57e3
#define Z_EUROPIUM_NEUTRON_MASS_ABSORPTION		0.6
#define Z_EUROPIUM_VALENCE				3
#define Z_EUROPIUM_ELECTRONEGATIVITY			REAL_MISSING
#define Z_EUROPIUM_ELECTRON_AFFINITY			50.0
#define Z_EUROPIUM_MELTING_POINT			822.0
#define Z_EUROPIUM_BOILING_POINT			1527.0
#define Z_EUROPIUM_CRITICAL_PRESSURE			REAL_MISSING
#define Z_EUROPIUM_CRITICAL_TEMPERATURE			REAL_MISSING
#define Z_EUROPIUM_FUSION_HEAT				9.3
#define Z_EUROPIUM_VAPORIZATION_HEAT			175.0
#define Z_EUROPIUM_SPECIFIC_HEAT			182.0
#define Z_EUROPIUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_EUROPIUM_NEEL_POINT				90.5
#define Z_EUROPIUM_THERMAL_CONDUCTIVITY			14.0
#define Z_EUROPIUM_THERMAL_EXPANSION			0.000035
#define Z_EUROPIUM_DENSITY				5244.0
#define Z_EUROPIUM_LIQUID_DENSITY			5130.0
#define Z_EUROPIUM_MOLAR_VOLUME				0.00002898
#define Z_EUROPIUM_BRINELL_HARDNESS			REAL_MISSING
#define Z_EUROPIUM_MOHS_HARDNESS			3.1
#define Z_EUROPIUM_VICKERS_HARDNESS			167.0
#define Z_EUROPIUM_BULK_MODULUS				8.3
#define Z_EUROPIUM_SHEAR_MODULUS			7.9
#define Z_EUROPIUM_YOUNG_MODULUS			18.0
#define Z_EUROPIUM_POISSON_RATIO			0.15
#define Z_EUROPIUM_REFRACTIVE_INDEX			REAL_MISSING
#define Z_EUROPIUM_SOUND_SPEED				REAL_MISSING
#define Z_EUROPIUM_ELECTRICAL_TYPE			Z_ELECTRICAL_TYPE_CONDUCTOR
#define Z_EUROPIUM_ELECTRICAL_CONDUCTIVITY		1.1e6
#define Z_EUROPIUM_ELECTRICAL_RESISTIVITY		9.0e-7
#define Z_EUROPIUM_SUPERCONDUCTING_POINT		1.8
#define Z_EUROPIUM_MAGNETIC_TYPE			Z_MAGNETIC_TYPE_PARAMAGNETIC
#define Z_EUROPIUM_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_EUROPIUM_MASS_MAGNETIC_SUSCEPTIBILITY		2.76e-7
#define Z_EUROPIUM_MOLAR_MAGNETIC_SUSCEPTIBILITY	4.1942e-8
#define Z_EUROPIUM_VOLUME_MAGNETIC_SUSCEPTIBILITY	0.0014473

/* MARK: - 64 - Gd */

#define Z_GADOLINIUM_SYMBOL				Gd
#define Z_GADOLINIUM_ATOMIC_NUMBER			64
#define Z_GADOLINIUM_GROUP				INTEGER_NOT_APPLICABLE
#define Z_GADOLINIUM_PERIOD				6
#define Z_GADOLINIUM_BLOCK				Z_CHEMICAL_ELEMENT_BLOCK_F
#define Z_GADOLINIUM_CLASS				Z_CHEMICAL_ELEMENT_CLASS_LANTHANIDE
#define Z_GADOLINIUM_METAL_STATE			Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_GADOLINIUM_SPACE_GROUP_NUMBER			194
#define Z_GADOLINIUM_ATOMIC_MASS			157.25
#define Z_GADOLINIUM_ATOMIC_RADIUS			233.0
#define Z_GADOLINIUM_COVALENT_RADIUS			196.0
#define Z_GADOLINIUM_VAN_DER_WAALS_RADIUS		REAL_MISSING
#define Z_GADOLINIUM_DECAY_MODE				KEY_NOT_APPLICABLE
#define Z_GADOLINIUM_NEUTRON_CROSS_SECTION		4.9e4
#define Z_GADOLINIUM_NEUTRON_MASS_ABSORPTION		7.3
#define Z_GADOLINIUM_VALENCE				3
#define Z_GADOLINIUM_ELECTRONEGATIVITY			1.20
#define Z_GADOLINIUM_ELECTRON_AFFINITY			50.0
#define Z_GADOLINIUM_MELTING_POINT			1313.0
#define Z_GADOLINIUM_BOILING_POINT			3250.0
#define Z_GADOLINIUM_CRITICAL_PRESSURE			REAL_MISSING
#define Z_GADOLINIUM_CRITICAL_TEMPERATURE		REAL_MISSING
#define Z_GADOLINIUM_FUSION_HEAT			10.0
#define Z_GADOLINIUM_VAPORIZATION_HEAT			305.0
#define Z_GADOLINIUM_SPECIFIC_HEAT			240.0
#define Z_GADOLINIUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_GADOLINIUM_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_GADOLINIUM_THERMAL_CONDUCTIVITY		11.0
#define Z_GADOLINIUM_THERMAL_EXPANSION			9.4e-6
#define Z_GADOLINIUM_DENSITY				7901.0
#define Z_GADOLINIUM_LIQUID_DENSITY			7400.0
#define Z_GADOLINIUM_MOLAR_VOLUME			0.0000199
#define Z_GADOLINIUM_BRINELL_HARDNESS			REAL_MISSING
#define Z_GADOLINIUM_MOHS_HARDNESS			5.1
#define Z_GADOLINIUM_VICKERS_HARDNESS			570.0
#define Z_GADOLINIUM_BULK_MODULUS			38.0
#define Z_GADOLINIUM_SHEAR_MODULUS			22.0
#define Z_GADOLINIUM_YOUNG_MODULUS			55.0
#define Z_GADOLINIUM_POISSON_RATIO			0.26
#define Z_GADOLINIUM_REFRACTIVE_INDEX			REAL_MISSING
#define Z_GADOLINIUM_SOUND_SPEED			2680.0
#define Z_GADOLINIUM_ELECTRICAL_TYPE			Z_ELECTRICAL_TYPE_CONDUCTOR
#define Z_GADOLINIUM_ELECTRICAL_CONDUCTIVITY		7.7e5
#define Z_GADOLINIUM_ELECTRICAL_RESISTIVITY		1.3e-6
#define Z_GADOLINIUM_SUPERCONDUCTING_POINT		REAL_MISSING
#define Z_GADOLINIUM_MAGNETIC_TYPE			Z_MAGNETIC_TYPE_FERROMAGNETIC
#define Z_GADOLINIUM_CURIE_POINT			292.0
#define Z_GADOLINIUM_MASS_MAGNETIC_SUSCEPTIBILITY	REAL_MISSING
#define Z_GADOLINIUM_MOLAR_MAGNETIC_SUSCEPTIBILITY	REAL_MISSING
#define Z_GADOLINIUM_VOLUME_MAGNETIC_SUSCEPTIBILITY	REAL_MISSING

/* MARK: - 65 - Tb */

#define Z_TERBIUM_SYMBOL				Tb
#define Z_TERBIUM_ATOMIC_NUMBER				65
#define Z_TERBIUM_GROUP					INTEGER_NOT_APPLICABLE
#define Z_TERBIUM_PERIOD				6
#define Z_TERBIUM_BLOCK					Z_CHEMICAL_ELEMENT_BLOCK_F
#define Z_TERBIUM_CLASS					Z_CHEMICAL_ELEMENT_CLASS_LANTHANIDE
#define Z_TERBIUM_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_TERBIUM_SPACE_GROUP_NUMBER			194
#define Z_TERBIUM_ATOMIC_MASS				158.92535
#define Z_TERBIUM_ATOMIC_RADIUS				225.0
#define Z_TERBIUM_COVALENT_RADIUS			194.0
#define Z_TERBIUM_VAN_DER_WAALS_RADIUS			REAL_MISSING
#define Z_TERBIUM_DECAY_MODE				KEY_NOT_APPLICABLE
#define Z_TERBIUM_NEUTRON_CROSS_SECTION			23.0
#define Z_TERBIUM_NEUTRON_MASS_ABSORPTION		0.009
#define Z_TERBIUM_VALENCE				3
#define Z_TERBIUM_ELECTRONEGATIVITY			REAL_MISSING
#define Z_TERBIUM_ELECTRON_AFFINITY			50.0
#define Z_TERBIUM_MELTING_POINT				1356.0
#define Z_TERBIUM_BOILING_POINT				3230.0
#define Z_TERBIUM_CRITICAL_PRESSURE			REAL_MISSING
#define Z_TERBIUM_CRITICAL_TEMPERATURE			REAL_MISSING
#define Z_TERBIUM_FUSION_HEAT				10.8
#define Z_TERBIUM_VAPORIZATION_HEAT			295.0
#define Z_TERBIUM_SPECIFIC_HEAT				182.0
#define Z_TERBIUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_TERBIUM_NEEL_POINT				230.0
#define Z_TERBIUM_THERMAL_CONDUCTIVITY			11.0
#define Z_TERBIUM_THERMAL_EXPANSION			0.0000103
#define Z_TERBIUM_DENSITY				8219.0
#define Z_TERBIUM_LIQUID_DENSITY			7650.0
#define Z_TERBIUM_MOLAR_VOLUME				0.00001934
#define Z_TERBIUM_BRINELL_HARDNESS			677.0
#define Z_TERBIUM_MOHS_HARDNESS				2.3
#define Z_TERBIUM_VICKERS_HARDNESS			863.0
#define Z_TERBIUM_BULK_MODULUS				38.7
#define Z_TERBIUM_SHEAR_MODULUS				22.0
#define Z_TERBIUM_YOUNG_MODULUS				56.0
#define Z_TERBIUM_POISSON_RATIO				0.26
#define Z_TERBIUM_REFRACTIVE_INDEX			REAL_MISSING
#define Z_TERBIUM_SOUND_SPEED				2620.0
#define Z_TERBIUM_ELECTRICAL_TYPE			Z_ELECTRICAL_TYPE_CONDUCTOR
#define Z_TERBIUM_ELECTRICAL_CONDUCTIVITY		8.3e5
#define Z_TERBIUM_ELECTRICAL_RESISTIVITY		1.2e-6
#define Z_TERBIUM_SUPERCONDUCTING_POINT			REAL_MISSING
#define Z_TERBIUM_MAGNETIC_TYPE				Z_MAGNETIC_TYPE_PARAMAGNETIC
#define Z_TERBIUM_CURIE_POINT				222.0
#define Z_TERBIUM_MASS_MAGNETIC_SUSCEPTIBILITY		0.0000136
#define Z_TERBIUM_MOLAR_MAGNETIC_SUSCEPTIBILITY		2.161385e-6
#define Z_TERBIUM_VOLUME_MAGNETIC_SUSCEPTIBILITY	0.1117784

/* MARK: - 66 - Dy */

#define Z_DYSPROSIUM_SYMBOL				Dy
#define Z_DYSPROSIUM_ATOMIC_NUMBER			66
#define Z_DYSPROSIUM_GROUP				INTEGER_NOT_APPLICABLE
#define Z_DYSPROSIUM_PERIOD				6
#define Z_DYSPROSIUM_BLOCK				Z_CHEMICAL_ELEMENT_BLOCK_F
#define Z_DYSPROSIUM_CLASS				Z_CHEMICAL_ELEMENT_CLASS_LANTHANIDE
#define Z_DYSPROSIUM_METAL_STATE			Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_DYSPROSIUM_SPACE_GROUP_NUMBER			194
#define Z_DYSPROSIUM_ATOMIC_MASS			162.5
#define Z_DYSPROSIUM_ATOMIC_RADIUS			228.0
#define Z_DYSPROSIUM_COVALENT_RADIUS			192.0
#define Z_DYSPROSIUM_VAN_DER_WAALS_RADIUS		REAL_MISSING
#define Z_DYSPROSIUM_DECAY_MODE				KEY_NOT_APPLICABLE
#define Z_DYSPROSIUM_NEUTRON_CROSS_SECTION		9.5e2
#define Z_DYSPROSIUM_NEUTRON_MASS_ABSORPTION		0.2
#define Z_DYSPROSIUM_VALENCE				3
#define Z_DYSPROSIUM_ELECTRONEGATIVITY			1.22
#define Z_DYSPROSIUM_ELECTRON_AFFINITY			50.0
#define Z_DYSPROSIUM_MELTING_POINT			1412.0
#define Z_DYSPROSIUM_BOILING_POINT			2567.0
#define Z_DYSPROSIUM_CRITICAL_PRESSURE			REAL_MISSING
#define Z_DYSPROSIUM_CRITICAL_TEMPERATURE		REAL_MISSING
#define Z_DYSPROSIUM_FUSION_HEAT			11.1
#define Z_DYSPROSIUM_VAPORIZATION_HEAT			280.0
#define Z_DYSPROSIUM_SPECIFIC_HEAT			167.0
#define Z_DYSPROSIUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_DYSPROSIUM_NEEL_POINT				178.0
#define Z_DYSPROSIUM_THERMAL_CONDUCTIVITY		11.0
#define Z_DYSPROSIUM_THERMAL_EXPANSION			1.0e-5
#define Z_DYSPROSIUM_DENSITY				8551.0
#define Z_DYSPROSIUM_LIQUID_DENSITY			8370.0
#define Z_DYSPROSIUM_MOLAR_VOLUME			0.000019
#define Z_DYSPROSIUM_BRINELL_HARDNESS			5.0e2
#define Z_DYSPROSIUM_MOHS_HARDNESS			1.8
#define Z_DYSPROSIUM_VICKERS_HARDNESS			540.0
#define Z_DYSPROSIUM_BULK_MODULUS			41.0
#define Z_DYSPROSIUM_SHEAR_MODULUS			25.0
#define Z_DYSPROSIUM_YOUNG_MODULUS			61.0
#define Z_DYSPROSIUM_POISSON_RATIO			0.25
#define Z_DYSPROSIUM_REFRACTIVE_INDEX			REAL_MISSING
#define Z_DYSPROSIUM_SOUND_SPEED			2710.0
#define Z_DYSPROSIUM_ELECTRICAL_TYPE			Z_ELECTRICAL_TYPE_CONDUCTOR
#define Z_DYSPROSIUM_ELECTRICAL_CONDUCTIVITY		1.1e6
#define Z_DYSPROSIUM_ELECTRICAL_RESISTIVITY		9.1e-7
#define Z_DYSPROSIUM_SUPERCONDUCTING_POINT		REAL_MISSING
#define Z_DYSPROSIUM_MAGNETIC_TYPE			Z_MAGNETIC_TYPE_PARAMAGNETIC
#define Z_DYSPROSIUM_CURIE_POINT			87.0
#define Z_DYSPROSIUM_MASS_MAGNETIC_SUSCEPTIBILITY	5.45e-6
#define Z_DYSPROSIUM_MOLAR_MAGNETIC_SUSCEPTIBILITY	8.85625e-7
#define Z_DYSPROSIUM_VOLUME_MAGNETIC_SUSCEPTIBILITY	0.046603

/* MARK: - 67 - Ho */

#define Z_HOLMIUM_SYMBOL				Ho
#define Z_HOLMIUM_ATOMIC_NUMBER				67
#define Z_HOLMIUM_GROUP					INTEGER_NOT_APPLICABLE
#define Z_HOLMIUM_PERIOD				6
#define Z_HOLMIUM_BLOCK					Z_CHEMICAL_ELEMENT_BLOCK_F
#define Z_HOLMIUM_CLASS					Z_CHEMICAL_ELEMENT_CLASS_LANTHANIDE
#define Z_HOLMIUM_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_HOLMIUM_SPACE_GROUP_NUMBER			194
#define Z_HOLMIUM_ATOMIC_MASS				164.93032
#define Z_HOLMIUM_ATOMIC_RADIUS				226.0
#define Z_HOLMIUM_COVALENT_RADIUS			192.0
#define Z_HOLMIUM_VAN_DER_WAALS_RADIUS			REAL_MISSING
#define Z_HOLMIUM_DECAY_MODE				KEY_NOT_APPLICABLE
#define Z_HOLMIUM_NEUTRON_CROSS_SECTION			64.0
#define Z_HOLMIUM_NEUTRON_MASS_ABSORPTION		0.015
#define Z_HOLMIUM_VALENCE				3
#define Z_HOLMIUM_ELECTRONEGATIVITY			1.23
#define Z_HOLMIUM_ELECTRON_AFFINITY			50.0
#define Z_HOLMIUM_MELTING_POINT				1474.0
#define Z_HOLMIUM_BOILING_POINT				2700.0
#define Z_HOLMIUM_CRITICAL_PRESSURE			REAL_MISSING
#define Z_HOLMIUM_CRITICAL_TEMPERATURE			REAL_MISSING
#define Z_HOLMIUM_FUSION_HEAT				17.0
#define Z_HOLMIUM_VAPORIZATION_HEAT			265.0
#define Z_HOLMIUM_SPECIFIC_HEAT				165.0
#define Z_HOLMIUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_HOLMIUM_NEEL_POINT				132.0
#define Z_HOLMIUM_THERMAL_CONDUCTIVITY			16.0
#define Z_HOLMIUM_THERMAL_EXPANSION			0.0000112
#define Z_HOLMIUM_DENSITY				8795.0
#define Z_HOLMIUM_LIQUID_DENSITY			8340.0
#define Z_HOLMIUM_MOLAR_VOLUME				0.00001875
#define Z_HOLMIUM_BRINELL_HARDNESS			746.0
#define Z_HOLMIUM_MOHS_HARDNESS				1.6
#define Z_HOLMIUM_VICKERS_HARDNESS			481.0
#define Z_HOLMIUM_BULK_MODULUS				40.0
#define Z_HOLMIUM_SHEAR_MODULUS				26.0
#define Z_HOLMIUM_YOUNG_MODULUS				64.0
#define Z_HOLMIUM_POISSON_RATIO				0.23
#define Z_HOLMIUM_REFRACTIVE_INDEX			REAL_MISSING
#define Z_HOLMIUM_SOUND_SPEED				2760.0
#define Z_HOLMIUM_ELECTRICAL_TYPE			Z_ELECTRICAL_TYPE_CONDUCTOR
#define Z_HOLMIUM_ELECTRICAL_CONDUCTIVITY		1.1e6
#define Z_HOLMIUM_ELECTRICAL_RESISTIVITY		9.4e-7
#define Z_HOLMIUM_SUPERCONDUCTING_POINT			REAL_MISSING
#define Z_HOLMIUM_MAGNETIC_TYPE				Z_MAGNETIC_TYPE_PARAMAGNETIC
#define Z_HOLMIUM_CURIE_POINT				20.0
#define Z_HOLMIUM_MASS_MAGNETIC_SUSCEPTIBILITY		5.49e-6
#define Z_HOLMIUM_MOLAR_MAGNETIC_SUSCEPTIBILITY		9.05467e-7
#define Z_HOLMIUM_VOLUME_MAGNETIC_SUSCEPTIBILITY	0.0482845

/* MARK: - 68 - Er */

#define Z_ERBIUM_SYMBOL					Er
#define Z_ERBIUM_ATOMIC_NUMBER				68
#define Z_ERBIUM_GROUP					INTEGER_NOT_APPLICABLE
#define Z_ERBIUM_PERIOD					6
#define Z_ERBIUM_BLOCK					Z_CHEMICAL_ELEMENT_BLOCK_F
#define Z_ERBIUM_CLASS					Z_CHEMICAL_ELEMENT_CLASS_LANTHANIDE
#define Z_ERBIUM_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_ERBIUM_SPACE_GROUP_NUMBER			194
#define Z_ERBIUM_ATOMIC_MASS				167.259
#define Z_ERBIUM_ATOMIC_RADIUS				226.0
#define Z_ERBIUM_COVALENT_RADIUS			189.0
#define Z_ERBIUM_VAN_DER_WAALS_RADIUS			REAL_MISSING
#define Z_ERBIUM_DECAY_MODE				KEY_NOT_APPLICABLE
#define Z_ERBIUM_NEUTRON_CROSS_SECTION			165.0
#define Z_ERBIUM_NEUTRON_MASS_ABSORPTION		0.036
#define Z_ERBIUM_VALENCE				3
#define Z_ERBIUM_ELECTRONEGATIVITY			1.24
#define Z_ERBIUM_ELECTRON_AFFINITY			50.0
#define Z_ERBIUM_MELTING_POINT				1497.0
#define Z_ERBIUM_BOILING_POINT				2868.0
#define Z_ERBIUM_CRITICAL_PRESSURE			REAL_MISSING
#define Z_ERBIUM_CRITICAL_TEMPERATURE			REAL_MISSING
#define Z_ERBIUM_FUSION_HEAT				19.9
#define Z_ERBIUM_VAPORIZATION_HEAT			285.0
#define Z_ERBIUM_SPECIFIC_HEAT				168.0
#define Z_ERBIUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_ERBIUM_NEEL_POINT				82.0
#define Z_ERBIUM_THERMAL_CONDUCTIVITY			15.0
#define Z_ERBIUM_THERMAL_EXPANSION			0.0000122
#define Z_ERBIUM_DENSITY				9066.0
#define Z_ERBIUM_LIQUID_DENSITY				8860.0
#define Z_ERBIUM_MOLAR_VOLUME				0.00001845
#define Z_ERBIUM_BRINELL_HARDNESS			814.0
#define Z_ERBIUM_MOHS_HARDNESS				2.0
#define Z_ERBIUM_VICKERS_HARDNESS			588.0
#define Z_ERBIUM_BULK_MODULUS				44.0
#define Z_ERBIUM_SHEAR_MODULUS				28.0
#define Z_ERBIUM_YOUNG_MODULUS				70.0
#define Z_ERBIUM_POISSON_RATIO				0.24
#define Z_ERBIUM_REFRACTIVE_INDEX			REAL_MISSING
#define Z_ERBIUM_SOUND_SPEED				2830.0
#define Z_ERBIUM_ELECTRICAL_TYPE			Z_ELECTRICAL_TYPE_CONDUCTOR
#define Z_ERBIUM_ELECTRICAL_CONDUCTIVITY		1.2e6
#define Z_ERBIUM_ELECTRICAL_RESISTIVITY			8.6e-7
#define Z_ERBIUM_SUPERCONDUCTING_POINT			REAL_MISSING
#define Z_ERBIUM_MAGNETIC_TYPE				Z_MAGNETIC_TYPE_PARAMAGNETIC
#define Z_ERBIUM_CURIE_POINT				32.0
#define Z_ERBIUM_MASS_MAGNETIC_SUSCEPTIBILITY		3.77e-6
#define Z_ERBIUM_MOLAR_MAGNETIC_SUSCEPTIBILITY		6.30566e-7
#define Z_ERBIUM_VOLUME_MAGNETIC_SUSCEPTIBILITY		0.0341788

/* MARK: - 69 - Tm */

#define Z_THULIUM_SYMBOL				Tm
#define Z_THULIUM_ATOMIC_NUMBER				69
#define Z_THULIUM_GROUP					INTEGER_NOT_APPLICABLE
#define Z_THULIUM_PERIOD				6
#define Z_THULIUM_BLOCK					Z_CHEMICAL_ELEMENT_BLOCK_F
#define Z_THULIUM_CLASS					Z_CHEMICAL_ELEMENT_CLASS_LANTHANIDE
#define Z_THULIUM_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_THULIUM_SPACE_GROUP_NUMBER			194
#define Z_THULIUM_ATOMIC_MASS				168.93421
#define Z_THULIUM_ATOMIC_RADIUS				222.0
#define Z_THULIUM_COVALENT_RADIUS			190.0
#define Z_THULIUM_VAN_DER_WAALS_RADIUS			REAL_MISSING
#define Z_THULIUM_DECAY_MODE				KEY_NOT_APPLICABLE
#define Z_THULIUM_NEUTRON_CROSS_SECTION			105.0
#define Z_THULIUM_NEUTRON_MASS_ABSORPTION		0.025
#define Z_THULIUM_VALENCE				3
#define Z_THULIUM_ELECTRONEGATIVITY			1.25
#define Z_THULIUM_ELECTRON_AFFINITY			50.0
#define Z_THULIUM_MELTING_POINT				1545.0
#define Z_THULIUM_BOILING_POINT				1950.0
#define Z_THULIUM_CRITICAL_PRESSURE			REAL_MISSING
#define Z_THULIUM_CRITICAL_TEMPERATURE			REAL_MISSING
#define Z_THULIUM_FUSION_HEAT				16.8
#define Z_THULIUM_VAPORIZATION_HEAT			250.0
#define Z_THULIUM_SPECIFIC_HEAT				160.0
#define Z_THULIUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_THULIUM_NEEL_POINT				56.0
#define Z_THULIUM_THERMAL_CONDUCTIVITY			17.0
#define Z_THULIUM_THERMAL_EXPANSION			0.0000133
#define Z_THULIUM_DENSITY				9320.0
#define Z_THULIUM_LIQUID_DENSITY			8560.0
#define Z_THULIUM_MOLAR_VOLUME				0.00001813
#define Z_THULIUM_BRINELL_HARDNESS			471.0
#define Z_THULIUM_MOHS_HARDNESS				1.8
#define Z_THULIUM_VICKERS_HARDNESS			520.0
#define Z_THULIUM_BULK_MODULUS				45.0
#define Z_THULIUM_SHEAR_MODULUS				31.0
#define Z_THULIUM_YOUNG_MODULUS				74.0
#define Z_THULIUM_POISSON_RATIO				0.21
#define Z_THULIUM_REFRACTIVE_INDEX			REAL_MISSING
#define Z_THULIUM_SOUND_SPEED				REAL_MISSING
#define Z_THULIUM_ELECTRICAL_TYPE			Z_ELECTRICAL_TYPE_CONDUCTOR
#define Z_THULIUM_ELECTRICAL_CONDUCTIVITY		1.4e6
#define Z_THULIUM_ELECTRICAL_RESISTIVITY		7.0e-7
#define Z_THULIUM_SUPERCONDUCTING_POINT			REAL_MISSING
#define Z_THULIUM_MAGNETIC_TYPE				Z_MAGNETIC_TYPE_PARAMAGNETIC
#define Z_THULIUM_CURIE_POINT				25.0
#define Z_THULIUM_MASS_MAGNETIC_SUSCEPTIBILITY		1.99e-6
#define Z_THULIUM_MOLAR_MAGNETIC_SUSCEPTIBILITY		3.36179e-7
#define Z_THULIUM_VOLUME_MAGNETIC_SUSCEPTIBILITY	0.0185488

/* MARK: - 70 - Yb */

#define Z_YTTERBIUM_SYMBOL				Yb
#define Z_YTTERBIUM_ATOMIC_NUMBER			70
#define Z_YTTERBIUM_GROUP				INTEGER_NOT_APPLICABLE
#define Z_YTTERBIUM_PERIOD				6
#define Z_YTTERBIUM_BLOCK				Z_CHEMICAL_ELEMENT_BLOCK_F
#define Z_YTTERBIUM_CLASS				Z_CHEMICAL_ELEMENT_CLASS_LANTHANIDE
#define Z_YTTERBIUM_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_YTTERBIUM_SPACE_GROUP_NUMBER			225
#define Z_YTTERBIUM_ATOMIC_MASS				173.054
#define Z_YTTERBIUM_ATOMIC_RADIUS			222.0
#define Z_YTTERBIUM_COVALENT_RADIUS			187.0
#define Z_YTTERBIUM_VAN_DER_WAALS_RADIUS		REAL_MISSING
#define Z_YTTERBIUM_DECAY_MODE				KEY_NOT_APPLICABLE
#define Z_YTTERBIUM_NEUTRON_CROSS_SECTION		35.0
#define Z_YTTERBIUM_NEUTRON_MASS_ABSORPTION		0.0076
#define Z_YTTERBIUM_VALENCE				3
#define Z_YTTERBIUM_ELECTRONEGATIVITY			REAL_MISSING
#define Z_YTTERBIUM_ELECTRON_AFFINITY			50.0
#define Z_YTTERBIUM_MELTING_POINT			819.0
#define Z_YTTERBIUM_BOILING_POINT			1196.0
#define Z_YTTERBIUM_CRITICAL_PRESSURE			REAL_MISSING
#define Z_YTTERBIUM_CRITICAL_TEMPERATURE		REAL_MISSING
#define Z_YTTERBIUM_FUSION_HEAT				7.7
#define Z_YTTERBIUM_VAPORIZATION_HEAT			160.0
#define Z_YTTERBIUM_SPECIFIC_HEAT			154.0
#define Z_YTTERBIUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_YTTERBIUM_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_YTTERBIUM_THERMAL_CONDUCTIVITY		39.0
#define Z_YTTERBIUM_THERMAL_EXPANSION			0.0000263
#define Z_YTTERBIUM_DENSITY				6570.0
#define Z_YTTERBIUM_LIQUID_DENSITY			6210.0
#define Z_YTTERBIUM_MOLAR_VOLUME			0.00002634
#define Z_YTTERBIUM_BRINELL_HARDNESS			343.0
#define Z_YTTERBIUM_MOHS_HARDNESS			REAL_MISSING
#define Z_YTTERBIUM_VICKERS_HARDNESS			206.0
#define Z_YTTERBIUM_BULK_MODULUS			31.0
#define Z_YTTERBIUM_SHEAR_MODULUS			10.0
#define Z_YTTERBIUM_YOUNG_MODULUS			24.0
#define Z_YTTERBIUM_POISSON_RATIO			0.21
#define Z_YTTERBIUM_REFRACTIVE_INDEX			REAL_MISSING
#define Z_YTTERBIUM_SOUND_SPEED				1590.0
#define Z_YTTERBIUM_ELECTRICAL_TYPE			Z_ELECTRICAL_TYPE_CONDUCTOR
#define Z_YTTERBIUM_ELECTRICAL_CONDUCTIVITY		3.6e6
#define Z_YTTERBIUM_ELECTRICAL_RESISTIVITY		2.8e-7
#define Z_YTTERBIUM_SUPERCONDUCTING_POINT		REAL_MISSING
#define Z_YTTERBIUM_MAGNETIC_TYPE			Z_MAGNETIC_TYPE_PARAMAGNETIC
#define Z_YTTERBIUM_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_YTTERBIUM_MASS_MAGNETIC_SUSCEPTIBILITY	5.9e-9
#define Z_YTTERBIUM_MOLAR_MAGNETIC_SUSCEPTIBILITY	1.02e-9
#define Z_YTTERBIUM_VOLUME_MAGNETIC_SUSCEPTIBILITY	0.0000388

/* MARK: - 71 - Lu */

#define Z_LUTETIUM_SYMBOL				Lu
#define Z_LUTETIUM_ATOMIC_NUMBER			71
#define Z_LUTETIUM_GROUP				3
#define Z_LUTETIUM_PERIOD				6
#define Z_LUTETIUM_BLOCK				Z_CHEMICAL_ELEMENT_BLOCK_D
#define Z_LUTETIUM_CLASS				Z_CHEMICAL_ELEMENT_CLASS_LANTHANIDE
#define Z_LUTETIUM_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_LUTETIUM_SPACE_GROUP_NUMBER			194
#define Z_LUTETIUM_ATOMIC_MASS				174.9668
#define Z_LUTETIUM_ATOMIC_RADIUS			217.0
#define Z_LUTETIUM_COVALENT_RADIUS			187.0
#define Z_LUTETIUM_VAN_DER_WAALS_RADIUS			REAL_UNKNOWN
#define Z_LUTETIUM_DECAY_MODE				KEY_NOT_APPLICABLE
#define Z_LUTETIUM_NEUTRON_CROSS_SECTION		85.0
#define Z_LUTETIUM_NEUTRON_MASS_ABSORPTION		0.022
#define Z_LUTETIUM_VALENCE				3
#define Z_LUTETIUM_ELECTRONEGATIVITY			1.27
#define Z_LUTETIUM_ELECTRON_AFFINITY			50.0
#define Z_LUTETIUM_MELTING_POINT			1663.0
#define Z_LUTETIUM_BOILING_POINT			3402.0
#define Z_LUTETIUM_CRITICAL_PRESSURE			REAL_UNKNOWN
#define Z_LUTETIUM_CRITICAL_TEMPERATURE			REAL_UNKNOWN
#define Z_LUTETIUM_FUSION_HEAT				22.0
#define Z_LUTETIUM_VAPORIZATION_HEAT			415.0
#define Z_LUTETIUM_SPECIFIC_HEAT			154.0
#define Z_LUTETIUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_LUTETIUM_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_LUTETIUM_THERMAL_CONDUCTIVITY			16.0
#define Z_LUTETIUM_THERMAL_EXPANSION			1.0e-5
#define Z_LUTETIUM_DENSITY				9841.0
#define Z_LUTETIUM_LIQUID_DENSITY			9300.0
#define Z_LUTETIUM_MOLAR_VOLUME				0.00001778
#define Z_LUTETIUM_BRINELL_HARDNESS			893.0
#define Z_LUTETIUM_MOHS_HARDNESS			2.6
#define Z_LUTETIUM_VICKERS_HARDNESS			1160.0
#define Z_LUTETIUM_BULK_MODULUS				48.0
#define Z_LUTETIUM_SHEAR_MODULUS			27.0
#define Z_LUTETIUM_YOUNG_MODULUS			67.0
#define Z_LUTETIUM_POISSON_RATIO			0.26
#define Z_LUTETIUM_REFRACTIVE_INDEX			REAL_UNKNOWN
#define Z_LUTETIUM_SOUND_SPEED				REAL_UNKNOWN
#define Z_LUTETIUM_ELECTRICAL_TYPE			Z_ELECTRICAL_TYPE_CONDUCTOR
#define Z_LUTETIUM_ELECTRICAL_CONDUCTIVITY		1.8e6
#define Z_LUTETIUM_ELECTRICAL_RESISTIVITY		5.7e-7
#define Z_LUTETIUM_SUPERCONDUCTING_POINT		0.10
#define Z_LUTETIUM_MAGNETIC_TYPE			Z_MAGNETIC_TYPE_PARAMAGNETIC
#define Z_LUTETIUM_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_LUTETIUM_MASS_MAGNETIC_SUSCEPTIBILITY		1.2e-9
#define Z_LUTETIUM_MOLAR_MAGNETIC_SUSCEPTIBILITY	2.1e-10
#define Z_LUTETIUM_VOLUME_MAGNETIC_SUSCEPTIBILITY	0.0000118

/* MARK: - 72 - Hf */

#define Z_HAFNIUM_SYMBOL				Hf
#define Z_HAFNIUM_ATOMIC_NUMBER				72
#define Z_HAFNIUM_GROUP					4
#define Z_HAFNIUM_PERIOD				6
#define Z_HAFNIUM_BLOCK					Z_CHEMICAL_ELEMENT_BLOCK_D
#define Z_HAFNIUM_CLASS					Z_CHEMICAL_ELEMENT_CLASS_TRANSITION_METAL
#define Z_HAFNIUM_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_HAFNIUM_SPACE_GROUP_NUMBER			194
#define Z_HAFNIUM_ATOMIC_MASS				178.49
#define Z_HAFNIUM_ATOMIC_RADIUS				208.0
#define Z_HAFNIUM_COVALENT_RADIUS			175.0
#define Z_HAFNIUM_VAN_DER_WAALS_RADIUS			REAL_MISSING
#define Z_HAFNIUM_DECAY_MODE				KEY_NOT_APPLICABLE
#define Z_HAFNIUM_NEUTRON_CROSS_SECTION			104.0
#define Z_HAFNIUM_NEUTRON_MASS_ABSORPTION		0.020
#define Z_HAFNIUM_VALENCE				4
#define Z_HAFNIUM_ELECTRONEGATIVITY			1.3
#define Z_HAFNIUM_ELECTRON_AFFINITY			0.0
#define Z_HAFNIUM_MELTING_POINT				2233.0
#define Z_HAFNIUM_BOILING_POINT				4603.0
#define Z_HAFNIUM_CRITICAL_PRESSURE			REAL_MISSING
#define Z_HAFNIUM_CRITICAL_TEMPERATURE			REAL_MISSING
#define Z_HAFNIUM_FUSION_HEAT				25.5
#define Z_HAFNIUM_VAPORIZATION_HEAT			630.0
#define Z_HAFNIUM_SPECIFIC_HEAT				144.0
#define Z_HAFNIUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_HAFNIUM_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_HAFNIUM_THERMAL_CONDUCTIVITY			23.0
#define Z_HAFNIUM_THERMAL_EXPANSION			5.9e-6
#define Z_HAFNIUM_DENSITY				13310.0
#define Z_HAFNIUM_LIQUID_DENSITY			12000.0
#define Z_HAFNIUM_MOLAR_VOLUME				0.00001341
#define Z_HAFNIUM_BRINELL_HARDNESS			1.7e3
#define Z_HAFNIUM_MOHS_HARDNESS				5.5
#define Z_HAFNIUM_VICKERS_HARDNESS			1760.0
#define Z_HAFNIUM_BULK_MODULUS				110.0
#define Z_HAFNIUM_SHEAR_MODULUS				30.0
#define Z_HAFNIUM_YOUNG_MODULUS				78.0
#define Z_HAFNIUM_POISSON_RATIO				0.37
#define Z_HAFNIUM_REFRACTIVE_INDEX			REAL_MISSING
#define Z_HAFNIUM_SOUND_SPEED				3010.0
#define Z_HAFNIUM_ELECTRICAL_TYPE			Z_ELECTRICAL_TYPE_CONDUCTOR
#define Z_HAFNIUM_ELECTRICAL_CONDUCTIVITY		3.3e6
#define Z_HAFNIUM_ELECTRICAL_RESISTIVITY		3.0e-7
#define Z_HAFNIUM_SUPERCONDUCTING_POINT			0.128
#define Z_HAFNIUM_MAGNETIC_TYPE				Z_MAGNETIC_TYPE_PARAMAGNETIC
#define Z_HAFNIUM_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_HAFNIUM_MASS_MAGNETIC_SUSCEPTIBILITY		5.3e-9
#define Z_HAFNIUM_MOLAR_MAGNETIC_SUSCEPTIBILITY		9.46e-10
#define Z_HAFNIUM_VOLUME_MAGNETIC_SUSCEPTIBILITY	0.0000705

/* MARK: - 73 - Ta */

#define Z_TANTALUM_SYMBOL				Ta
#define Z_TANTALUM_ATOMIC_NUMBER			73
#define Z_TANTALUM_GROUP				5
#define Z_TANTALUM_PERIOD				6
#define Z_TANTALUM_BLOCK				Z_CHEMICAL_ELEMENT_BLOCK_D
#define Z_TANTALUM_CLASS				Z_CHEMICAL_ELEMENT_CLASS_TRANSITION_METAL
#define Z_TANTALUM_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_TANTALUM_SPACE_GROUP_NUMBER			229
#define Z_TANTALUM_ATOMIC_MASS				180.94788
#define Z_TANTALUM_ATOMIC_RADIUS			2.0e2
#define Z_TANTALUM_COVALENT_RADIUS			170.0
#define Z_TANTALUM_VAN_DER_WAALS_RADIUS			REAL_MISSING
#define Z_TANTALUM_DECAY_MODE				KEY_NOT_APPLICABLE
#define Z_TANTALUM_NEUTRON_CROSS_SECTION		20.5
#define Z_TANTALUM_NEUTRON_MASS_ABSORPTION		0.0041
#define Z_TANTALUM_VALENCE				5
#define Z_TANTALUM_ELECTRONEGATIVITY			1.5
#define Z_TANTALUM_ELECTRON_AFFINITY			31.0
#define Z_TANTALUM_MELTING_POINT			3017.0
#define Z_TANTALUM_BOILING_POINT			5458.0
#define Z_TANTALUM_CRITICAL_PRESSURE			REAL_MISSING
#define Z_TANTALUM_CRITICAL_TEMPERATURE			REAL_MISSING
#define Z_TANTALUM_FUSION_HEAT				36.0
#define Z_TANTALUM_VAPORIZATION_HEAT			736.0
#define Z_TANTALUM_SPECIFIC_HEAT			140.0
#define Z_TANTALUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_TANTALUM_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_TANTALUM_THERMAL_CONDUCTIVITY			57.0
#define Z_TANTALUM_THERMAL_EXPANSION			6.3e-6
#define Z_TANTALUM_DENSITY				16650.0
#define Z_TANTALUM_LIQUID_DENSITY			15000.0
#define Z_TANTALUM_MOLAR_VOLUME				0.000010868
#define Z_TANTALUM_BRINELL_HARDNESS			8.0e2
#define Z_TANTALUM_MOHS_HARDNESS			6.5
#define Z_TANTALUM_VICKERS_HARDNESS			873.0
#define Z_TANTALUM_BULK_MODULUS				2.0e2
#define Z_TANTALUM_SHEAR_MODULUS			67.0
#define Z_TANTALUM_YOUNG_MODULUS			186.0
#define Z_TANTALUM_POISSON_RATIO			0.34
#define Z_TANTALUM_REFRACTIVE_INDEX			REAL_MISSING
#define Z_TANTALUM_SOUND_SPEED				3.4e3
#define Z_TANTALUM_ELECTRICAL_TYPE			Z_ELECTRICAL_TYPE_CONDUCTOR
#define Z_TANTALUM_ELECTRICAL_CONDUCTIVITY		7.7e6
#define Z_TANTALUM_ELECTRICAL_RESISTIVITY		1.3e-7
#define Z_TANTALUM_SUPERCONDUCTING_POINT		4.47
#define Z_TANTALUM_MAGNETIC_TYPE			Z_MAGNETIC_TYPE_PARAMAGNETIC
#define Z_TANTALUM_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_TANTALUM_MASS_MAGNETIC_SUSCEPTIBILITY		1.07e-8
#define Z_TANTALUM_MOLAR_MAGNETIC_SUSCEPTIBILITY	1.936e-9
#define Z_TANTALUM_VOLUME_MAGNETIC_SUSCEPTIBILITY	0.0001782

/* MARK: - 74 - W */

#define Z_WOLFRAM_SYMBOL				W
#define Z_WOLFRAM_ATOMIC_NUMBER				74
#define Z_WOLFRAM_GROUP					6
#define Z_WOLFRAM_PERIOD				6
#define Z_WOLFRAM_BLOCK					Z_CHEMICAL_ELEMENT_BLOCK_D
#define Z_WOLFRAM_CLASS					Z_CHEMICAL_ELEMENT_CLASS_TRANSITION_METAL
#define Z_WOLFRAM_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_WOLFRAM_SPACE_GROUP_NUMBER			229
#define Z_WOLFRAM_ATOMIC_MASS				183.84
#define Z_WOLFRAM_ATOMIC_RADIUS				193.0
#define Z_WOLFRAM_COVALENT_RADIUS			162.0
#define Z_WOLFRAM_VAN_DER_WAALS_RADIUS			REAL_MISSING
#define Z_WOLFRAM_DECAY_MODE				KEY_NOT_APPLICABLE
#define Z_WOLFRAM_NEUTRON_CROSS_SECTION			18.4
#define Z_WOLFRAM_NEUTRON_MASS_ABSORPTION		0.0036
#define Z_WOLFRAM_VALENCE				6
#define Z_WOLFRAM_ELECTRONEGATIVITY			2.36
#define Z_WOLFRAM_ELECTRON_AFFINITY			78.6
#define Z_WOLFRAM_MELTING_POINT				3422.0
#define Z_WOLFRAM_BOILING_POINT				5555.0
#define Z_WOLFRAM_CRITICAL_PRESSURE			REAL_MISSING
#define Z_WOLFRAM_CRITICAL_TEMPERATURE			REAL_MISSING
#define Z_WOLFRAM_FUSION_HEAT				35.0
#define Z_WOLFRAM_VAPORIZATION_HEAT			8.0e2
#define Z_WOLFRAM_SPECIFIC_HEAT				132.0
#define Z_WOLFRAM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_WOLFRAM_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_WOLFRAM_THERMAL_CONDUCTIVITY			170.0
#define Z_WOLFRAM_THERMAL_EXPANSION			4.5e-6
#define Z_WOLFRAM_DENSITY				19250.0
#define Z_WOLFRAM_LIQUID_DENSITY			17600.0
#define Z_WOLFRAM_MOLAR_VOLUME				9.55e-6
#define Z_WOLFRAM_BRINELL_HARDNESS			2570.0
#define Z_WOLFRAM_MOHS_HARDNESS				7.5
#define Z_WOLFRAM_VICKERS_HARDNESS			3430.0
#define Z_WOLFRAM_BULK_MODULUS				310.0
#define Z_WOLFRAM_SHEAR_MODULUS				161.0
#define Z_WOLFRAM_YOUNG_MODULUS				411.0
#define Z_WOLFRAM_POISSON_RATIO				0.28
#define Z_WOLFRAM_REFRACTIVE_INDEX			REAL_MISSING
#define Z_WOLFRAM_SOUND_SPEED				5174.0
#define Z_WOLFRAM_ELECTRICAL_TYPE			Z_ELECTRICAL_TYPE_CONDUCTOR
#define Z_WOLFRAM_ELECTRICAL_CONDUCTIVITY		2.0e7
#define Z_WOLFRAM_ELECTRICAL_RESISTIVITY		5.0e-8
#define Z_WOLFRAM_SUPERCONDUCTING_POINT			0.015
#define Z_WOLFRAM_MAGNETIC_TYPE				Z_MAGNETIC_TYPE_PARAMAGNETIC
#define Z_WOLFRAM_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_WOLFRAM_MASS_MAGNETIC_SUSCEPTIBILITY		4.59e-9
#define Z_WOLFRAM_MOLAR_MAGNETIC_SUSCEPTIBILITY		8.44e-10
#define Z_WOLFRAM_VOLUME_MAGNETIC_SUSCEPTIBILITY	0.0000884

#define Z_TUNGSTEN_SYMBOL				Z_WOLFRAM_SYMBOL
#define Z_TUNGSTEN_ATOMIC_NUMBER			Z_WOLFRAM_ATOMIC_NUMBER
#define Z_TUNGSTEN_GROUP				Z_WOLFRAM_GROUP
#define Z_TUNGSTEN_PERIOD				Z_WOLFRAM_PERIOD
#define Z_TUNGSTEN_BLOCK				Z_WOLFRAM_BLOCK
#define Z_TUNGSTEN_CLASS				Z_WOLFRAM_CLASS
#define Z_TUNGSTEN_METAL_STATE				Z_WOLFRAM_METAL_STATE
#define Z_TUNGSTEN_SPACE_GROUP_NUMBER			Z_WOLFRAM_SPACE_GROUP_NUMBER
#define Z_TUNGSTEN_ATOMIC_MASS				Z_WOLFRAM_ATOMIC_MASS
#define Z_TUNGSTEN_ATOMIC_RADIUS			Z_WOLFRAM_ATOMIC_RADIUS
#define Z_TUNGSTEN_COVALENT_RADIUS			Z_WOLFRAM_COVALENT_RADIUS
#define Z_TUNGSTEN_VAN_DER_WAALS_RADIUS			Z_WOLFRAM_VAN_DER_WAALS_RADIUS
#define Z_TUNGSTEN_DECAY_MODE				Z_WOLFRAM_DECAY_MODE
#define Z_TUNGSTEN_NEUTRON_CROSS_SECTION		Z_WOLFRAM_NEUTRON_CROSS_SECTION
#define Z_TUNGSTEN_NEUTRON_MASS_ABSORPTION		Z_WOLFRAM_NEUTRON_MASS_ABSORPTION
#define Z_TUNGSTEN_VALENCE				Z_WOLFRAM_VALENCE
#define Z_TUNGSTEN_ELECTRONEGATIVITY			Z_WOLFRAM_ELECTRONEGATIVITY
#define Z_TUNGSTEN_ELECTRON_AFFINITY			Z_WOLFRAM_ELECTRON_AFFINITY
#define Z_TUNGSTEN_MELTING_POINT			Z_WOLFRAM_MELTING_POINT
#define Z_TUNGSTEN_BOILING_POINT			Z_WOLFRAM_BOILING_POINT
#define Z_TUNGSTEN_CRITICAL_PRESSURE			Z_WOLFRAM_CRITICAL_PRESSURE
#define Z_TUNGSTEN_CRITICAL_TEMPERATURE			Z_WOLFRAM_CRITICAL_TEMPERATURE
#define Z_TUNGSTEN_FUSION_HEAT				Z_WOLFRAM_FUSION_HEAT
#define Z_TUNGSTEN_VAPORIZATION_HEAT			Z_WOLFRAM_VAPORIZATION_HEAT
#define Z_TUNGSTEN_SPECIFIC_HEAT			Z_WOLFRAM_SPECIFIC_HEAT
#define Z_TUNGSTEN_ADIABATIC_INDEX			Z_WOLFRAM_ADIABATIC_INDEX
#define Z_TUNGSTEN_NEEL_POINT				Z_WOLFRAM_NEEL_POINT
#define Z_TUNGSTEN_THERMAL_CONDUCTIVITY			Z_WOLFRAM_THERMAL_CONDUCTIVITY
#define Z_TUNGSTEN_THERMAL_EXPANSION			Z_WOLFRAM_THERMAL_EXPANSION
#define Z_TUNGSTEN_DENSITY				Z_WOLFRAM_DENSITY
#define Z_TUNGSTEN_LIQUID_DENSITY			Z_WOLFRAM_LIQUID_DENSITY
#define Z_TUNGSTEN_MOLAR_VOLUME				Z_WOLFRAM_MOLAR_VOLUME
#define Z_TUNGSTEN_BRINELL_HARDNESS			Z_WOLFRAM_BRINELL_HARDNESS
#define Z_TUNGSTEN_MOHS_HARDNESS			Z_WOLFRAM_MOHS_HARDNESS
#define Z_TUNGSTEN_VICKERS_HARDNESS			Z_WOLFRAM_VICKERS_HARDNESS
#define Z_TUNGSTEN_BULK_MODULUS				Z_WOLFRAM_BULK_MODULUS
#define Z_TUNGSTEN_SHEAR_MODULUS			Z_WOLFRAM_SHEAR_MODULUS
#define Z_TUNGSTEN_YOUNG_MODULUS			Z_WOLFRAM_YOUNG_MODULUS
#define Z_TUNGSTEN_POISSON_RATIO			Z_WOLFRAM_POISSON_RATIO
#define Z_TUNGSTEN_REFRACTIVE_INDEX			Z_WOLFRAM_REFRACTIVE_INDEX
#define Z_TUNGSTEN_SOUND_SPEED				Z_WOLFRAM_SOUND_SPEED
#define Z_TUNGSTEN_ELECTRICAL_TYPE			Z_WOLFRAM_ELECTRICAL_TYPE
#define Z_TUNGSTEN_ELECTRICAL_CONDUCTIVITY		Z_WOLFRAM_ELECTRICAL_CONDUCTIVITY
#define Z_TUNGSTEN_ELECTRICAL_RESISTIVITY		Z_WOLFRAM_ELECTRICAL_RESISTIVITY
#define Z_TUNGSTEN_SUPERCONDUCTING_POINT		Z_WOLFRAM_SUPERCONDUCTING_POINT
#define Z_TUNGSTEN_MAGNETIC_TYPE			Z_WOLFRAM_MAGNETIC_TYPE
#define Z_TUNGSTEN_CURIE_POINT				Z_WOLFRAM_CURIE_POINT
#define Z_TUNGSTEN_MASS_MAGNETIC_SUSCEPTIBILITY		Z_WOLFRAM_MASS_MAGNETIC_SUSCEPTIBILITY
#define Z_TUNGSTEN_MOLAR_MAGNETIC_SUSCEPTIBILITY	Z_WOLFRAM_MOLAR_MAGNETIC_SUSCEPTIBILITY
#define Z_TUNGSTEN_VOLUME_MAGNETIC_SUSCEPTIBILITY	Z_WOLFRAM_VOLUME_MAGNETIC_SUSCEPTIBILITY

/* MARK: - 75 - Re */

#define Z_RHENIUM_SYMBOL				Re
#define Z_RHENIUM_ATOMIC_NUMBER				75
#define Z_RHENIUM_GROUP					7
#define Z_RHENIUM_PERIOD				6
#define Z_RHENIUM_BLOCK					Z_CHEMICAL_ELEMENT_BLOCK_D
#define Z_RHENIUM_CLASS					Z_CHEMICAL_ELEMENT_CLASS_TRANSITION_METAL
#define Z_RHENIUM_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_RHENIUM_SPACE_GROUP_NUMBER			194
#define Z_RHENIUM_ATOMIC_MASS				186.207
#define Z_RHENIUM_ATOMIC_RADIUS				188.0
#define Z_RHENIUM_COVALENT_RADIUS			151.0
#define Z_RHENIUM_VAN_DER_WAALS_RADIUS			REAL_MISSING
#define Z_RHENIUM_DECAY_MODE				KEY_NOT_APPLICABLE
#define Z_RHENIUM_NEUTRON_CROSS_SECTION			90.0
#define Z_RHENIUM_NEUTRON_MASS_ABSORPTION		0.016
#define Z_RHENIUM_VALENCE				7
#define Z_RHENIUM_ELECTRONEGATIVITY			1.9
#define Z_RHENIUM_ELECTRON_AFFINITY			14.5
#define Z_RHENIUM_MELTING_POINT				3186.0
#define Z_RHENIUM_BOILING_POINT				5596.0
#define Z_RHENIUM_CRITICAL_PRESSURE			REAL_MISSING
#define Z_RHENIUM_CRITICAL_TEMPERATURE			REAL_MISSING
#define Z_RHENIUM_FUSION_HEAT				33.0
#define Z_RHENIUM_VAPORIZATION_HEAT			705.0
#define Z_RHENIUM_SPECIFIC_HEAT				137.0
#define Z_RHENIUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_RHENIUM_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_RHENIUM_THERMAL_CONDUCTIVITY			48.0
#define Z_RHENIUM_THERMAL_EXPANSION			6.2e-6
#define Z_RHENIUM_DENSITY				21020.0
#define Z_RHENIUM_LIQUID_DENSITY			18900.0
#define Z_RHENIUM_MOLAR_VOLUME				8.8586e-6
#define Z_RHENIUM_BRINELL_HARDNESS			1320.0
#define Z_RHENIUM_MOHS_HARDNESS				7.0
#define Z_RHENIUM_VICKERS_HARDNESS			2450.0
#define Z_RHENIUM_BULK_MODULUS				370.0
#define Z_RHENIUM_SHEAR_MODULUS				178.0
#define Z_RHENIUM_YOUNG_MODULUS				463.0
#define Z_RHENIUM_POISSON_RATIO				0.30
#define Z_RHENIUM_REFRACTIVE_INDEX			REAL_MISSING
#define Z_RHENIUM_SOUND_SPEED				4.7e3
#define Z_RHENIUM_ELECTRICAL_TYPE			Z_ELECTRICAL_TYPE_CONDUCTOR
#define Z_RHENIUM_ELECTRICAL_CONDUCTIVITY		5.6e6
#define Z_RHENIUM_ELECTRICAL_RESISTIVITY		1.8e-7
#define Z_RHENIUM_SUPERCONDUCTING_POINT			1.70
#define Z_RHENIUM_MAGNETIC_TYPE				Z_MAGNETIC_TYPE_PARAMAGNETIC
#define Z_RHENIUM_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_RHENIUM_MASS_MAGNETIC_SUSCEPTIBILITY		4.56e-9
#define Z_RHENIUM_MOLAR_MAGNETIC_SUSCEPTIBILITY		8.49e-10
#define Z_RHENIUM_VOLUME_MAGNETIC_SUSCEPTIBILITY	0.0000959

/* MARK: - 76 - Os */

#define Z_OSMIUM_SYMBOL					Os
#define Z_OSMIUM_ATOMIC_NUMBER				76
#define Z_OSMIUM_GROUP					8
#define Z_OSMIUM_PERIOD					6
#define Z_OSMIUM_BLOCK					Z_CHEMICAL_ELEMENT_BLOCK_D
#define Z_OSMIUM_CLASS					Z_CHEMICAL_ELEMENT_CLASS_TRANSITION_METAL
#define Z_OSMIUM_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_OSMIUM_SPACE_GROUP_NUMBER			194
#define Z_OSMIUM_ATOMIC_MASS				190.23
#define Z_OSMIUM_ATOMIC_RADIUS				185.0
#define Z_OSMIUM_COVALENT_RADIUS			144.0
#define Z_OSMIUM_VAN_DER_WAALS_RADIUS			REAL_MISSING
#define Z_OSMIUM_DECAY_MODE				KEY_NOT_APPLICABLE
#define Z_OSMIUM_NEUTRON_CROSS_SECTION			15.0
#define Z_OSMIUM_NEUTRON_MASS_ABSORPTION		0.0023
#define Z_OSMIUM_VALENCE				6
#define Z_OSMIUM_ELECTRONEGATIVITY			2.2
#define Z_OSMIUM_ELECTRON_AFFINITY			106.1
#define Z_OSMIUM_MELTING_POINT				3033.0
#define Z_OSMIUM_BOILING_POINT				5012.0
#define Z_OSMIUM_CRITICAL_PRESSURE			REAL_MISSING
#define Z_OSMIUM_CRITICAL_TEMPERATURE			REAL_MISSING
#define Z_OSMIUM_FUSION_HEAT				31.0
#define Z_OSMIUM_VAPORIZATION_HEAT			630.0
#define Z_OSMIUM_SPECIFIC_HEAT				130.0
#define Z_OSMIUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_OSMIUM_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_OSMIUM_THERMAL_CONDUCTIVITY			87.0
#define Z_OSMIUM_THERMAL_EXPANSION			5.1e-6
#define Z_OSMIUM_DENSITY				2.259e4
#define Z_OSMIUM_LIQUID_DENSITY				20000.0
#define Z_OSMIUM_MOLAR_VOLUME				8.421e-6
#define Z_OSMIUM_BRINELL_HARDNESS			3920.0
#define Z_OSMIUM_MOHS_HARDNESS				7.0
#define Z_OSMIUM_VICKERS_HARDNESS			4.14e3
#define Z_OSMIUM_BULK_MODULUS				REAL_MISSING
#define Z_OSMIUM_SHEAR_MODULUS				222.0
#define Z_OSMIUM_YOUNG_MODULUS				REAL_MISSING
#define Z_OSMIUM_POISSON_RATIO				0.25
#define Z_OSMIUM_REFRACTIVE_INDEX			REAL_MISSING
#define Z_OSMIUM_SOUND_SPEED				4940.0
#define Z_OSMIUM_ELECTRICAL_TYPE			Z_ELECTRICAL_TYPE_CONDUCTOR
#define Z_OSMIUM_ELECTRICAL_CONDUCTIVITY		1.2e7
#define Z_OSMIUM_ELECTRICAL_RESISTIVITY			8.0e-8
#define Z_OSMIUM_SUPERCONDUCTING_POINT			0.66
#define Z_OSMIUM_MAGNETIC_TYPE				Z_MAGNETIC_TYPE_PARAMAGNETIC
#define Z_OSMIUM_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_OSMIUM_MASS_MAGNETIC_SUSCEPTIBILITY		6.0e-10
#define Z_OSMIUM_MOLAR_MAGNETIC_SUSCEPTIBILITY		1.1e-10
#define Z_OSMIUM_VOLUME_MAGNETIC_SUSCEPTIBILITY		0.000014

/* MARK: - 77 - Ir */

#define Z_IRIDIUM_SYMBOL				Ir
#define Z_IRIDIUM_ATOMIC_NUMBER				77
#define Z_IRIDIUM_GROUP					9
#define Z_IRIDIUM_PERIOD				6
#define Z_IRIDIUM_BLOCK					Z_CHEMICAL_ELEMENT_BLOCK_D
#define Z_IRIDIUM_CLASS					Z_CHEMICAL_ELEMENT_CLASS_TRANSITION_METAL
#define Z_IRIDIUM_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_IRIDIUM_SPACE_GROUP_NUMBER			225
#define Z_IRIDIUM_ATOMIC_MASS				192.217
#define Z_IRIDIUM_ATOMIC_RADIUS				180.0
#define Z_IRIDIUM_COVALENT_RADIUS			141.0
#define Z_IRIDIUM_VAN_DER_WAALS_RADIUS			REAL_MISSING
#define Z_IRIDIUM_DECAY_MODE				KEY_NOT_APPLICABLE
#define Z_IRIDIUM_NEUTRON_CROSS_SECTION			425.0
#define Z_IRIDIUM_NEUTRON_MASS_ABSORPTION		0.081
#define Z_IRIDIUM_VALENCE				6
#define Z_IRIDIUM_ELECTRONEGATIVITY			2.20
#define Z_IRIDIUM_ELECTRON_AFFINITY			151.0
#define Z_IRIDIUM_MELTING_POINT				2466.0
#define Z_IRIDIUM_BOILING_POINT				4428.0
#define Z_IRIDIUM_CRITICAL_PRESSURE			REAL_MISSING
#define Z_IRIDIUM_CRITICAL_TEMPERATURE			REAL_MISSING
#define Z_IRIDIUM_FUSION_HEAT				26.0
#define Z_IRIDIUM_VAPORIZATION_HEAT			560.0
#define Z_IRIDIUM_SPECIFIC_HEAT				131.0
#define Z_IRIDIUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_IRIDIUM_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_IRIDIUM_THERMAL_CONDUCTIVITY			150.0
#define Z_IRIDIUM_THERMAL_EXPANSION			6.4e-6
#define Z_IRIDIUM_DENSITY				2.256e4
#define Z_IRIDIUM_LIQUID_DENSITY			19000.0
#define Z_IRIDIUM_MOLAR_VOLUME				8.52e-6
#define Z_IRIDIUM_BRINELL_HARDNESS			1670.0
#define Z_IRIDIUM_MOHS_HARDNESS				6.5
#define Z_IRIDIUM_VICKERS_HARDNESS			1760.0
#define Z_IRIDIUM_BULK_MODULUS				320.0
#define Z_IRIDIUM_SHEAR_MODULUS				210.0
#define Z_IRIDIUM_YOUNG_MODULUS				528.0
#define Z_IRIDIUM_POISSON_RATIO				0.26
#define Z_IRIDIUM_REFRACTIVE_INDEX			REAL_MISSING
#define Z_IRIDIUM_SOUND_SPEED				4825.0
#define Z_IRIDIUM_ELECTRICAL_TYPE			Z_ELECTRICAL_TYPE_CONDUCTOR
#define Z_IRIDIUM_ELECTRICAL_CONDUCTIVITY		2.1e7
#define Z_IRIDIUM_ELECTRICAL_RESISTIVITY		4.7e-8
#define Z_IRIDIUM_SUPERCONDUCTING_POINT			0.11
#define Z_IRIDIUM_MAGNETIC_TYPE				Z_MAGNETIC_TYPE_PARAMAGNETIC
#define Z_IRIDIUM_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_IRIDIUM_MASS_MAGNETIC_SUSCEPTIBILITY		1.67e-9
#define Z_IRIDIUM_MOLAR_MAGNETIC_SUSCEPTIBILITY		3.21e-10
#define Z_IRIDIUM_VOLUME_MAGNETIC_SUSCEPTIBILITY	0.0000377

/* MARK: - 78 - Pt */

#define Z_PLATINUM_SYMBOL				Pt
#define Z_PLATINUM_ATOMIC_NUMBER			78
#define Z_PLATINUM_GROUP				10
#define Z_PLATINUM_PERIOD				6
#define Z_PLATINUM_BLOCK				Z_CHEMICAL_ELEMENT_BLOCK_D
#define Z_PLATINUM_CLASS				Z_CHEMICAL_ELEMENT_CLASS_TRANSITION_METAL
#define Z_PLATINUM_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_PLATINUM_SPACE_GROUP_NUMBER			225
#define Z_PLATINUM_ATOMIC_MASS				195.084
#define Z_PLATINUM_ATOMIC_RADIUS			177.0
#define Z_PLATINUM_COVALENT_RADIUS			136.0
#define Z_PLATINUM_VAN_DER_WAALS_RADIUS			175.0
#define Z_PLATINUM_DECAY_MODE				KEY_NOT_APPLICABLE
#define Z_PLATINUM_NEUTRON_CROSS_SECTION		10.0
#define Z_PLATINUM_NEUTRON_MASS_ABSORPTION		0.0020
#define Z_PLATINUM_VALENCE				6
#define Z_PLATINUM_ELECTRONEGATIVITY			2.28
#define Z_PLATINUM_ELECTRON_AFFINITY			205.3
#define Z_PLATINUM_MELTING_POINT			1768.3
#define Z_PLATINUM_BOILING_POINT			3825.0
#define Z_PLATINUM_CRITICAL_PRESSURE			REAL_MISSING
#define Z_PLATINUM_CRITICAL_TEMPERATURE			REAL_MISSING
#define Z_PLATINUM_FUSION_HEAT				20.0
#define Z_PLATINUM_VAPORIZATION_HEAT			490.0
#define Z_PLATINUM_SPECIFIC_HEAT			133.0
#define Z_PLATINUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_PLATINUM_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_PLATINUM_THERMAL_CONDUCTIVITY			71.0
#define Z_PLATINUM_THERMAL_EXPANSION			8.9e-6
#define Z_PLATINUM_DENSITY				21450.0
#define Z_PLATINUM_LIQUID_DENSITY			19770.0
#define Z_PLATINUM_MOLAR_VOLUME				9.095e-6
#define Z_PLATINUM_BRINELL_HARDNESS			392.0
#define Z_PLATINUM_MOHS_HARDNESS			3.5
#define Z_PLATINUM_VICKERS_HARDNESS			549.0
#define Z_PLATINUM_BULK_MODULUS				230.0
#define Z_PLATINUM_SHEAR_MODULUS			61.0
#define Z_PLATINUM_YOUNG_MODULUS			168.0
#define Z_PLATINUM_POISSON_RATIO			0.38
#define Z_PLATINUM_REFRACTIVE_INDEX			REAL_MISSING
#define Z_PLATINUM_SOUND_SPEED				2680.0
#define Z_PLATINUM_ELECTRICAL_TYPE			Z_ELECTRICAL_TYPE_CONDUCTOR
#define Z_PLATINUM_ELECTRICAL_CONDUCTIVITY		9.4e6
#define Z_PLATINUM_ELECTRICAL_RESISTIVITY		1.1e-7
#define Z_PLATINUM_SUPERCONDUCTING_POINT		REAL_MISSING
#define Z_PLATINUM_MAGNETIC_TYPE			Z_MAGNETIC_TYPE_PARAMAGNETIC
#define Z_PLATINUM_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_PLATINUM_MASS_MAGNETIC_SUSCEPTIBILITY		1.22e-8
#define Z_PLATINUM_MOLAR_MAGNETIC_SUSCEPTIBILITY	2.38e-9
#define Z_PLATINUM_VOLUME_MAGNETIC_SUSCEPTIBILITY	0.0002573

/* MARK: - 79 - Au */

#define Z_GOLD_SYMBOL					Au
#define Z_GOLD_ATOMIC_NUMBER				79
#define Z_GOLD_GROUP					11
#define Z_GOLD_PERIOD					6
#define Z_GOLD_BLOCK					Z_CHEMICAL_ELEMENT_BLOCK_D
#define Z_GOLD_CLASS					Z_CHEMICAL_ELEMENT_CLASS_TRANSITION_METAL
#define Z_GOLD_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_GOLD_SPACE_GROUP_NUMBER			225
#define Z_GOLD_ATOMIC_MASS				196.966569
#define Z_GOLD_ATOMIC_RADIUS				174.0
#define Z_GOLD_COVALENT_RADIUS				136.0
#define Z_GOLD_VAN_DER_WAALS_RADIUS			166.0
#define Z_GOLD_DECAY_MODE				KEY_NOT_APPLICABLE
#define Z_GOLD_NEUTRON_CROSS_SECTION			98.7
#define Z_GOLD_NEUTRON_MASS_ABSORPTION			0.017
#define Z_GOLD_VALENCE					5
#define Z_GOLD_ELECTRONEGATIVITY			2.54
#define Z_GOLD_ELECTRON_AFFINITY			222.8
#define Z_GOLD_MELTING_POINT				1064.18
#define Z_GOLD_BOILING_POINT				2856.0
#define Z_GOLD_CRITICAL_PRESSURE			REAL_MISSING
#define Z_GOLD_CRITICAL_TEMPERATURE			REAL_MISSING
#define Z_GOLD_FUSION_HEAT				12.5
#define Z_GOLD_VAPORIZATION_HEAT			330.0
#define Z_GOLD_SPECIFIC_HEAT				129.1
#define Z_GOLD_ADIABATIC_INDEX				KEY_NOT_APPLICABLE
#define Z_GOLD_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_GOLD_THERMAL_CONDUCTIVITY			320.0
#define Z_GOLD_THERMAL_EXPANSION			0.0000142
#define Z_GOLD_DENSITY					1.93e4
#define Z_GOLD_LIQUID_DENSITY				17310.0
#define Z_GOLD_MOLAR_VOLUME				0.0000102
#define Z_GOLD_BRINELL_HARDNESS				2450.0
#define Z_GOLD_MOHS_HARDNESS				2.5
#define Z_GOLD_VICKERS_HARDNESS				216.0
#define Z_GOLD_BULK_MODULUS				220.0
#define Z_GOLD_SHEAR_MODULUS				27.0
#define Z_GOLD_YOUNG_MODULUS				78.0
#define Z_GOLD_POISSON_RATIO				0.44
#define Z_GOLD_REFRACTIVE_INDEX				REAL_MISSING
#define Z_GOLD_SOUND_SPEED				1740.0
#define Z_GOLD_ELECTRICAL_TYPE				Z_ELECTRICAL_TYPE_CONDUCTOR
#define Z_GOLD_ELECTRICAL_CONDUCTIVITY			4.5e7
#define Z_GOLD_ELECTRICAL_RESISTIVITY			2.2e-8
#define Z_GOLD_SUPERCONDUCTING_POINT			REAL_MISSING
#define Z_GOLD_MAGNETIC_TYPE				Z_MAGNETIC_TYPE_DIAMAGNETIC
#define Z_GOLD_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_GOLD_MASS_MAGNETIC_SUSCEPTIBILITY		-1.78e-9
#define Z_GOLD_MOLAR_MAGNETIC_SUSCEPTIBILITY		-3.51e-10
#define Z_GOLD_VOLUME_MAGNETIC_SUSCEPTIBILITY		-0.0000344

/* MARK: - 80 - Hg */

#define Z_MERCURY_SYMBOL				Hg
#define Z_MERCURY_ATOMIC_NUMBER				80
#define Z_MERCURY_GROUP					12
#define Z_MERCURY_PERIOD				6
#define Z_MERCURY_BLOCK					Z_CHEMICAL_ELEMENT_BLOCK_D
#define Z_MERCURY_CLASS					Z_CHEMICAL_ELEMENT_CLASS_TRANSITION_METAL
#define Z_MERCURY_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_MERCURY_SPACE_GROUP_NUMBER			166
#define Z_MERCURY_ATOMIC_MASS				200.59
#define Z_MERCURY_ATOMIC_RADIUS				171.0
#define Z_MERCURY_COVALENT_RADIUS			132.0
#define Z_MERCURY_VAN_DER_WAALS_RADIUS			155.0
#define Z_MERCURY_DECAY_MODE				KEY_NOT_APPLICABLE
#define Z_MERCURY_NEUTRON_CROSS_SECTION			374.0
#define Z_MERCURY_NEUTRON_MASS_ABSORPTION		0.063
#define Z_MERCURY_VALENCE				2
#define Z_MERCURY_ELECTRONEGATIVITY			2.0
#define Z_MERCURY_ELECTRON_AFFINITY			0.0
#define Z_MERCURY_MELTING_POINT				-38.83
#define Z_MERCURY_BOILING_POINT				356.73
#define Z_MERCURY_CRITICAL_PRESSURE			172.0
#define Z_MERCURY_CRITICAL_TEMPERATURE			1750.0
#define Z_MERCURY_FUSION_HEAT				2.29
#define Z_MERCURY_VAPORIZATION_HEAT			59.2
#define Z_MERCURY_SPECIFIC_HEAT				139.5
#define Z_MERCURY_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_MERCURY_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_MERCURY_THERMAL_CONDUCTIVITY			8.3
#define Z_MERCURY_THERMAL_EXPANSION			0.000181
#define Z_MERCURY_DENSITY				13534.0
#define Z_MERCURY_LIQUID_DENSITY			13534.0
#define Z_MERCURY_MOLAR_VOLUME				0.000014821
#define Z_MERCURY_BRINELL_HARDNESS			REAL_NOT_APPLICABLE
#define Z_MERCURY_MOHS_HARDNESS				REAL_NOT_APPLICABLE
#define Z_MERCURY_VICKERS_HARDNESS			REAL_NOT_APPLICABLE
#define Z_MERCURY_BULK_MODULUS				25.0
#define Z_MERCURY_SHEAR_MODULUS				REAL_NOT_APPLICABLE
#define Z_MERCURY_YOUNG_MODULUS				REAL_NOT_APPLICABLE
#define Z_MERCURY_POISSON_RATIO				REAL_NOT_APPLICABLE
#define Z_MERCURY_REFRACTIVE_INDEX			1.000933
#define Z_MERCURY_SOUND_SPEED				1407.0
#define Z_MERCURY_ELECTRICAL_TYPE			Z_ELECTRICAL_TYPE_CONDUCTOR
#define Z_MERCURY_ELECTRICAL_CONDUCTIVITY		1.0e6
#define Z_MERCURY_ELECTRICAL_RESISTIVITY		9.6e-7
#define Z_MERCURY_SUPERCONDUCTING_POINT			4.154
#define Z_MERCURY_MAGNETIC_TYPE				Z_MAGNETIC_TYPE_DIAMAGNETIC
#define Z_MERCURY_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_MERCURY_MASS_MAGNETIC_SUSCEPTIBILITY		-2.1e-9
#define Z_MERCURY_MOLAR_MAGNETIC_SUSCEPTIBILITY		-4.21e-10
#define Z_MERCURY_VOLUME_MAGNETIC_SUSCEPTIBILITY	-0.0000284

/* MARK: - 81 - Tl */

#define Z_THALLIUM_SYMBOL				Tl
#define Z_THALLIUM_ATOMIC_NUMBER			81
#define Z_THALLIUM_GROUP				13
#define Z_THALLIUM_PERIOD				6
#define Z_THALLIUM_BLOCK				Z_CHEMICAL_ELEMENT_BLOCK_P
#define Z_THALLIUM_CLASS				Z_CHEMICAL_ELEMENT_CLASS_POOR_METAL
#define Z_THALLIUM_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_THALLIUM_SPACE_GROUP_NUMBER			194
#define Z_THALLIUM_ATOMIC_MASS				204.3833
#define Z_THALLIUM_ATOMIC_RADIUS			156.0
#define Z_THALLIUM_COVALENT_RADIUS			145.0
#define Z_THALLIUM_VAN_DER_WAALS_RADIUS			196.0
#define Z_THALLIUM_DECAY_MODE				KEY_NOT_APPLICABLE
#define Z_THALLIUM_NEUTRON_CROSS_SECTION		3.4
#define Z_THALLIUM_NEUTRON_MASS_ABSORPTION		0.00059
#define Z_THALLIUM_VALENCE				3
#define Z_THALLIUM_ELECTRONEGATIVITY			1.62
#define Z_THALLIUM_ELECTRON_AFFINITY			19.2
#define Z_THALLIUM_MELTING_POINT			304.0
#define Z_THALLIUM_BOILING_POINT			1473.0
#define Z_THALLIUM_CRITICAL_PRESSURE			REAL_MISSING
#define Z_THALLIUM_CRITICAL_TEMPERATURE			REAL_MISSING
#define Z_THALLIUM_FUSION_HEAT				4.2
#define Z_THALLIUM_VAPORIZATION_HEAT			165.0
#define Z_THALLIUM_SPECIFIC_HEAT			129.0
#define Z_THALLIUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_THALLIUM_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_THALLIUM_THERMAL_CONDUCTIVITY			46.0
#define Z_THALLIUM_THERMAL_EXPANSION			0.0000299
#define Z_THALLIUM_DENSITY				11850.0
#define Z_THALLIUM_LIQUID_DENSITY			11220.0
#define Z_THALLIUM_MOLAR_VOLUME				0.000017248
#define Z_THALLIUM_BRINELL_HARDNESS			26.4
#define Z_THALLIUM_MOHS_HARDNESS			1.2
#define Z_THALLIUM_VICKERS_HARDNESS			REAL_MISSING
#define Z_THALLIUM_BULK_MODULUS				43.0
#define Z_THALLIUM_SHEAR_MODULUS			2.8
#define Z_THALLIUM_YOUNG_MODULUS			8.0
#define Z_THALLIUM_POISSON_RATIO			0.45
#define Z_THALLIUM_REFRACTIVE_INDEX			REAL_MISSING
#define Z_THALLIUM_SOUND_SPEED				818.0
#define Z_THALLIUM_ELECTRICAL_TYPE			Z_ELECTRICAL_TYPE_CONDUCTOR
#define Z_THALLIUM_ELECTRICAL_CONDUCTIVITY		6.7e6
#define Z_THALLIUM_ELECTRICAL_RESISTIVITY		1.5e-7
#define Z_THALLIUM_SUPERCONDUCTING_POINT		2.38
#define Z_THALLIUM_MAGNETIC_TYPE			Z_MAGNETIC_TYPE_DIAMAGNETIC
#define Z_THALLIUM_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_THALLIUM_MASS_MAGNETIC_SUSCEPTIBILITY		-3.0e-9
#define Z_THALLIUM_MOLAR_MAGNETIC_SUSCEPTIBILITY	-6.13e-10
#define Z_THALLIUM_VOLUME_MAGNETIC_SUSCEPTIBILITY	-0.0000356

/* MARK: - 82 - Pb */

#define Z_LEAD_SYMBOL					Pb
#define Z_LEAD_ATOMIC_NUMBER				82
#define Z_LEAD_GROUP					14
#define Z_LEAD_PERIOD					6
#define Z_LEAD_BLOCK					Z_CHEMICAL_ELEMENT_BLOCK_P
#define Z_LEAD_CLASS					Z_CHEMICAL_ELEMENT_CLASS_POOR_METAL
#define Z_LEAD_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_LEAD_SPACE_GROUP_NUMBER			225
#define Z_LEAD_ATOMIC_MASS				207.2
#define Z_LEAD_ATOMIC_RADIUS				154.0
#define Z_LEAD_COVALENT_RADIUS				146.0
#define Z_LEAD_VAN_DER_WAALS_RADIUS			202.0
#define Z_LEAD_DECAY_MODE				KEY_NOT_APPLICABLE
#define Z_LEAD_NEUTRON_CROSS_SECTION			0.171
#define Z_LEAD_NEUTRON_MASS_ABSORPTION			0.00003
#define Z_LEAD_VALENCE					4
#define Z_LEAD_ELECTRONEGATIVITY			2.33
#define Z_LEAD_ELECTRON_AFFINITY			35.1
#define Z_LEAD_MELTING_POINT				327.46
#define Z_LEAD_BOILING_POINT				1749.0
#define Z_LEAD_CRITICAL_PRESSURE			REAL_MISSING
#define Z_LEAD_CRITICAL_TEMPERATURE			REAL_MISSING
#define Z_LEAD_FUSION_HEAT				4.77
#define Z_LEAD_VAPORIZATION_HEAT			178.0
#define Z_LEAD_SPECIFIC_HEAT				127.0
#define Z_LEAD_ADIABATIC_INDEX				KEY_NOT_APPLICABLE
#define Z_LEAD_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_LEAD_THERMAL_CONDUCTIVITY			35.0
#define Z_LEAD_THERMAL_EXPANSION			0.0000289
#define Z_LEAD_DENSITY					11340.0
#define Z_LEAD_LIQUID_DENSITY				10660.0
#define Z_LEAD_MOLAR_VOLUME				0.00001827
#define Z_LEAD_BRINELL_HARDNESS				38.3
#define Z_LEAD_MOHS_HARDNESS				1.5
#define Z_LEAD_VICKERS_HARDNESS				REAL_MISSING
#define Z_LEAD_BULK_MODULUS				46.0
#define Z_LEAD_SHEAR_MODULUS				5.6
#define Z_LEAD_YOUNG_MODULUS				16.0
#define Z_LEAD_POISSON_RATIO				0.44
#define Z_LEAD_REFRACTIVE_INDEX				REAL_MISSING
#define Z_LEAD_SOUND_SPEED				1260.0
#define Z_LEAD_ELECTRICAL_TYPE				Z_ELECTRICAL_TYPE_CONDUCTOR
#define Z_LEAD_ELECTRICAL_CONDUCTIVITY			4.8e6
#define Z_LEAD_ELECTRICAL_RESISTIVITY			2.1e-7
#define Z_LEAD_SUPERCONDUCTING_POINT			7.2
#define Z_LEAD_MAGNETIC_TYPE				Z_MAGNETIC_TYPE_DIAMAGNETIC
#define Z_LEAD_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_LEAD_MASS_MAGNETIC_SUSCEPTIBILITY		-1.5e-9
#define Z_LEAD_MOLAR_MAGNETIC_SUSCEPTIBILITY		-3.11e-10
#define Z_LEAD_VOLUME_MAGNETIC_SUSCEPTIBILITY		-0.000017

/* MARK: - 83 - Bi */

#define Z_BISMUTH_SYMBOL				Bi
#define Z_BISMUTH_ATOMIC_NUMBER				83
#define Z_BISMUTH_GROUP					15
#define Z_BISMUTH_PERIOD				6
#define Z_BISMUTH_BLOCK					Z_CHEMICAL_ELEMENT_BLOCK_P
#define Z_BISMUTH_CLASS					Z_CHEMICAL_ELEMENT_CLASS_POOR_METAL
#define Z_BISMUTH_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_BISMUTH_SPACE_GROUP_NUMBER			12
#define Z_BISMUTH_ATOMIC_MASS				208.98040
#define Z_BISMUTH_ATOMIC_RADIUS				143.0
#define Z_BISMUTH_COVALENT_RADIUS			148.0
#define Z_BISMUTH_VAN_DER_WAALS_RADIUS			REAL_MISSING
#define Z_BISMUTH_DECAY_MODE				Z_CHEMICAL_ELEMENT_DECAY_MODE_ALPHA_EMISSION
#define Z_BISMUTH_NEUTRON_CROSS_SECTION			0.034
#define Z_BISMUTH_NEUTRON_MASS_ABSORPTION		6.0e-6
#define Z_BISMUTH_VALENCE				5
#define Z_BISMUTH_ELECTRONEGATIVITY			2.02
#define Z_BISMUTH_ELECTRON_AFFINITY			91.2
#define Z_BISMUTH_MELTING_POINT				271.3
#define Z_BISMUTH_BOILING_POINT				1564.0
#define Z_BISMUTH_CRITICAL_PRESSURE			REAL_MISSING
#define Z_BISMUTH_CRITICAL_TEMPERATURE			REAL_MISSING
#define Z_BISMUTH_FUSION_HEAT				10.9
#define Z_BISMUTH_VAPORIZATION_HEAT			160.0
#define Z_BISMUTH_SPECIFIC_HEAT				122.0
#define Z_BISMUTH_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_BISMUTH_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_BISMUTH_THERMAL_CONDUCTIVITY			8.0
#define Z_BISMUTH_THERMAL_EXPANSION			0.0000134
#define Z_BISMUTH_DENSITY				9780.0
#define Z_BISMUTH_LIQUID_DENSITY			10050.0
#define Z_BISMUTH_MOLAR_VOLUME				0.00002137
#define Z_BISMUTH_BRINELL_HARDNESS			94.2
#define Z_BISMUTH_MOHS_HARDNESS				2.25
#define Z_BISMUTH_VICKERS_HARDNESS			REAL_MISSING
#define Z_BISMUTH_BULK_MODULUS				31.0
#define Z_BISMUTH_SHEAR_MODULUS				12.0
#define Z_BISMUTH_YOUNG_MODULUS				32.0
#define Z_BISMUTH_POISSON_RATIO				0.33
#define Z_BISMUTH_REFRACTIVE_INDEX			REAL_MISSING
#define Z_BISMUTH_SOUND_SPEED				1790.0
#define Z_BISMUTH_ELECTRICAL_TYPE			Z_ELECTRICAL_TYPE_CONDUCTOR
#define Z_BISMUTH_ELECTRICAL_CONDUCTIVITY		7.7e5
#define Z_BISMUTH_ELECTRICAL_RESISTIVITY		1.3e-6
#define Z_BISMUTH_SUPERCONDUCTING_POINT			REAL_MISSING
#define Z_BISMUTH_MAGNETIC_TYPE				Z_MAGNETIC_TYPE_DIAMAGNETIC
#define Z_BISMUTH_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_BISMUTH_MASS_MAGNETIC_SUSCEPTIBILITY		-1.7e-8
#define Z_BISMUTH_MOLAR_MAGNETIC_SUSCEPTIBILITY		-3.6e-9
#define Z_BISMUTH_VOLUME_MAGNETIC_SUSCEPTIBILITY	-0.00017

/* MARK: - 84 - Po */

#define Z_POLONIUM_SYMBOL				Po
#define Z_POLONIUM_ATOMIC_NUMBER			84
#define Z_POLONIUM_GROUP				16
#define Z_POLONIUM_PERIOD				6
#define Z_POLONIUM_BLOCK				Z_CHEMICAL_ELEMENT_BLOCK_P
#define Z_POLONIUM_CLASS				Z_CHEMICAL_ELEMENT_CLASS_METALLOID
#define Z_POLONIUM_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_METALLOID
#define Z_POLONIUM_SPACE_GROUP_NUMBER			221
#define Z_POLONIUM_ATOMIC_MASS				209.0
#define Z_POLONIUM_ATOMIC_RADIUS			135.0
#define Z_POLONIUM_COVALENT_RADIUS			140.0
#define Z_POLONIUM_VAN_DER_WAALS_RADIUS			REAL_MISSING
#define Z_POLONIUM_DECAY_MODE				Z_CHEMICAL_ELEMENT_DECAY_MODE_ALPHA_EMISSION
#define Z_POLONIUM_NEUTRON_CROSS_SECTION		0.5
#define Z_POLONIUM_NEUTRON_MASS_ABSORPTION		REAL_MISSING
#define Z_POLONIUM_VALENCE				6
#define Z_POLONIUM_ELECTRONEGATIVITY			2.0
#define Z_POLONIUM_ELECTRON_AFFINITY			183.3
#define Z_POLONIUM_MELTING_POINT			254.0
#define Z_POLONIUM_BOILING_POINT			962.0
#define Z_POLONIUM_CRITICAL_PRESSURE			REAL_MISSING
#define Z_POLONIUM_CRITICAL_TEMPERATURE			REAL_MISSING
#define Z_POLONIUM_FUSION_HEAT				13.0
#define Z_POLONIUM_VAPORIZATION_HEAT			1.0e2
#define Z_POLONIUM_SPECIFIC_HEAT			REAL_MISSING
#define Z_POLONIUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_POLONIUM_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_POLONIUM_THERMAL_CONDUCTIVITY			REAL_MISSING
#define Z_POLONIUM_THERMAL_EXPANSION			REAL_MISSING
#define Z_POLONIUM_DENSITY				9196.0
#define Z_POLONIUM_LIQUID_DENSITY			REAL_MISSING
#define Z_POLONIUM_MOLAR_VOLUME				0.00002273
#define Z_POLONIUM_BRINELL_HARDNESS			REAL_MISSING
#define Z_POLONIUM_MOHS_HARDNESS			REAL_MISSING
#define Z_POLONIUM_VICKERS_HARDNESS			REAL_MISSING
#define Z_POLONIUM_BULK_MODULUS				REAL_MISSING
#define Z_POLONIUM_SHEAR_MODULUS			REAL_MISSING
#define Z_POLONIUM_YOUNG_MODULUS			REAL_MISSING
#define Z_POLONIUM_POISSON_RATIO			REAL_MISSING
#define Z_POLONIUM_REFRACTIVE_INDEX			REAL_MISSING
#define Z_POLONIUM_SOUND_SPEED				REAL_MISSING
#define Z_POLONIUM_ELECTRICAL_TYPE			Z_ELECTRICAL_TYPE_CONDUCTOR
#define Z_POLONIUM_ELECTRICAL_CONDUCTIVITY		2.3e6
#define Z_POLONIUM_ELECTRICAL_RESISTIVITY		4.3e-7
#define Z_POLONIUM_SUPERCONDUCTING_POINT		REAL_MISSING
#define Z_POLONIUM_MAGNETIC_TYPE			KEY_MISSING
#define Z_POLONIUM_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_POLONIUM_MASS_MAGNETIC_SUSCEPTIBILITY		REAL_MISSING
#define Z_POLONIUM_MOLAR_MAGNETIC_SUSCEPTIBILITY	REAL_MISSING
#define Z_POLONIUM_VOLUME_MAGNETIC_SUSCEPTIBILITY	REAL_MISSING

/* MARK: - 85 - At */

#define Z_ASTATINE_SYMBOL				At
#define Z_ASTATINE_ATOMIC_NUMBER			85
#define Z_ASTATINE_GROUP				17
#define Z_ASTATINE_PERIOD				6
#define Z_ASTATINE_BLOCK				Z_CHEMICAL_ELEMENT_BLOCK_P
#define Z_ASTATINE_CLASS				Z_CHEMICAL_ELEMENT_CLASS_HALOGEN
#define Z_ASTATINE_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_NON_METAL
#define Z_ASTATINE_SPACE_GROUP_NUMBER			INTEGER_MISSING
#define Z_ASTATINE_ATOMIC_MASS				210.0
#define Z_ASTATINE_ATOMIC_RADIUS			127.0
#define Z_ASTATINE_COVALENT_RADIUS			150.0
#define Z_ASTATINE_VAN_DER_WAALS_RADIUS			REAL_MISSING
#define Z_ASTATINE_DECAY_MODE				Z_CHEMICAL_ELEMENT_DECAY_MODE_BETA_PLUS_DECAY
#define Z_ASTATINE_NEUTRON_CROSS_SECTION		REAL_MISSING
#define Z_ASTATINE_NEUTRON_MASS_ABSORPTION		REAL_MISSING
#define Z_ASTATINE_VALENCE				7
#define Z_ASTATINE_ELECTRONEGATIVITY			2.2
#define Z_ASTATINE_ELECTRON_AFFINITY			270.1
#define Z_ASTATINE_MELTING_POINT			302.0
#define Z_ASTATINE_BOILING_POINT			REAL_MISSING
#define Z_ASTATINE_CRITICAL_PRESSURE			REAL_MISSING
#define Z_ASTATINE_CRITICAL_TEMPERATURE			REAL_MISSING
#define Z_ASTATINE_FUSION_HEAT				6.0
#define Z_ASTATINE_VAPORIZATION_HEAT			40.0
#define Z_ASTATINE_SPECIFIC_HEAT			REAL_MISSING
#define Z_ASTATINE_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_ASTATINE_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_ASTATINE_THERMAL_CONDUCTIVITY			2.0
#define Z_ASTATINE_THERMAL_EXPANSION			REAL_MISSING
#define Z_ASTATINE_DENSITY				REAL_MISSING
#define Z_ASTATINE_LIQUID_DENSITY			REAL_MISSING
#define Z_ASTATINE_MOLAR_VOLUME				REAL_MISSING
#define Z_ASTATINE_BRINELL_HARDNESS			REAL_MISSING
#define Z_ASTATINE_MOHS_HARDNESS			REAL_MISSING
#define Z_ASTATINE_VICKERS_HARDNESS			REAL_MISSING
#define Z_ASTATINE_BULK_MODULUS				REAL_MISSING
#define Z_ASTATINE_SHEAR_MODULUS			REAL_MISSING
#define Z_ASTATINE_YOUNG_MODULUS			REAL_MISSING
#define Z_ASTATINE_POISSON_RATIO			REAL_MISSING
#define Z_ASTATINE_REFRACTIVE_INDEX			REAL_MISSING
#define Z_ASTATINE_SOUND_SPEED				REAL_MISSING
#define Z_ASTATINE_ELECTRICAL_TYPE			KEY_MISSING
#define Z_ASTATINE_ELECTRICAL_CONDUCTIVITY		REAL_MISSING
#define Z_ASTATINE_ELECTRICAL_RESISTIVITY		REAL_MISSING
#define Z_ASTATINE_SUPERCONDUCTING_POINT		REAL_MISSING
#define Z_ASTATINE_MAGNETIC_TYPE			KEY_MISSING
#define Z_ASTATINE_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_ASTATINE_MASS_MAGNETIC_SUSCEPTIBILITY		REAL_MISSING
#define Z_ASTATINE_MOLAR_MAGNETIC_SUSCEPTIBILITY	REAL_MISSING
#define Z_ASTATINE_VOLUME_MAGNETIC_SUSCEPTIBILITY	REAL_MISSING

/* MARK: - 86 - Rn */

#define Z_RADON_SYMBOL					Rn
#define Z_RADON_ATOMIC_NUMBER				86
#define Z_RADON_GROUP					18
#define Z_RADON_PERIOD					6
#define Z_RADON_BLOCK					Z_CHEMICAL_ELEMENT_BLOCK_P
#define Z_RADON_CLASS					Z_CHEMICAL_ELEMENT_CLASS_NOBLE_GAS
#define Z_RADON_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_NON_METAL
#define Z_RADON_SPACE_GROUP_NUMBER			INTEGER_MISSING
#define Z_RADON_ATOMIC_MASS				222.0
#define Z_RADON_ATOMIC_RADIUS				120.0
#define Z_RADON_COVALENT_RADIUS				150.0
#define Z_RADON_VAN_DER_WAALS_RADIUS			REAL_MISSING
#define Z_RADON_DECAY_MODE				Z_CHEMICAL_ELEMENT_DECAY_MODE_ALPHA_EMISSION
#define Z_RADON_NEUTRON_CROSS_SECTION			0.7
#define Z_RADON_NEUTRON_MASS_ABSORPTION			REAL_MISSING
#define Z_RADON_VALENCE					6
#define Z_RADON_ELECTRONEGATIVITY			REAL_NOT_APPLICABLE
#define Z_RADON_ELECTRON_AFFINITY			0.0
#define Z_RADON_MELTING_POINT				-71.0
#define Z_RADON_BOILING_POINT				-61.7
#define Z_RADON_CRITICAL_PRESSURE			6.28
#define Z_RADON_CRITICAL_TEMPERATURE			377.0
#define Z_RADON_FUSION_HEAT				3.0
#define Z_RADON_VAPORIZATION_HEAT			17.0
#define Z_RADON_SPECIFIC_HEAT				93.65
#define Z_RADON_ADIABATIC_INDEX				Z_CHEMICAL_ELEMENT_ADIABATIC_INDEX_5_3
#define Z_RADON_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_RADON_THERMAL_CONDUCTIVITY			0.00361
#define Z_RADON_THERMAL_EXPANSION			REAL_MISSING
#define Z_RADON_DENSITY					9.73
#define Z_RADON_LIQUID_DENSITY				REAL_MISSING
#define Z_RADON_MOLAR_VOLUME				0.0228
#define Z_RADON_BRINELL_HARDNESS			REAL_NOT_APPLICABLE
#define Z_RADON_MOHS_HARDNESS				REAL_NOT_APPLICABLE
#define Z_RADON_VICKERS_HARDNESS			REAL_MISSING
#define Z_RADON_BULK_MODULUS				REAL_NOT_APPLICABLE
#define Z_RADON_SHEAR_MODULUS				REAL_NOT_APPLICABLE
#define Z_RADON_YOUNG_MODULUS				REAL_MISSING
#define Z_RADON_POISSON_RATIO				REAL_MISSING
#define Z_RADON_REFRACTIVE_INDEX			REAL_MISSING
#define Z_RADON_SOUND_SPEED				REAL_MISSING
#define Z_RADON_ELECTRICAL_TYPE				KEY_MISSING
#define Z_RADON_ELECTRICAL_CONDUCTIVITY			REAL_MISSING
#define Z_RADON_ELECTRICAL_RESISTIVITY			REAL_NOT_APPLICABLE
#define Z_RADON_SUPERCONDUCTING_POINT			REAL_MISSING
#define Z_RADON_MAGNETIC_TYPE				KEY_MISSING
#define Z_RADON_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_RADON_MASS_MAGNETIC_SUSCEPTIBILITY		REAL_MISSING
#define Z_RADON_MOLAR_MAGNETIC_SUSCEPTIBILITY		REAL_MISSING
#define Z_RADON_VOLUME_MAGNETIC_SUSCEPTIBILITY		REAL_MISSING

/* MARK: - 87 - Fr */

#define Z_FRANCIUM_SYMBOL				Fr
#define Z_FRANCIUM_ATOMIC_NUMBER			87
#define Z_FRANCIUM_GROUP				1
#define Z_FRANCIUM_PERIOD				7
#define Z_FRANCIUM_BLOCK				Z_CHEMICAL_ELEMENT_BLOCK_S
#define Z_FRANCIUM_CLASS				Z_CHEMICAL_ELEMENT_CLASS_ALKALI_METAL
#define Z_FRANCIUM_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_FRANCIUM_SPACE_GROUP_NUMBER			INTEGER_MISSING
#define Z_FRANCIUM_ATOMIC_MASS				223.0
#define Z_FRANCIUM_ATOMIC_RADIUS			REAL_MISSING
#define Z_FRANCIUM_COVALENT_RADIUS			260.0
#define Z_FRANCIUM_VAN_DER_WAALS_RADIUS			REAL_MISSING
#define Z_FRANCIUM_DECAY_MODE				Z_CHEMICAL_ELEMENT_DECAY_MODE_BETA_DECAY
#define Z_FRANCIUM_NEUTRON_CROSS_SECTION		REAL_MISSING
#define Z_FRANCIUM_NEUTRON_MASS_ABSORPTION		REAL_MISSING
#define Z_FRANCIUM_VALENCE				1
#define Z_FRANCIUM_ELECTRONEGATIVITY			0.7
#define Z_FRANCIUM_ELECTRON_AFFINITY			REAL_MISSING
#define Z_FRANCIUM_MELTING_POINT			REAL_MISSING
#define Z_FRANCIUM_BOILING_POINT			REAL_MISSING
#define Z_FRANCIUM_CRITICAL_PRESSURE			REAL_MISSING
#define Z_FRANCIUM_CRITICAL_TEMPERATURE			REAL_MISSING
#define Z_FRANCIUM_FUSION_HEAT				2.0
#define Z_FRANCIUM_VAPORIZATION_HEAT			64.0
#define Z_FRANCIUM_SPECIFIC_HEAT			REAL_MISSING
#define Z_FRANCIUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_FRANCIUM_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_FRANCIUM_THERMAL_CONDUCTIVITY			REAL_MISSING
#define Z_FRANCIUM_THERMAL_EXPANSION			REAL_MISSING
#define Z_FRANCIUM_DENSITY				REAL_MISSING
#define Z_FRANCIUM_LIQUID_DENSITY			REAL_MISSING
#define Z_FRANCIUM_MOLAR_VOLUME				REAL_MISSING
#define Z_FRANCIUM_BRINELL_HARDNESS			REAL_MISSING
#define Z_FRANCIUM_MOHS_HARDNESS			REAL_MISSING
#define Z_FRANCIUM_VICKERS_HARDNESS			REAL_MISSING
#define Z_FRANCIUM_BULK_MODULUS				REAL_MISSING
#define Z_FRANCIUM_SHEAR_MODULUS			REAL_MISSING
#define Z_FRANCIUM_YOUNG_MODULUS			REAL_MISSING
#define Z_FRANCIUM_POISSON_RATIO			REAL_MISSING
#define Z_FRANCIUM_REFRACTIVE_INDEX			REAL_MISSING
#define Z_FRANCIUM_SOUND_SPEED				REAL_MISSING
#define Z_FRANCIUM_ELECTRICAL_TYPE			KEY_MISSING
#define Z_FRANCIUM_ELECTRICAL_CONDUCTIVITY		REAL_MISSING
#define Z_FRANCIUM_ELECTRICAL_RESISTIVITY		REAL_MISSING
#define Z_FRANCIUM_SUPERCONDUCTING_POINT		REAL_MISSING
#define Z_FRANCIUM_MAGNETIC_TYPE			KEY_MISSING
#define Z_FRANCIUM_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_FRANCIUM_MASS_MAGNETIC_SUSCEPTIBILITY		REAL_MISSING
#define Z_FRANCIUM_MOLAR_MAGNETIC_SUSCEPTIBILITY	REAL_MISSING
#define Z_FRANCIUM_VOLUME_MAGNETIC_SUSCEPTIBILITY	REAL_MISSING

/* MARK: - 88 - Ra */

#define Z_RADIUM_SYMBOL					Ra
#define Z_RADIUM_ATOMIC_NUMBER				88
#define Z_RADIUM_GROUP					2
#define Z_RADIUM_PERIOD					7
#define Z_RADIUM_BLOCK					Z_CHEMICAL_ELEMENT_BLOCK_S
#define Z_RADIUM_CLASS					Z_CHEMICAL_ELEMENT_CLASS_ALKALINE_EARTH_METAL
#define Z_RADIUM_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_RADIUM_SPACE_GROUP_NUMBER			229
#define Z_RADIUM_ATOMIC_MASS				226.0
#define Z_RADIUM_ATOMIC_RADIUS				REAL_MISSING
#define Z_RADIUM_COVALENT_RADIUS			221.0
#define Z_RADIUM_VAN_DER_WAALS_RADIUS			REAL_MISSING
#define Z_RADIUM_DECAY_MODE				Z_CHEMICAL_ELEMENT_DECAY_MODE_ALPHA_EMISSION
#define Z_RADIUM_NEUTRON_CROSS_SECTION			13.0
#define Z_RADIUM_NEUTRON_MASS_ABSORPTION		REAL_MISSING
#define Z_RADIUM_VALENCE				2
#define Z_RADIUM_ELECTRONEGATIVITY			0.9
#define Z_RADIUM_ELECTRON_AFFINITY			REAL_MISSING
#define Z_RADIUM_MELTING_POINT				7.0e2
#define Z_RADIUM_BOILING_POINT				1737.0
#define Z_RADIUM_CRITICAL_PRESSURE			REAL_MISSING
#define Z_RADIUM_CRITICAL_TEMPERATURE			REAL_MISSING
#define Z_RADIUM_FUSION_HEAT				8.0
#define Z_RADIUM_VAPORIZATION_HEAT			125.0
#define Z_RADIUM_SPECIFIC_HEAT				92.0
#define Z_RADIUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_RADIUM_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_RADIUM_THERMAL_CONDUCTIVITY			19.0
#define Z_RADIUM_THERMAL_EXPANSION			REAL_MISSING
#define Z_RADIUM_DENSITY				5.0e3
#define Z_RADIUM_LIQUID_DENSITY				REAL_MISSING
#define Z_RADIUM_MOLAR_VOLUME				0.000045
#define Z_RADIUM_BRINELL_HARDNESS			REAL_MISSING
#define Z_RADIUM_MOHS_HARDNESS				REAL_MISSING
#define Z_RADIUM_VICKERS_HARDNESS			REAL_MISSING
#define Z_RADIUM_BULK_MODULUS				REAL_MISSING
#define Z_RADIUM_SHEAR_MODULUS				REAL_MISSING
#define Z_RADIUM_YOUNG_MODULUS				REAL_MISSING
#define Z_RADIUM_POISSON_RATIO				REAL_MISSING
#define Z_RADIUM_REFRACTIVE_INDEX			REAL_MISSING
#define Z_RADIUM_SOUND_SPEED				REAL_MISSING
#define Z_RADIUM_ELECTRICAL_TYPE			Z_ELECTRICAL_TYPE_CONDUCTOR
#define Z_RADIUM_ELECTRICAL_CONDUCTIVITY		1.0e6
#define Z_RADIUM_ELECTRICAL_RESISTIVITY			1.0e-6
#define Z_RADIUM_SUPERCONDUCTING_POINT			REAL_MISSING
#define Z_RADIUM_MAGNETIC_TYPE				KEY_MISSING
#define Z_RADIUM_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_RADIUM_MASS_MAGNETIC_SUSCEPTIBILITY		REAL_MISSING
#define Z_RADIUM_MOLAR_MAGNETIC_SUSCEPTIBILITY		REAL_MISSING
#define Z_RADIUM_VOLUME_MAGNETIC_SUSCEPTIBILITY		REAL_MISSING

/* MARK: - 89 - Ac */

#define Z_ACTINIUM_SYMBOL				Ac
#define Z_ACTINIUM_ATOMIC_NUMBER			89
#define Z_ACTINIUM_GROUP				INTEGER_NOT_APPLICABLE
#define Z_ACTINIUM_PERIOD				7
#define Z_ACTINIUM_BLOCK				Z_CHEMICAL_ELEMENT_BLOCK_F
#define Z_ACTINIUM_CLASS				Z_CHEMICAL_ELEMENT_CLASS_ACTINIDE
#define Z_ACTINIUM_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_ACTINIUM_SPACE_GROUP_NUMBER			225
#define Z_ACTINIUM_ATOMIC_MASS				227.0
#define Z_ACTINIUM_ATOMIC_RADIUS			REAL_MISSING
#define Z_ACTINIUM_COVALENT_RADIUS			215.0
#define Z_ACTINIUM_VAN_DER_WAALS_RADIUS			REAL_MISSING
#define Z_ACTINIUM_DECAY_MODE				Z_CHEMICAL_ELEMENT_DECAY_MODE_BETA_DECAY
#define Z_ACTINIUM_NEUTRON_CROSS_SECTION		8.8e2
#define Z_ACTINIUM_NEUTRON_MASS_ABSORPTION		0.078
#define Z_ACTINIUM_VALENCE				3
#define Z_ACTINIUM_ELECTRONEGATIVITY			1.1
#define Z_ACTINIUM_ELECTRON_AFFINITY			REAL_MISSING
#define Z_ACTINIUM_MELTING_POINT			1050.0
#define Z_ACTINIUM_BOILING_POINT			3200.0
#define Z_ACTINIUM_CRITICAL_PRESSURE			REAL_MISSING
#define Z_ACTINIUM_CRITICAL_TEMPERATURE			REAL_MISSING
#define Z_ACTINIUM_FUSION_HEAT				14.0
#define Z_ACTINIUM_VAPORIZATION_HEAT			4.0e2
#define Z_ACTINIUM_SPECIFIC_HEAT			120.0
#define Z_ACTINIUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_ACTINIUM_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_ACTINIUM_THERMAL_CONDUCTIVITY			12.0
#define Z_ACTINIUM_THERMAL_EXPANSION			REAL_MISSING
#define Z_ACTINIUM_DENSITY				10070.0
#define Z_ACTINIUM_LIQUID_DENSITY			REAL_MISSING
#define Z_ACTINIUM_MOLAR_VOLUME				0.000022542
#define Z_ACTINIUM_BRINELL_HARDNESS			REAL_MISSING
#define Z_ACTINIUM_MOHS_HARDNESS			REAL_MISSING
#define Z_ACTINIUM_VICKERS_HARDNESS			REAL_MISSING
#define Z_ACTINIUM_BULK_MODULUS				REAL_MISSING
#define Z_ACTINIUM_SHEAR_MODULUS			REAL_MISSING
#define Z_ACTINIUM_YOUNG_MODULUS			REAL_MISSING
#define Z_ACTINIUM_POISSON_RATIO			REAL_MISSING
#define Z_ACTINIUM_REFRACTIVE_INDEX			REAL_MISSING
#define Z_ACTINIUM_SOUND_SPEED				REAL_MISSING
#define Z_ACTINIUM_ELECTRICAL_TYPE			KEY_MISSING
#define Z_ACTINIUM_ELECTRICAL_CONDUCTIVITY		REAL_MISSING
#define Z_ACTINIUM_ELECTRICAL_RESISTIVITY		REAL_MISSING
#define Z_ACTINIUM_SUPERCONDUCTING_POINT		REAL_MISSING
#define Z_ACTINIUM_MAGNETIC_TYPE			KEY_MISSING
#define Z_ACTINIUM_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_ACTINIUM_MASS_MAGNETIC_SUSCEPTIBILITY		REAL_MISSING
#define Z_ACTINIUM_MOLAR_MAGNETIC_SUSCEPTIBILITY	REAL_MISSING
#define Z_ACTINIUM_VOLUME_MAGNETIC_SUSCEPTIBILITY	REAL_MISSING

/* MARK: - 90 - Th */

#define Z_THORIUM_SYMBOL				Th
#define Z_THORIUM_ATOMIC_NUMBER				90
#define Z_THORIUM_GROUP					INTEGER_NOT_APPLICABLE
#define Z_THORIUM_PERIOD				7
#define Z_THORIUM_BLOCK					Z_CHEMICAL_ELEMENT_BLOCK_F
#define Z_THORIUM_CLASS					Z_CHEMICAL_ELEMENT_CLASS_ACTINIDE
#define Z_THORIUM_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_THORIUM_SPACE_GROUP_NUMBER			225
#define Z_THORIUM_ATOMIC_MASS				232.03806
#define Z_THORIUM_ATOMIC_RADIUS				REAL_UNKNOWN
#define Z_THORIUM_COVALENT_RADIUS			206.0
#define Z_THORIUM_VAN_DER_WAALS_RADIUS			REAL_UNKNOWN
#define Z_THORIUM_DECAY_MODE				Z_CHEMICAL_ELEMENT_DECAY_MODE_ALPHA_EMISSION
#define Z_THORIUM_NEUTRON_CROSS_SECTION			7.4
#define Z_THORIUM_NEUTRON_MASS_ABSORPTION		0.001
#define Z_THORIUM_VALENCE				4
#define Z_THORIUM_ELECTRONEGATIVITY			1.3
#define Z_THORIUM_ELECTRON_AFFINITY			REAL_UNKNOWN
#define Z_THORIUM_MELTING_POINT				1750.0
#define Z_THORIUM_BOILING_POINT				4820.0
#define Z_THORIUM_CRITICAL_PRESSURE			REAL_UNKNOWN
#define Z_THORIUM_CRITICAL_TEMPERATURE			REAL_UNKNOWN
#define Z_THORIUM_FUSION_HEAT				16.0
#define Z_THORIUM_VAPORIZATION_HEAT			530.0
#define Z_THORIUM_SPECIFIC_HEAT				118.0
#define Z_THORIUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_THORIUM_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_THORIUM_THERMAL_CONDUCTIVITY			54.0
#define Z_THORIUM_THERMAL_EXPANSION			0.000011
#define Z_THORIUM_DENSITY				11724.0
#define Z_THORIUM_LIQUID_DENSITY			REAL_UNKNOWN
#define Z_THORIUM_MOLAR_VOLUME				0.000019792
#define Z_THORIUM_BRINELL_HARDNESS			4.0e2
#define Z_THORIUM_MOHS_HARDNESS				3.0
#define Z_THORIUM_VICKERS_HARDNESS			350.0
#define Z_THORIUM_BULK_MODULUS				54.0
#define Z_THORIUM_SHEAR_MODULUS				31.0
#define Z_THORIUM_YOUNG_MODULUS				79.0
#define Z_THORIUM_POISSON_RATIO				0.27
#define Z_THORIUM_REFRACTIVE_INDEX			REAL_UNKNOWN
#define Z_THORIUM_SOUND_SPEED				2490.0
#define Z_THORIUM_ELECTRICAL_TYPE			Z_ELECTRICAL_TYPE_CONDUCTOR
#define Z_THORIUM_ELECTRICAL_CONDUCTIVITY		6.7e6
#define Z_THORIUM_ELECTRICAL_RESISTIVITY		1.5e-7
#define Z_THORIUM_SUPERCONDUCTING_POINT			1.38
#define Z_THORIUM_MAGNETIC_TYPE				Z_MAGNETIC_TYPE_PARAMAGNETIC
#define Z_THORIUM_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_THORIUM_MASS_MAGNETIC_SUSCEPTIBILITY		7.2e-9
#define Z_THORIUM_MOLAR_MAGNETIC_SUSCEPTIBILITY		1.7e-9
#define Z_THORIUM_VOLUME_MAGNETIC_SUSCEPTIBILITY	0.000084

/* MARK: - 91 - Pa */

#define Z_PROTACTINIUM_SYMBOL				Pa
#define Z_PROTACTINIUM_ATOMIC_NUMBER			91
#define Z_PROTACTINIUM_GROUP				INTEGER_NOT_APPLICABLE
#define Z_PROTACTINIUM_PERIOD				7
#define Z_PROTACTINIUM_BLOCK				Z_CHEMICAL_ELEMENT_BLOCK_F
#define Z_PROTACTINIUM_CLASS				Z_CHEMICAL_ELEMENT_CLASS_ACTINIDE
#define Z_PROTACTINIUM_METAL_STATE			Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_PROTACTINIUM_SPACE_GROUP_NUMBER		139
#define Z_PROTACTINIUM_ATOMIC_MASS			231.03586
#define Z_PROTACTINIUM_ATOMIC_RADIUS			REAL_UNKNOWN
#define Z_PROTACTINIUM_COVALENT_RADIUS			200.0
#define Z_PROTACTINIUM_VAN_DER_WAALS_RADIUS		REAL_UNKNOWN
#define Z_PROTACTINIUM_DECAY_MODE			Z_CHEMICAL_ELEMENT_DECAY_MODE_ALPHA_EMISSION
#define Z_PROTACTINIUM_NEUTRON_CROSS_SECTION		2.0e2
#define Z_PROTACTINIUM_NEUTRON_MASS_ABSORPTION		REAL_MISSING
#define Z_PROTACTINIUM_VALENCE				5
#define Z_PROTACTINIUM_ELECTRONEGATIVITY		1.5
#define Z_PROTACTINIUM_ELECTRON_AFFINITY		REAL_UNKNOWN
#define Z_PROTACTINIUM_MELTING_POINT			1572.0
#define Z_PROTACTINIUM_BOILING_POINT			4000.0
#define Z_PROTACTINIUM_CRITICAL_PRESSURE		REAL_UNKNOWN
#define Z_PROTACTINIUM_CRITICAL_TEMPERATURE		REAL_UNKNOWN
#define Z_PROTACTINIUM_FUSION_HEAT			15.0
#define Z_PROTACTINIUM_VAPORIZATION_HEAT		470.0
#define Z_PROTACTINIUM_SPECIFIC_HEAT			99.1
#define Z_PROTACTINIUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_PROTACTINIUM_NEEL_POINT			REAL_NOT_APPLICABLE
#define Z_PROTACTINIUM_THERMAL_CONDUCTIVITY		47.0
#define Z_PROTACTINIUM_THERMAL_EXPANSION		REAL_UNKNOWN
#define Z_PROTACTINIUM_DENSITY				15370.0
#define Z_PROTACTINIUM_LIQUID_DENSITY			REAL_UNKNOWN
#define Z_PROTACTINIUM_MOLAR_VOLUME			0.000015032
#define Z_PROTACTINIUM_BRINELL_HARDNESS			REAL_UNKNOWN
#define Z_PROTACTINIUM_MOHS_HARDNESS			REAL_UNKNOWN
#define Z_PROTACTINIUM_VICKERS_HARDNESS			REAL_UNKNOWN
#define Z_PROTACTINIUM_BULK_MODULUS			REAL_UNKNOWN
#define Z_PROTACTINIUM_SHEAR_MODULUS			REAL_UNKNOWN
#define Z_PROTACTINIUM_YOUNG_MODULUS			REAL_UNKNOWN
#define Z_PROTACTINIUM_POISSON_RATIO			REAL_UNKNOWN
#define Z_PROTACTINIUM_REFRACTIVE_INDEX			REAL_UNKNOWN
#define Z_PROTACTINIUM_SOUND_SPEED			REAL_UNKNOWN
#define Z_PROTACTINIUM_ELECTRICAL_TYPE			Z_ELECTRICAL_TYPE_CONDUCTOR
#define Z_PROTACTINIUM_ELECTRICAL_CONDUCTIVITY		5.6e6
#define Z_PROTACTINIUM_ELECTRICAL_RESISTIVITY		1.8e-7
#define Z_PROTACTINIUM_SUPERCONDUCTING_POINT		1.4
#define Z_PROTACTINIUM_MAGNETIC_TYPE			Z_MAGNETIC_TYPE_PARAMAGNETIC
#define Z_PROTACTINIUM_CURIE_POINT			REAL_NOT_APPLICABLE
#define Z_PROTACTINIUM_MASS_MAGNETIC_SUSCEPTIBILITY	3.25e-8
#define Z_PROTACTINIUM_MOLAR_MAGNETIC_SUSCEPTIBILITY	7.509e-9
#define Z_PROTACTINIUM_VOLUME_MAGNETIC_SUSCEPTIBILITY	0.0004995

/* MARK: - 92 - U */

#define Z_URANIUM_SYMBOL				U
#define Z_URANIUM_ATOMIC_NUMBER				92
#define Z_URANIUM_GROUP					INTEGER_NOT_APPLICABLE
#define Z_URANIUM_PERIOD				7
#define Z_URANIUM_BLOCK					Z_CHEMICAL_ELEMENT_BLOCK_F
#define Z_URANIUM_CLASS					Z_CHEMICAL_ELEMENT_CLASS_ACTINIDE
#define Z_URANIUM_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_URANIUM_SPACE_GROUP_NUMBER			63
#define Z_URANIUM_ATOMIC_MASS				238.02891
#define Z_URANIUM_ATOMIC_RADIUS				REAL_UNKNOWN
#define Z_URANIUM_COVALENT_RADIUS			196.0
#define Z_URANIUM_VAN_DER_WAALS_RADIUS			186.0
#define Z_URANIUM_DECAY_MODE				Z_CHEMICAL_ELEMENT_DECAY_MODE_ALPHA_EMISSION
#define Z_URANIUM_NEUTRON_CROSS_SECTION			7.57
#define Z_URANIUM_NEUTRON_MASS_ABSORPTION		0.0005
#define Z_URANIUM_VALENCE				6
#define Z_URANIUM_ELECTRONEGATIVITY			1.38
#define Z_URANIUM_ELECTRON_AFFINITY			REAL_UNKNOWN
#define Z_URANIUM_MELTING_POINT				1135.0
#define Z_URANIUM_BOILING_POINT				3927.0
#define Z_URANIUM_CRITICAL_PRESSURE			REAL_UNKNOWN
#define Z_URANIUM_CRITICAL_TEMPERATURE			REAL_UNKNOWN
#define Z_URANIUM_FUSION_HEAT				14.0
#define Z_URANIUM_VAPORIZATION_HEAT			420.0
#define Z_URANIUM_SPECIFIC_HEAT				116.0
#define Z_URANIUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_URANIUM_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_URANIUM_THERMAL_CONDUCTIVITY			27.0
#define Z_URANIUM_THERMAL_EXPANSION			0.0000139
#define Z_URANIUM_DENSITY				19050.0
#define Z_URANIUM_LIQUID_DENSITY			17300.0
#define Z_URANIUM_MOLAR_VOLUME				0.000012495
#define Z_URANIUM_BRINELL_HARDNESS			2.4e3
#define Z_URANIUM_MOHS_HARDNESS				6.0
#define Z_URANIUM_VICKERS_HARDNESS			1960.0
#define Z_URANIUM_BULK_MODULUS				1.0e2
#define Z_URANIUM_SHEAR_MODULUS				111.0
#define Z_URANIUM_YOUNG_MODULUS				208.0
#define Z_URANIUM_POISSON_RATIO				0.23
#define Z_URANIUM_REFRACTIVE_INDEX			REAL_UNKNOWN
#define Z_URANIUM_SOUND_SPEED				3155.0
#define Z_URANIUM_ELECTRICAL_TYPE			Z_ELECTRICAL_TYPE_CONDUCTOR
#define Z_URANIUM_ELECTRICAL_CONDUCTIVITY		3.6e6
#define Z_URANIUM_ELECTRICAL_RESISTIVITY		2.8e-7
#define Z_URANIUM_SUPERCONDUCTING_POINT			0.69
#define Z_URANIUM_MAGNETIC_TYPE				Z_MAGNETIC_TYPE_PARAMAGNETIC
#define Z_URANIUM_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_URANIUM_MASS_MAGNETIC_SUSCEPTIBILITY		2.16e-8
#define Z_URANIUM_MOLAR_MAGNETIC_SUSCEPTIBILITY		5.14e-9
#define Z_URANIUM_VOLUME_MAGNETIC_SUSCEPTIBILITY	0.000411

/* MARK: - 93 - Np */

#define Z_NEPTUNIUM_SYMBOL				Np
#define Z_NEPTUNIUM_ATOMIC_NUMBER			93
#define Z_NEPTUNIUM_GROUP				INTEGER_NOT_APPLICABLE
#define Z_NEPTUNIUM_PERIOD				7
#define Z_NEPTUNIUM_BLOCK				Z_CHEMICAL_ELEMENT_BLOCK_F
#define Z_NEPTUNIUM_CLASS				Z_CHEMICAL_ELEMENT_CLASS_ACTINIDE
#define Z_NEPTUNIUM_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_NEPTUNIUM_SPACE_GROUP_NUMBER			62
#define Z_NEPTUNIUM_ATOMIC_MASS				237.0
#define Z_NEPTUNIUM_ATOMIC_RADIUS			REAL_UNKNOWN
#define Z_NEPTUNIUM_COVALENT_RADIUS			190.0
#define Z_NEPTUNIUM_VAN_DER_WAALS_RADIUS		REAL_UNKNOWN
#define Z_NEPTUNIUM_DECAY_MODE				Z_CHEMICAL_ELEMENT_DECAY_MODE_ALPHA_EMISSION
#define Z_NEPTUNIUM_NEUTRON_CROSS_SECTION		180.0
#define Z_NEPTUNIUM_NEUTRON_MASS_ABSORPTION		REAL_MISSING
#define Z_NEPTUNIUM_VALENCE				6
#define Z_NEPTUNIUM_ELECTRONEGATIVITY			1.36
#define Z_NEPTUNIUM_ELECTRON_AFFINITY			REAL_UNKNOWN
#define Z_NEPTUNIUM_MELTING_POINT			644.0
#define Z_NEPTUNIUM_BOILING_POINT			4.0e3
#define Z_NEPTUNIUM_CRITICAL_PRESSURE			REAL_UNKNOWN
#define Z_NEPTUNIUM_CRITICAL_TEMPERATURE		REAL_UNKNOWN
#define Z_NEPTUNIUM_FUSION_HEAT				10.0
#define Z_NEPTUNIUM_VAPORIZATION_HEAT			335.0
#define Z_NEPTUNIUM_SPECIFIC_HEAT			REAL_MISSING
#define Z_NEPTUNIUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_NEPTUNIUM_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_NEPTUNIUM_THERMAL_CONDUCTIVITY		6.0
#define Z_NEPTUNIUM_THERMAL_EXPANSION			REAL_UNKNOWN
#define Z_NEPTUNIUM_DENSITY				20450.0
#define Z_NEPTUNIUM_LIQUID_DENSITY			REAL_UNKNOWN
#define Z_NEPTUNIUM_MOLAR_VOLUME			0.000011589
#define Z_NEPTUNIUM_BRINELL_HARDNESS			REAL_UNKNOWN
#define Z_NEPTUNIUM_MOHS_HARDNESS			REAL_UNKNOWN
#define Z_NEPTUNIUM_VICKERS_HARDNESS			REAL_UNKNOWN
#define Z_NEPTUNIUM_BULK_MODULUS			REAL_UNKNOWN
#define Z_NEPTUNIUM_SHEAR_MODULUS			REAL_UNKNOWN
#define Z_NEPTUNIUM_YOUNG_MODULUS			REAL_UNKNOWN
#define Z_NEPTUNIUM_POISSON_RATIO			REAL_UNKNOWN
#define Z_NEPTUNIUM_REFRACTIVE_INDEX			REAL_UNKNOWN
#define Z_NEPTUNIUM_SOUND_SPEED				REAL_UNKNOWN
#define Z_NEPTUNIUM_ELECTRICAL_TYPE			Z_ELECTRICAL_TYPE_CONDUCTOR
#define Z_NEPTUNIUM_ELECTRICAL_CONDUCTIVITY		8.3e5
#define Z_NEPTUNIUM_ELECTRICAL_RESISTIVITY		1.2e-6
#define Z_NEPTUNIUM_SUPERCONDUCTING_POINT		REAL_UNKNOWN
#define Z_NEPTUNIUM_MAGNETIC_TYPE			KEY_UNKNOWN
#define Z_NEPTUNIUM_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_NEPTUNIUM_MASS_MAGNETIC_SUSCEPTIBILITY	REAL_MISSING
#define Z_NEPTUNIUM_MOLAR_MAGNETIC_SUSCEPTIBILITY	REAL_MISSING
#define Z_NEPTUNIUM_VOLUME_MAGNETIC_SUSCEPTIBILITY	REAL_MISSING

/* MARK: - 94 - Pu */

#define Z_PLUTONIUM_SYMBOL				Pu
#define Z_PLUTONIUM_ATOMIC_NUMBER			94
#define Z_PLUTONIUM_GROUP				INTEGER_NOT_APPLICABLE
#define Z_PLUTONIUM_PERIOD				7
#define Z_PLUTONIUM_BLOCK				Z_CHEMICAL_ELEMENT_BLOCK_F
#define Z_PLUTONIUM_CLASS				Z_CHEMICAL_ELEMENT_CLASS_ACTINIDE
#define Z_PLUTONIUM_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_PLUTONIUM_SPACE_GROUP_NUMBER			11
#define Z_PLUTONIUM_ATOMIC_MASS				244.0
#define Z_PLUTONIUM_ATOMIC_RADIUS			REAL_UNKNOWN
#define Z_PLUTONIUM_COVALENT_RADIUS			187.0
#define Z_PLUTONIUM_VAN_DER_WAALS_RADIUS		REAL_UNKNOWN
#define Z_PLUTONIUM_DECAY_MODE				Z_CHEMICAL_ELEMENT_DECAY_MODE_ALPHA_EMISSION
#define Z_PLUTONIUM_NEUTRON_CROSS_SECTION		1.7
#define Z_PLUTONIUM_NEUTRON_MASS_ABSORPTION		REAL_MISSING
#define Z_PLUTONIUM_VALENCE				6
#define Z_PLUTONIUM_ELECTRONEGATIVITY			1.28
#define Z_PLUTONIUM_ELECTRON_AFFINITY			REAL_UNKNOWN
#define Z_PLUTONIUM_MELTING_POINT			640.0
#define Z_PLUTONIUM_BOILING_POINT			3230.0
#define Z_PLUTONIUM_CRITICAL_PRESSURE			REAL_UNKNOWN
#define Z_PLUTONIUM_CRITICAL_TEMPERATURE		REAL_UNKNOWN
#define Z_PLUTONIUM_FUSION_HEAT				REAL_UNKNOWN
#define Z_PLUTONIUM_VAPORIZATION_HEAT			325.0
#define Z_PLUTONIUM_SPECIFIC_HEAT			REAL_MISSING
#define Z_PLUTONIUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_PLUTONIUM_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_PLUTONIUM_THERMAL_CONDUCTIVITY		6.0
#define Z_PLUTONIUM_THERMAL_EXPANSION			REAL_UNKNOWN
#define Z_PLUTONIUM_DENSITY				19816.0
#define Z_PLUTONIUM_LIQUID_DENSITY			16630.0
#define Z_PLUTONIUM_MOLAR_VOLUME			0.000012313
#define Z_PLUTONIUM_BRINELL_HARDNESS			REAL_UNKNOWN
#define Z_PLUTONIUM_MOHS_HARDNESS			REAL_UNKNOWN
#define Z_PLUTONIUM_VICKERS_HARDNESS			REAL_UNKNOWN
#define Z_PLUTONIUM_BULK_MODULUS			REAL_UNKNOWN
#define Z_PLUTONIUM_SHEAR_MODULUS			43.0
#define Z_PLUTONIUM_YOUNG_MODULUS			96.0
#define Z_PLUTONIUM_POISSON_RATIO			0.21
#define Z_PLUTONIUM_REFRACTIVE_INDEX			REAL_UNKNOWN
#define Z_PLUTONIUM_SOUND_SPEED				2260.0
#define Z_PLUTONIUM_ELECTRICAL_TYPE			Z_ELECTRICAL_TYPE_CONDUCTOR
#define Z_PLUTONIUM_ELECTRICAL_CONDUCTIVITY		6.7e5
#define Z_PLUTONIUM_ELECTRICAL_RESISTIVITY		1.5e-6
#define Z_PLUTONIUM_SUPERCONDUCTING_POINT		REAL_UNKNOWN
#define Z_PLUTONIUM_MAGNETIC_TYPE			Z_MAGNETIC_TYPE_PARAMAGNETIC
#define Z_PLUTONIUM_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_PLUTONIUM_MASS_MAGNETIC_SUSCEPTIBILITY	3.17e-8
#define Z_PLUTONIUM_MOLAR_MAGNETIC_SUSCEPTIBILITY	7.735e-9
#define Z_PLUTONIUM_VOLUME_MAGNETIC_SUSCEPTIBILITY	0.0006282

/* MARK: - 95 - Am */

#define Z_AMERICIUM_SYMBOL				Am
#define Z_AMERICIUM_ATOMIC_NUMBER			95
#define Z_AMERICIUM_GROUP				INTEGER_NOT_APPLICABLE
#define Z_AMERICIUM_PERIOD				7
#define Z_AMERICIUM_BLOCK				Z_CHEMICAL_ELEMENT_BLOCK_F
#define Z_AMERICIUM_CLASS				Z_CHEMICAL_ELEMENT_CLASS_ACTINIDE
#define Z_AMERICIUM_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_AMERICIUM_SPACE_GROUP_NUMBER			194
#define Z_AMERICIUM_ATOMIC_MASS				243.0
#define Z_AMERICIUM_ATOMIC_RADIUS			REAL_UNKNOWN
#define Z_AMERICIUM_COVALENT_RADIUS			180.0
#define Z_AMERICIUM_VAN_DER_WAALS_RADIUS		REAL_UNKNOWN
#define Z_AMERICIUM_DECAY_MODE				Z_CHEMICAL_ELEMENT_DECAY_MODE_ALPHA_EMISSION
#define Z_AMERICIUM_NEUTRON_CROSS_SECTION		74.0
#define Z_AMERICIUM_NEUTRON_MASS_ABSORPTION		REAL_MISSING
#define Z_AMERICIUM_VALENCE				4
#define Z_AMERICIUM_ELECTRONEGATIVITY			1.3
#define Z_AMERICIUM_ELECTRON_AFFINITY			REAL_UNKNOWN
#define Z_AMERICIUM_MELTING_POINT			1176.0
#define Z_AMERICIUM_BOILING_POINT			2011.0
#define Z_AMERICIUM_CRITICAL_PRESSURE			REAL_UNKNOWN
#define Z_AMERICIUM_CRITICAL_TEMPERATURE		REAL_UNKNOWN
#define Z_AMERICIUM_FUSION_HEAT				REAL_UNKNOWN
#define Z_AMERICIUM_VAPORIZATION_HEAT			REAL_UNKNOWN
#define Z_AMERICIUM_SPECIFIC_HEAT			REAL_MISSING
#define Z_AMERICIUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_AMERICIUM_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_AMERICIUM_THERMAL_CONDUCTIVITY		10.0
#define Z_AMERICIUM_THERMAL_EXPANSION			REAL_UNKNOWN
#define Z_AMERICIUM_DENSITY				1.367e4
#define Z_AMERICIUM_LIQUID_DENSITY			REAL_UNKNOWN
#define Z_AMERICIUM_MOLAR_VOLUME			0.00001778
#define Z_AMERICIUM_BRINELL_HARDNESS			REAL_UNKNOWN
#define Z_AMERICIUM_MOHS_HARDNESS			REAL_UNKNOWN
#define Z_AMERICIUM_VICKERS_HARDNESS			REAL_UNKNOWN
#define Z_AMERICIUM_BULK_MODULUS			REAL_UNKNOWN
#define Z_AMERICIUM_SHEAR_MODULUS			REAL_UNKNOWN
#define Z_AMERICIUM_YOUNG_MODULUS			REAL_UNKNOWN
#define Z_AMERICIUM_POISSON_RATIO			REAL_UNKNOWN
#define Z_AMERICIUM_REFRACTIVE_INDEX			REAL_UNKNOWN
#define Z_AMERICIUM_SOUND_SPEED				REAL_UNKNOWN
#define Z_AMERICIUM_ELECTRICAL_TYPE			KEY_MISSING
#define Z_AMERICIUM_ELECTRICAL_CONDUCTIVITY		REAL_UNKNOWN
#define Z_AMERICIUM_ELECTRICAL_RESISTIVITY		REAL_UNKNOWN
#define Z_AMERICIUM_SUPERCONDUCTING_POINT		0.6
#define Z_AMERICIUM_MAGNETIC_TYPE			Z_MAGNETIC_TYPE_PARAMAGNETIC
#define Z_AMERICIUM_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_AMERICIUM_MASS_MAGNETIC_SUSCEPTIBILITY	5.150e-8
#define Z_AMERICIUM_MOLAR_MAGNETIC_SUSCEPTIBILITY	1.251e-8
#define Z_AMERICIUM_VOLUME_MAGNETIC_SUSCEPTIBILITY	0.000704

/* MARK: - 96 - Cm */

#define Z_CURIUM_SYMBOL					Cm
#define Z_CURIUM_ATOMIC_NUMBER				96
#define Z_CURIUM_GROUP					INTEGER_NOT_APPLICABLE
#define Z_CURIUM_PERIOD					7
#define Z_CURIUM_BLOCK					Z_CHEMICAL_ELEMENT_BLOCK_F
#define Z_CURIUM_CLASS					Z_CHEMICAL_ELEMENT_CLASS_ACTINIDE
#define Z_CURIUM_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_CURIUM_SPACE_GROUP_NUMBER			194
#define Z_CURIUM_ATOMIC_MASS				247.0
#define Z_CURIUM_ATOMIC_RADIUS				REAL_UNKNOWN
#define Z_CURIUM_COVALENT_RADIUS			169.0
#define Z_CURIUM_VAN_DER_WAALS_RADIUS			REAL_UNKNOWN
#define Z_CURIUM_DECAY_MODE				Z_CHEMICAL_ELEMENT_DECAY_MODE_ALPHA_EMISSION
#define Z_CURIUM_NEUTRON_CROSS_SECTION			60.0
#define Z_CURIUM_NEUTRON_MASS_ABSORPTION		REAL_MISSING
#define Z_CURIUM_VALENCE				4
#define Z_CURIUM_ELECTRONEGATIVITY			1.3
#define Z_CURIUM_ELECTRON_AFFINITY			REAL_UNKNOWN
#define Z_CURIUM_MELTING_POINT				1345.0
#define Z_CURIUM_BOILING_POINT				3110.0
#define Z_CURIUM_CRITICAL_PRESSURE			REAL_UNKNOWN
#define Z_CURIUM_CRITICAL_TEMPERATURE			REAL_UNKNOWN
#define Z_CURIUM_FUSION_HEAT				REAL_UNKNOWN
#define Z_CURIUM_VAPORIZATION_HEAT			REAL_UNKNOWN
#define Z_CURIUM_SPECIFIC_HEAT				REAL_MISSING
#define Z_CURIUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_CURIUM_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_CURIUM_THERMAL_CONDUCTIVITY			REAL_UNKNOWN
#define Z_CURIUM_THERMAL_EXPANSION			REAL_UNKNOWN
#define Z_CURIUM_DENSITY				13510.0
#define Z_CURIUM_LIQUID_DENSITY				REAL_UNKNOWN
#define Z_CURIUM_MOLAR_VOLUME				0.000018283
#define Z_CURIUM_BRINELL_HARDNESS			REAL_UNKNOWN
#define Z_CURIUM_MOHS_HARDNESS				REAL_UNKNOWN
#define Z_CURIUM_VICKERS_HARDNESS			REAL_UNKNOWN
#define Z_CURIUM_BULK_MODULUS				REAL_UNKNOWN
#define Z_CURIUM_SHEAR_MODULUS				REAL_UNKNOWN
#define Z_CURIUM_YOUNG_MODULUS				REAL_UNKNOWN
#define Z_CURIUM_POISSON_RATIO				REAL_UNKNOWN
#define Z_CURIUM_REFRACTIVE_INDEX			REAL_UNKNOWN
#define Z_CURIUM_SOUND_SPEED				REAL_UNKNOWN
#define Z_CURIUM_ELECTRICAL_TYPE			KEY_MISSING
#define Z_CURIUM_ELECTRICAL_CONDUCTIVITY		REAL_UNKNOWN
#define Z_CURIUM_ELECTRICAL_RESISTIVITY			REAL_UNKNOWN
#define Z_CURIUM_SUPERCONDUCTING_POINT			REAL_UNKNOWN
#define Z_CURIUM_MAGNETIC_TYPE				KEY_UNKNOWN
#define Z_CURIUM_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_CURIUM_MASS_MAGNETIC_SUSCEPTIBILITY		REAL_MISSING
#define Z_CURIUM_MOLAR_MAGNETIC_SUSCEPTIBILITY		REAL_MISSING
#define Z_CURIUM_VOLUME_MAGNETIC_SUSCEPTIBILITY		REAL_MISSING

/* MARK: - 97 - Bk */

#define Z_BERKELIUM_SYMBOL				Bk
#define Z_BERKELIUM_ATOMIC_NUMBER			97
#define Z_BERKELIUM_GROUP				INTEGER_NOT_APPLICABLE
#define Z_BERKELIUM_PERIOD				7
#define Z_BERKELIUM_BLOCK				Z_CHEMICAL_ELEMENT_BLOCK_F
#define Z_BERKELIUM_CLASS				Z_CHEMICAL_ELEMENT_CLASS_ACTINIDE
#define Z_BERKELIUM_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_BERKELIUM_SPACE_GROUP_NUMBER			194
#define Z_BERKELIUM_ATOMIC_MASS				247.0
#define Z_BERKELIUM_ATOMIC_RADIUS			REAL_MISSING
#define Z_BERKELIUM_COVALENT_RADIUS			REAL_MISSING
#define Z_BERKELIUM_VAN_DER_WAALS_RADIUS		REAL_MISSING
#define Z_BERKELIUM_DECAY_MODE				Z_CHEMICAL_ELEMENT_DECAY_MODE_ALPHA_EMISSION
#define Z_BERKELIUM_NEUTRON_CROSS_SECTION		710.0
#define Z_BERKELIUM_NEUTRON_MASS_ABSORPTION		REAL_MISSING
#define Z_BERKELIUM_VALENCE				4
#define Z_BERKELIUM_ELECTRONEGATIVITY			1.3
#define Z_BERKELIUM_ELECTRON_AFFINITY			REAL_MISSING
#define Z_BERKELIUM_MELTING_POINT			1050.0
#define Z_BERKELIUM_BOILING_POINT			REAL_MISSING
#define Z_BERKELIUM_CRITICAL_PRESSURE			REAL_MISSING
#define Z_BERKELIUM_CRITICAL_TEMPERATURE		REAL_MISSING
#define Z_BERKELIUM_FUSION_HEAT				REAL_MISSING
#define Z_BERKELIUM_VAPORIZATION_HEAT			REAL_MISSING
#define Z_BERKELIUM_SPECIFIC_HEAT			REAL_MISSING
#define Z_BERKELIUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_BERKELIUM_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_BERKELIUM_THERMAL_CONDUCTIVITY		10.0
#define Z_BERKELIUM_THERMAL_EXPANSION			REAL_MISSING
#define Z_BERKELIUM_DENSITY				14780.0
#define Z_BERKELIUM_LIQUID_DENSITY			REAL_MISSING
#define Z_BERKELIUM_MOLAR_VOLUME			0.000016712
#define Z_BERKELIUM_BRINELL_HARDNESS			REAL_MISSING
#define Z_BERKELIUM_MOHS_HARDNESS			REAL_MISSING
#define Z_BERKELIUM_VICKERS_HARDNESS			REAL_MISSING
#define Z_BERKELIUM_BULK_MODULUS			REAL_MISSING
#define Z_BERKELIUM_SHEAR_MODULUS			REAL_MISSING
#define Z_BERKELIUM_YOUNG_MODULUS			REAL_MISSING
#define Z_BERKELIUM_POISSON_RATIO			REAL_MISSING
#define Z_BERKELIUM_REFRACTIVE_INDEX			REAL_MISSING
#define Z_BERKELIUM_SOUND_SPEED				REAL_MISSING
#define Z_BERKELIUM_ELECTRICAL_TYPE			KEY_MISSING
#define Z_BERKELIUM_ELECTRICAL_CONDUCTIVITY		REAL_MISSING
#define Z_BERKELIUM_ELECTRICAL_RESISTIVITY		REAL_MISSING
#define Z_BERKELIUM_SUPERCONDUCTING_POINT		REAL_MISSING
#define Z_BERKELIUM_MAGNETIC_TYPE			KEY_MISSING
#define Z_BERKELIUM_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_BERKELIUM_MASS_MAGNETIC_SUSCEPTIBILITY	REAL_MISSING
#define Z_BERKELIUM_MOLAR_MAGNETIC_SUSCEPTIBILITY	REAL_MISSING
#define Z_BERKELIUM_VOLUME_MAGNETIC_SUSCEPTIBILITY	REAL_MISSING

/* MARK: - 98 - Cf */

#define Z_CALIFORNIUM_SYMBOL				Cf
#define Z_CALIFORNIUM_ATOMIC_NUMBER			98
#define Z_CALIFORNIUM_GROUP				INTEGER_NOT_APPLICABLE
#define Z_CALIFORNIUM_PERIOD				7
#define Z_CALIFORNIUM_BLOCK				Z_CHEMICAL_ELEMENT_BLOCK_F
#define Z_CALIFORNIUM_CLASS				Z_CHEMICAL_ELEMENT_CLASS_ACTINIDE
#define Z_CALIFORNIUM_METAL_STATE			Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_CALIFORNIUM_SPACE_GROUP_NUMBER		194
#define Z_CALIFORNIUM_ATOMIC_MASS			251.0
#define Z_CALIFORNIUM_ATOMIC_RADIUS			REAL_MISSING
#define Z_CALIFORNIUM_COVALENT_RADIUS			REAL_MISSING
#define Z_CALIFORNIUM_VAN_DER_WAALS_RADIUS		REAL_MISSING
#define Z_CALIFORNIUM_DECAY_MODE			Z_CHEMICAL_ELEMENT_DECAY_MODE_ALPHA_EMISSION
#define Z_CALIFORNIUM_NEUTRON_CROSS_SECTION		2.9e3
#define Z_CALIFORNIUM_NEUTRON_MASS_ABSORPTION		REAL_MISSING
#define Z_CALIFORNIUM_VALENCE				4
#define Z_CALIFORNIUM_ELECTRONEGATIVITY			1.3
#define Z_CALIFORNIUM_ELECTRON_AFFINITY			REAL_MISSING
#define Z_CALIFORNIUM_MELTING_POINT			900.0
#define Z_CALIFORNIUM_BOILING_POINT			REAL_MISSING
#define Z_CALIFORNIUM_CRITICAL_PRESSURE			REAL_MISSING
#define Z_CALIFORNIUM_CRITICAL_TEMPERATURE		REAL_MISSING
#define Z_CALIFORNIUM_FUSION_HEAT			REAL_MISSING
#define Z_CALIFORNIUM_VAPORIZATION_HEAT			REAL_MISSING
#define Z_CALIFORNIUM_SPECIFIC_HEAT			REAL_MISSING
#define Z_CALIFORNIUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_CALIFORNIUM_NEEL_POINT			REAL_NOT_APPLICABLE
#define Z_CALIFORNIUM_THERMAL_CONDUCTIVITY		REAL_MISSING
#define Z_CALIFORNIUM_THERMAL_EXPANSION			REAL_MISSING
#define Z_CALIFORNIUM_DENSITY				1.51e4
#define Z_CALIFORNIUM_LIQUID_DENSITY			REAL_MISSING
#define Z_CALIFORNIUM_MOLAR_VOLUME			0.0000166
#define Z_CALIFORNIUM_BRINELL_HARDNESS			REAL_MISSING
#define Z_CALIFORNIUM_MOHS_HARDNESS			REAL_MISSING
#define Z_CALIFORNIUM_VICKERS_HARDNESS			REAL_MISSING
#define Z_CALIFORNIUM_BULK_MODULUS			REAL_MISSING
#define Z_CALIFORNIUM_SHEAR_MODULUS			REAL_MISSING
#define Z_CALIFORNIUM_YOUNG_MODULUS			REAL_MISSING
#define Z_CALIFORNIUM_POISSON_RATIO			REAL_MISSING
#define Z_CALIFORNIUM_REFRACTIVE_INDEX			REAL_MISSING
#define Z_CALIFORNIUM_SOUND_SPEED			REAL_MISSING
#define Z_CALIFORNIUM_ELECTRICAL_TYPE			KEY_MISSING
#define Z_CALIFORNIUM_ELECTRICAL_CONDUCTIVITY		REAL_MISSING
#define Z_CALIFORNIUM_ELECTRICAL_RESISTIVITY		REAL_MISSING
#define Z_CALIFORNIUM_SUPERCONDUCTING_POINT		REAL_MISSING
#define Z_CALIFORNIUM_MAGNETIC_TYPE			KEY_MISSING
#define Z_CALIFORNIUM_CURIE_POINT			REAL_NOT_APPLICABLE
#define Z_CALIFORNIUM_MASS_MAGNETIC_SUSCEPTIBILITY	REAL_MISSING
#define Z_CALIFORNIUM_MOLAR_MAGNETIC_SUSCEPTIBILITY	REAL_MISSING
#define Z_CALIFORNIUM_VOLUME_MAGNETIC_SUSCEPTIBILITY	REAL_MISSING

/* MARK: - 99 - Es */

#define Z_EINSTEINIUM_SYMBOL				Es
#define Z_EINSTEINIUM_ATOMIC_NUMBER			99
#define Z_EINSTEINIUM_GROUP				INTEGER_NOT_APPLICABLE
#define Z_EINSTEINIUM_PERIOD				7
#define Z_EINSTEINIUM_BLOCK				Z_CHEMICAL_ELEMENT_BLOCK_F
#define Z_EINSTEINIUM_CLASS				Z_CHEMICAL_ELEMENT_CLASS_ACTINIDE
#define Z_EINSTEINIUM_METAL_STATE			Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_EINSTEINIUM_SPACE_GROUP_NUMBER		INTEGER_MISSING
#define Z_EINSTEINIUM_ATOMIC_MASS			252.0
#define Z_EINSTEINIUM_ATOMIC_RADIUS			REAL_MISSING
#define Z_EINSTEINIUM_COVALENT_RADIUS			REAL_MISSING
#define Z_EINSTEINIUM_VAN_DER_WAALS_RADIUS		REAL_MISSING
#define Z_EINSTEINIUM_DECAY_MODE			Z_CHEMICAL_ELEMENT_DECAY_MODE_ALPHA_EMISSION
#define Z_EINSTEINIUM_NEUTRON_CROSS_SECTION		160.0
#define Z_EINSTEINIUM_NEUTRON_MASS_ABSORPTION		REAL_MISSING
#define Z_EINSTEINIUM_VALENCE				4
#define Z_EINSTEINIUM_ELECTRONEGATIVITY			1.3
#define Z_EINSTEINIUM_ELECTRON_AFFINITY			REAL_MISSING
#define Z_EINSTEINIUM_MELTING_POINT			860.0
#define Z_EINSTEINIUM_BOILING_POINT			REAL_MISSING
#define Z_EINSTEINIUM_CRITICAL_PRESSURE			REAL_MISSING
#define Z_EINSTEINIUM_CRITICAL_TEMPERATURE		REAL_MISSING
#define Z_EINSTEINIUM_FUSION_HEAT			REAL_MISSING
#define Z_EINSTEINIUM_VAPORIZATION_HEAT			REAL_MISSING
#define Z_EINSTEINIUM_SPECIFIC_HEAT			REAL_MISSING
#define Z_EINSTEINIUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_EINSTEINIUM_NEEL_POINT			REAL_NOT_APPLICABLE
#define Z_EINSTEINIUM_THERMAL_CONDUCTIVITY		REAL_MISSING
#define Z_EINSTEINIUM_THERMAL_EXPANSION			REAL_MISSING
#define Z_EINSTEINIUM_DENSITY				REAL_MISSING
#define Z_EINSTEINIUM_LIQUID_DENSITY			REAL_MISSING
#define Z_EINSTEINIUM_MOLAR_VOLUME			REAL_MISSING
#define Z_EINSTEINIUM_BRINELL_HARDNESS			REAL_MISSING
#define Z_EINSTEINIUM_MOHS_HARDNESS			REAL_MISSING
#define Z_EINSTEINIUM_VICKERS_HARDNESS			REAL_MISSING
#define Z_EINSTEINIUM_BULK_MODULUS			REAL_MISSING
#define Z_EINSTEINIUM_SHEAR_MODULUS			REAL_MISSING
#define Z_EINSTEINIUM_YOUNG_MODULUS			REAL_MISSING
#define Z_EINSTEINIUM_POISSON_RATIO			REAL_MISSING
#define Z_EINSTEINIUM_REFRACTIVE_INDEX			REAL_MISSING
#define Z_EINSTEINIUM_SOUND_SPEED			REAL_MISSING
#define Z_EINSTEINIUM_ELECTRICAL_TYPE			KEY_MISSING
#define Z_EINSTEINIUM_ELECTRICAL_CONDUCTIVITY		REAL_MISSING
#define Z_EINSTEINIUM_ELECTRICAL_RESISTIVITY		REAL_MISSING
#define Z_EINSTEINIUM_SUPERCONDUCTING_POINT		REAL_MISSING
#define Z_EINSTEINIUM_MAGNETIC_TYPE			KEY_MISSING
#define Z_EINSTEINIUM_CURIE_POINT			REAL_NOT_APPLICABLE
#define Z_EINSTEINIUM_MASS_MAGNETIC_SUSCEPTIBILITY	REAL_MISSING
#define Z_EINSTEINIUM_MOLAR_MAGNETIC_SUSCEPTIBILITY	REAL_MISSING
#define Z_EINSTEINIUM_VOLUME_MAGNETIC_SUSCEPTIBILITY	REAL_MISSING

/* MARK: - 100 - Fm */

#define Z_FERMIUM_SYMBOL				Fm
#define Z_FERMIUM_ATOMIC_NUMBER				100
#define Z_FERMIUM_GROUP					INTEGER_NOT_APPLICABLE
#define Z_FERMIUM_PERIOD				7
#define Z_FERMIUM_BLOCK					Z_CHEMICAL_ELEMENT_BLOCK_F
#define Z_FERMIUM_CLASS					Z_CHEMICAL_ELEMENT_CLASS_ACTINIDE
#define Z_FERMIUM_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_FERMIUM_SPACE_GROUP_NUMBER			INTEGER_MISSING
#define Z_FERMIUM_ATOMIC_MASS				257.0
#define Z_FERMIUM_ATOMIC_RADIUS				REAL_MISSING
#define Z_FERMIUM_COVALENT_RADIUS			REAL_MISSING
#define Z_FERMIUM_VAN_DER_WAALS_RADIUS			REAL_MISSING
#define Z_FERMIUM_DECAY_MODE				Z_CHEMICAL_ELEMENT_DECAY_MODE_ALPHA_EMISSION
#define Z_FERMIUM_NEUTRON_CROSS_SECTION			5.8e3
#define Z_FERMIUM_NEUTRON_MASS_ABSORPTION		REAL_UNKNOWN
#define Z_FERMIUM_VALENCE				3
#define Z_FERMIUM_ELECTRONEGATIVITY			1.3
#define Z_FERMIUM_ELECTRON_AFFINITY			REAL_MISSING
#define Z_FERMIUM_MELTING_POINT				1527.0
#define Z_FERMIUM_BOILING_POINT				REAL_MISSING
#define Z_FERMIUM_CRITICAL_PRESSURE			REAL_MISSING
#define Z_FERMIUM_CRITICAL_TEMPERATURE			REAL_MISSING
#define Z_FERMIUM_FUSION_HEAT				REAL_MISSING
#define Z_FERMIUM_VAPORIZATION_HEAT			REAL_MISSING
#define Z_FERMIUM_SPECIFIC_HEAT				REAL_UNKNOWN
#define Z_FERMIUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_FERMIUM_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_FERMIUM_THERMAL_CONDUCTIVITY			REAL_MISSING
#define Z_FERMIUM_THERMAL_EXPANSION			REAL_MISSING
#define Z_FERMIUM_DENSITY				REAL_MISSING
#define Z_FERMIUM_LIQUID_DENSITY			REAL_MISSING
#define Z_FERMIUM_MOLAR_VOLUME				REAL_MISSING
#define Z_FERMIUM_BRINELL_HARDNESS			REAL_MISSING
#define Z_FERMIUM_MOHS_HARDNESS				REAL_MISSING
#define Z_FERMIUM_VICKERS_HARDNESS			REAL_MISSING
#define Z_FERMIUM_BULK_MODULUS				REAL_MISSING
#define Z_FERMIUM_SHEAR_MODULUS				REAL_MISSING
#define Z_FERMIUM_YOUNG_MODULUS				REAL_MISSING
#define Z_FERMIUM_POISSON_RATIO				REAL_MISSING
#define Z_FERMIUM_REFRACTIVE_INDEX			REAL_MISSING
#define Z_FERMIUM_SOUND_SPEED				REAL_MISSING
#define Z_FERMIUM_ELECTRICAL_TYPE			KEY_UNKNOWN
#define Z_FERMIUM_ELECTRICAL_CONDUCTIVITY		REAL_MISSING
#define Z_FERMIUM_ELECTRICAL_RESISTIVITY		REAL_MISSING
#define Z_FERMIUM_SUPERCONDUCTING_POINT			REAL_MISSING
#define Z_FERMIUM_MAGNETIC_TYPE				KEY_MISSING
#define Z_FERMIUM_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_FERMIUM_MASS_MAGNETIC_SUSCEPTIBILITY		REAL_UNKNOWN
#define Z_FERMIUM_MOLAR_MAGNETIC_SUSCEPTIBILITY		REAL_UNKNOWN
#define Z_FERMIUM_VOLUME_MAGNETIC_SUSCEPTIBILITY	REAL_UNKNOWN

/* MARK: - 101 - Md */

#define Z_MENDELEVIUM_SYMBOL				Md
#define Z_MENDELEVIUM_ATOMIC_NUMBER			101
#define Z_MENDELEVIUM_GROUP				INTEGER_NOT_APPLICABLE
#define Z_MENDELEVIUM_PERIOD				7
#define Z_MENDELEVIUM_BLOCK				Z_CHEMICAL_ELEMENT_BLOCK_F
#define Z_MENDELEVIUM_CLASS				Z_CHEMICAL_ELEMENT_CLASS_ACTINIDE
#define Z_MENDELEVIUM_METAL_STATE			Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_MENDELEVIUM_SPACE_GROUP_NUMBER		INTEGER_MISSING
#define Z_MENDELEVIUM_ATOMIC_MASS			258.0
#define Z_MENDELEVIUM_ATOMIC_RADIUS			REAL_MISSING
#define Z_MENDELEVIUM_COVALENT_RADIUS			REAL_MISSING
#define Z_MENDELEVIUM_VAN_DER_WAALS_RADIUS		REAL_MISSING
#define Z_MENDELEVIUM_DECAY_MODE			Z_CHEMICAL_ELEMENT_DECAY_MODE_ALPHA_EMISSION
#define Z_MENDELEVIUM_NEUTRON_CROSS_SECTION		REAL_UNKNOWN
#define Z_MENDELEVIUM_NEUTRON_MASS_ABSORPTION		REAL_UNKNOWN
#define Z_MENDELEVIUM_VALENCE				3
#define Z_MENDELEVIUM_ELECTRONEGATIVITY			1.3
#define Z_MENDELEVIUM_ELECTRON_AFFINITY			REAL_MISSING
#define Z_MENDELEVIUM_MELTING_POINT			828.0
#define Z_MENDELEVIUM_BOILING_POINT			REAL_MISSING
#define Z_MENDELEVIUM_CRITICAL_PRESSURE			REAL_MISSING
#define Z_MENDELEVIUM_CRITICAL_TEMPERATURE		REAL_MISSING
#define Z_MENDELEVIUM_FUSION_HEAT			REAL_MISSING
#define Z_MENDELEVIUM_VAPORIZATION_HEAT			REAL_MISSING
#define Z_MENDELEVIUM_SPECIFIC_HEAT			REAL_UNKNOWN
#define Z_MENDELEVIUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_MENDELEVIUM_NEEL_POINT			REAL_NOT_APPLICABLE
#define Z_MENDELEVIUM_THERMAL_CONDUCTIVITY		REAL_MISSING
#define Z_MENDELEVIUM_THERMAL_EXPANSION			REAL_MISSING
#define Z_MENDELEVIUM_DENSITY				REAL_MISSING
#define Z_MENDELEVIUM_LIQUID_DENSITY			REAL_MISSING
#define Z_MENDELEVIUM_MOLAR_VOLUME			REAL_MISSING
#define Z_MENDELEVIUM_BRINELL_HARDNESS			REAL_MISSING
#define Z_MENDELEVIUM_MOHS_HARDNESS			REAL_MISSING
#define Z_MENDELEVIUM_VICKERS_HARDNESS			REAL_MISSING
#define Z_MENDELEVIUM_BULK_MODULUS			REAL_MISSING
#define Z_MENDELEVIUM_SHEAR_MODULUS			REAL_MISSING
#define Z_MENDELEVIUM_YOUNG_MODULUS			REAL_MISSING
#define Z_MENDELEVIUM_POISSON_RATIO			REAL_MISSING
#define Z_MENDELEVIUM_REFRACTIVE_INDEX			REAL_MISSING
#define Z_MENDELEVIUM_SOUND_SPEED			REAL_MISSING
#define Z_MENDELEVIUM_ELECTRICAL_TYPE			KEY_UNKNOWN
#define Z_MENDELEVIUM_ELECTRICAL_CONDUCTIVITY		REAL_MISSING
#define Z_MENDELEVIUM_ELECTRICAL_RESISTIVITY		REAL_MISSING
#define Z_MENDELEVIUM_SUPERCONDUCTING_POINT		REAL_MISSING
#define Z_MENDELEVIUM_MAGNETIC_TYPE			KEY_MISSING
#define Z_MENDELEVIUM_CURIE_POINT			REAL_NOT_APPLICABLE
#define Z_MENDELEVIUM_MASS_MAGNETIC_SUSCEPTIBILITY	REAL_UNKNOWN
#define Z_MENDELEVIUM_MOLAR_MAGNETIC_SUSCEPTIBILITY	REAL_UNKNOWN
#define Z_MENDELEVIUM_VOLUME_MAGNETIC_SUSCEPTIBILITY	REAL_UNKNOWN

/* MARK: - 102 - No */

#define Z_NOBELIUM_SYMBOL				No
#define Z_NOBELIUM_ATOMIC_NUMBER			102
#define Z_NOBELIUM_GROUP				INTEGER_NOT_APPLICABLE
#define Z_NOBELIUM_PERIOD				7
#define Z_NOBELIUM_BLOCK				Z_CHEMICAL_ELEMENT_BLOCK_F
#define Z_NOBELIUM_CLASS				Z_CHEMICAL_ELEMENT_CLASS_ACTINIDE
#define Z_NOBELIUM_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_NOBELIUM_SPACE_GROUP_NUMBER			INTEGER_MISSING
#define Z_NOBELIUM_ATOMIC_MASS				259.0
#define Z_NOBELIUM_ATOMIC_RADIUS			REAL_MISSING
#define Z_NOBELIUM_COVALENT_RADIUS			REAL_MISSING
#define Z_NOBELIUM_VAN_DER_WAALS_RADIUS			REAL_MISSING
#define Z_NOBELIUM_DECAY_MODE				Z_CHEMICAL_ELEMENT_DECAY_MODE_ALPHA_EMISSION
#define Z_NOBELIUM_NEUTRON_CROSS_SECTION		REAL_UNKNOWN
#define Z_NOBELIUM_NEUTRON_MASS_ABSORPTION		REAL_UNKNOWN
#define Z_NOBELIUM_VALENCE				3
#define Z_NOBELIUM_ELECTRONEGATIVITY			1.3
#define Z_NOBELIUM_ELECTRON_AFFINITY			REAL_MISSING
#define Z_NOBELIUM_MELTING_POINT			828.0
#define Z_NOBELIUM_BOILING_POINT			REAL_MISSING
#define Z_NOBELIUM_CRITICAL_PRESSURE			REAL_MISSING
#define Z_NOBELIUM_CRITICAL_TEMPERATURE			REAL_MISSING
#define Z_NOBELIUM_FUSION_HEAT				REAL_MISSING
#define Z_NOBELIUM_VAPORIZATION_HEAT			REAL_MISSING
#define Z_NOBELIUM_SPECIFIC_HEAT			REAL_UNKNOWN
#define Z_NOBELIUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_NOBELIUM_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_NOBELIUM_THERMAL_CONDUCTIVITY			REAL_MISSING
#define Z_NOBELIUM_THERMAL_EXPANSION			REAL_MISSING
#define Z_NOBELIUM_DENSITY				REAL_MISSING
#define Z_NOBELIUM_LIQUID_DENSITY			REAL_MISSING
#define Z_NOBELIUM_MOLAR_VOLUME				REAL_MISSING
#define Z_NOBELIUM_BRINELL_HARDNESS			REAL_MISSING
#define Z_NOBELIUM_MOHS_HARDNESS			REAL_MISSING
#define Z_NOBELIUM_VICKERS_HARDNESS			REAL_MISSING
#define Z_NOBELIUM_BULK_MODULUS				REAL_MISSING
#define Z_NOBELIUM_SHEAR_MODULUS			REAL_MISSING
#define Z_NOBELIUM_YOUNG_MODULUS			REAL_MISSING
#define Z_NOBELIUM_POISSON_RATIO			REAL_MISSING
#define Z_NOBELIUM_REFRACTIVE_INDEX			REAL_MISSING
#define Z_NOBELIUM_SOUND_SPEED				REAL_MISSING
#define Z_NOBELIUM_ELECTRICAL_TYPE			KEY_UNKNOWN
#define Z_NOBELIUM_ELECTRICAL_CONDUCTIVITY		REAL_MISSING
#define Z_NOBELIUM_ELECTRICAL_RESISTIVITY		REAL_MISSING
#define Z_NOBELIUM_SUPERCONDUCTING_POINT		REAL_MISSING
#define Z_NOBELIUM_MAGNETIC_TYPE			KEY_MISSING
#define Z_NOBELIUM_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_NOBELIUM_MASS_MAGNETIC_SUSCEPTIBILITY		REAL_UNKNOWN
#define Z_NOBELIUM_MOLAR_MAGNETIC_SUSCEPTIBILITY	REAL_UNKNOWN
#define Z_NOBELIUM_VOLUME_MAGNETIC_SUSCEPTIBILITY	REAL_UNKNOWN

/* MARK: - 103 - Lr */

#define Z_LAWRENCIUM_SYMBOL				Lr
#define Z_LAWRENCIUM_ATOMIC_NUMBER			103
#define Z_LAWRENCIUM_GROUP				3
#define Z_LAWRENCIUM_PERIOD				7
#define Z_LAWRENCIUM_BLOCK				Z_CHEMICAL_ELEMENT_BLOCK_D
#define Z_LAWRENCIUM_CLASS				Z_CHEMICAL_ELEMENT_CLASS_ACTINIDE
#define Z_LAWRENCIUM_METAL_STATE			Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_LAWRENCIUM_SPACE_GROUP_NUMBER			INTEGER_MISSING
#define Z_LAWRENCIUM_ATOMIC_MASS			262.0
#define Z_LAWRENCIUM_ATOMIC_RADIUS			REAL_MISSING
#define Z_LAWRENCIUM_COVALENT_RADIUS			REAL_MISSING
#define Z_LAWRENCIUM_VAN_DER_WAALS_RADIUS		REAL_MISSING
#define Z_LAWRENCIUM_DECAY_MODE				Z_CHEMICAL_ELEMENT_DECAY_MODE_ALPHA_EMISSION
#define Z_LAWRENCIUM_NEUTRON_CROSS_SECTION		REAL_UNKNOWN
#define Z_LAWRENCIUM_NEUTRON_MASS_ABSORPTION		REAL_UNKNOWN
#define Z_LAWRENCIUM_VALENCE				3
#define Z_LAWRENCIUM_ELECTRONEGATIVITY			REAL_MISSING
#define Z_LAWRENCIUM_ELECTRON_AFFINITY			REAL_MISSING
#define Z_LAWRENCIUM_MELTING_POINT			1627.0
#define Z_LAWRENCIUM_BOILING_POINT			REAL_MISSING
#define Z_LAWRENCIUM_CRITICAL_PRESSURE			REAL_MISSING
#define Z_LAWRENCIUM_CRITICAL_TEMPERATURE		REAL_MISSING
#define Z_LAWRENCIUM_FUSION_HEAT			REAL_MISSING
#define Z_LAWRENCIUM_VAPORIZATION_HEAT			REAL_MISSING
#define Z_LAWRENCIUM_SPECIFIC_HEAT			REAL_UNKNOWN
#define Z_LAWRENCIUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_LAWRENCIUM_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_LAWRENCIUM_THERMAL_CONDUCTIVITY		REAL_MISSING
#define Z_LAWRENCIUM_THERMAL_EXPANSION			REAL_MISSING
#define Z_LAWRENCIUM_DENSITY				REAL_MISSING
#define Z_LAWRENCIUM_LIQUID_DENSITY			REAL_MISSING
#define Z_LAWRENCIUM_MOLAR_VOLUME			REAL_MISSING
#define Z_LAWRENCIUM_BRINELL_HARDNESS			REAL_MISSING
#define Z_LAWRENCIUM_MOHS_HARDNESS			REAL_MISSING
#define Z_LAWRENCIUM_VICKERS_HARDNESS			REAL_MISSING
#define Z_LAWRENCIUM_BULK_MODULUS			REAL_MISSING
#define Z_LAWRENCIUM_SHEAR_MODULUS			REAL_MISSING
#define Z_LAWRENCIUM_YOUNG_MODULUS			REAL_MISSING
#define Z_LAWRENCIUM_POISSON_RATIO			REAL_MISSING
#define Z_LAWRENCIUM_REFRACTIVE_INDEX			REAL_MISSING
#define Z_LAWRENCIUM_SOUND_SPEED			REAL_MISSING
#define Z_LAWRENCIUM_ELECTRICAL_TYPE			KEY_UNKNOWN
#define Z_LAWRENCIUM_ELECTRICAL_CONDUCTIVITY		REAL_MISSING
#define Z_LAWRENCIUM_ELECTRICAL_RESISTIVITY		REAL_MISSING
#define Z_LAWRENCIUM_SUPERCONDUCTING_POINT		REAL_MISSING
#define Z_LAWRENCIUM_MAGNETIC_TYPE			KEY_MISSING
#define Z_LAWRENCIUM_CURIE_POINT			REAL_NOT_APPLICABLE
#define Z_LAWRENCIUM_MASS_MAGNETIC_SUSCEPTIBILITY	REAL_UNKNOWN
#define Z_LAWRENCIUM_MOLAR_MAGNETIC_SUSCEPTIBILITY	REAL_UNKNOWN
#define Z_LAWRENCIUM_VOLUME_MAGNETIC_SUSCEPTIBILITY	REAL_UNKNOWN

/* MARK: - 104 - Rf */

#define Z_RUTHERFORDIUM_SYMBOL				Rf
#define Z_RUTHERFORDIUM_ATOMIC_NUMBER			104
#define Z_RUTHERFORDIUM_GROUP				4
#define Z_RUTHERFORDIUM_PERIOD				7
#define Z_RUTHERFORDIUM_BLOCK				Z_CHEMICAL_ELEMENT_BLOCK_D
#define Z_RUTHERFORDIUM_CLASS				Z_CHEMICAL_ELEMENT_CLASS_TRANSITION_METAL
#define Z_RUTHERFORDIUM_METAL_STATE			Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_RUTHERFORDIUM_SPACE_GROUP_NUMBER		INTEGER_MISSING
#define Z_RUTHERFORDIUM_ATOMIC_MASS			265.0
#define Z_RUTHERFORDIUM_ATOMIC_RADIUS			REAL_MISSING
#define Z_RUTHERFORDIUM_COVALENT_RADIUS			REAL_MISSING
#define Z_RUTHERFORDIUM_VAN_DER_WAALS_RADIUS		REAL_MISSING
#define Z_RUTHERFORDIUM_DECAY_MODE			Z_CHEMICAL_ELEMENT_DECAY_MODE_ALPHA_EMISSION
#define Z_RUTHERFORDIUM_NEUTRON_CROSS_SECTION		REAL_UNKNOWN
#define Z_RUTHERFORDIUM_NEUTRON_MASS_ABSORPTION		REAL_UNKNOWN
#define Z_RUTHERFORDIUM_VALENCE				4
#define Z_RUTHERFORDIUM_ELECTRONEGATIVITY		REAL_MISSING
#define Z_RUTHERFORDIUM_ELECTRON_AFFINITY		REAL_MISSING
#define Z_RUTHERFORDIUM_MELTING_POINT			REAL_MISSING
#define Z_RUTHERFORDIUM_BOILING_POINT			REAL_MISSING
#define Z_RUTHERFORDIUM_CRITICAL_PRESSURE		REAL_MISSING
#define Z_RUTHERFORDIUM_CRITICAL_TEMPERATURE		REAL_MISSING
#define Z_RUTHERFORDIUM_FUSION_HEAT			REAL_MISSING
#define Z_RUTHERFORDIUM_VAPORIZATION_HEAT		REAL_MISSING
#define Z_RUTHERFORDIUM_SPECIFIC_HEAT			REAL_UNKNOWN
#define Z_RUTHERFORDIUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_RUTHERFORDIUM_NEEL_POINT			REAL_NOT_APPLICABLE
#define Z_RUTHERFORDIUM_THERMAL_CONDUCTIVITY		REAL_MISSING
#define Z_RUTHERFORDIUM_THERMAL_EXPANSION		REAL_MISSING
#define Z_RUTHERFORDIUM_DENSITY				REAL_MISSING
#define Z_RUTHERFORDIUM_LIQUID_DENSITY			REAL_MISSING
#define Z_RUTHERFORDIUM_MOLAR_VOLUME			REAL_MISSING
#define Z_RUTHERFORDIUM_BRINELL_HARDNESS		REAL_MISSING
#define Z_RUTHERFORDIUM_MOHS_HARDNESS			REAL_MISSING
#define Z_RUTHERFORDIUM_VICKERS_HARDNESS		REAL_MISSING
#define Z_RUTHERFORDIUM_BULK_MODULUS			REAL_MISSING
#define Z_RUTHERFORDIUM_SHEAR_MODULUS			REAL_MISSING
#define Z_RUTHERFORDIUM_YOUNG_MODULUS			REAL_MISSING
#define Z_RUTHERFORDIUM_POISSON_RATIO			REAL_MISSING
#define Z_RUTHERFORDIUM_REFRACTIVE_INDEX		REAL_MISSING
#define Z_RUTHERFORDIUM_SOUND_SPEED			REAL_MISSING
#define Z_RUTHERFORDIUM_ELECTRICAL_TYPE			KEY_UNKNOWN
#define Z_RUTHERFORDIUM_ELECTRICAL_CONDUCTIVITY		REAL_MISSING
#define Z_RUTHERFORDIUM_ELECTRICAL_RESISTIVITY		REAL_MISSING
#define Z_RUTHERFORDIUM_SUPERCONDUCTING_POINT		REAL_MISSING
#define Z_RUTHERFORDIUM_MAGNETIC_TYPE			KEY_MISSING
#define Z_RUTHERFORDIUM_CURIE_POINT			REAL_NOT_APPLICABLE
#define Z_RUTHERFORDIUM_MASS_MAGNETIC_SUSCEPTIBILITY	REAL_UNKNOWN
#define Z_RUTHERFORDIUM_MOLAR_MAGNETIC_SUSCEPTIBILITY	REAL_UNKNOWN
#define Z_RUTHERFORDIUM_VOLUME_MAGNETIC_SUSCEPTIBILITY	REAL_UNKNOWN

/* MARK: - 105 - Db */

#define Z_DUBNIUM_SYMBOL				Db
#define Z_DUBNIUM_ATOMIC_NUMBER				105
#define Z_DUBNIUM_GROUP					5
#define Z_DUBNIUM_PERIOD				7
#define Z_DUBNIUM_BLOCK					Z_CHEMICAL_ELEMENT_BLOCK_D
#define Z_DUBNIUM_CLASS					Z_CHEMICAL_ELEMENT_CLASS_TRANSITION_METAL
#define Z_DUBNIUM_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_DUBNIUM_SPACE_GROUP_NUMBER			INTEGER_MISSING
#define Z_DUBNIUM_ATOMIC_MASS				268.0
#define Z_DUBNIUM_ATOMIC_RADIUS				REAL_MISSING
#define Z_DUBNIUM_COVALENT_RADIUS			REAL_MISSING
#define Z_DUBNIUM_VAN_DER_WAALS_RADIUS			REAL_MISSING
#define Z_DUBNIUM_DECAY_MODE				Z_CHEMICAL_ELEMENT_DECAY_MODE_ALPHA_EMISSION
#define Z_DUBNIUM_NEUTRON_CROSS_SECTION			REAL_UNKNOWN
#define Z_DUBNIUM_NEUTRON_MASS_ABSORPTION		REAL_UNKNOWN
#define Z_DUBNIUM_VALENCE				5
#define Z_DUBNIUM_ELECTRONEGATIVITY			REAL_MISSING
#define Z_DUBNIUM_ELECTRON_AFFINITY			REAL_MISSING
#define Z_DUBNIUM_MELTING_POINT				REAL_MISSING
#define Z_DUBNIUM_BOILING_POINT				REAL_MISSING
#define Z_DUBNIUM_CRITICAL_PRESSURE			REAL_MISSING
#define Z_DUBNIUM_CRITICAL_TEMPERATURE			REAL_MISSING
#define Z_DUBNIUM_FUSION_HEAT				REAL_MISSING
#define Z_DUBNIUM_VAPORIZATION_HEAT			REAL_MISSING
#define Z_DUBNIUM_SPECIFIC_HEAT				REAL_UNKNOWN
#define Z_DUBNIUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_DUBNIUM_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_DUBNIUM_THERMAL_CONDUCTIVITY			REAL_MISSING
#define Z_DUBNIUM_THERMAL_EXPANSION			REAL_MISSING
#define Z_DUBNIUM_DENSITY				REAL_MISSING
#define Z_DUBNIUM_LIQUID_DENSITY			REAL_MISSING
#define Z_DUBNIUM_MOLAR_VOLUME				REAL_MISSING
#define Z_DUBNIUM_BRINELL_HARDNESS			REAL_MISSING
#define Z_DUBNIUM_MOHS_HARDNESS				REAL_MISSING
#define Z_DUBNIUM_VICKERS_HARDNESS			REAL_MISSING
#define Z_DUBNIUM_BULK_MODULUS				REAL_MISSING
#define Z_DUBNIUM_SHEAR_MODULUS				REAL_MISSING
#define Z_DUBNIUM_YOUNG_MODULUS				REAL_MISSING
#define Z_DUBNIUM_POISSON_RATIO				REAL_MISSING
#define Z_DUBNIUM_REFRACTIVE_INDEX			REAL_MISSING
#define Z_DUBNIUM_SOUND_SPEED				REAL_MISSING
#define Z_DUBNIUM_ELECTRICAL_TYPE			KEY_UNKNOWN
#define Z_DUBNIUM_ELECTRICAL_CONDUCTIVITY		REAL_MISSING
#define Z_DUBNIUM_ELECTRICAL_RESISTIVITY		REAL_MISSING
#define Z_DUBNIUM_SUPERCONDUCTING_POINT			REAL_MISSING
#define Z_DUBNIUM_MAGNETIC_TYPE				KEY_MISSING
#define Z_DUBNIUM_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_DUBNIUM_MASS_MAGNETIC_SUSCEPTIBILITY		REAL_UNKNOWN
#define Z_DUBNIUM_MOLAR_MAGNETIC_SUSCEPTIBILITY		REAL_UNKNOWN
#define Z_DUBNIUM_VOLUME_MAGNETIC_SUSCEPTIBILITY	REAL_UNKNOWN

/* MARK: - 106 - Sg */

#define Z_SEABORGIUM_SYMBOL				Sg
#define Z_SEABORGIUM_ATOMIC_NUMBER			106
#define Z_SEABORGIUM_GROUP				6
#define Z_SEABORGIUM_PERIOD				7
#define Z_SEABORGIUM_BLOCK				Z_CHEMICAL_ELEMENT_BLOCK_D
#define Z_SEABORGIUM_CLASS				Z_CHEMICAL_ELEMENT_CLASS_TRANSITION_METAL
#define Z_SEABORGIUM_METAL_STATE			Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_SEABORGIUM_SPACE_GROUP_NUMBER			INTEGER_MISSING
#define Z_SEABORGIUM_ATOMIC_MASS			271.0
#define Z_SEABORGIUM_ATOMIC_RADIUS			REAL_MISSING
#define Z_SEABORGIUM_COVALENT_RADIUS			REAL_MISSING
#define Z_SEABORGIUM_VAN_DER_WAALS_RADIUS		REAL_MISSING
#define Z_SEABORGIUM_DECAY_MODE				Z_CHEMICAL_ELEMENT_DECAY_MODE_ALPHA_EMISSION
#define Z_SEABORGIUM_NEUTRON_CROSS_SECTION		REAL_UNKNOWN
#define Z_SEABORGIUM_NEUTRON_MASS_ABSORPTION		REAL_UNKNOWN
#define Z_SEABORGIUM_VALENCE				6
#define Z_SEABORGIUM_ELECTRONEGATIVITY			REAL_MISSING
#define Z_SEABORGIUM_ELECTRON_AFFINITY			REAL_MISSING
#define Z_SEABORGIUM_MELTING_POINT			REAL_MISSING
#define Z_SEABORGIUM_BOILING_POINT			REAL_MISSING
#define Z_SEABORGIUM_CRITICAL_PRESSURE			REAL_MISSING
#define Z_SEABORGIUM_CRITICAL_TEMPERATURE		REAL_MISSING
#define Z_SEABORGIUM_FUSION_HEAT			REAL_MISSING
#define Z_SEABORGIUM_VAPORIZATION_HEAT			REAL_MISSING
#define Z_SEABORGIUM_SPECIFIC_HEAT			REAL_UNKNOWN
#define Z_SEABORGIUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_SEABORGIUM_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_SEABORGIUM_THERMAL_CONDUCTIVITY		REAL_MISSING
#define Z_SEABORGIUM_THERMAL_EXPANSION			REAL_MISSING
#define Z_SEABORGIUM_DENSITY				REAL_MISSING
#define Z_SEABORGIUM_LIQUID_DENSITY			REAL_MISSING
#define Z_SEABORGIUM_MOLAR_VOLUME			REAL_MISSING
#define Z_SEABORGIUM_BRINELL_HARDNESS			REAL_MISSING
#define Z_SEABORGIUM_MOHS_HARDNESS			REAL_MISSING
#define Z_SEABORGIUM_VICKERS_HARDNESS			REAL_MISSING
#define Z_SEABORGIUM_BULK_MODULUS			REAL_MISSING
#define Z_SEABORGIUM_SHEAR_MODULUS			REAL_MISSING
#define Z_SEABORGIUM_YOUNG_MODULUS			REAL_MISSING
#define Z_SEABORGIUM_POISSON_RATIO			REAL_MISSING
#define Z_SEABORGIUM_REFRACTIVE_INDEX			REAL_MISSING
#define Z_SEABORGIUM_SOUND_SPEED			REAL_MISSING
#define Z_SEABORGIUM_ELECTRICAL_TYPE			KEY_UNKNOWN
#define Z_SEABORGIUM_ELECTRICAL_CONDUCTIVITY		REAL_MISSING
#define Z_SEABORGIUM_ELECTRICAL_RESISTIVITY		REAL_MISSING
#define Z_SEABORGIUM_SUPERCONDUCTING_POINT		REAL_MISSING
#define Z_SEABORGIUM_MAGNETIC_TYPE			KEY_MISSING
#define Z_SEABORGIUM_CURIE_POINT			REAL_NOT_APPLICABLE
#define Z_SEABORGIUM_MASS_MAGNETIC_SUSCEPTIBILITY	REAL_UNKNOWN
#define Z_SEABORGIUM_MOLAR_MAGNETIC_SUSCEPTIBILITY	REAL_UNKNOWN
#define Z_SEABORGIUM_VOLUME_MAGNETIC_SUSCEPTIBILITY	REAL_UNKNOWN

/* MARK: - 107 - Bh */

#define Z_BOHRIUM_SYMBOL				Bh
#define Z_BOHRIUM_ATOMIC_NUMBER				107
#define Z_BOHRIUM_GROUP					7
#define Z_BOHRIUM_PERIOD				7
#define Z_BOHRIUM_BLOCK					Z_CHEMICAL_ELEMENT_BLOCK_D
#define Z_BOHRIUM_CLASS					Z_CHEMICAL_ELEMENT_CLASS_TRANSITION_METAL
#define Z_BOHRIUM_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_BOHRIUM_SPACE_GROUP_NUMBER			INTEGER_MISSING
#define Z_BOHRIUM_ATOMIC_MASS				272.0
#define Z_BOHRIUM_ATOMIC_RADIUS				REAL_MISSING
#define Z_BOHRIUM_COVALENT_RADIUS			REAL_MISSING
#define Z_BOHRIUM_VAN_DER_WAALS_RADIUS			REAL_MISSING
#define Z_BOHRIUM_DECAY_MODE				Z_CHEMICAL_ELEMENT_DECAY_MODE_ALPHA_EMISSION
#define Z_BOHRIUM_NEUTRON_CROSS_SECTION			REAL_UNKNOWN
#define Z_BOHRIUM_NEUTRON_MASS_ABSORPTION		REAL_UNKNOWN
#define Z_BOHRIUM_VALENCE				7
#define Z_BOHRIUM_ELECTRONEGATIVITY			REAL_MISSING
#define Z_BOHRIUM_ELECTRON_AFFINITY			REAL_MISSING
#define Z_BOHRIUM_MELTING_POINT				REAL_MISSING
#define Z_BOHRIUM_BOILING_POINT				REAL_MISSING
#define Z_BOHRIUM_CRITICAL_PRESSURE			REAL_MISSING
#define Z_BOHRIUM_CRITICAL_TEMPERATURE			REAL_MISSING
#define Z_BOHRIUM_FUSION_HEAT				REAL_MISSING
#define Z_BOHRIUM_VAPORIZATION_HEAT			REAL_MISSING
#define Z_BOHRIUM_SPECIFIC_HEAT				REAL_UNKNOWN
#define Z_BOHRIUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_BOHRIUM_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_BOHRIUM_THERMAL_CONDUCTIVITY			REAL_MISSING
#define Z_BOHRIUM_THERMAL_EXPANSION			REAL_MISSING
#define Z_BOHRIUM_DENSITY				REAL_MISSING
#define Z_BOHRIUM_LIQUID_DENSITY			REAL_MISSING
#define Z_BOHRIUM_MOLAR_VOLUME				REAL_MISSING
#define Z_BOHRIUM_BRINELL_HARDNESS			REAL_MISSING
#define Z_BOHRIUM_MOHS_HARDNESS				REAL_MISSING
#define Z_BOHRIUM_VICKERS_HARDNESS			REAL_MISSING
#define Z_BOHRIUM_BULK_MODULUS				REAL_MISSING
#define Z_BOHRIUM_SHEAR_MODULUS				REAL_MISSING
#define Z_BOHRIUM_YOUNG_MODULUS				REAL_MISSING
#define Z_BOHRIUM_POISSON_RATIO				REAL_MISSING
#define Z_BOHRIUM_REFRACTIVE_INDEX			REAL_MISSING
#define Z_BOHRIUM_SOUND_SPEED				REAL_MISSING
#define Z_BOHRIUM_ELECTRICAL_TYPE			KEY_UNKNOWN
#define Z_BOHRIUM_ELECTRICAL_CONDUCTIVITY		REAL_MISSING
#define Z_BOHRIUM_ELECTRICAL_RESISTIVITY		REAL_MISSING
#define Z_BOHRIUM_SUPERCONDUCTING_POINT			REAL_MISSING
#define Z_BOHRIUM_MAGNETIC_TYPE				KEY_MISSING
#define Z_BOHRIUM_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_BOHRIUM_MASS_MAGNETIC_SUSCEPTIBILITY		REAL_UNKNOWN
#define Z_BOHRIUM_MOLAR_MAGNETIC_SUSCEPTIBILITY		REAL_UNKNOWN
#define Z_BOHRIUM_VOLUME_MAGNETIC_SUSCEPTIBILITY	REAL_UNKNOWN

/* MARK: - 108 - Hs */

#define Z_HASSIUM_SYMBOL				Hs
#define Z_HASSIUM_ATOMIC_NUMBER				108
#define Z_HASSIUM_GROUP					8
#define Z_HASSIUM_PERIOD				7
#define Z_HASSIUM_BLOCK					Z_CHEMICAL_ELEMENT_BLOCK_D
#define Z_HASSIUM_CLASS					Z_CHEMICAL_ELEMENT_CLASS_TRANSITION_METAL
#define Z_HASSIUM_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_HASSIUM_SPACE_GROUP_NUMBER			INTEGER_MISSING
#define Z_HASSIUM_ATOMIC_MASS				270.0
#define Z_HASSIUM_ATOMIC_RADIUS				REAL_MISSING
#define Z_HASSIUM_COVALENT_RADIUS			REAL_MISSING
#define Z_HASSIUM_VAN_DER_WAALS_RADIUS			REAL_MISSING
#define Z_HASSIUM_DECAY_MODE				Z_CHEMICAL_ELEMENT_DECAY_MODE_ALPHA_EMISSION
#define Z_HASSIUM_NEUTRON_CROSS_SECTION			REAL_UNKNOWN
#define Z_HASSIUM_NEUTRON_MASS_ABSORPTION		REAL_UNKNOWN
#define Z_HASSIUM_VALENCE				INTEGER_UNKNOWN
#define Z_HASSIUM_ELECTRONEGATIVITY			REAL_MISSING
#define Z_HASSIUM_ELECTRON_AFFINITY			REAL_MISSING
#define Z_HASSIUM_MELTING_POINT				REAL_MISSING
#define Z_HASSIUM_BOILING_POINT				REAL_MISSING
#define Z_HASSIUM_CRITICAL_PRESSURE			REAL_MISSING
#define Z_HASSIUM_CRITICAL_TEMPERATURE			REAL_MISSING
#define Z_HASSIUM_FUSION_HEAT				REAL_MISSING
#define Z_HASSIUM_VAPORIZATION_HEAT			REAL_MISSING
#define Z_HASSIUM_SPECIFIC_HEAT				REAL_UNKNOWN
#define Z_HASSIUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_HASSIUM_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_HASSIUM_THERMAL_CONDUCTIVITY			REAL_MISSING
#define Z_HASSIUM_THERMAL_EXPANSION			REAL_MISSING
#define Z_HASSIUM_DENSITY				REAL_MISSING
#define Z_HASSIUM_LIQUID_DENSITY			REAL_MISSING
#define Z_HASSIUM_MOLAR_VOLUME				REAL_MISSING
#define Z_HASSIUM_BRINELL_HARDNESS			REAL_MISSING
#define Z_HASSIUM_MOHS_HARDNESS				REAL_MISSING
#define Z_HASSIUM_VICKERS_HARDNESS			REAL_MISSING
#define Z_HASSIUM_BULK_MODULUS				REAL_MISSING
#define Z_HASSIUM_SHEAR_MODULUS				REAL_MISSING
#define Z_HASSIUM_YOUNG_MODULUS				REAL_MISSING
#define Z_HASSIUM_POISSON_RATIO				REAL_MISSING
#define Z_HASSIUM_REFRACTIVE_INDEX			REAL_MISSING
#define Z_HASSIUM_SOUND_SPEED				REAL_MISSING
#define Z_HASSIUM_ELECTRICAL_TYPE			KEY_UNKNOWN
#define Z_HASSIUM_ELECTRICAL_CONDUCTIVITY		REAL_MISSING
#define Z_HASSIUM_ELECTRICAL_RESISTIVITY		REAL_MISSING
#define Z_HASSIUM_SUPERCONDUCTING_POINT			REAL_MISSING
#define Z_HASSIUM_MAGNETIC_TYPE				KEY_MISSING
#define Z_HASSIUM_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_HASSIUM_MASS_MAGNETIC_SUSCEPTIBILITY		REAL_UNKNOWN
#define Z_HASSIUM_MOLAR_MAGNETIC_SUSCEPTIBILITY		REAL_UNKNOWN
#define Z_HASSIUM_VOLUME_MAGNETIC_SUSCEPTIBILITY	REAL_UNKNOWN

/* MARK: - 109 - Mt */

#define Z_MEITNERIUM_SYMBOL				Mt
#define Z_MEITNERIUM_ATOMIC_NUMBER			109
#define Z_MEITNERIUM_GROUP				9
#define Z_MEITNERIUM_PERIOD				7
#define Z_MEITNERIUM_BLOCK				Z_CHEMICAL_ELEMENT_BLOCK_D
#define Z_MEITNERIUM_CLASS				Z_CHEMICAL_ELEMENT_CLASS_TRANSITION_METAL
#define Z_MEITNERIUM_METAL_STATE			Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_MEITNERIUM_SPACE_GROUP_NUMBER			INTEGER_MISSING
#define Z_MEITNERIUM_ATOMIC_MASS			276.0
#define Z_MEITNERIUM_ATOMIC_RADIUS			REAL_MISSING
#define Z_MEITNERIUM_COVALENT_RADIUS			REAL_MISSING
#define Z_MEITNERIUM_VAN_DER_WAALS_RADIUS		REAL_MISSING
#define Z_MEITNERIUM_DECAY_MODE				Z_CHEMICAL_ELEMENT_DECAY_MODE_ALPHA_EMISSION
#define Z_MEITNERIUM_NEUTRON_CROSS_SECTION		REAL_UNKNOWN
#define Z_MEITNERIUM_NEUTRON_MASS_ABSORPTION		REAL_UNKNOWN
#define Z_MEITNERIUM_VALENCE				INTEGER_UNKNOWN
#define Z_MEITNERIUM_ELECTRONEGATIVITY			REAL_MISSING
#define Z_MEITNERIUM_ELECTRON_AFFINITY			REAL_MISSING
#define Z_MEITNERIUM_MELTING_POINT			REAL_MISSING
#define Z_MEITNERIUM_BOILING_POINT			REAL_MISSING
#define Z_MEITNERIUM_CRITICAL_PRESSURE			REAL_MISSING
#define Z_MEITNERIUM_CRITICAL_TEMPERATURE		REAL_MISSING
#define Z_MEITNERIUM_FUSION_HEAT			REAL_MISSING
#define Z_MEITNERIUM_VAPORIZATION_HEAT			REAL_MISSING
#define Z_MEITNERIUM_SPECIFIC_HEAT			REAL_UNKNOWN
#define Z_MEITNERIUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_MEITNERIUM_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_MEITNERIUM_THERMAL_CONDUCTIVITY		REAL_MISSING
#define Z_MEITNERIUM_THERMAL_EXPANSION			REAL_MISSING
#define Z_MEITNERIUM_DENSITY				REAL_MISSING
#define Z_MEITNERIUM_LIQUID_DENSITY			REAL_MISSING
#define Z_MEITNERIUM_MOLAR_VOLUME			REAL_MISSING
#define Z_MEITNERIUM_BRINELL_HARDNESS			REAL_MISSING
#define Z_MEITNERIUM_MOHS_HARDNESS			REAL_MISSING
#define Z_MEITNERIUM_VICKERS_HARDNESS			REAL_MISSING
#define Z_MEITNERIUM_BULK_MODULUS			REAL_MISSING
#define Z_MEITNERIUM_SHEAR_MODULUS			REAL_MISSING
#define Z_MEITNERIUM_YOUNG_MODULUS			REAL_MISSING
#define Z_MEITNERIUM_POISSON_RATIO			REAL_MISSING
#define Z_MEITNERIUM_REFRACTIVE_INDEX			REAL_MISSING
#define Z_MEITNERIUM_SOUND_SPEED			REAL_MISSING
#define Z_MEITNERIUM_ELECTRICAL_TYPE			KEY_UNKNOWN
#define Z_MEITNERIUM_ELECTRICAL_CONDUCTIVITY		REAL_MISSING
#define Z_MEITNERIUM_ELECTRICAL_RESISTIVITY		REAL_MISSING
#define Z_MEITNERIUM_SUPERCONDUCTING_POINT		REAL_MISSING
#define Z_MEITNERIUM_MAGNETIC_TYPE			KEY_MISSING
#define Z_MEITNERIUM_CURIE_POINT			REAL_NOT_APPLICABLE
#define Z_MEITNERIUM_MASS_MAGNETIC_SUSCEPTIBILITY	REAL_UNKNOWN
#define Z_MEITNERIUM_MOLAR_MAGNETIC_SUSCEPTIBILITY	REAL_UNKNOWN
#define Z_MEITNERIUM_VOLUME_MAGNETIC_SUSCEPTIBILITY	REAL_UNKNOWN

/* MARK: - 110 - Ds */

#define Z_DARMSTADTIUM_SYMBOL				Ds
#define Z_DARMSTADTIUM_ATOMIC_NUMBER			110
#define Z_DARMSTADTIUM_GROUP				10
#define Z_DARMSTADTIUM_PERIOD				7
#define Z_DARMSTADTIUM_BLOCK				Z_CHEMICAL_ELEMENT_BLOCK_D
#define Z_DARMSTADTIUM_CLASS				Z_CHEMICAL_ELEMENT_CLASS_TRANSITION_METAL
#define Z_DARMSTADTIUM_METAL_STATE			Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_DARMSTADTIUM_SPACE_GROUP_NUMBER		INTEGER_UNKNOWN
#define Z_DARMSTADTIUM_ATOMIC_MASS			281.0
#define Z_DARMSTADTIUM_ATOMIC_RADIUS			REAL_UNKNOWN
#define Z_DARMSTADTIUM_COVALENT_RADIUS			REAL_UNKNOWN
#define Z_DARMSTADTIUM_VAN_DER_WAALS_RADIUS		REAL_UNKNOWN
#define Z_DARMSTADTIUM_DECAY_MODE			Z_CHEMICAL_ELEMENT_DECAY_MODE_ALPHA_EMISSION
#define Z_DARMSTADTIUM_NEUTRON_CROSS_SECTION		REAL_UNKNOWN
#define Z_DARMSTADTIUM_NEUTRON_MASS_ABSORPTION		REAL_UNKNOWN
#define Z_DARMSTADTIUM_VALENCE				INTEGER_UNKNOWN
#define Z_DARMSTADTIUM_ELECTRONEGATIVITY		REAL_UNKNOWN
#define Z_DARMSTADTIUM_ELECTRON_AFFINITY		REAL_UNKNOWN
#define Z_DARMSTADTIUM_MELTING_POINT			REAL_UNKNOWN
#define Z_DARMSTADTIUM_BOILING_POINT			REAL_UNKNOWN
#define Z_DARMSTADTIUM_CRITICAL_PRESSURE		REAL_UNKNOWN
#define Z_DARMSTADTIUM_CRITICAL_TEMPERATURE		REAL_UNKNOWN
#define Z_DARMSTADTIUM_FUSION_HEAT			REAL_UNKNOWN
#define Z_DARMSTADTIUM_VAPORIZATION_HEAT		REAL_UNKNOWN
#define Z_DARMSTADTIUM_SPECIFIC_HEAT			REAL_UNKNOWN
#define Z_DARMSTADTIUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_DARMSTADTIUM_NEEL_POINT			REAL_NOT_APPLICABLE
#define Z_DARMSTADTIUM_THERMAL_CONDUCTIVITY		REAL_UNKNOWN
#define Z_DARMSTADTIUM_THERMAL_EXPANSION		REAL_UNKNOWN
#define Z_DARMSTADTIUM_DENSITY				REAL_UNKNOWN
#define Z_DARMSTADTIUM_LIQUID_DENSITY			REAL_UNKNOWN
#define Z_DARMSTADTIUM_MOLAR_VOLUME			REAL_MISSING
#define Z_DARMSTADTIUM_BRINELL_HARDNESS			REAL_UNKNOWN
#define Z_DARMSTADTIUM_MOHS_HARDNESS			REAL_UNKNOWN
#define Z_DARMSTADTIUM_VICKERS_HARDNESS			REAL_UNKNOWN
#define Z_DARMSTADTIUM_BULK_MODULUS			REAL_UNKNOWN
#define Z_DARMSTADTIUM_SHEAR_MODULUS			REAL_UNKNOWN
#define Z_DARMSTADTIUM_YOUNG_MODULUS			REAL_UNKNOWN
#define Z_DARMSTADTIUM_POISSON_RATIO			REAL_UNKNOWN
#define Z_DARMSTADTIUM_REFRACTIVE_INDEX			REAL_UNKNOWN
#define Z_DARMSTADTIUM_SOUND_SPEED			REAL_UNKNOWN
#define Z_DARMSTADTIUM_ELECTRICAL_TYPE			KEY_UNKNOWN
#define Z_DARMSTADTIUM_ELECTRICAL_CONDUCTIVITY		REAL_UNKNOWN
#define Z_DARMSTADTIUM_ELECTRICAL_RESISTIVITY		REAL_UNKNOWN
#define Z_DARMSTADTIUM_SUPERCONDUCTING_POINT		REAL_UNKNOWN
#define Z_DARMSTADTIUM_MAGNETIC_TYPE			KEY_UNKNOWN
#define Z_DARMSTADTIUM_CURIE_POINT			REAL_NOT_APPLICABLE
#define Z_DARMSTADTIUM_MASS_MAGNETIC_SUSCEPTIBILITY	REAL_UNKNOWN
#define Z_DARMSTADTIUM_MOLAR_MAGNETIC_SUSCEPTIBILITY	REAL_UNKNOWN
#define Z_DARMSTADTIUM_VOLUME_MAGNETIC_SUSCEPTIBILITY	REAL_UNKNOWN

/* MARK: - 111 - Rg */

#define Z_ROENTGENIUM_SYMBOL				Rg
#define Z_ROENTGENIUM_ATOMIC_NUMBER			111
#define Z_ROENTGENIUM_GROUP				11
#define Z_ROENTGENIUM_PERIOD				7
#define Z_ROENTGENIUM_BLOCK				Z_CHEMICAL_ELEMENT_BLOCK_D
#define Z_ROENTGENIUM_CLASS				Z_CHEMICAL_ELEMENT_CLASS_TRANSITION_METAL
#define Z_ROENTGENIUM_METAL_STATE			Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_ROENTGENIUM_SPACE_GROUP_NUMBER		INTEGER_MISSING
#define Z_ROENTGENIUM_ATOMIC_MASS			280.0
#define Z_ROENTGENIUM_ATOMIC_RADIUS			REAL_MISSING
#define Z_ROENTGENIUM_COVALENT_RADIUS			REAL_UNKNOWN
#define Z_ROENTGENIUM_VAN_DER_WAALS_RADIUS		REAL_MISSING
#define Z_ROENTGENIUM_DECAY_MODE			Z_CHEMICAL_ELEMENT_DECAY_MODE_ALPHA_EMISSION
#define Z_ROENTGENIUM_NEUTRON_CROSS_SECTION		REAL_UNKNOWN
#define Z_ROENTGENIUM_NEUTRON_MASS_ABSORPTION		REAL_UNKNOWN
#define Z_ROENTGENIUM_VALENCE				INTEGER_UNKNOWN
#define Z_ROENTGENIUM_ELECTRONEGATIVITY			REAL_MISSING
#define Z_ROENTGENIUM_ELECTRON_AFFINITY			REAL_MISSING
#define Z_ROENTGENIUM_MELTING_POINT			REAL_MISSING
#define Z_ROENTGENIUM_BOILING_POINT			REAL_MISSING
#define Z_ROENTGENIUM_CRITICAL_PRESSURE			REAL_MISSING
#define Z_ROENTGENIUM_CRITICAL_TEMPERATURE		REAL_MISSING
#define Z_ROENTGENIUM_FUSION_HEAT			REAL_MISSING
#define Z_ROENTGENIUM_VAPORIZATION_HEAT			REAL_MISSING
#define Z_ROENTGENIUM_SPECIFIC_HEAT			REAL_UNKNOWN
#define Z_ROENTGENIUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_ROENTGENIUM_NEEL_POINT			REAL_NOT_APPLICABLE
#define Z_ROENTGENIUM_THERMAL_CONDUCTIVITY		REAL_MISSING
#define Z_ROENTGENIUM_THERMAL_EXPANSION			REAL_MISSING
#define Z_ROENTGENIUM_DENSITY				REAL_MISSING
#define Z_ROENTGENIUM_LIQUID_DENSITY			REAL_MISSING
#define Z_ROENTGENIUM_MOLAR_VOLUME			REAL_MISSING
#define Z_ROENTGENIUM_BRINELL_HARDNESS			REAL_MISSING
#define Z_ROENTGENIUM_MOHS_HARDNESS			REAL_MISSING
#define Z_ROENTGENIUM_VICKERS_HARDNESS			REAL_MISSING
#define Z_ROENTGENIUM_BULK_MODULUS			REAL_MISSING
#define Z_ROENTGENIUM_SHEAR_MODULUS			REAL_MISSING
#define Z_ROENTGENIUM_YOUNG_MODULUS			REAL_MISSING
#define Z_ROENTGENIUM_POISSON_RATIO			REAL_MISSING
#define Z_ROENTGENIUM_REFRACTIVE_INDEX			REAL_MISSING
#define Z_ROENTGENIUM_SOUND_SPEED			REAL_MISSING
#define Z_ROENTGENIUM_ELECTRICAL_TYPE			KEY_UNKNOWN
#define Z_ROENTGENIUM_ELECTRICAL_CONDUCTIVITY		REAL_MISSING
#define Z_ROENTGENIUM_ELECTRICAL_RESISTIVITY		REAL_MISSING
#define Z_ROENTGENIUM_SUPERCONDUCTING_POINT		REAL_MISSING
#define Z_ROENTGENIUM_MAGNETIC_TYPE			KEY_MISSING
#define Z_ROENTGENIUM_CURIE_POINT			REAL_NOT_APPLICABLE
#define Z_ROENTGENIUM_MASS_MAGNETIC_SUSCEPTIBILITY	REAL_UNKNOWN
#define Z_ROENTGENIUM_MOLAR_MAGNETIC_SUSCEPTIBILITY	REAL_UNKNOWN
#define Z_ROENTGENIUM_VOLUME_MAGNETIC_SUSCEPTIBILITY	REAL_UNKNOWN

/* MARK: - 112 - Cn */

#define Z_COPERNICIUM_SYMBOL				Cn
#define Z_COPERNICIUM_ATOMIC_NUMBER			112
#define Z_COPERNICIUM_GROUP				12
#define Z_COPERNICIUM_PERIOD				7
#define Z_COPERNICIUM_BLOCK				Z_CHEMICAL_ELEMENT_BLOCK_D
#define Z_COPERNICIUM_CLASS				Z_CHEMICAL_ELEMENT_CLASS_TRANSITION_METAL
#define Z_COPERNICIUM_METAL_STATE			Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_COPERNICIUM_SPACE_GROUP_NUMBER		INTEGER_MISSING
#define Z_COPERNICIUM_ATOMIC_MASS			285.0
#define Z_COPERNICIUM_ATOMIC_RADIUS			REAL_MISSING
#define Z_COPERNICIUM_COVALENT_RADIUS			REAL_UNKNOWN
#define Z_COPERNICIUM_VAN_DER_WAALS_RADIUS		REAL_MISSING
#define Z_COPERNICIUM_DECAY_MODE			Z_CHEMICAL_ELEMENT_DECAY_MODE_ALPHA_EMISSION
#define Z_COPERNICIUM_NEUTRON_CROSS_SECTION		REAL_UNKNOWN
#define Z_COPERNICIUM_NEUTRON_MASS_ABSORPTION		REAL_UNKNOWN
#define Z_COPERNICIUM_VALENCE				INTEGER_UNKNOWN
#define Z_COPERNICIUM_ELECTRONEGATIVITY			REAL_MISSING
#define Z_COPERNICIUM_ELECTRON_AFFINITY			REAL_MISSING
#define Z_COPERNICIUM_MELTING_POINT			REAL_MISSING
#define Z_COPERNICIUM_BOILING_POINT			REAL_MISSING
#define Z_COPERNICIUM_CRITICAL_PRESSURE			REAL_MISSING
#define Z_COPERNICIUM_CRITICAL_TEMPERATURE		REAL_MISSING
#define Z_COPERNICIUM_FUSION_HEAT			REAL_MISSING
#define Z_COPERNICIUM_VAPORIZATION_HEAT			REAL_MISSING
#define Z_COPERNICIUM_SPECIFIC_HEAT			REAL_UNKNOWN
#define Z_COPERNICIUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_COPERNICIUM_NEEL_POINT			REAL_NOT_APPLICABLE
#define Z_COPERNICIUM_THERMAL_CONDUCTIVITY		REAL_MISSING
#define Z_COPERNICIUM_THERMAL_EXPANSION			REAL_MISSING
#define Z_COPERNICIUM_DENSITY				REAL_MISSING
#define Z_COPERNICIUM_LIQUID_DENSITY			REAL_MISSING
#define Z_COPERNICIUM_MOLAR_VOLUME			REAL_MISSING
#define Z_COPERNICIUM_BRINELL_HARDNESS			REAL_MISSING
#define Z_COPERNICIUM_MOHS_HARDNESS			REAL_MISSING
#define Z_COPERNICIUM_VICKERS_HARDNESS			REAL_MISSING
#define Z_COPERNICIUM_BULK_MODULUS			REAL_MISSING
#define Z_COPERNICIUM_SHEAR_MODULUS			REAL_MISSING
#define Z_COPERNICIUM_YOUNG_MODULUS			REAL_MISSING
#define Z_COPERNICIUM_POISSON_RATIO			REAL_MISSING
#define Z_COPERNICIUM_REFRACTIVE_INDEX			REAL_MISSING
#define Z_COPERNICIUM_SOUND_SPEED			REAL_MISSING
#define Z_COPERNICIUM_ELECTRICAL_TYPE			KEY_UNKNOWN
#define Z_COPERNICIUM_ELECTRICAL_CONDUCTIVITY		REAL_MISSING
#define Z_COPERNICIUM_ELECTRICAL_RESISTIVITY		REAL_MISSING
#define Z_COPERNICIUM_SUPERCONDUCTING_POINT		REAL_MISSING
#define Z_COPERNICIUM_MAGNETIC_TYPE			KEY_MISSING
#define Z_COPERNICIUM_CURIE_POINT			REAL_NOT_APPLICABLE
#define Z_COPERNICIUM_MASS_MAGNETIC_SUSCEPTIBILITY	REAL_UNKNOWN
#define Z_COPERNICIUM_MOLAR_MAGNETIC_SUSCEPTIBILITY	REAL_UNKNOWN
#define Z_COPERNICIUM_VOLUME_MAGNETIC_SUSCEPTIBILITY	REAL_UNKNOWN

/* MARK: - 113 - Nh */

#define Z_NIHONIUM_SYMBOL				Nh
#define Z_NIHONIUM_ATOMIC_NUMBER			113
#define Z_NIHONIUM_GROUP				13
#define Z_NIHONIUM_PERIOD				7
#define Z_NIHONIUM_BLOCK				Z_CHEMICAL_ELEMENT_BLOCK_P
#define Z_NIHONIUM_CLASS				Z_CHEMICAL_ELEMENT_CLASS_TRANSITION_METAL
#define Z_NIHONIUM_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_NIHONIUM_SPACE_GROUP_NUMBER			INTEGER_UNKNOWN
#define Z_NIHONIUM_ATOMIC_MASS				284.0
#define Z_NIHONIUM_ATOMIC_RADIUS			REAL_UNKNOWN
#define Z_NIHONIUM_COVALENT_RADIUS			REAL_UNKNOWN
#define Z_NIHONIUM_VAN_DER_WAALS_RADIUS			REAL_UNKNOWN
#define Z_NIHONIUM_DECAY_MODE				Z_CHEMICAL_ELEMENT_DECAY_MODE_ALPHA_EMISSION
#define Z_NIHONIUM_NEUTRON_CROSS_SECTION		REAL_UNKNOWN
#define Z_NIHONIUM_NEUTRON_MASS_ABSORPTION		REAL_UNKNOWN
#define Z_NIHONIUM_VALENCE				INTEGER_UNKNOWN
#define Z_NIHONIUM_ELECTRONEGATIVITY			REAL_UNKNOWN
#define Z_NIHONIUM_ELECTRON_AFFINITY			REAL_UNKNOWN
#define Z_NIHONIUM_MELTING_POINT			REAL_UNKNOWN
#define Z_NIHONIUM_BOILING_POINT			REAL_UNKNOWN
#define Z_NIHONIUM_CRITICAL_PRESSURE			REAL_UNKNOWN
#define Z_NIHONIUM_CRITICAL_TEMPERATURE			REAL_UNKNOWN
#define Z_NIHONIUM_FUSION_HEAT				REAL_UNKNOWN
#define Z_NIHONIUM_VAPORIZATION_HEAT			REAL_UNKNOWN
#define Z_NIHONIUM_SPECIFIC_HEAT			REAL_UNKNOWN
#define Z_NIHONIUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_NIHONIUM_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_NIHONIUM_THERMAL_CONDUCTIVITY			REAL_UNKNOWN
#define Z_NIHONIUM_THERMAL_EXPANSION			REAL_UNKNOWN
#define Z_NIHONIUM_DENSITY				REAL_UNKNOWN
#define Z_NIHONIUM_LIQUID_DENSITY			REAL_UNKNOWN
#define Z_NIHONIUM_MOLAR_VOLUME				REAL_MISSING
#define Z_NIHONIUM_BRINELL_HARDNESS			REAL_UNKNOWN
#define Z_NIHONIUM_MOHS_HARDNESS			REAL_UNKNOWN
#define Z_NIHONIUM_VICKERS_HARDNESS			REAL_UNKNOWN
#define Z_NIHONIUM_BULK_MODULUS				REAL_UNKNOWN
#define Z_NIHONIUM_SHEAR_MODULUS			REAL_UNKNOWN
#define Z_NIHONIUM_YOUNG_MODULUS			REAL_UNKNOWN
#define Z_NIHONIUM_POISSON_RATIO			REAL_UNKNOWN
#define Z_NIHONIUM_REFRACTIVE_INDEX			REAL_UNKNOWN
#define Z_NIHONIUM_SOUND_SPEED				REAL_UNKNOWN
#define Z_NIHONIUM_ELECTRICAL_TYPE			KEY_UNKNOWN
#define Z_NIHONIUM_ELECTRICAL_CONDUCTIVITY		REAL_UNKNOWN
#define Z_NIHONIUM_ELECTRICAL_RESISTIVITY		REAL_UNKNOWN
#define Z_NIHONIUM_SUPERCONDUCTING_POINT		REAL_UNKNOWN
#define Z_NIHONIUM_MAGNETIC_TYPE			KEY_UNKNOWN
#define Z_NIHONIUM_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_NIHONIUM_MASS_MAGNETIC_SUSCEPTIBILITY		REAL_UNKNOWN
#define Z_NIHONIUM_MOLAR_MAGNETIC_SUSCEPTIBILITY	REAL_UNKNOWN
#define Z_NIHONIUM_VOLUME_MAGNETIC_SUSCEPTIBILITY	REAL_UNKNOWN

/* MARK: - 114 - Fl */

#define Z_FLEROVIUM_SYMBOL				Fl
#define Z_FLEROVIUM_ATOMIC_NUMBER			114
#define Z_FLEROVIUM_GROUP				14
#define Z_FLEROVIUM_PERIOD				7
#define Z_FLEROVIUM_BLOCK				Z_CHEMICAL_ELEMENT_BLOCK_P
#define Z_FLEROVIUM_CLASS				Z_CHEMICAL_ELEMENT_CLASS_TRANSITION_METAL
#define Z_FLEROVIUM_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_FLEROVIUM_SPACE_GROUP_NUMBER			INTEGER_MISSING
#define Z_FLEROVIUM_ATOMIC_MASS				289.0
#define Z_FLEROVIUM_ATOMIC_RADIUS			REAL_MISSING
#define Z_FLEROVIUM_COVALENT_RADIUS			REAL_UNKNOWN
#define Z_FLEROVIUM_VAN_DER_WAALS_RADIUS		REAL_MISSING
#define Z_FLEROVIUM_DECAY_MODE				Z_CHEMICAL_ELEMENT_DECAY_MODE_ALPHA_EMISSION
#define Z_FLEROVIUM_NEUTRON_CROSS_SECTION		REAL_UNKNOWN
#define Z_FLEROVIUM_NEUTRON_MASS_ABSORPTION		REAL_UNKNOWN
#define Z_FLEROVIUM_VALENCE				INTEGER_UNKNOWN
#define Z_FLEROVIUM_ELECTRONEGATIVITY			REAL_MISSING
#define Z_FLEROVIUM_ELECTRON_AFFINITY			REAL_MISSING
#define Z_FLEROVIUM_MELTING_POINT			REAL_MISSING
#define Z_FLEROVIUM_BOILING_POINT			REAL_MISSING
#define Z_FLEROVIUM_CRITICAL_PRESSURE			REAL_MISSING
#define Z_FLEROVIUM_CRITICAL_TEMPERATURE		REAL_MISSING
#define Z_FLEROVIUM_FUSION_HEAT				REAL_MISSING
#define Z_FLEROVIUM_VAPORIZATION_HEAT			REAL_MISSING
#define Z_FLEROVIUM_SPECIFIC_HEAT			REAL_UNKNOWN
#define Z_FLEROVIUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_FLEROVIUM_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_FLEROVIUM_THERMAL_CONDUCTIVITY		REAL_MISSING
#define Z_FLEROVIUM_THERMAL_EXPANSION			REAL_MISSING
#define Z_FLEROVIUM_DENSITY				REAL_MISSING
#define Z_FLEROVIUM_LIQUID_DENSITY			REAL_MISSING
#define Z_FLEROVIUM_MOLAR_VOLUME			REAL_MISSING
#define Z_FLEROVIUM_BRINELL_HARDNESS			REAL_MISSING
#define Z_FLEROVIUM_MOHS_HARDNESS			REAL_MISSING
#define Z_FLEROVIUM_VICKERS_HARDNESS			REAL_MISSING
#define Z_FLEROVIUM_BULK_MODULUS			REAL_MISSING
#define Z_FLEROVIUM_SHEAR_MODULUS			REAL_MISSING
#define Z_FLEROVIUM_YOUNG_MODULUS			REAL_MISSING
#define Z_FLEROVIUM_POISSON_RATIO			REAL_MISSING
#define Z_FLEROVIUM_REFRACTIVE_INDEX			REAL_MISSING
#define Z_FLEROVIUM_SOUND_SPEED				REAL_MISSING
#define Z_FLEROVIUM_ELECTRICAL_TYPE			KEY_UNKNOWN
#define Z_FLEROVIUM_ELECTRICAL_CONDUCTIVITY		REAL_MISSING
#define Z_FLEROVIUM_ELECTRICAL_RESISTIVITY		REAL_MISSING
#define Z_FLEROVIUM_SUPERCONDUCTING_POINT		REAL_MISSING
#define Z_FLEROVIUM_MAGNETIC_TYPE			KEY_MISSING
#define Z_FLEROVIUM_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_FLEROVIUM_MASS_MAGNETIC_SUSCEPTIBILITY	REAL_UNKNOWN
#define Z_FLEROVIUM_MOLAR_MAGNETIC_SUSCEPTIBILITY	REAL_UNKNOWN
#define Z_FLEROVIUM_VOLUME_MAGNETIC_SUSCEPTIBILITY	REAL_UNKNOWN

/* MARK: - 115 - Mc */

#define Z_MOSCOVIUM_SYMBOL				Mc
#define Z_MOSCOVIUM_ATOMIC_NUMBER			115
#define Z_MOSCOVIUM_GROUP				15
#define Z_MOSCOVIUM_PERIOD				7
#define Z_MOSCOVIUM_BLOCK				Z_CHEMICAL_ELEMENT_BLOCK_P
#define Z_MOSCOVIUM_CLASS				Z_CHEMICAL_ELEMENT_CLASS_TRANSITION_METAL
#define Z_MOSCOVIUM_METAL_STATE				Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_MOSCOVIUM_SPACE_GROUP_NUMBER			INTEGER_MISSING
#define Z_MOSCOVIUM_ATOMIC_MASS				288.0
#define Z_MOSCOVIUM_ATOMIC_RADIUS			REAL_MISSING
#define Z_MOSCOVIUM_COVALENT_RADIUS			REAL_UNKNOWN
#define Z_MOSCOVIUM_VAN_DER_WAALS_RADIUS		REAL_MISSING
#define Z_MOSCOVIUM_DECAY_MODE				Z_CHEMICAL_ELEMENT_DECAY_MODE_ALPHA_EMISSION
#define Z_MOSCOVIUM_NEUTRON_CROSS_SECTION		REAL_UNKNOWN
#define Z_MOSCOVIUM_NEUTRON_MASS_ABSORPTION		REAL_UNKNOWN
#define Z_MOSCOVIUM_VALENCE				INTEGER_UNKNOWN
#define Z_MOSCOVIUM_ELECTRONEGATIVITY			REAL_MISSING
#define Z_MOSCOVIUM_ELECTRON_AFFINITY			REAL_MISSING
#define Z_MOSCOVIUM_MELTING_POINT			REAL_MISSING
#define Z_MOSCOVIUM_BOILING_POINT			REAL_MISSING
#define Z_MOSCOVIUM_CRITICAL_PRESSURE			REAL_MISSING
#define Z_MOSCOVIUM_CRITICAL_TEMPERATURE		REAL_MISSING
#define Z_MOSCOVIUM_FUSION_HEAT				REAL_MISSING
#define Z_MOSCOVIUM_VAPORIZATION_HEAT			REAL_MISSING
#define Z_MOSCOVIUM_SPECIFIC_HEAT			REAL_UNKNOWN
#define Z_MOSCOVIUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_MOSCOVIUM_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_MOSCOVIUM_THERMAL_CONDUCTIVITY		REAL_MISSING
#define Z_MOSCOVIUM_THERMAL_EXPANSION			REAL_MISSING
#define Z_MOSCOVIUM_DENSITY				REAL_MISSING
#define Z_MOSCOVIUM_LIQUID_DENSITY			REAL_MISSING
#define Z_MOSCOVIUM_MOLAR_VOLUME			REAL_MISSING
#define Z_MOSCOVIUM_BRINELL_HARDNESS			REAL_MISSING
#define Z_MOSCOVIUM_MOHS_HARDNESS			REAL_MISSING
#define Z_MOSCOVIUM_VICKERS_HARDNESS			REAL_MISSING
#define Z_MOSCOVIUM_BULK_MODULUS			REAL_MISSING
#define Z_MOSCOVIUM_SHEAR_MODULUS			REAL_MISSING
#define Z_MOSCOVIUM_YOUNG_MODULUS			REAL_MISSING
#define Z_MOSCOVIUM_POISSON_RATIO			REAL_MISSING
#define Z_MOSCOVIUM_REFRACTIVE_INDEX			REAL_MISSING
#define Z_MOSCOVIUM_SOUND_SPEED				REAL_MISSING
#define Z_MOSCOVIUM_ELECTRICAL_TYPE			KEY_UNKNOWN
#define Z_MOSCOVIUM_ELECTRICAL_CONDUCTIVITY		REAL_MISSING
#define Z_MOSCOVIUM_ELECTRICAL_RESISTIVITY		REAL_MISSING
#define Z_MOSCOVIUM_SUPERCONDUCTING_POINT		REAL_MISSING
#define Z_MOSCOVIUM_MAGNETIC_TYPE			KEY_MISSING
#define Z_MOSCOVIUM_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_MOSCOVIUM_MASS_MAGNETIC_SUSCEPTIBILITY	REAL_UNKNOWN
#define Z_MOSCOVIUM_MOLAR_MAGNETIC_SUSCEPTIBILITY	REAL_UNKNOWN
#define Z_MOSCOVIUM_VOLUME_MAGNETIC_SUSCEPTIBILITY	REAL_UNKNOWN

/* MARK: - 116 - Lv */

#define Z_LIVERMORIUM_SYMBOL				Lv
#define Z_LIVERMORIUM_ATOMIC_NUMBER			116
#define Z_LIVERMORIUM_GROUP				16
#define Z_LIVERMORIUM_PERIOD				7
#define Z_LIVERMORIUM_BLOCK				Z_CHEMICAL_ELEMENT_BLOCK_P
#define Z_LIVERMORIUM_CLASS				Z_CHEMICAL_ELEMENT_CLASS_TRANSITION_METAL
#define Z_LIVERMORIUM_METAL_STATE			Z_CHEMICAL_ELEMENT_METAL_STATE_METAL
#define Z_LIVERMORIUM_SPACE_GROUP_NUMBER		INTEGER_UNKNOWN
#define Z_LIVERMORIUM_ATOMIC_MASS			293.0
#define Z_LIVERMORIUM_ATOMIC_RADIUS			REAL_UNKNOWN
#define Z_LIVERMORIUM_COVALENT_RADIUS			REAL_UNKNOWN
#define Z_LIVERMORIUM_VAN_DER_WAALS_RADIUS		REAL_UNKNOWN
#define Z_LIVERMORIUM_DECAY_MODE			Z_CHEMICAL_ELEMENT_DECAY_MODE_ALPHA_EMISSION
#define Z_LIVERMORIUM_NEUTRON_CROSS_SECTION		REAL_UNKNOWN
#define Z_LIVERMORIUM_NEUTRON_MASS_ABSORPTION		REAL_UNKNOWN
#define Z_LIVERMORIUM_VALENCE				INTEGER_UNKNOWN
#define Z_LIVERMORIUM_ELECTRONEGATIVITY			REAL_UNKNOWN
#define Z_LIVERMORIUM_ELECTRON_AFFINITY			REAL_UNKNOWN
#define Z_LIVERMORIUM_MELTING_POINT			REAL_UNKNOWN
#define Z_LIVERMORIUM_BOILING_POINT			REAL_UNKNOWN
#define Z_LIVERMORIUM_CRITICAL_PRESSURE			REAL_UNKNOWN
#define Z_LIVERMORIUM_CRITICAL_TEMPERATURE		REAL_UNKNOWN
#define Z_LIVERMORIUM_FUSION_HEAT			REAL_UNKNOWN
#define Z_LIVERMORIUM_VAPORIZATION_HEAT			REAL_UNKNOWN
#define Z_LIVERMORIUM_SPECIFIC_HEAT			REAL_UNKNOWN
#define Z_LIVERMORIUM_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_LIVERMORIUM_NEEL_POINT			REAL_NOT_APPLICABLE
#define Z_LIVERMORIUM_THERMAL_CONDUCTIVITY		REAL_UNKNOWN
#define Z_LIVERMORIUM_THERMAL_EXPANSION			REAL_UNKNOWN
#define Z_LIVERMORIUM_DENSITY				REAL_UNKNOWN
#define Z_LIVERMORIUM_LIQUID_DENSITY			REAL_UNKNOWN
#define Z_LIVERMORIUM_MOLAR_VOLUME			REAL_MISSING
#define Z_LIVERMORIUM_BRINELL_HARDNESS			REAL_UNKNOWN
#define Z_LIVERMORIUM_MOHS_HARDNESS			REAL_UNKNOWN
#define Z_LIVERMORIUM_VICKERS_HARDNESS			REAL_UNKNOWN
#define Z_LIVERMORIUM_BULK_MODULUS			REAL_UNKNOWN
#define Z_LIVERMORIUM_SHEAR_MODULUS			REAL_UNKNOWN
#define Z_LIVERMORIUM_YOUNG_MODULUS			REAL_UNKNOWN
#define Z_LIVERMORIUM_POISSON_RATIO			REAL_UNKNOWN
#define Z_LIVERMORIUM_REFRACTIVE_INDEX			REAL_UNKNOWN
#define Z_LIVERMORIUM_SOUND_SPEED			REAL_UNKNOWN
#define Z_LIVERMORIUM_ELECTRICAL_TYPE			KEY_UNKNOWN
#define Z_LIVERMORIUM_ELECTRICAL_CONDUCTIVITY		REAL_UNKNOWN
#define Z_LIVERMORIUM_ELECTRICAL_RESISTIVITY		REAL_UNKNOWN
#define Z_LIVERMORIUM_SUPERCONDUCTING_POINT		REAL_UNKNOWN
#define Z_LIVERMORIUM_MAGNETIC_TYPE			KEY_UNKNOWN
#define Z_LIVERMORIUM_CURIE_POINT			REAL_NOT_APPLICABLE
#define Z_LIVERMORIUM_MASS_MAGNETIC_SUSCEPTIBILITY	REAL_UNKNOWN
#define Z_LIVERMORIUM_MOLAR_MAGNETIC_SUSCEPTIBILITY	REAL_UNKNOWN
#define Z_LIVERMORIUM_VOLUME_MAGNETIC_SUSCEPTIBILITY	REAL_UNKNOWN

/* MARK: - 117 - Ts */

#define Z_TENNESSINE_SYMBOL				Ts
#define Z_TENNESSINE_ATOMIC_NUMBER			117
#define Z_TENNESSINE_GROUP				17
#define Z_TENNESSINE_PERIOD				7
#define Z_TENNESSINE_BLOCK				Z_CHEMICAL_ELEMENT_BLOCK_P
#define Z_TENNESSINE_CLASS				Z_CHEMICAL_ELEMENT_CLASS_HALOGEN
#define Z_TENNESSINE_METAL_STATE			KEY_UNKNOWN
#define Z_TENNESSINE_SPACE_GROUP_NUMBER			INTEGER_MISSING
#define Z_TENNESSINE_ATOMIC_MASS			294.0
#define Z_TENNESSINE_ATOMIC_RADIUS			REAL_MISSING
#define Z_TENNESSINE_COVALENT_RADIUS			REAL_UNKNOWN
#define Z_TENNESSINE_VAN_DER_WAALS_RADIUS		REAL_MISSING
#define Z_TENNESSINE_DECAY_MODE				KEY_UNKNOWN
#define Z_TENNESSINE_NEUTRON_CROSS_SECTION		REAL_UNKNOWN
#define Z_TENNESSINE_NEUTRON_MASS_ABSORPTION		REAL_UNKNOWN
#define Z_TENNESSINE_VALENCE				INTEGER_UNKNOWN
#define Z_TENNESSINE_ELECTRONEGATIVITY			REAL_MISSING
#define Z_TENNESSINE_ELECTRON_AFFINITY			REAL_MISSING
#define Z_TENNESSINE_MELTING_POINT			REAL_MISSING
#define Z_TENNESSINE_BOILING_POINT			REAL_MISSING
#define Z_TENNESSINE_CRITICAL_PRESSURE			REAL_MISSING
#define Z_TENNESSINE_CRITICAL_TEMPERATURE		REAL_MISSING
#define Z_TENNESSINE_FUSION_HEAT			REAL_MISSING
#define Z_TENNESSINE_VAPORIZATION_HEAT			REAL_MISSING
#define Z_TENNESSINE_SPECIFIC_HEAT			REAL_UNKNOWN
#define Z_TENNESSINE_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_TENNESSINE_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_TENNESSINE_THERMAL_CONDUCTIVITY		REAL_MISSING
#define Z_TENNESSINE_THERMAL_EXPANSION			REAL_MISSING
#define Z_TENNESSINE_DENSITY				REAL_MISSING
#define Z_TENNESSINE_LIQUID_DENSITY			REAL_MISSING
#define Z_TENNESSINE_MOLAR_VOLUME			REAL_MISSING
#define Z_TENNESSINE_BRINELL_HARDNESS			REAL_MISSING
#define Z_TENNESSINE_MOHS_HARDNESS			REAL_MISSING
#define Z_TENNESSINE_VICKERS_HARDNESS			REAL_MISSING
#define Z_TENNESSINE_BULK_MODULUS			REAL_MISSING
#define Z_TENNESSINE_SHEAR_MODULUS			REAL_MISSING
#define Z_TENNESSINE_YOUNG_MODULUS			REAL_MISSING
#define Z_TENNESSINE_POISSON_RATIO			REAL_MISSING
#define Z_TENNESSINE_REFRACTIVE_INDEX			REAL_MISSING
#define Z_TENNESSINE_SOUND_SPEED			REAL_MISSING
#define Z_TENNESSINE_ELECTRICAL_TYPE			KEY_UNKNOWN
#define Z_TENNESSINE_ELECTRICAL_CONDUCTIVITY		REAL_MISSING
#define Z_TENNESSINE_ELECTRICAL_RESISTIVITY		REAL_MISSING
#define Z_TENNESSINE_SUPERCONDUCTING_POINT		REAL_MISSING
#define Z_TENNESSINE_MAGNETIC_TYPE			KEY_MISSING
#define Z_TENNESSINE_CURIE_POINT			REAL_NOT_APPLICABLE
#define Z_TENNESSINE_MASS_MAGNETIC_SUSCEPTIBILITY	REAL_UNKNOWN
#define Z_TENNESSINE_MOLAR_MAGNETIC_SUSCEPTIBILITY	REAL_UNKNOWN
#define Z_TENNESSINE_VOLUME_MAGNETIC_SUSCEPTIBILITY	REAL_UNKNOWN

/* MARK: - 118 - Og */

#define Z_OGANESSON_SYMBOL				Og
#define Z_OGANESSON_ATOMIC_NUMBER			118
#define Z_OGANESSON_GROUP				18
#define Z_OGANESSON_PERIOD				7
#define Z_OGANESSON_BLOCK				Z_CHEMICAL_ELEMENT_BLOCK_P
#define Z_OGANESSON_CLASS				Z_CHEMICAL_ELEMENT_CLASS_NOBLE_GAS
#define Z_OGANESSON_METAL_STATE				KEY_UNKNOWN
#define Z_OGANESSON_SPACE_GROUP_NUMBER			INTEGER_MISSING
#define Z_OGANESSON_ATOMIC_MASS				294.0
#define Z_OGANESSON_ATOMIC_RADIUS			REAL_MISSING
#define Z_OGANESSON_COVALENT_RADIUS			REAL_UNKNOWN
#define Z_OGANESSON_VAN_DER_WAALS_RADIUS		REAL_MISSING
#define Z_OGANESSON_DECAY_MODE				Z_CHEMICAL_ELEMENT_DECAY_MODE_ALPHA_EMISSION
#define Z_OGANESSON_NEUTRON_CROSS_SECTION		REAL_UNKNOWN
#define Z_OGANESSON_NEUTRON_MASS_ABSORPTION		REAL_UNKNOWN
#define Z_OGANESSON_VALENCE				INTEGER_UNKNOWN
#define Z_OGANESSON_ELECTRONEGATIVITY			REAL_MISSING
#define Z_OGANESSON_ELECTRON_AFFINITY			REAL_MISSING
#define Z_OGANESSON_MELTING_POINT			REAL_MISSING
#define Z_OGANESSON_BOILING_POINT			REAL_MISSING
#define Z_OGANESSON_CRITICAL_PRESSURE			REAL_MISSING
#define Z_OGANESSON_CRITICAL_TEMPERATURE		REAL_MISSING
#define Z_OGANESSON_FUSION_HEAT				REAL_MISSING
#define Z_OGANESSON_VAPORIZATION_HEAT			REAL_MISSING
#define Z_OGANESSON_SPECIFIC_HEAT			REAL_UNKNOWN
#define Z_OGANESSON_ADIABATIC_INDEX			KEY_NOT_APPLICABLE
#define Z_OGANESSON_NEEL_POINT				REAL_NOT_APPLICABLE
#define Z_OGANESSON_THERMAL_CONDUCTIVITY		REAL_MISSING
#define Z_OGANESSON_THERMAL_EXPANSION			REAL_MISSING
#define Z_OGANESSON_DENSITY				REAL_MISSING
#define Z_OGANESSON_LIQUID_DENSITY			REAL_MISSING
#define Z_OGANESSON_MOLAR_VOLUME			REAL_MISSING
#define Z_OGANESSON_BRINELL_HARDNESS			REAL_MISSING
#define Z_OGANESSON_MOHS_HARDNESS			REAL_MISSING
#define Z_OGANESSON_VICKERS_HARDNESS			REAL_MISSING
#define Z_OGANESSON_BULK_MODULUS			REAL_MISSING
#define Z_OGANESSON_SHEAR_MODULUS			REAL_MISSING
#define Z_OGANESSON_YOUNG_MODULUS			REAL_MISSING
#define Z_OGANESSON_POISSON_RATIO			REAL_MISSING
#define Z_OGANESSON_REFRACTIVE_INDEX			REAL_MISSING
#define Z_OGANESSON_SOUND_SPEED				REAL_MISSING
#define Z_OGANESSON_ELECTRICAL_TYPE			KEY_UNKNOWN
#define Z_OGANESSON_ELECTRICAL_CONDUCTIVITY		REAL_MISSING
#define Z_OGANESSON_ELECTRICAL_RESISTIVITY		REAL_MISSING
#define Z_OGANESSON_SUPERCONDUCTING_POINT		REAL_MISSING
#define Z_OGANESSON_MAGNETIC_TYPE			KEY_MISSING
#define Z_OGANESSON_CURIE_POINT				REAL_NOT_APPLICABLE
#define Z_OGANESSON_MASS_MAGNETIC_SUSCEPTIBILITY	REAL_UNKNOWN
#define Z_OGANESSON_MOLAR_MAGNETIC_SUSCEPTIBILITY	REAL_UNKNOWN
#define Z_OGANESSON_VOLUME_MAGNETIC_SUSCEPTIBILITY	REAL_UNKNOWN

#endif /* _Z_constants_chemical_elements_H_ */
